/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * (c) Copyright 2006,2007,2008,2009 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2009 Dmitry Gromada <gromada@jarios.org>
 *
 * Transport layer driver for IDE hard disk
 */

#include <ata/libata.h>
#include <debug.h>
#include <time.h>
#include "drv.h"

ata_driver_t idedisk_driver = {
  .media_types = (1 << ATA_MEDIA_DISK),
  .phys_ifaces = (1 << ATA_IFACE_IDE),

  .match = idedisk_match,
  .init = idedisk_init,
  .prepare_cmd = pata_prepare_cmd,
  .post_issue_cmd = idedisk_post_issue_cmd,
  .process_rq = ata_process_rq,
  .timeout_handler = pata_timeout_handler,
  .ready_cmd_tag = pata_ready_cmd_tag,
  .prep_cmd_retry = idedisk_prep_cmd_retry,
  .error_handler = idedisk_error_handler,
  .irq_handler = pata_irq_handler,
  .write_taskfile = ata_direct_write_taskfile,
  .read_taskfile = ata_direct_read_taskfile,
  .read_status = ata_direct_read_status
};

bool idedisk_match(ata_host_t *host, ata_device_t *dev)
{
  return true;
}

int idedisk_init(ata_device_t *dev)
{
  ata_set_pio_mode(dev, 4);
  dev->io_prot = ATA_PROT_PIO;
  dev->ctl_cmd |= DEVCTL_NIEN;  /* disable intrq asserting */
  list_init_head(&dev->cmd_queue.cmds);

  return 0;
}

bool idedisk_cmd_nestable(ata_device_t *dev, ata_cmd_t *cmd)
{
  return false;
}

void idedisk_prep_cmd_retry(ata_cmd_t *cmd)
{

}

void idedisk_error_handler(ata_device_t *dev, int err_type, uint32_t status)
{
}

int idedisk_post_issue_cmd(ata_device_t *dev, ata_cmd_t *cmd)
{
  int ret = 0;
  uint32_t nsects;
  void (*pio_fn)(ata_cmd_t*, void*, uint32_t);
  struct timespec ts = {0, ATA_STATUS_TIMEOUT};
  ata_status_t stat;
  blk_request_t *rq = cmd->rq;

  /* write the first block in case of pio out protocol with enabled intrq */
  if (ata_pio_protocol(cmd->protocol) && blkreq_rw(rq) && ata_intrq_enabled(dev)) {
    nanosleep(&ts, NULL);
    if (wait_dev_ready(dev, &stat)) {
      if (cmd->protocol == ATA_PROT_MULTI_PIO)
        nsects = MIN(dev->mult_count, rq->nr_sectors);
      else
        nsects = 1;

      pio_fn = dev->set_features.pio_8bit ? pio_arrayb : pio_arrayw;
      do_blk_pio(cmd, nsects, pio_fn);
    } else {
      ret = -1;
    }
  }

  return ret;
}
