/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * (c) Copyright 2006,2007,2008,2009 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2009 Dmitry Gromada <gromada@jarios.org>
 *
 * Base PCI ata controller definitions
 */

#ifndef __GENERIC_H__
#define __GENERIC_H__

#include <sys/types.h>
#include <ata/ata.h>
#include <pci/pci.h>
#include <dma.h>

#define DECLARE_FIND_DESC_FN(fn_name, descriptors)                              \
  static typeof(descriptors[0]) *fn_name(pci_dev_t *dev)                        \
  {                                                                             \
    int i;                                                                      \
                                                                                \
    for (i = 0; i < ARRAY_LENGTH(descriptors); i++) {                           \
      if ((dev->ident.vendor_id == descriptors[i].vend_id) &&                   \
          (dev->ident.device_id == descriptors[i].dev_id))                      \
        break;                                                                  \
    }                                                                           \
                                                                                \
    ASSERT(i < ARRAY_LENGTH(descriptors), "");                                  \
                                                                                \
    return &descriptors[i];                                                     \
  }                                                                             \

#define DECLARE_SCAN_HOSTS_FN(fn_name, descriptors, devices, nr_devices)        \
  static int fn_name(void)                                                      \
  {                                                                             \
    int i;                                                                      \
    pci_devid_t ids[ARRAY_LENGTH(descriptors)];                                 \
                                                                                \
    if (devices != NULL)                                                        \
      return nr_devices;                                                        \
                                                                                \
    for (i = 0; i < ARRAY_LENGTH(descriptors); i++) {                           \
      ids[i].vendor = descriptors[i].vend_id;                                   \
      ids[i].product = descriptors[i].dev_id;                                   \
    }                                                                           \
    nr_devices = pci_search_devices(&devices, ids, ARRAY_LENGTH(descriptors));  \
    ata_print_pci_devices(devices, nr_devices);                                 \
                                                                                \
    return (nr_devices == -1) ? -errno : nr_devices;                            \
  }                                                                             \

void ata_print_pci_devices(pci_dev_t *devs, int nr_devs);
int ata_pci_alloc_dma_res(ata_port_t *port, uint32_t min_chunk_size,
                          uint32_t max_tsize);
int ata_pci_init_dma_chan(ata_port_t *port, uint32_t desc_tbl_boundary,
                          uint32_t res_area_size, uint32_t max_single_tsize);
void ata_pci_destroy_dma_chan(dma_channel_t *chan);

static inline void ata_pci_init_host(ata_host_t *host, pci_dev_t *dev)
{
  host->bus_private = dev;
}

#endif /* __GENERIC_H__ */
