/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * (c) Copyright 2006,2007,2008,2009 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2009 Dmitry Gromada <gromada@jarios.org>
 *
 * Generic IDE pci controller definitions
 */

#ifndef __GENERIC_IDE_H__
#define __GENERIC_IDE_H__

#include <sys/types.h>
#include <pci/pci.h>
#include "generic.h"

#define IDE_IO_AREA_SIZE 16

/* resources */
enum {
  IDE_PCMD_BAR,               /* primary command block BAR */
  IDE_PCNL_BAR,               /* primary control block BAR */
  IDE_SCMD_BAR,               /* secondary command block BAR */
  IDE_SCNL_BAR,               /* secondary control block BAR */
  IDE_BM_BASE_BAR             /* bus master BAR */
};

/*
 * I/O space registers; for the secondary channel
 * the register have the additional offset 0x08
 */
enum {
  IDE_BMICX = 0,               /* bus master command register */
  IDE_BMISX = 0x02,            /* primary channel's */
  IDE_BMIDTPX = 0x04           /* bus master IDE descriptor table pointer register */
};

/* bus master command register bits */
enum {
  BMICX_SSBM = 0x01,          /* start/stop the bus master */
  BMICX_RWCON = 0x08          /* bus master read/write control */
};

/* bus master status register bits */
enum {
  BMISX_BMIDEA = 0x01,        /* bus master ide active */
  BMISX_DMA_ERR = 0x02,       /* dma error */
  BMISX_IDEINTS = 0x04,       /* IDE interrupt status */
  BMISX_DMA0CAP = 0x20,       /* drive 0 dma capable */
  BMISX_DMA1CAP = 0x40        /* drive 1 dma capable */
};

/* PRD high word */
typedef union __prd_highw {
  struct {
    uint16_t __res : 15;
    uint16_t eot : 1; /* end of table */
  };
  uint16_t value;
} __attribute__ ((packed)) prd_highw_t;

typedef struct __ide_prd {
  uint32_t base_addr;
  uint16_t count;
  prd_highw_t highw;
} __attribute__ ((packed)) ide_prd_t;

int ide_pci_alloc_bmus_ports(ata_host_t *host);
void ide_pci_free_bmus_ports(ata_host_t *host);
/* generic ide port initialization routine */
int ide_pci_init_port(ata_port_t *port, bool legacy);
/* generic release port routine */
void ide_pci_release_port(ata_port_t *port);
int ide_pci_init_dma(ata_port_t *port, uint32_t min_chunk_size, uint32_t max_tsize);
int ide_pci_init_dma_chan(ata_port_t *port, uint16_t max_descs,
                          uint32_t res_area_size, uint32_t max_single_tsize);

port_status_t ide_pci_read_status(ata_port_t *port);
void ide_pci_set_irq(ata_port_t *port, uint16_t irq);
void ide_pci_clear_irq(ata_port_t *port);

/* set all drives capable to dma */
void ide_pci_init_bmdma(ata_host_t *host);

uint32_t ide_pci_read_reg(ata_port_t *port, int addr);
void ide_pci_clear_bits(ata_port_t *port, int reg_addr, uint8_t mask);
void ide_pci_set_bits(ata_port_t *port, int reg_addr, uint8_t mask);
void ide_pci_write_reg(ata_port_t *port, int addr, uint32_t val);

#endif /* __GENERIC_IDE_H__ */
