/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 * 
 * (c) Copyright 2006,2007,2008 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2008 Paul Graphov <graphov@jarios.org> 
 *
 * PCI direct access through IO ports
 *
 */

#ifndef __PCI_IO_H__
#define __PCI_IO_H__

#include <sys/types.h>

#include "pci.h"

#define PCI_CONFIG_ADDRESS 0xCF8
#define PCI_CONFIG_DATA 0xCFC

typedef struct __pci_methods_t {
  int (*read) (pci_addr_t *, uint16_t, uint8_t, void *);
  int (*write) (pci_addr_t *, uint16_t, uint8_t, void *);
  int (*detect) (void);
} pci_methods_t;

/* bus, device, function, position */
#define PCI_CONF1_MAKE_ADDRESS(b, d, f, p)           \
  0x80000000            |                            \
  (((p) & 0xF00) << 16) |                            \
  ((f) << 8)            |                            \
  ((d) << 11)           |                            \
  ((b) << 16)           |                            \
  ((p) & 0xFC)

#define PCI_CONF2_MAKE_ADDRESS(d, p)            \
  0xC000 | (d << 8) | p

int init_pci_access(void);
int pci_write(pci_addr_t *d, uint16_t pos, uint8_t len, void *buf);
int pci_read(pci_addr_t *d, uint16_t pos, uint8_t len, void *buf);
int pci_generic_read(pci_addr_t *d, uint16_t pos, uint16_t len, void *buf);

#endif
