#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <byteorder.h>

#include <sys/kcontrol.h>

#include "pcfio.h"

uint8_t pcfio_read_char(pcfio_t *io)
{
  char c = (io->buffer[io->pos]);
  if(io->pos >= io->len || io->error) {
    io->error = 1;
    return 0;
  }
  io->pos++;
  return c;
}

int pcfio_read_block_rel(pcfio_t *io, char *dst, int size)
{
  if(io->pos + size <= io->len && !io->error) {
    memcpy(dst, &io->buffer[io->pos], size);
    return 0;
  } else {
    io->error = 1;
    return -1;
  }
}


int pcfio_pad4(pcfio_t *io)
{
  io->pos += (4 - (io->pos & 3)) & 3;
  if(io->pos >= io->len) {
    io->error = 1;
    return -1;
  }
  return 0;
}

PCF_READ_FUNC(16);
PCF_READ_FUNC(32);
PCF_READ_FUNC(64);

PCF_READ_FUNC_LSB(16);
PCF_READ_FUNC_LSB(32);
PCF_READ_FUNC_LSB(64);

static const char *be = "BIG ENDIAN";
static const char *le = "LITTLE ENDIAN";

void pcfio_set_format(pcfio_t *io, int format)
{
  io->format = format;
}

// FIXME: Now this function reads the whole file.
// It's not good but in future it will be mmap
int pcfio_init(const char *path, pcfio_t *io)
{
  int fd, r, ret = 0;
  struct stat stat;
  int ready = 0;
  ulong_t size;
  char *buf;

  char tmp[256];

  fd = open(path, O_RDONLY);
  if(fd < 0) {
    return -1;
  }

  r = fstat(fd, &stat);
  if(r < 0) {
    ret = -2;
    goto ii_cleanup;
  }

  size = stat.st_size;

  buf = malloc(size);
  if(!buf) {
    ret = -3;
    errno = ENOMEM;
    goto ii_cleanup;
  }

  io->error = 0;

  while(ready < size) {
    r = read(fd, &buf[ready], size-ready);
    if(r <= 0) {
      ret = -4;
      goto ii_cleanup2;
    }
    ready += size;
  }

  io->buffer = buf;
  io->len = size;
  io->pos = 0;
  io->format = PCF_LE;

  goto ii_cleanup;
  
  ii_cleanup2:
  free(buf);
  
  ii_cleanup:
  close(fd);
  return ret;
}

void pcfio_seekto(pcfio_t *io, int pos)
{
  io->pos = pos;
}

int pcfio_haserror(pcfio_t *io)
{
  return io->error;
}

void pcfio_close(pcfio_t *io)
{
  free(io->buffer);
}

void pcfio_swap_to_be2(uint16_t *data, int size, int byte_order)
{
  int i;
  
  for(i = 0; i < size / 2; i++) {
    if(byte_order == PCF_LE) {
      data[i] = __swab16(data[i]);
    }
  }
}

void pcfio_swap_to_be4(uint32_t *data, int size, int byte_order)
{
  int i;
  
  for(i = 0; i < size / 4; i++) {
    if(byte_order == PCF_LE) {
      data[i] = __swab32(data[i]);
    }
  }
}

