#include <sys/types.h>
#include <stdbool.h>
#include <stdlib.h>
#include <string.h>
#include "internal.h"

#include <sys/kcontrol.h>

static int x_pos = 0;
static int y_pos = 0;

static bool scroll_enable = true;

static bool cur_visible = true;

static int bg_color;

static console_ops_t console;

extern console_ops_t gc_ops;

int con_init(void)
{
  if(gc_ops.init() == 0) {
    memcpy(&console, &gc_ops, sizeof(console));
    return 0;
  } else {
    return -1;
  }
}

static void kill_char(int x, int y)
{
  console.fill_rect(x, y, 1, 1, bg_color);
}

// TODO: abstract out something like newline()

void con_scroll_enable(void)
{
  scroll_enable = true;
  if(x_pos >= console.width) {
    x_pos = 0;
    y_pos++;
    if(y_pos >= console.width) {
      y_pos = console.width - 1;
      console.scroll_area(0, console.height - 1, 1, bg_color);
    }
  }
}

void con_scroll_disable(void)
{
  scroll_enable = false;
}

void con_gotoxy(int x, int y)
{
  CHECK_BOUNDS(x, 0, console.width - 1);
  CHECK_BOUNDS(y, 0, console.height - 1);
  x_pos = x;
  y_pos = y;
}

void con_print(cschar_t *buf, int length)
{
  int i;

  if(cur_visible) {
    console.hide_cursor();
  }
  
  for(i = 0; i < length; i++) {
    switch(buf[i].ch) {
      case L'\n':
        y_pos++;
        x_pos = 0;
        if(y_pos >= console.height) {
          y_pos = console.height - 1;
          console.scroll_area(0, console.height - 1, 1, bg_color);
        }
        break;
      case L'\b':
        if(x_pos == 0) {
          y_pos--;
          if(y_pos >= 0) {
            x_pos = console.width - 1;
            kill_char(x_pos, y_pos);
          } else {
            y_pos = 0;
          }
        } else {
          x_pos--;
        kill_char(x_pos, y_pos);
        }
        break;
      default:
        if(scroll_enable) {
          console.put_char(x_pos, y_pos, buf[i]);
          x_pos++;
          if(x_pos >= console.width) {
            x_pos = 0;
            y_pos++;
            if(y_pos >= console.height) {
              y_pos = console.height - 1;
              console.scroll_area(0, console.height - 1, 1, bg_color);
            }
          }
        } else {
          if(x_pos < console.width) {
            console.put_char(x_pos, y_pos, buf[i]);
            x_pos++;
          }
        }
    }
  }

  if(cur_visible && x_pos < console.width && y_pos < console.height) {
    console.set_cursor(x_pos, y_pos);
  }
}

void con_show_cursor(void)
{
  cur_visible = true;
  console.set_cursor(x_pos, y_pos);
}

void con_hide_cursor(void)
{
  cur_visible = false;
  console.hide_cursor();
}

void con_scroll_area(int startline, int stopline, int lines)
{
  CHECK_BOUNDS(startline, 0, console.height);
  CHECK_BOUNDS(stopline, 0, console.height);
  CHECK_BOUNDS(lines, -(stopline - startline - 1), stopline - startline - 1);

  console.scroll_area(startline, stopline, lines, bg_color);
}

void con_clear_rect(int x1, int y1, int x2, int y2, int color)
{
  CHECK_BOUNDS(x1, 0, console.width - 1);
  CHECK_BOUNDS(y1, 0, console.height - 1);
  CHECK_BOUNDS(x2, x1, console.width - 1);
  CHECK_BOUNDS(y2, y1, console.height - 1);

  console.fill_rect(x1, y1, x2 - x1 + 1, y2 - y1 + 1, color);
}

int con_width(void)
{
  return console.width;
}

int con_height(void)
{
  return console.height;
}
