#ifndef __INTERNAL_H__
#define __INTERNAL_H__

#include <sys/types.h>
#include <sys/console.h>
#include <wchar.h>
#include "palette.h"

typedef struct __pixmap {
  int height;
  int width;
  uint32_t *data;
} pixmap_t;

typedef struct __font_ops {
  int width;
  int height;
  int padding;
  int (*init)(const char *path);
  void (*destroy)(void);
  uint8_t *(*get_bitmap)(wchar_t ch);
} font_ops_t;

typedef struct __viewport_ops {
  int width;
  int height;
  int bytes_per_pixel;
  int (*init)(void);
  void (*draw_bitmap)(int x, int y, int fgcolor, int bgcolor,
                      uint8_t *bitmap, int width, int height, int padding);
  int (*set_bg)(uint8_t *bg);
  void (*invert_rect)(int x, int y, int width, int height);
  void (*draw_pixmap)(int x, int y, pixmap_t *pixmap);
  void (*scroll_area)(int startline, int stopline, int lines, int color);
  void (*fill_rect)(int x, int y, int width, int height, int color);
  void (*sync_rect)(int x, int y, int width, int height);
  void (*palette_transform)(uint8_t *src, uint8_t *dest, int num);
  void (*destroy)(void);
} viewport_ops_t;

typedef struct __console_ops {
  int (*init)(void);
  int width;
  int height;
  void (*set_cursor)(int x, int y);
  void (*hide_cursor)(void);
  void (*put_char)(int x, int y, cschar_t ch);
  void (*scroll_area)(int startline, int stopline, int lines, int color);
  void (*fill_rect)(int x, int y, int width, int height, int color);
} console_ops_t;

// interface console operations
int con_init(void);
void con_print(cschar_t *buf, int length);
void con_show_cursor(void);
void con_hide_cursor(void);
void con_scroll_area(int startline, int stopline, int lines);
void con_clear_rect(int x1, int y1, int x2, int y2, int color);
void con_scroll_enable(void);
void con_gotoxy(int x, int y);
void con_scroll_disable(void);
int con_width(void);
int con_height(void);


// Put value into an interval.
#define CHECK_BOUNDS(x, min, max)               \
  {                                             \
    if((x) < (min)) {                           \
      (x) = (min);                              \
    } else if((x) > (max)) {                    \
      (x) = (max);                              \
    }                                           \
  }

// Pad a to be multiple of b
#define PAD(a,b) (((a) + (b) - 1) & (~((b) - 1)))


// background
int load_background(const char *path, viewport_ops_t *viewport);
  
#endif
