#include <sys/types.h>
#include <wchar.h>
#include <string.h>
#include <sys/ioctl.h>
#include <stdlib.h>
#include <errno.h>
#include <signal.h>
#include <service.h>

#include "internal.h"
#include <chardev.h>
#include <sys/kcontrol.h>

static status_t newcon_write(int devno, void *buf, size_t size, size_t offset, void **data);
static status_t newcon_read(int devno, void *buf, size_t size, size_t offset, void **data);
static status_t newcon_ioctl(int devno, int type, int nr, void *buf, size_t size, void *obuf, size_t osize);

static void easylog(const char *str) {
  klog(str);
}

struct device_ops dev_op = {
  .read = newcon_read,
  .write = newcon_write,  
  .ioctl = newcon_ioctl,
  .log = easylog,
};

#define DRVNAME "newcons.d"
#define DEVNAME "/dev/console"

void debug(const char *c, uint8_t col)
{
  cschar_t b[100];
  int len, i;

  len = strlen(c);
  for(i = 0; i < len; i++) {
    b[i].ch = (wchar_t)c[i];
    b[i].cattr = col;
  }

  con_print(b, len);

}

int main(void)
{
  int ret;

#ifndef CONFIG_VESACONS_GRUB
  service_start_t sstart;

  service_start_init(&sstart, SIGUSR1);
  launch_service(&sstart);
#endif

  ret = con_init();
  if(ret != 0) {
    abort();
  }

  ret = init_simple_device_server(65536, 65536, DRVNAME, DEVNAME, &dev_op);

  if(ret != 0) {
    abort();
  }

#ifndef CONFIG_VESACONS_GRUB
  service_start_finished(&sstart);
#endif
  device_server_start();

  // Unreachable under normal circumstances
  abort();
  return 0;
}

static status_t newcon_write(int devno, void *buf, size_t size, size_t offset, void **data)
{
  con_print((cschar_t*)buf, size / sizeof(cschar_t));
  return size;
}

void ascii_puts(const char *msg)
{
  return;
}

static status_t newcon_read(int devno, void *buf, size_t size, size_t offset, void **data)
{
  return -ENOSYS;
}

static status_t newcon_ioctl(int devno, int type, int nr, void *buf, size_t size, void *obuf, size_t osize)
{
  
  if(type != CON_IOCTL) return -ENOSYS;
  
//  if(size < get_ioctl_size(CON_IOCTL, nr)) return -EINVAL;
  //TODO: add sane size checking

  switch(nr) {
    case PHYT_DISABLEAUTOSCROLL:
      con_scroll_disable();
      break;
    case PHYT_ENABLEAUTOSCROLL:
      con_scroll_enable();
      break;
    case PHYT_GETSIZE:
      ((coord_t*)obuf)->x = con_width();
      ((coord_t*)obuf)->y = con_height();
      break;
    case PHYT_CLEAR:
      con_clear_rect(((clear_desc_t *)buf)->rect.x1, ((clear_desc_t *)buf)->rect.y1,
                       ((clear_desc_t *)buf)->rect.x2, ((clear_desc_t *)buf)->rect.y2,
                       ((clear_desc_t *)buf)->color);
      break;
    case PHYT_SCROLL:
      con_scroll_area(((scroll_desc_t *)buf)->startline, ((scroll_desc_t *)buf)->stopline,
                      ((scroll_desc_t *)buf)->lines);
      break;
    case PHYT_SETCURPOSITION:
      con_gotoxy(((coord_t *)buf)->x, ((coord_t *)buf)->y);
      break;
    case PHYT_SHOWCURSOR:
      con_show_cursor();
      break;
    case PHYT_HIDECURSOR:
      con_hide_cursor();
      break;
    default:
      return -EINVAL;
      break;
  }

  return 0;
}

