#define RED(x, bits)    (((x) >> (24 - (bits))) & ((1 << (bits)) - 1))
#define GREEN(x, bits)  (((x) >> (16 - (bits))) & ((1 << (bits)) - 1))
#define BLUE(x, bits)   (((x) >> (8 - (bits))) & ((1 << (bits)) - 1))

#define RGB_TO_565(a)                                                 \
  ((uint16_t)(RED((a), 5) << 11 | GREEN((a), 6) << 5 | BLUE((a), 5)))

#define RGB_TO_556(a)                                                 \
  ((uint16_t)(RED((a), 5) << 11 | GREEN((a), 5) << 6 | BLUE((a), 6)))

#define RGB_TO_555(a)                                                 \
  ((uint16_t)(RED((a), 5) << 10 | GREEN((a), 5) << 5 | BLUE((a), 5)))

#define RGB_TO_0888(a)                          \
  ((uint32_t)(a))
  
#define MK_RGB(r, g, b) (((r) << 16) | ((g) << 8) | (b))

void rgb_to_565_transform(uint8_t *src, uint8_t *dest, int num);
void rgb_to_555_transform(uint8_t *src, uint8_t *dest, int num);
void rgb_to_0888_transform(uint8_t *src, uint8_t *dest, int num);

#define MK_PALETTE_TRANSFORM(type, name, conv)                          \
  void rgb_to_##name##_transform(uint8_t *src, uint8_t *dest, int num)  \
  {                                                                     \
    int i;                                                              \
    for(i = 0; i < num; i++) {                                          \
      ((type *)dest)[i] = conv(((uint32_t *)src)[i]);                   \
    }                                                                   \
  }

#define FG_COLOR(n) ((n) & 0x0F)
#define BG_COLOR(n) ((n) >> 4)

#define RGB_BLACK          MK_RGB(0x00,0x00,0x00)
#define RGB_BLUE           MK_RGB(0x00,0x00,0xAA)
#define RGB_GREEN          MK_RGB(0x00,0xAA,0x00)
#define RGB_CYAN           MK_RGB(0x00,0xAA,0xAA)
#define RGB_RED            MK_RGB(0xAA,0x00,0x00)
#define RGB_MAGENTA        MK_RGB(0xAA,0x00,0xAA)
#define RGB_BROWN          MK_RGB(0xAA,0x55,0x00)
#define RGB_WHITE          MK_RGB(0xAA,0xAA,0xAA)
#define RGB_DARK_GRAY      MK_RGB(0x55,0x55,0x55)
#define RGB_BRIGHT_BLUE    MK_RGB(0x55,0x55,0xFF)
#define RGB_BRIGHT_GREEN   MK_RGB(0x55,0xFF,0x55)
#define RGB_BRIGHT_CYAN    MK_RGB(0x55,0xFF,0xFF)
#define RGB_BRIGHT_RED     MK_RGB(0xFF,0x55,0x55)
#define RGB_BRIGHT_MAGENTA MK_RGB(0xFF,0x55,0xFF)
#define RGB_BRIGHT_YELLOW  MK_RGB(0xFF,0xFF,0x55)
#define RGB_BRIGHT_WHITE   MK_RGB(0xFF,0xFF,0xFF)

extern uint16_t vesa_565_palette[];
extern uint16_t vesa_555_palette[];
extern uint32_t vesa_0888_palette[];

#define MK_PALETTE(type,name,conv)              \
  type vesa_##name##_palette[] = {              \
    conv(RGB_BLACK),                            \
    conv(RGB_BLUE),                             \
    conv(RGB_GREEN),                            \
    conv(RGB_CYAN),                             \
    conv(RGB_RED),                              \
    conv(RGB_MAGENTA),                          \
    conv(RGB_BROWN),                            \
    conv(RGB_WHITE),                            \
    conv(RGB_DARK_GRAY),                        \
    conv(RGB_BRIGHT_BLUE),                      \
    conv(RGB_BRIGHT_GREEN),                     \
    conv(RGB_BRIGHT_CYAN),                      \
    conv(RGB_BRIGHT_RED),                       \
    conv(RGB_BRIGHT_MAGENTA),                   \
    conv(RGB_BRIGHT_YELLOW),                    \
    conv(RGB_BRIGHT_WHITE)}

