#include "pcf.h"

enum {
  PCF_BE,
  PCF_LE,
};

typedef struct __pcfio {
  char *buffer;
  size_t len;
  int pos;
  int error;
  uint32_t format;
} pcfio_t;

uint8_t pcfio_read_char(pcfio_t *io);

uint16_t pcfio_read16(pcfio_t *io);
uint32_t pcfio_read32(pcfio_t *io);
uint64_t pcfio_read64(pcfio_t *io);

uint16_t pcfio_readlsb16(pcfio_t *io);
uint32_t pcfio_readlsb32(pcfio_t *io);
uint64_t pcfio_readlsb64(pcfio_t *io);

int pcfio_haserror(pcfio_t *io);
int pcfio_read_block_rel(pcfio_t *io, char *dst, int size);

int pcfio_init(const char *path, pcfio_t *io);
void pcfio_set_format(pcfio_t *io, int format);
void pcfio_seekto(pcfio_t *io, int pos);
int pcfio_pad4(pcfio_t *io);
void pcfio_close(pcfio_t *io);

#define CURRENT_PTR(io) (&io->buffer[io->pos])

#define PCF_READ_FUNC(bits)                                 \
  uint##bits##_t pcfio_read##bits(pcfio_t *io)              \
  {                                                         \
    uint##bits##_t v;                                       \
    if((ulong_t)io->pos + bits/8 > io->len || io->error) {  \
      io->error = 1;                                        \
      return 0;                                             \
    }                                                       \
    switch(PCF_BYTE_ORDER(io->format)) {                    \
      case PCF_LE:                                          \
        v = le_to_cpu##bits(                                \
          *(uint##bits##_t*)CURRENT_PTR(io));               \
        break;                                              \
      case PCF_BE:                                          \
        v = be_to_cpu##bits(                                \
          *(uint##bits##_t*)CURRENT_PTR(io));               \
        break;                                              \
    }                                                       \
    io->pos += bits/8;                                      \
    return v;                                               \
  }

#define PCF_READ_FUNC_LSB(bits)                              \
  uint##bits##_t pcfio_readlsb##bits(pcfio_t *io)            \
  {                                                          \
    uint##bits##_t v;                                        \
    if((ulong_t)io->pos + bits / 8 > io->len || io->error) { \
      io->error = 1;                                         \
      return 0;                                              \
    }                                                        \
    v = le_to_cpu##bits(*(uint##bits##_t*)CURRENT_PTR(io));  \
    io->pos += bits/8;                                       \
    return v;                                                \
  }


void pcfio_swap_to_be2(uint16_t *data, int size, int byte_order);

void pcfio_swap_to_be4(uint32_t *data, int size, int byte_order);
