/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * (c) Copyright 2006,2007,2008 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2008 MadTirra <madtirra@jarios.org>
 *
 * vgacons.d: Generic VGA console driver
 * console.c: main entry and IPC internals
 *
 */

#include <stdbool.h>
#include <sys/types.h>
#include <sys/mman.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <wchar.h>
#include <errno.h>
#include <time.h>
#include <datalib/usrtc.h>
#include <ns/ns.h>
#include <sys/scheduler.h>
#include <ipc/rpc.h>
#include <devio/devio.h>
#include <stdio.h>
#include <sys/ioctl.h>

#include <sys/console.h>

#include "vga.h"
#include "hwops.h"

#include <chardev.h>

#define DRVNAME  "vgacons.d"
#define DEVNAME  "/dev/console"

static status_t vga_write(int devno, void *buf, size_t size, size_t offset, void **data);
static status_t vga_read(int devno, void *buf, size_t size, size_t offset, void **data);
static status_t vga_ioctl(int devno, int type, int nr, void *buf, size_t size, void *obuf, size_t osize);

struct device_ops dev_op = {
  .read = vga_read,
  .write = vga_write,
  .ioctl = vga_ioctl,
  .log = vga_print,
};

int main(int argc,char **argv)
{
  int r=0;
  void *iobuf;
  char buff[256];

#ifndef CONFIG_VGACONS_GRUB
  start_service();
#endif

  iobuf=mmap(NULL, 4096, PROT_READ|PROT_WRITE, MAP_PHYS|MAP_PRIVATE, NOFD, VGA_TM_BASE);
  if(iobuf==MAP_FAILED) abort();

  task_scheduler_control(0,0,SYS_SCHED_CTL_SET_PRIORITY,30);

  vga_init(iobuf);

  vga_print("[vgacons] i've started\n");

  r = init_simple_device_server(65536, 65536, DRVNAME, DEVNAME, &dev_op);

  if(r != 0) {
    sprintf(buff, "[vgacons] device service init failed (%d, errno = %d)\n", r, errno);
    vga_print(buff);
    abort();
  }

  vga_print("[vgacons] device server run\n");

  device_server_start();

  vga_print("[vgacons] :((\n");

  return 0;
}

static status_t vga_write(int devno, void *buf, size_t size, size_t offset, void **data)
{

  uni_print((cschar_t*)buf,size);
  return size;
}

void ascii_puts(const char *msg)
{
  return;
}

static status_t vga_read(int devno, void *buf, size_t size, size_t offset, void **data)
{

  return -ENOSYS;

  return 0;
}
/*
#include <drv_ioctl.h>

IOCTL_GETSIZE_BEGIN {
  CIOC_CLEAR;
  CIOC_SCROLL;
  CIOC_SETCURPOSITION;
  CIOC_SHOWCURSOR;
  CIOC_HIDECURSOR;
  CIOC_ENABLEAUTOSCROLL;
  CIOC_DISABLEAUTOSCROLL;
  CIOC_GETSIZE;
} IOCTL_GETSIZE_END
*/

static status_t vga_ioctl(int devno, int type, int nr, void *buf, size_t size, void *obuf, size_t osize)
{
  //TODO: add sane size checking;
  
  if(type != CON_IOCTL) return -ENOSYS;

  switch(nr) {
    case PHYT_DISABLEAUTOSCROLL:
      uni_scrolldisable();
      break;
    case PHYT_ENABLEAUTOSCROLL:
      uni_scrollenable();
      break;
    case PHYT_GETSIZE:
      ((coord_t*)obuf)->x = uni_width();
      ((coord_t*)obuf)->y = uni_height();
      break;
    case PHYT_CLEAR:
      uni_clear_rect(((clear_desc_t *)buf)->rect.x1, ((clear_desc_t *)buf)->rect.y1,
                     ((clear_desc_t *)buf)->rect.x2, ((clear_desc_t *)buf)->rect.y2,
                     ((clear_desc_t *)buf)->color);
      break;
    case PHYT_HIDECURSOR:
      vga_cursor(false);
      break;
    case PHYT_SHOWCURSOR:
      vga_cursor(true);
      break;
    case PHYT_SCROLL:
      uni_scroll_area(((scroll_desc_t *)buf)->startline, ((scroll_desc_t *)buf)->stopline,
                      ((scroll_desc_t *)buf)->lines);
      break;
    case PHYT_SETCURPOSITION:
      vga_setcursor(((coord_t *)buf)->x, ((coord_t *)buf)->y);
      break;
    default:
      return EINVAL;
      break;
  }

  return 0;
}
