/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 * 
 * (c) Copyright 2006,2007,2008 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2005,2008 MadTirra <madtirra@jarios.org> 
 *
 * i8042 chip driver (ports for keyboard and mouse in pc arch)
 *
 */

#ifndef __I8042_H__
#define __I8042_H__

#include <sys/types.h>

#define I8042_IN_FULL   0x1
#define I8042_OUT_FULL  0x2

#define OUT_BUF_FULL   0x1
#define INPT_BUF_FULL  0x2
#define SYS_FLAG       0x4
#define CMD_DATA       0x8
#define KEYBD_INH      0x10
#define TRANS_TMOUT    0x20
#define RCV_TMOUT      0x40
#define PARITY_EVEN    0x80
#define INH_KEYBOARD   0x10
#define KBD_ENA        0xAE
#define KBD_DIS        0xAD
/* Keyboard Commands */
#define KB_MENU        0xF1
#define KB_ENABLE      0xF4
#define KB_MAKEBREAK   0xF7
#define KB_ECHO        0xFE
#define KB_RESET       0xFF
#define KB_LED_CMD     0xED
/* Keyboard responses */
#define KB_OK          0xAA
#define KB_ACK         0xFA
#define KB_OVERRUN     0xFF
#define KB_RESEND      0xFE
#define KB_BREAK       0xF0
#define KB_FA          0x10
#define KB_FE          0x20
#define KB_PR_LED      0x40


#define STATUS_PORT  0x64
#define DATA_PORT    0x60



int init_i8042(void);

static inline uint8_t read_status_i8042(void)
{
  return inb(STATUS_PORT);
}

static inline uint8_t read_data_i8042(void)
{
  return inb(DATA_PORT);
}

static inline void write_data_i8042(uint8_t db)
{
  outb(DATA_PORT,db);
}

static inline void write_command_i8042(uint8_t cmd)
{
  outb(STATUS_PORT,cmd);
}

static inline void wait_i8042_in(void)
{
  while(!(read_status_i8042() & I8042_IN_FULL));
}

static inline void wait_i8042_out(void)
{
  while(read_status_i8042() & I8042_OUT_FULL);
}

static inline int i8042_in_ready(void)
{
  return (read_status_i8042() & I8042_IN_FULL);
}

static inline void send_to_kbd(uint8_t c)
{
  wait_i8042_out();
  write_data_i8042(c);
}

#endif /* __I8042_H__ */
