#ifndef __E1000_H__
#define __E1000_H__

#include <arch/atomic.h>
#include <lwip/pbuf.h>
#include <lwip/netif.h>
#include <pci/pci.h>
#include <sys/ievent.h>
#include "e1000_hw.h"

struct e1000_adapter;
struct e1000_tx_ring;

void e1000_reset(struct e1000_adapter *adapter);
err_t e1000_netdev_linkoutput(struct netif *nif,struct pbuf *p);
bool e1000_clean_tx_irq(struct e1000_adapter *adapter,
                        struct e1000_tx_ring *tx_ring);

#define E1000_MNG_VLAN_NONE -1
/* Number of packet split data buffers (not including the header buffer) */
#define PS_PAGE_BUFFERS MAX_PS_BUFFERS-1

#define MAXIMUM_ETHERNET_VLAN_SIZE 1522

/* Supported Rx Buffer Sizes */
#define E1000_RXBUFFER_128   128    /* Used for packet split */

/* The sizes (in bytes) of a ethernet packet */
#define ENET_HEADER_SIZE             14
#define MAXIMUM_ETHERNET_FRAME_SIZE  1518 /* With FCS */
#define MINIMUM_ETHERNET_FRAME_SIZE  64   /* With FCS */
#define ETHERNET_FCS_SIZE            4

/* Flow Control Watermarks */
#define E1000_FC_HIGH_DIFF 0x1638  /* High: 5688 bytes below Rx FIFO size */
#define E1000_FC_LOW_DIFF 0x1640   /* Low:  5696 bytes below Rx FIFO size */

#define AUTO_ALL_MODES 0

/* Supported Rx Buffer Sizes */
#define E1000_RXBUFFER_128   128    /* Used for packet split */
#define E1000_RXBUFFER_256   256    /* Used for packet split */
#define E1000_RXBUFFER_512   512
#define E1000_RXBUFFER_1024  1024
#define E1000_RXBUFFER_2048  2048
#define E1000_RXBUFFER_4096  4096
#define E1000_RXBUFFER_8192  8192
#define E1000_RXBUFFER_16384 16384

#define E1000_DEFAULT_TXBUFFER_SIZE  E1000_RXBUFFER_2048

/* Packet Buffer allocations */
#define E1000_PBA_BYTES_SHIFT 0xA
#define E1000_TX_HEAD_ADDR_SHIFT 7
#define E1000_PBA_TX_MASK 0xFFFF0000

#define E1000_FC_PAUSE_TIME 0x0680 /* 858 usec */

/* 802.1q VLAN Packet Sizes */
#define VLAN_TAG_SIZE  4     /* 802.3ac tag (not DMAed) */

#ifndef E1000_MASTER_SLAVE
/* Switch to override PHY master/slave setting */
#define E1000_MASTER_SLAVE      e1000_ms_hw_default
#endif

#define AUTO_ALL_MODES            0
#define E1000_EEPROM_82544_APM    0x0004
#define E1000_EEPROM_ICH8_APME    0x0004
#define E1000_EEPROM_APME         0x0400

struct pci_device {
  int irq;
};

/* wrapper around a pointer to a socket buffer,
 * so a DMA handle can be stored along with the buffer */
struct e1000_buffer {
  struct pbuf *skb;
  long dma;
  unsigned long time_stamp;
  uint16_t length;
  uint16_t next_to_watch;
};

struct e1000_tx_ring {
  /* pointer to the descriptor ring memory */
  void *desc;
  /* physical address of the descriptor ring */
  long dma;
  /* length of descriptor ring in bytes */
  unsigned int size;
  /* number of descriptors in the ring */
  unsigned int count;
  /* next descriptor to associate a buffer with */
  unsigned int next_to_use;
  /* next descriptor to check for DD status bit */
  unsigned int next_to_clean;
  /* array of buffer information structs */
  struct e1000_buffer *buffer_info;

  spinlock_t tx_lock;
  uint16_t tdh;
  uint16_t tdt;
  bool last_tx_tso;
};

struct e1000_rx_ring {
  /* pointer to the descriptor ring memory */
  void *desc;
  /* physical address of the descriptor ring */
  long dma;
  /* length of descriptor ring in bytes */
  unsigned int size;
  /* number of descriptors in the ring */
  unsigned int count;
  /* next descriptor to associate a buffer with */
  unsigned int next_to_use;
  /* next descriptor to check for DD status bit */
  unsigned int next_to_clean;
  /* array of buffer information structs */
  struct e1000_buffer *buffer_info;

  /* cpu for rx queue */
  int cpu;

  uint16_t rdh;
  uint16_t rdt;
};

#define E1000_DESC_UNUSED(R) \
	((((R)->next_to_clean > (R)->next_to_use) ? 0 : (R)->count) + \
	(R)->next_to_clean - (R)->next_to_use - 1)

#define E1000_RX_DESC_PS(R, i)	    \
	(&(((union e1000_rx_desc_packet_split *)((R).desc))[i]))
#define E1000_RX_DESC_EXT(R, i)	    \
	(&(((union e1000_rx_desc_extended *)((R).desc))[i]))
#define E1000_GET_DESC(R, i, type)	(&(((struct type *)((R).desc))[i]))
#define E1000_RX_DESC(R, i)             E1000_GET_DESC(R, i, e1000_rx_desc)
#define E1000_TX_DESC(R, i)		E1000_GET_DESC(R, i, e1000_tx_desc)
#define E1000_CONTEXT_DESC(R, i)	E1000_GET_DESC(R, i, e1000_context_desc

/* TX/RX descriptor defines */
#define E1000_DEFAULT_TXD                  32 /* 256 */
#define E1000_MAX_TXD                      256
#define E1000_MIN_TXD                       80
#define E1000_MAX_82544_TXD               4096

#define E1000_DEFAULT_RXD                  32 /* 256 */
#define E1000_MAX_RXD                      256
#define E1000_MIN_RXD                       80
#define E1000_MAX_82544_RXD               4096

/* How many Tx Descriptors do we need to call netif_wake_queue ? */
#define E1000_TX_QUEUE_WAKE	16
/* How many Rx Buffers do we bundle into one write to the hardware ? */
#define E1000_RX_BUFFER_WRITE	16	/* Must be power of 2 */

#define ETH_DATA_LEN    1500            /* Max. octets in payload */

#ifndef NET_IP_ALIGN
#define NET_IP_ALIGN    2
#endif

struct e1000_adapter {
  struct e1000_hw hw;
  struct netif *netdev;
  int rx_buffer_len;
  uint16_t rx_ps_bsize0;
  int num_tx_queues;
  int num_rx_queues;
  atomic_t irq_sem;
  spinlock_t stats_lock;
  pci_dev_t *pdev;
  unsigned long flags;
  uint32_t en_mng_pt;
  struct e1000_phy_info phy_info;
  bool smart_power_down;	/* phy smart power down */
  uint16_t mng_vlan_id;
  uint32_t eeprom_wol;
  bool quad_port_a;
  uint32_t wol;
  int bd_number;
  uint8_t fc_autoneg;
  unsigned int rx_ps_pages;
  unsigned int hw_csum_err;
  unsigned int total_rx_packets,total_rx_bytes;
  unsigned int total_tx_packets,total_tx_bytes;
  struct e1000_hw_stats stats;
  uint32_t alloc_rx_buff_failed;
  bool have_msi;
  int link_speed;

  bool (*clean_rx) (struct e1000_adapter *adapter,
                    struct e1000_rx_ring *rx_ring);
  void (*alloc_rx_buf) (struct e1000_adapter *adapter,
                        struct e1000_rx_ring *rx_ring,
                        int cleaned_count);

  /* Interrupt Throttle Rate */
  uint32_t itr;
  uint32_t itr_setting;
  uint16_t tx_itr;
  uint16_t rx_itr;

  /* TX */
  struct e1000_tx_ring *tx_ring;      /* One per active queue */
  unsigned int restart_queue;
  unsigned long tx_queue_len;
  uint32_t txd_cmd;
  uint32_t tx_int_delay;
  uint32_t tx_abs_int_delay;
  uint32_t gotcl;
  uint64_t gotcl_old;
  uint64_t tpt_old;
  uint64_t colc_old;
  uint32_t tx_timeout_count;
  uint32_t tx_fifo_head;
  uint32_t tx_head_addr;
  uint32_t tx_fifo_size;
  uint8_t  tx_timeout_factor;
  atomic_t tx_fifo_stall;
  bool pcix_82544;
  bool detect_tx_hung;
  irq_t irq_line;

  /* RX */
  struct e1000_rx_ring *rx_ring;      /* One per active queue */
  uint32_t rx_int_delay;
  uint32_t rx_abs_int_delay;
  bool rx_csum;

  void *vlgrp;
};

enum e1000_state_t {
  __E1000_TESTING,
  __E1000_RESETTING,
  __E1000_DOWN
};

enum ethtool_test_flags {
  ETH_TEST_FL_OFFLINE	= (1 << 0),	/* online / offline */
  ETH_TEST_FL_FAILED	= (1 << 1),	/* test passed / failed */
};

#define SPEED_10    10
#define SPEED_100   100
#define SPEED_1000  1000
#define HALF_DUPLEX 1
#define FULL_DUPLEX 2

#define E1000_MAX_INTR 1000

#endif
