/*******************************************************************************

  Intel PRO/1000 Linux driver
  Copyright(c) 1999 - 2006 Intel Corporation.

  This program is free software; you can redistribute it and/or modify it
  under the terms and conditions of the GNU General Public License,
  version 2, as published by the Free Software Foundation.

  This program is distributed in the hope it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
  more details.

  You should have received a copy of the GNU General Public License along with
  this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA.

  The full GNU General Public License is included in this distribution in
  the file called "COPYING".

  Contact Information:
  Linux NICS <linux.nics@intel.com>
  e1000-devel Mailing List <e1000-devel@lists.sourceforge.net>
  Intel Corporation, 5200 N.E. Elam Young Parkway, Hillsboro, OR 97124-6497

*******************************************************************************/


/* glue for the OS independent part of e1000
 * includes register access macros
 */

#ifndef _E1000_OSDEP_H_
#define _E1000_OSDEP_H_

#include <sys/types.h>
#include <time.h>

static inline void writel(unsigned int c,void *p)
{
  *(unsigned int *)p = c;
}

static inline void writew(unsigned short c,void *p)
{
  *(unsigned short *)p = c;
}

static inline void writeb(unsigned char c,void *p)
{
  *(unsigned int *)p = c;
}

static inline unsigned char readb(void *ptr)
{
  return *(unsigned char *)ptr;
}

static inline unsigned short readw(void *ptr)
{
  return *(unsigned short *)ptr;
}

static inline unsigned int readl(void *ptr)
{
  return *(unsigned int *)ptr;
}

#define E1000_WRITE_REG(a, reg, value) ( \
    writel((value), ((a)->hw_addr + \
        (((a)->mac_type >= e1000_82543) ? E1000_##reg : E1000_82542_##reg))))

#define E1000_READ_REG(a, reg) ( \
    readl((a)->hw_addr + \
        (((a)->mac_type >= e1000_82543) ? E1000_##reg : E1000_82542_##reg)))

#define E1000_WRITE_REG_ARRAY(a, reg, offset, value) ( \
    writel((value), ((a)->hw_addr + \
        (((a)->mac_type >= e1000_82543) ? E1000_##reg : E1000_82542_##reg) + \
        ((offset) << 2))))

#define E1000_READ_REG_ARRAY(a, reg, offset) ( \
    readl((a)->hw_addr + \
        (((a)->mac_type >= e1000_82543) ? E1000_##reg : E1000_82542_##reg) + \
        ((offset) << 2)))

#define E1000_READ_REG_ARRAY_DWORD E1000_READ_REG_ARRAY
#define E1000_WRITE_REG_ARRAY_DWORD E1000_WRITE_REG_ARRAY

#define E1000_WRITE_REG_ARRAY_WORD(a, reg, offset, value) ( \
    writew((value), ((a)->hw_addr + \
        (((a)->mac_type >= e1000_82543) ? E1000_##reg : E1000_82542_##reg) + \
        ((offset) << 1))))

#define E1000_READ_REG_ARRAY_WORD(a, reg, offset) ( \
    readw((a)->hw_addr + \
        (((a)->mac_type >= e1000_82543) ? E1000_##reg : E1000_82542_##reg) + \
        ((offset) << 1)))

#define E1000_WRITE_REG_ARRAY_BYTE(a, reg, offset, value) ( \
    writeb((value), ((a)->hw_addr + \
        (((a)->mac_type >= e1000_82543) ? E1000_##reg : E1000_82542_##reg) + \
        (offset))))

#define E1000_READ_REG_ARRAY_BYTE(a, reg, offset) ( \
    readb((a)->hw_addr + \
        (((a)->mac_type >= e1000_82543) ? E1000_##reg : E1000_82542_##reg) + \
        (offset)))

#define E1000_WRITE_FLUSH(a) E1000_READ_REG(a, STATUS)

#define E1000_WRITE_ICH_FLASH_REG(a, reg, value) ( \
    writel((value), ((a)->flash_address + reg)))

#define E1000_READ_ICH_FLASH_REG(a, reg) ( \
    readl((a)->flash_address + reg))

#define E1000_WRITE_ICH_FLASH_REG16(a, reg, value) ( \
    writew((value), ((a)->flash_address + reg)))

#define E1000_READ_ICH_FLASH_REG16(a, reg) ( \
    readw((a)->flash_address + reg))


static inline int mdelay(unsigned long msecs)
{
  struct timespec ts;
  unsigned long secs,nsecs;

  if( msecs >= 1000000000 ) {
    return -EINVAL;
  }

  if( msecs >= 1000 ) {
    secs=msecs/1000;
    nsecs=(msecs % 1000)*1000000;
  } else {
    secs=0;
    nsecs=msecs*1000000;
  }

  ts.tv_sec=secs;
  ts.tv_nsec=nsecs;

  return nanosleep(&ts,NULL);
}

static inline int udelay(unsigned long usecs)
{
  struct timespec ts;
  long secs,nsecs;

  if( usecs >= 1000000000 ) {
    return -EINVAL;
  }

  if( usecs >= 1000000 ) {
    secs=usecs/1000000;
    nsecs=(usecs % 1000000)*1000;
  } else {
    secs=0;
    nsecs=usecs*1000;
  }

  ts.tv_sec=secs;
  ts.tv_nsec=nsecs;

  return nanosleep(&ts,NULL);
}

#define msleep(x) mdelay((x))
#define ALIGN(x,a) (((x)+(a)-1)&~((a)-1))
#define pci_map_single(a,b,c,d)  NULL
#define pci_unmap_single(a,b,c,d)
#define pci_unmap_page(a,b,c,d)

#define pci_enable_msi(x) false
#define pci_disable_msi(x)
#define time_after(a,b) false
#define pci_free_consistent(a,b,c,d)

#define spin_lock_irqsave(x,f)
#define spin_unlock_irqrestore(x,f)
#define jiffies 0
#define HZ 1000

#define smp_mb()
#define mmiowb()

#define netif_carrier_ok(t) true
#define netif_queue_stopped(t) false
#define netif_wake_queue(t)

#define le16_to_cpu(x) (x)

#endif /* _E1000_OSDEP_H_ */
