#include <sys/types.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <datalib/usrtc.h>
#include <errno.h>
#include <ipc/port.h>
#include <ipc/channel.h>
#include <ipc/rpc.h>
#include <ns/ns.h>
#include <errno.h>
#include <devio/devio.h>
#include <sys/poll.h>
#include <sys/thread.h>
#include <debug.h>
#include <ns/ns.h>
#include <sys/procmgr.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <stddef.h>
#include <sys/ievent.h>
#include <arch/atomic.h>
#include <arch/bitwise.h>
#include <sys/mman.h>
#include <pci/pci.h>
#include <net/network.h>
#include <pthread.h>
#include <sys/dma.h>
#include <netif/etharp.h>

#include "e1000.h"
#include "e1000_hw.h"
#include "e1000_osdep.h"

#define E1000_TX_FLAGS_CSUM		0x00000001
#define E1000_TX_FLAGS_VLAN		0x00000002
#define E1000_TX_FLAGS_TSO		0x00000004
#define E1000_TX_FLAGS_IPV4		0x00000008
#define E1000_TX_FLAGS_VLAN_MASK	0xffff0000
#define E1000_TX_FLAGS_VLAN_SHIFT	16

#define E1000_MAX_TXD_PWR	12
#define E1000_MAX_DATA_PER_TXD	(1<<E1000_MAX_TXD_PWR)

#define skb_is_gso(x) false

static int e1000_tx_map(struct e1000_adapter *adapter, struct e1000_tx_ring *tx_ring,
                        struct pbuf *skb)
{
  struct e1000_buffer *buffer_info;
  struct pbuf *mapped_skb;
  int i,size;

  i = tx_ring->next_to_use;
  buffer_info = &tx_ring->buffer_info[i];
  mapped_skb = buffer_info->skb;

  size = MIN(skb->tot_len,mapped_skb->tot_len);
  if( pbuf_copy(mapped_skb,skb) ) {
    console_printf("[e1000] e1000_tx_map(): failed to copy SKB !\n");
    return -EFAULT;
  }

  buffer_info->length = size;
  buffer_info->next_to_watch = i;
  buffer_info->dma=(dma_addr_t)mapped_skb->dma;

  /* TODO: [mt] map skb */
  //console_printf("[e1000] e1000_tx_map(): Using Tx buffer N %d, (PAYLOAD=%p, DMA=%p\n",
  //               i,mapped_skb->payload,buffer_info->dma);

  return 1;
}

static int __e1000_maybe_stop_tx(struct netif *netdev, int size)
{
  struct e1000_adapter *adapter = netdev->state;
  struct e1000_tx_ring *tx_ring = adapter->tx_ring;

  ++adapter->restart_queue;
  return 0;
}

static int e1000_maybe_stop_tx(struct netif *netdev,
                               struct e1000_tx_ring *tx_ring, int size)
{
  return 0;
  //return __e1000_maybe_stop_tx(netdev, size);
}

static void
e1000_tx_queue(struct e1000_adapter *adapter, struct e1000_tx_ring *tx_ring,
               int tx_flags, int count)
{
  struct e1000_tx_desc *tx_desc = NULL;
  struct e1000_buffer *buffer_info;
  uint32_t txd_upper = 0, txd_lower = E1000_TXD_CMD_IFCS;
  unsigned int i;

  if (tx_flags & E1000_TX_FLAGS_TSO) {
    txd_lower |= E1000_TXD_CMD_DEXT | E1000_TXD_DTYP_D |
      E1000_TXD_CMD_TSE;
    txd_upper |= E1000_TXD_POPTS_TXSM << 8;

    if (tx_flags & E1000_TX_FLAGS_IPV4)
      txd_upper |= E1000_TXD_POPTS_IXSM << 8;
  }

  if (tx_flags & E1000_TX_FLAGS_CSUM) {
    txd_lower |= E1000_TXD_CMD_DEXT | E1000_TXD_DTYP_D;
    txd_upper |= E1000_TXD_POPTS_TXSM << 8;
  }

  if (tx_flags & E1000_TX_FLAGS_VLAN) {
    txd_lower |= E1000_TXD_CMD_VLE;
    txd_upper |= (tx_flags & E1000_TX_FLAGS_VLAN_MASK);
  }

  i = tx_ring->next_to_use;
  while (count--) {
    buffer_info = &tx_ring->buffer_info[i];

    tx_desc = E1000_TX_DESC(*tx_ring, i);
    tx_desc->buffer_addr = cpu_to_le64(buffer_info->dma);
    tx_desc->lower.data = cpu_to_le32(txd_lower | buffer_info->length);
    tx_desc->upper.data = cpu_to_le32(txd_upper);
    if (++i == tx_ring->count)
      i = 0;
  }

  tx_desc->lower.data |= cpu_to_le32(adapter->txd_cmd);

  /* Force memory writes to complete before letting h/w
   * know there are new descriptors to fetch.  (Only
   * applicable for weak-ordered memory model archs,
   * such as IA-64). */
  wmb();

  tx_ring->next_to_use = i;
  writel(i, adapter->hw.hw_addr + tx_ring->tdt);
  /* we need this if more than one processor can write to our tail
   * at a time, it syncronizes IO on IA64/Altix systems */
  mmiowb();
}

int e1000_xmit_frame(struct pbuf *skb, struct netif *netdev)
{
  struct e1000_adapter *adapter = netdev->state;
  struct e1000_tx_ring *tx_ring;
  unsigned int first, max_per_txd = E1000_MAX_DATA_PER_TXD;
  unsigned int max_txd_pwr = E1000_MAX_TXD_PWR;
  unsigned int tx_flags = 0;
  unsigned int len = skb->len;
  unsigned long flags;
  unsigned int nr_frags;
  unsigned int mss;
  int count = 0;
  int tso;
  unsigned int f;

  /* This goes back to the question of how to logically map a tx queue
   * to a flow.  Right now, performance is impacted slightly negatively
   * if using multiple tx queues.  If the stack breaks away from a
   * single qdisc implementation, we can look at this again. */
  tx_ring = adapter->tx_ring;

  if (skb->len <= 0) {
    return 0;
  }

  /* 82571 and newer doesn't need the workaround that limited descriptor
   * length to 4kB */
  if (adapter->hw.mac_type >= e1000_82571)
    max_per_txd = 8192;

  /* TODO: [mt] TSO support was deprecated. */

  /* need: count + 2 desc gap to keep tail from touching
   * head, otherwise try next time */
  if (e1000_maybe_stop_tx(netdev, tx_ring, count + 2)) {
    return -EBUSY;
  }

  /*
  if (adapter->hw.mac_type == e1000_82547) {
    if (e1000_82547_fifo_workaround(adapter, skb)) {
      netif_stop_queue(netdev);
      mod_timer(&adapter->tx_fifo_stall_timer, jiffies + 1);
      spin_unlock_irqrestore(&tx_ring->tx_lock, flags);
      return NETDEV_TX_BUSY;
    }
    }
  */

  /*
  if (adapter->vlgrp && vlan_tx_tag_present(skb)) {
    tx_flags |= E1000_TX_FLAGS_VLAN;
    tx_flags |= (vlan_tx_tag_get(skb) << E1000_TX_FLAGS_VLAN_SHIFT);
    }
  */

  //first = tx_ring->next_to_use;

  /* TODO: [mt] TSO mode disabled.
  tso = e1000_tso(adapter, tx_ring, skb);
  if (tso < 0) {
    dev_kfree_skb_any(skb);
    spin_unlock_irqrestore(&tx_ring->tx_lock, flags);
    return NETDEV_TX_OK;
  }

  if (likely(tso)) {
    tx_ring->last_tx_tso = 1;
    tx_flags |= E1000_TX_FLAGS_TSO;
  } else if (likely(e1000_tx_csum(adapter, tx_ring, skb)))
  tx_flags |= E1000_TX_FLAGS_CSUM;
  */

  /* TODO: [mt] stack-related checksum calculation disabled.
  if (likely(e1000_tx_csum(adapter, tx_ring, skb)))
  tx_flags |= E1000_TX_FLAGS_CSUM;
  */

  /* Old method was to assume IPv4 packet by default if TSO was enabled.
   * 82571 hardware supports TSO capabilities for IPv6 as well...
   * no longer assume, we must. */
  /* TODO: [mt] Protocol is assumed to be IPV4 always. */
  //if (likely(skb->protocol == htons(ETH_P_IP)))

  tx_flags |= E1000_TX_FLAGS_IPV4;

  e1000_tx_queue(adapter, tx_ring, tx_flags,
                 e1000_tx_map(adapter, tx_ring, skb));

  /* Make sure there is space in the ring for the next send. */
  e1000_maybe_stop_tx(netdev, tx_ring, 1);

  return 0;
}

/**
 * e1000_clean_tx_irq - Reclaim resources after transmit completes
 * @adapter: board private structure
 **/
bool e1000_clean_tx_irq(struct e1000_adapter *adapter,
                        struct e1000_tx_ring *tx_ring)
{
  struct netif *netdev = adapter->netdev;
  struct e1000_tx_desc *tx_desc, *eop_desc;
  struct e1000_buffer *buffer_info;
  unsigned int i, eop;
  bool cleaned = false;
  unsigned int total_tx_bytes=0, total_tx_packets=0;

  i = tx_ring->next_to_clean;
  eop = tx_ring->buffer_info[i].next_to_watch;
  eop_desc = E1000_TX_DESC(*tx_ring, eop);

  while (eop_desc->upper.data & cpu_to_le32(E1000_TXD_STAT_DD)) {
    for (cleaned = false; !cleaned; ) {
      tx_desc = E1000_TX_DESC(*tx_ring, i);
      buffer_info = &tx_ring->buffer_info[i];
      cleaned = (i == eop);

      if (cleaned) {
        struct pbuf *skb = buffer_info->skb;
        unsigned int segs = 1, bytecount;

        /* multiply data chunks by size of headers */
        bytecount = skb->tot_len;
        total_tx_packets += segs;
        total_tx_bytes += bytecount;
      }
      /* TODO: [mt] we don't unmap Tx buffer after transmission ! */
      tx_desc->upper.data = 0;

      if (++i == tx_ring->count) i = 0;
    }

    eop = tx_ring->buffer_info[i].next_to_watch;
    eop_desc = E1000_TX_DESC(*tx_ring, eop);
  }

  tx_ring->next_to_clean = i;

#define TX_WAKE_THRESHOLD 32
  if (cleaned && E1000_DESC_UNUSED(tx_ring) >= TX_WAKE_THRESHOLD) {
    /* Make sure that anybody stopping the queue after this
     * sees the new next_to_clean.
     */
    smp_mb();
    ++adapter->restart_queue;
  }

  /* Detect a transmit hang in hardware, this serializes the
   * check with the clearing of time_stamp and movement of i */  
  /* if (adapter->detect_tx_hung) {
     ....
     }
  */

  adapter->total_tx_bytes += total_tx_bytes;
  adapter->total_tx_packets += total_tx_packets;
  return cleaned;
}

static void dump_ethernet_header(struct eth_hdr *ehdr)
{
  char buf[256];

  sprintf(buf,"Destination MAC: %x:%x:%x:%x:%x:%x, Source MAC: %x:%x:%x:%x:%x:%x, Type: 0x%X\n",
          ehdr->dest.addr[0],ehdr->dest.addr[1],ehdr->dest.addr[2],
          ehdr->dest.addr[3],ehdr->dest.addr[4],ehdr->dest.addr[5],
          ehdr->src.addr[0],ehdr->src.addr[1],ehdr->src.addr[2],
          ehdr->src.addr[3],ehdr->src.addr[4],ehdr->src.addr[5],
          ehdr->type);
  console_printf(buf);
}

err_t e1000_netdev_linkoutput(struct netif *nif,struct pbuf *p)
{
  //struct e1000_adapter *adapter=nif->state;
  //console_printf("[e1000] __e1000_netdev_linkoutput(): transmitting a packet via %s.\n",
  //               adapter->netdev->name);
  e1000_xmit_frame(p,nif);

  return ERR_OK;
}
