#include <sys/types.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <datalib/usrtc.h>
#include <errno.h>
#include <ipc/port.h>
#include <ipc/channel.h>
#include <ipc/rpc.h>
#include <ns/ns.h>
#include <errno.h>
#include <devio/devio.h>
#include <sys/poll.h>
#include <sys/thread.h>
#include <debug.h>
#include <ns/ns.h>
#include <sys/procmgr.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <stddef.h>
#include <sys/ievent.h>
#include <arch/atomic.h>
#include <arch/bitwise.h>
#include <sys/mman.h>
#include <pci/pci.h>
#include <net/network.h>
#include <pthread.h>
#include <sys/dma.h>
#include <netif/etharp.h>

#include "e1000.h"
#include "e1000_hw.h"
#include "e1000_osdep.h"

pci_dev_t devs[10];

#define VENDOR_ID_INTEL 0x8086
#define E1000_DEV_ID_82571EB_COPPER  0x105E
#define E1000_MAX_IRQS  32
#define E1000_TX_QUEUE_LEN 32
#define MAXCARDS  32

#define PCI_DMA_TODEVICE   0
#define PCI_DMA_FROMDEVICE 1

static int __cards_found=0;
static struct e1000_adapter *__adapter;

static void __e1000_get_hw_control(struct e1000_adapter *adapter);
static int __e1000_check_phy_reset_block(struct e1000_hw *hw);
static void e1000_clean_tx_ring(struct e1000_adapter *adapter,
                                struct e1000_tx_ring *tx_ring);
static void e1000_clean_rx_ring(struct e1000_adapter *adapter,
                                struct e1000_rx_ring *rx_ring);
static bool e1000_check_64k_bound(struct e1000_adapter *adapter,
                                  void *start, unsigned long len);

extern int e1000_diag_test(struct e1000_adapter *adapter,bool online,
                           uint64_t *data);

#define vmalloc(s)  malloc((s))
#define pci_free_consistent(a,b,c,d)

static struct e1000_adapter *__adapter;
static pthread_t __intr_threads[MAXCARDS];
static 

void *pci_alloc_consistent(pci_dev_t *pdev,size_t size,dma_addr_t *dma_handle)
{
  int pages=PAGE_ALIGN(size)>>PAGE_WIDTH;
  void *pa=dma_alloc_pages(pages);
  void *va;

  if( pa != MAP_FAILED ) {
    va=mmap(NULL,PAGE_ALIGN(size),PROT_READ | PROT_WRITE,MAP_PHYS|MAP_PRIVATE,
            NOFD,(long)pa);
    if( va != MAP_FAILED ) {
      *dma_handle=(dma_addr_t)pa;
      return va;
    }
    /* TODO: [mt] free DMA pages properly ! */
  }

  return NULL;
}

/**
 * e1000_free_tx_resources - Free Tx Resources per Queue
 * @adapter: board private structure
 * @tx_ring: Tx descriptor ring for a specific queue
 *
 * Free all transmit software resources
 **/

static void e1000_free_tx_resources(struct e1000_adapter *adapter,
                                    struct e1000_tx_ring *tx_ring)
{
  pci_dev_t *pdev = adapter->pdev;

  e1000_clean_tx_ring(adapter, tx_ring);

  free(tx_ring->buffer_info);
  tx_ring->buffer_info = NULL;

  pci_free_consistent(pdev, tx_ring->size, tx_ring->desc, tx_ring->dma);

  tx_ring->desc = NULL;
}

/**
 * e1000_free_all_tx_resources - Free Tx Resources for All Queues
 * @adapter: board private structure
 *
 * Free all transmit software resources
 **/
void e1000_free_all_tx_resources(struct e1000_adapter *adapter)
{
  int i;

  for (i = 0; i < adapter->num_tx_queues; i++) {
    e1000_free_tx_resources(adapter, &adapter->tx_ring[i]);
  }
}

/**
 * e1000_free_rx_resources - Free Rx Resources
 * @adapter: board private structure
 * @rx_ring: ring to clean the resources from
 *
 * Free all receive software resources
 **/

static void e1000_free_rx_resources(struct e1000_adapter *adapter,
                                    struct e1000_rx_ring *rx_ring)
{
  struct pci_dev *pdev = adapter->pdev;

  e1000_clean_rx_ring(adapter, rx_ring);

  free(rx_ring->buffer_info);
  rx_ring->buffer_info = NULL;

  pci_free_consistent(pdev, rx_ring->size, rx_ring->desc, rx_ring->dma);

  rx_ring->desc = NULL;
}

/**
 * e1000_free_all_rx_resources - Free Rx Resources for All Queues
 * @adapter: board private structure
 *
 * Free all receive software resources
 **/
void e1000_free_all_rx_resources(struct e1000_adapter *adapter)
{
  int i;

  for (i = 0; i < adapter->num_rx_queues; i++)
    e1000_free_rx_resources(adapter, &adapter->rx_ring[i]);
}

/**
 * e1000_clean_rx_ring - Free Rx Buffers per Queue
 * @adapter: board private structure
 * @rx_ring: ring to free buffers from
 **/

static void e1000_clean_rx_ring(struct e1000_adapter *adapter,
                                struct e1000_rx_ring *rx_ring)
{
  struct e1000_buffer *buffer_info;
  struct netif *netdev=adapter->netdev;
  pci_dev_t *pdev = adapter->pdev;
  unsigned long size;
  unsigned int i;

  /* Free all the Rx ring sk_buffs */
  for (i = 0; i < rx_ring->count; i++) {
    buffer_info = &rx_ring->buffer_info[i];
    if (buffer_info->skb) {
      pci_unmap_single(pdev,
                       buffer_info->dma,
                       buffer_info->length,
                       PCI_DMA_FROMDEVICE);

      netdev->pbuf_free(buffer_info->skb);
      buffer_info->skb = NULL;
    }
  }

  size = sizeof(struct e1000_buffer) * rx_ring->count;
  memset(rx_ring->buffer_info, 0, size);

  /* Zero out the descriptor ring */
  memset(rx_ring->desc, 0, rx_ring->size);

  rx_ring->next_to_clean = 0;
  rx_ring->next_to_use = 0;

  writel(0, adapter->hw.hw_addr + rx_ring->rdh);
  writel(0, adapter->hw.hw_addr + rx_ring->rdt);
  /* TODO: [mt] what is 'rx_ring->rdt' ? */
}

/**
 * e1000_setup_rx_resources - allocate Rx resources (Descriptors)
 * @adapter: board private structure
 * @rxdr:    rx descriptor ring (for a specific queue) to setup
 *
 * Returns 0 on success, negative on failure
 **/
static int e1000_setup_rx_resources(struct e1000_adapter *adapter,
                                    struct e1000_rx_ring *rxdr)
{
  pci_dev_t *pdev = adapter->pdev;
  int size, desc_len;

  size = sizeof(struct e1000_buffer) * rxdr->count;
  rxdr->buffer_info = malloc(size);
  if (!rxdr->buffer_info) {
    console_printf("Unable to allocate memory for the receive descriptor ring\n");
    return -ENOMEM;
  }
  memset(rxdr->buffer_info, 0, size);

  if (adapter->hw.mac_type <= e1000_82547_rev_2)
    desc_len = sizeof(struct e1000_rx_desc);
  else
    desc_len = sizeof(union e1000_rx_desc_packet_split);

  /* Round up to nearest 4K */

  rxdr->size = rxdr->count * desc_len;
  rxdr->size = ALIGN(rxdr->size, 4096);

  rxdr->desc = pci_alloc_consistent(pdev, rxdr->size, &rxdr->dma);
  if (!rxdr->desc) {
    console_printf("Unable to allocate memory for the receive descriptor ring\n");
    free(rxdr->buffer_info);
    return -ENOMEM;
  }

  /* Fix for errata 23, can't cross 64kB boundary */
  if (!e1000_check_64k_bound(adapter, rxdr->desc, rxdr->size)) {
    console_printf("[e1000]: Feature not implemented ! (#5)\n");
    abort();
  }

  memset(rxdr->desc, 0, rxdr->size);

  rxdr->next_to_clean = 0;
  rxdr->next_to_use = 0;

  return 0;
}

/**
 * e1000_setup_all_rx_resources - wrapper to allocate Rx resources
 * 				  (Descriptors) for all queues
 * @adapter: board private structure
 *
 * Return 0 on success, negative on failure
 **/

int e1000_setup_all_rx_resources(struct e1000_adapter *adapter)
{
  int i, err = 0;

  for (i = 0; i < adapter->num_rx_queues; i++) {
    err = e1000_setup_rx_resources(adapter, &adapter->rx_ring[i]);
    if (err) {
      console_printf("Allocation for Rx Queue %u failed\n", i);
      for (i-- ; i >= 0; i--)
        e1000_free_rx_resources(adapter,
                                &adapter->rx_ring[i]);
      break;
    }
  }

  return err;
}

static void
e1000_unmap_and_free_tx_resource(struct e1000_adapter *adapter,
			struct e1000_buffer *buffer_info)
{
  if (buffer_info->dma) {
    pci_unmap_page(adapter->pdev,
                   buffer_info->dma,
                   buffer_info->length,
                   PCI_DMA_TODEVICE);
    buffer_info->dma = 0;
  }
  if (buffer_info->skb) {
    adapter->netdev->pbuf_free(buffer_info->skb);
    buffer_info->skb = NULL;
  }
  /* buffer_info must be completely set up in the transmit path */
}

/**
 * e1000_clean_tx_ring - Free Tx Buffers
 * @adapter: board private structure
 * @tx_ring: ring to be cleaned
 **/

static void e1000_clean_tx_ring(struct e1000_adapter *adapter,
                                struct e1000_tx_ring *tx_ring)
{
  struct e1000_buffer *buffer_info;
  unsigned long size;
  unsigned int i;

  /* Free all the Tx ring sk_buffs */

  for (i = 0; i < tx_ring->count; i++) {
    buffer_info = &tx_ring->buffer_info[i];
    e1000_unmap_and_free_tx_resource(adapter, buffer_info);
  }

  size = sizeof(struct e1000_buffer) * tx_ring->count;
  memset(tx_ring->buffer_info, 0, size);

  /* Zero out the descriptor ring */

  memset(tx_ring->desc, 0, tx_ring->size);

  tx_ring->next_to_use = 0;
  tx_ring->next_to_clean = 0;
  tx_ring->last_tx_tso = 0;

  writel(0, adapter->hw.hw_addr + tx_ring->tdh);
  writel(0, adapter->hw.hw_addr + tx_ring->tdt);
}

/******************************************************************************
 * Adjusts the statistic counters when a frame is accepted by TBI_ACCEPT
 *
 * hw - Struct containing variables accessed by shared code
 * frame_len - The length of the frame in question
 * mac_addr - The Ethernet destination address of the frame in question
 *****************************************************************************/
void
e1000_tbi_adjust_stats(struct e1000_hw *hw,
                       struct e1000_hw_stats *stats,
                       uint32_t frame_len,
                       uint8_t *mac_addr)
{
    uint64_t carry_bit;

    /* First adjust the frame length. */
    frame_len--;
    /* We need to adjust the statistics counters, since the hardware
     * counters overcount this packet as a CRC error and undercount
     * the packet as a good packet
     */
    /* This packet should not be counted as a CRC error.    */
    stats->crcerrs--;
    /* This packet does count as a Good Packet Received.    */
    stats->gprc++;

    /* Adjust the Good Octets received counters             */
    carry_bit = 0x80000000 & stats->gorcl;
    stats->gorcl += frame_len;
    /* If the high bit of Gorcl (the low 32 bits of the Good Octets
     * Received Count) was one before the addition,
     * AND it is zero after, then we lost the carry out,
     * need to add one to Gorch (Good Octets Received Count High).
     * This could be simplified if all environments supported
     * 64-bit integers.
     */
    if (carry_bit && ((stats->gorcl & 0x80000000) == 0))
        stats->gorch++;
    /* Is this a broadcast or multicast?  Check broadcast first,
     * since the test for a multicast frame will test positive on
     * a broadcast frame.
     */
    if ((mac_addr[0] == (uint8_t) 0xff) && (mac_addr[1] == (uint8_t) 0xff))
        /* Broadcast packet */
        stats->bprc++;
    else if (*mac_addr & 0x01)
        /* Multicast packet */
        stats->mprc++;

    if (frame_len == hw->max_frame_size) {
        /* In this case, the hardware has overcounted the number of
         * oversize frames.
         */
        if (stats->roc > 0)
            stats->roc--;
    }

    /* Adjust the bin counters when the extra byte put the frame in the
     * wrong bin. Remember that the frame_len was adjusted above.
     */
    if (frame_len == 64) {
        stats->prc64++;
        stats->prc127--;
    } else if (frame_len == 127) {
        stats->prc127++;
        stats->prc255--;
    } else if (frame_len == 255) {
        stats->prc255++;
        stats->prc511--;
    } else if (frame_len == 511) {
        stats->prc511++;
        stats->prc1023--;
    } else if (frame_len == 1023) {
        stats->prc1023++;
        stats->prc1522--;
    } else if (frame_len == 1522) {
        stats->prc1522++;
    }
}

/**
 * e1000_check_64k_bound - check that memory doesn't cross 64kB boundary
 * @adapter: address of board private structure
 * @start: address of beginning of memory
 * @len: length of memory
 **/
static bool e1000_check_64k_bound(struct e1000_adapter *adapter,
                                  void *start, unsigned long len)
{
  unsigned long begin = (unsigned long) start;
  unsigned long end = begin + len;

  /* First rev 82545 and 82546 need to not allow any memory
   * write location to cross 64k boundary due to errata 23 */
  if (adapter->hw.mac_type == e1000_82545 ||
      adapter->hw.mac_type == e1000_82546) {
    return ((begin ^ (end - 1)) >> 16) != 0 ? false : true;
  }

  return true;
}

/**
 * e1000_setup_tx_resources - allocate Tx resources (Descriptors)
 * @adapter: board private structure
 * @txdr:    tx descriptor ring (for a specific queue) to setup
 *
 * Return 0 on success, negative on failure
 **/

static int e1000_setup_tx_resources(struct e1000_adapter *adapter,
                                    struct e1000_tx_ring *txdr)
{
  pci_dev_t *pdev = adapter->pdev;
  struct netif *netdev=adapter->netdev;
  int size,i;

  size = sizeof(struct e1000_buffer) * txdr->count;
  txdr->buffer_info = malloc(size);
  if (!txdr->buffer_info) {
    console_printf("Unable to allocate memory for the transmit descriptor ring\n");
    return -ENOMEM;
  }
  memset(txdr->buffer_info, 0, size);

  /* round up to nearest 4K */

  txdr->size = txdr->count * sizeof(struct e1000_tx_desc);
  txdr->size = PAGE_ALIGN(txdr->size);

  txdr->desc = pci_alloc_consistent(pdev, txdr->size, &txdr->dma);
  if (!txdr->desc) {
    free(txdr->buffer_info);
    console_printf("Unable to allocate memory for the transmit descriptor ring\n");
    return -ENOMEM;
  }

  memset(txdr->desc,0,txdr->size);

  /* Fix for errata 23, can't cross 64kB boundary */
  if (!e1000_check_64k_bound(adapter, txdr->desc, txdr->size)) {
    console_printf("[e1000]: Feature not implemented ! (#6)\n");
    abort();
  }

  memset(txdr->desc, 0, txdr->size);
  txdr->next_to_use = 0;
  txdr->next_to_clean = 0;
  spin_lock_init(&txdr->tx_lock);

  /* TODO: [mt] we're using pre-allocated array of Tx descriptors. */
  /* Now we should initialize pre-allocated array of packet buffers
   * for Tx purposes.
   */

  for( i=0; i<txdr->count; i++ ) {
    struct e1000_buffer *buffer_info=&txdr->buffer_info[i];
    struct pbuf *pb=netdev->pbuf_alloc(PBUF_RAW,E1000_DEFAULT_TXBUFFER_SIZE,PBUF_DMA);

    if( !pb ) {
      console_printf("Failed to pre-allocate Rx buffer N %d of %d bytes !\n",
                     i,E1000_DEFAULT_TXBUFFER_SIZE);
      for(;;);
    }

    buffer_info->skb=pb;
    buffer_info->dma=pb->dma;
    buffer_info->length=pb->tot_len;
  }

descr_failed:
  return 0;
}

int e1000_setup_all_tx_resources(struct e1000_adapter *adapter)
{
  int i, err = 0;

  for (i = 0; i < adapter->num_tx_queues; i++) {
    err = e1000_setup_tx_resources(adapter, &adapter->tx_ring[i]);
    if( err ) {
      console_printf("Allocation for Tx Queue %u failed\n", i);
      for( i-- ; i >= 0; i-- ) {
        e1000_free_tx_resources(adapter,&adapter->tx_ring[i]);
      }
      break;
    }
  }
  return err;
}

static void __e1000_power_down_phy(struct e1000_adapter *adapter)
{
  /* Power down the PHY so no link is implied when interface is down *
   * The PHY cannot be powered down if any of the following is TRUE *
   * (a) WoL is enabled
   * (b) AMT is active
   * (c) SoL/IDER session is active */
  if (!adapter->wol && adapter->hw.mac_type >= e1000_82540 &&
      adapter->hw.media_type == e1000_media_type_copper) {
    uint16_t mii_reg = 0;

    switch (adapter->hw.mac_type) {
      case e1000_82540:
      case e1000_82545:
      case e1000_82545_rev_3:
      case e1000_82546:
      case e1000_82546_rev_3:
      case e1000_82541:
      case e1000_82541_rev_2:
      case e1000_82547:
      case e1000_82547_rev_2:
        if (E1000_READ_REG(&adapter->hw, MANC) &
            E1000_MANC_SMBUS_EN)
          goto out;
        break;
      case e1000_82571:
      case e1000_82572:
      case e1000_82573:
      case e1000_80003es2lan:
      case e1000_ich8lan:
        if (e1000_check_mng_mode(&adapter->hw) ||
            __e1000_check_phy_reset_block(&adapter->hw))
          goto out;
        break;
      default:
        goto out;
    }
    e1000_read_phy_reg(&adapter->hw, PHY_CTRL, &mii_reg);
    mii_reg |= MII_CR_POWER_DOWN;
    e1000_write_phy_reg(&adapter->hw, PHY_CTRL, mii_reg);
    mdelay(1);
  }
out:
  return;
}

void dump_ethernet_header(struct eth_hdr *ehdr)
{
  char buf[256];

  sprintf(buf,"Destination MAC: %x:%x:%x:%x:%x:%x, Source MAC: %x:%x:%x:%x:%x:%x, Type: 0x%X\n",
          ehdr->dest.addr[0],ehdr->dest.addr[1],ehdr->dest.addr[2],
          ehdr->dest.addr[3],ehdr->dest.addr[4],ehdr->dest.addr[5],
          ehdr->src.addr[0],ehdr->src.addr[1],ehdr->src.addr[2],
          ehdr->src.addr[3],ehdr->src.addr[4],ehdr->src.addr[5],
          ehdr->type);
  console_printf(buf);
}

static bool e1000_clean_rx_irq(struct e1000_adapter *adapter,
                               struct e1000_rx_ring *rx_ring)
{
  struct netif *netdev = adapter->netdev;
  pci_dev_t *pdev = adapter->pdev;
  struct e1000_rx_desc *rx_desc, *next_rxd;
  struct e1000_buffer *buffer_info, *next_buffer;
  uint32_t length;
  uint8_t last_byte;
  unsigned int i;
  int cleaned_count = 0;
  bool cleaned = false;
  unsigned int total_rx_bytes=0, total_rx_packets=0;

  for(i = rx_ring->next_to_use;;) {
    rx_desc = E1000_RX_DESC(*rx_ring, i);
    buffer_info = &rx_ring->buffer_info[i];

    if (rx_desc->status & E1000_RXD_STAT_DD) {
      struct pbuf *skb,*new_skb,*q;
      uint8_t status;
      unsigned char *in_data;

      status = rx_desc->status;
      skb = buffer_info->skb;

//      console_printf("[e1000] e1000_clean_rx_irq() descriptor %d contains data (%d bytes), BUFFER=%p\n",
//                     i,rx_desc->length,buffer_info);
//      dump_ethernet_header((struct eth_hdr *)buffer_info->skb->payload);

      rx_desc->status = 0;

      length = le16_to_cpu(rx_desc->length);

      if( !(status & E1000_RXD_STAT_EOP) ) {
        /* All receives must fit into a single buffer */
        console_printf("[e1000]: %s: Receive packet consumed multiple"
                     " buffers\n", netdev->name);
        /* recycle */
        buffer_info->skb = skb;
        goto next_desc;
      }

      /* probably a little skewed due to removing CRC */
      total_rx_bytes += length;
      total_rx_packets++;

      /* Let the stack to process the packet. */
      new_skb=pbuf_alloc(PBUF_RAW,length,PBUF_RAM);
      if( !new_skb ) {
        console_printf("Failed to allocate a new SKB of %d bytes !\n",length);
        for(;;);
      }

      in_data=(unsigned char *)buffer_info->skb->payload;
      for( q=new_skb; q != NULL; q=q->next ) {
        memcpy(q->payload,in_data,q->len);
        in_data+=q->len;
      }

      netdev->input(new_skb,netdev);

      buffer_info->skb = NULL;
      cleaned_count++;
next_desc:
      rx_desc->status = 0;

      /* use prefetched values */
      rx_desc = next_rxd;
      buffer_info = next_buffer;
    }

    if( ++i >= rx_ring->count )
      i=0;

    if( i == rx_ring->next_to_use ) {
      rx_ring->next_to_clean = i;
      break;
    }
  }

  if( cleaned_count ) {
    adapter->alloc_rx_buf(adapter, rx_ring, cleaned_count);
  }

  adapter->total_rx_packets += total_rx_packets;
  adapter->total_rx_bytes += total_rx_bytes;
  return cleaned;
}

/**
 * e1000_alloc_rx_buffers - Replace used receive buffers; legacy & extended
 * @adapter: address of board private structure
 **/
static void e1000_alloc_rx_buffers(struct e1000_adapter *adapter,
                                   struct e1000_rx_ring *rx_ring,
                                   int cleaned_count)
{
  struct netif *netdev = adapter->netdev;
  pci_dev_t *pdev = adapter->pdev;
  struct e1000_rx_desc *rx_desc;
  struct e1000_buffer *buffer_info;
  struct pbuf *skb;
  unsigned int i;
  unsigned int bufsz = adapter->rx_buffer_len + NET_IP_ALIGN;

  //console_printf("e1000_alloc_rx_buffer(): cleaned_count=%d\n",
  //               cleaned_count);

  i = rx_ring->next_to_use;

  while( cleaned_count-- ) {
    buffer_info = &rx_ring->buffer_info[i];
    skb = buffer_info->skb;

    if (skb) {
      //console_printf("Reusing PBUF %p, payload=%p, DMA=%p\n",
      //skb,skb->payload,skb->dma);
      goto map_skb;
    }

    skb=netdev->pbuf_alloc(PBUF_RAW,bufsz,PBUF_DMA);
    if( !skb ) {
      console_printf("e1000_alloc_rx_buffers(): Failed to allocate pbuf of %d bytes !\n",
                     bufsz);
      for(;;);
    }
    //console_printf("Allocated PBUF %p, payload=%p, DMA=%p\n",
    //               skb,skb->payload,skb->dma);
map_skb:
    buffer_info->skb=skb;
    buffer_info->dma=skb->dma;

    /* Fix for errata 23, can't cross 64kB boundary */
    if (!e1000_check_64k_bound(adapter,(void *)(unsigned long)buffer_info->dma,
                             adapter->rx_buffer_len)) {
      console_printf("[e1000]: Feature not implemented ! (#1)\n");
      abort();
    }

    rx_desc = E1000_RX_DESC(*rx_ring, i);
    rx_desc->buffer_addr = cpu_to_le64(buffer_info->dma);

    /* Force memory writes to complete before letting h/w
     * know there are new descriptors to fetch.  (Only
     * applicable for weak-ordered memory model archs,
     * such as IA-64). */
    wmb();
    writel(i, adapter->hw.hw_addr + rx_ring->rdt);

    if( ++i >= rx_ring->count ) {
      i=0;
    }
  }
  rx_ring->next_to_use = i;  
}

/**
 * e1000_configure_rx - Configure 8254xe1000_alloc_rx_buffers Receive Unit after Reset
 * @adapter: board private structure
 *
 * Configure the Rx unit of the MAC after a reset.
 **/
static void e1000_configure_rx(struct e1000_adapter *adapter)
{
  uint64_t rdba;
  struct e1000_hw *hw = &adapter->hw;
  uint32_t rdlen, rctl, rxcsum, ctrl_ext;

  rdlen = adapter->rx_ring[0].count * sizeof(struct e1000_rx_desc);
  adapter->clean_rx = e1000_clean_rx_irq;
  adapter->alloc_rx_buf = e1000_alloc_rx_buffers;

  /* disable receives while setting up the descriptors */
  rctl = E1000_READ_REG(hw, RCTL);
  E1000_WRITE_REG(hw, RCTL, rctl & ~E1000_RCTL_EN);

  /* set the Receive Delay Timer Register */
  E1000_WRITE_REG(hw, RDTR, adapter->rx_int_delay);

  if (hw->mac_type >= e1000_82540) {
    E1000_WRITE_REG(hw, RADV, adapter->rx_abs_int_delay);
    if (adapter->itr_setting != 0)
      E1000_WRITE_REG(hw, ITR,
                      1000000000 / (adapter->itr * 256));
  }

  if (hw->mac_type >= e1000_82571) {
    ctrl_ext = E1000_READ_REG(hw, CTRL_EXT);
    /* Reset delay timers after every interrupt */
    ctrl_ext |= E1000_CTRL_EXT_INT_TIMER_CLR;
#ifdef CONFIG_E1000_NAPI
    /* Auto-Mask interrupts upon ICR access */
    ctrl_ext |= E1000_CTRL_EXT_IAME;
    E1000_WRITE_REG(hw, IAM, 0xffffffff);
#endif
    E1000_WRITE_REG(hw, CTRL_EXT, ctrl_ext);
    E1000_WRITE_FLUSH(hw);
  }

  /* Setup the HW Rx Head and Tail Descriptor Pointers and
   * the Base and Length of the Rx Descriptor Ring */
  switch (adapter->num_rx_queues) {
    case 1:
    default:
      console_printf("Setting up DMA: %p\n",adapter->rx_ring[0].dma);
      rdba = adapter->rx_ring[0].dma;
      E1000_WRITE_REG(hw, RDLEN, rdlen);
      E1000_WRITE_REG(hw, RDBAH, (rdba >> 32));
      E1000_WRITE_REG(hw, RDBAL, (rdba & 0x00000000ffffffffULL));
      E1000_WRITE_REG(hw, RDT, 0);
      E1000_WRITE_REG(hw, RDH, 0);
      adapter->rx_ring[0].rdh = ((hw->mac_type >= e1000_82543) ? E1000_RDH : E1000_82542_RDH);
      adapter->rx_ring[0].rdt = ((hw->mac_type >= e1000_82543) ? E1000_RDT : E1000_82542_RDT);
      break;
  }

  /* Enable 82543 Receive Checksum Offload for TCP and UDP */
  if (hw->mac_type >= e1000_82543) {
    rxcsum = E1000_READ_REG(hw, RXCSUM);
    if (adapter->rx_csum == true) {
      rxcsum |= E1000_RXCSUM_TUOFL;

      /* Enable 82571 IPv4 payload checksum for UDP fragments
       * Must be used in conjunction with packet-split. */
      if ((hw->mac_type >= e1000_82571) &&
          (adapter->rx_ps_pages)) {
        rxcsum |= E1000_RXCSUM_IPPCSE;
      }
    } else {
      rxcsum &= ~E1000_RXCSUM_TUOFL;
      /* don't need to clear IPPCSE as it defaults to 0 */
    }
    E1000_WRITE_REG(hw, RXCSUM, rxcsum);
  }

  /* enable early receives on 82573, only takes effect if using > 2048
   * byte total frame size.  for example only for jumbo frames */
#define E1000_ERT_2048 0x100
  if (hw->mac_type == e1000_82573)
    E1000_WRITE_REG(hw, ERT, E1000_ERT_2048);

  /* Enable Receives */
  E1000_WRITE_REG(hw, RCTL, rctl);
  console_printf("==> RCTL = 0x%X\n",rctl);
}

/**
 * e1000_setup_rctl - configure the receive control registers
 * @adapter: Board private structure
 **/
#define PAGE_USE_COUNT(S) (((S) >> PAGE_SHIFT) + \
                           (((S) & (PAGE_SIZE - 1)) ? 1 : 0))

static void e1000_setup_rctl(struct e1000_adapter *adapter)
{
  uint32_t rctl;

  rctl = E1000_READ_REG(&adapter->hw, RCTL);

  rctl &= ~(3 << E1000_RCTL_MO_SHIFT);

  rctl |= E1000_RCTL_EN | E1000_RCTL_BAM |
    E1000_RCTL_LBM_NO | E1000_RCTL_RDMTS_HALF |
    (adapter->hw.mc_filter_type << E1000_RCTL_MO_SHIFT);

  if (adapter->hw.tbi_compatibility_on == 1)
    rctl |= E1000_RCTL_SBP;
  else
    rctl &= ~E1000_RCTL_SBP;

  console_printf("===> MTU: %d, ETH_DATA_LEN: %d\n",
                 adapter->netdev->mtu,ETH_DATA_LEN);
  if (adapter->netdev->mtu <= ETH_DATA_LEN)
    rctl &= ~E1000_RCTL_LPE;
  else
    rctl |= E1000_RCTL_LPE;

  /* Setup buffer sizes */
  rctl &= ~E1000_RCTL_SZ_4096;
  rctl |= E1000_RCTL_BSEX;

  console_printf("===> adapter->rx_buffer_len=%d\n",
                 adapter->rx_buffer_len);
  switch (adapter->rx_buffer_len) {
    case E1000_RXBUFFER_256:
      rctl |= E1000_RCTL_SZ_256;
      rctl &= ~E1000_RCTL_BSEX;
      break;
    case E1000_RXBUFFER_512:
      rctl |= E1000_RCTL_SZ_512;
      rctl &= ~E1000_RCTL_BSEX;
      break;
    case E1000_RXBUFFER_1024:
      rctl |= E1000_RCTL_SZ_1024;
      rctl &= ~E1000_RCTL_BSEX;
      break;
    case E1000_RXBUFFER_2048:
    default:
      rctl |= E1000_RCTL_SZ_2048;
      rctl &= ~E1000_RCTL_BSEX;
      break;
    case E1000_RXBUFFER_4096:
      rctl |= E1000_RCTL_SZ_4096;
      break;
    case E1000_RXBUFFER_8192:
      rctl |= E1000_RCTL_SZ_8192;
      break;
    case E1000_RXBUFFER_16384:
      rctl |= E1000_RCTL_SZ_16384;
      break;
  }

#if 0 /* [mt] Packet-Splitting deprecated. */
#ifndef CONFIG_E1000_DISABLE_PACKET_SPLIT
  /* 82571 and greater support packet-split where the protocol
   * header is placed in skb->data and the packet data is
   * placed in pages hanging off of skb_shinfo(skb)->nr_frags.
   * In the case of a non-split, skb->data is linearly filled,
   * followed by the page buffers.  Therefore, skb->data is
   * sized to hold the largest protocol header.
   */
  /* allocations using alloc_page take too long for regular MTU
   * so only enable packet split for jumbo frames */
  pages = PAGE_USE_COUNT(adapter->netdev->mtu);
  if ((adapter->hw.mac_type >= e1000_82571) && (pages <= 3) &&
      PAGE_SIZE <= 16384 && (rctl & E1000_RCTL_LPE))
    adapter->rx_ps_pages = pages;
  else
    adapter->rx_ps_pages = 0;
#endif
  if (adapter->rx_ps_pages) {
    /* Configure extra packet-split registers */
    rfctl = E1000_READ_REG(&adapter->hw, RFCTL);
    rfctl |= E1000_RFCTL_EXTEN;
    /* disable packet split support for IPv6 extension headers,
     * because some malformed IPv6 headers can hang the RX */
    rfctl |= (E1000_RFCTL_IPV6_EX_DIS |
              E1000_RFCTL_NEW_IPV6_EXT_DIS);

    E1000_WRITE_REG(&adapter->hw, RFCTL, rfctl);

    rctl |= E1000_RCTL_DTYP_PS;

    psrctl |= adapter->rx_ps_bsize0 >>
      E1000_PSRCTL_BSIZE0_SHIFT;

    switch (adapter->rx_ps_pages) {
      case 3:
        psrctl |= PAGE_SIZE <<
          E1000_PSRCTL_BSIZE3_SHIFT;
      case 2:
        psrctl |= PAGE_SIZE <<
          E1000_PSRCTL_BSIZE2_SHIFT;
      case 1:
        psrctl |= PAGE_SIZE >>
          E1000_PSRCTL_BSIZE1_SHIFT;
        break;
    }
    
    E1000_WRITE_REG(&adapter->hw, PSRCTL, psrctl);
  }
#endif /* #if 0 */
  console_printf("====> writing to RCTL: 0x%X\n",rctl);
  E1000_WRITE_REG(&adapter->hw, RCTL, rctl);
}

/**
 * e1000_configure_tx - Configure 8254x Transmit Unit after Reset
 * @adapter: board private structure
 *
 * Configure the Tx unit of the MAC after a reset.
 **/

#define DEFAULT_TIDV                   2
#define DEFAULT_TADV                   32

static void
e1000_configure_tx(struct e1000_adapter *adapter)
{
  uint64_t tdba;
  struct e1000_hw *hw = &adapter->hw;
  uint32_t tdlen, tctl, tipg, tarc;
  uint32_t ipgr1, ipgr2;

  /* TODO: [mt] setup interrupt delays properly, not hardcode them ! */
  adapter->tx_int_delay = DEFAULT_TIDV;
  adapter->tx_abs_int_delay = DEFAULT_TADV;

  /* Setup the HW Tx Head and Tail descriptor pointers */

  switch (adapter->num_tx_queues) {
    case 1:
    default:
      tdba = adapter->tx_ring[0].dma;
      tdlen = adapter->tx_ring[0].count * sizeof(struct e1000_tx_desc);
      E1000_WRITE_REG(hw, TDLEN, tdlen);
      E1000_WRITE_REG(hw, TDBAH, (tdba >> 32));
      E1000_WRITE_REG(hw, TDBAL, (tdba & 0x00000000ffffffffULL));
      E1000_WRITE_REG(hw, TDT, 0);
      E1000_WRITE_REG(hw, TDH, 0);
      adapter->tx_ring[0].tdh = ((hw->mac_type >= e1000_82543) ? E1000_TDH : E1000_82542_TDH);
      adapter->tx_ring[0].tdt = ((hw->mac_type >= e1000_82543) ? E1000_TDT : E1000_82542_TDT);
      break;
  }

  /* Set the default values for the Tx Inter Packet Gap timer */
  if (adapter->hw.mac_type <= e1000_82547_rev_2 &&
      (hw->media_type == e1000_media_type_fiber ||
       hw->media_type == e1000_media_type_internal_serdes))
    tipg = DEFAULT_82543_TIPG_IPGT_FIBER;
  else
    tipg = DEFAULT_82543_TIPG_IPGT_COPPER;

  switch (hw->mac_type) {
    case e1000_82542_rev2_0:
    case e1000_82542_rev2_1:
      tipg = DEFAULT_82542_TIPG_IPGT;
      ipgr1 = DEFAULT_82542_TIPG_IPGR1;
      ipgr2 = DEFAULT_82542_TIPG_IPGR2;
      break;
    case e1000_80003es2lan:
      ipgr1 = DEFAULT_82543_TIPG_IPGR1;
      ipgr2 = DEFAULT_80003ES2LAN_TIPG_IPGR2;
      break;
    default:
      ipgr1 = DEFAULT_82543_TIPG_IPGR1;
      ipgr2 = DEFAULT_82543_TIPG_IPGR2;
      break;
  }
  tipg |= ipgr1 << E1000_TIPG_IPGR1_SHIFT;
  tipg |= ipgr2 << E1000_TIPG_IPGR2_SHIFT;
  E1000_WRITE_REG(hw, TIPG, tipg);

  /* Set the Tx Interrupt Delay register */

  E1000_WRITE_REG(hw, TIDV, adapter->tx_int_delay);
  if (hw->mac_type >= e1000_82540)
    E1000_WRITE_REG(hw, TADV, adapter->tx_abs_int_delay);

  /* Program the Transmit Control Register */

  tctl = E1000_READ_REG(hw, TCTL);
  tctl &= ~E1000_TCTL_CT;
  tctl |= E1000_TCTL_PSP | E1000_TCTL_RTLC |
    (E1000_COLLISION_THRESHOLD << E1000_CT_SHIFT);

  if (hw->mac_type == e1000_82571 || hw->mac_type == e1000_82572) {
    tarc = E1000_READ_REG(hw, TARC0);
    /* set the speed mode bit, we'll clear it if we're not at
     * gigabit link later */
    tarc |= (1 << 21);
    E1000_WRITE_REG(hw, TARC0, tarc);
  } else if (hw->mac_type == e1000_80003es2lan) {
    tarc = E1000_READ_REG(hw, TARC0);
    tarc |= 1;
    E1000_WRITE_REG(hw, TARC0, tarc);
    tarc = E1000_READ_REG(hw, TARC1);
    tarc |= 1;
    E1000_WRITE_REG(hw, TARC1, tarc);
  }

  e1000_config_collision_dist(hw);

  /* Setup Transmit Descriptor Settings for eop descriptor */
  adapter->txd_cmd = E1000_TXD_CMD_EOP | E1000_TXD_CMD_IFCS;

  /* only set IDE if we are delaying interrupts using the timers */
  if (adapter->tx_int_delay)
    adapter->txd_cmd |= E1000_TXD_CMD_IDE;

  if (hw->mac_type < e1000_82543)
    adapter->txd_cmd |= E1000_TXD_CMD_RPS;
  else
    adapter->txd_cmd |= E1000_TXD_CMD_RS;

  /* Cache if we're 82544 running in PCI-X because we'll
   * need this to apply a workaround later in the send path. */
  if (hw->mac_type == e1000_82544 &&
      hw->bus_type == e1000_bus_type_pcix)
    adapter->pcix_82544 = 1;

  /* TODO: [mt] Should we setup E1000_TCTL_EN flag in TCTL explicitely ? */
  tctl |= E1000_TCTL_EN;

  E1000_WRITE_REG(hw, TCTL, tctl);
}

static void
e1000_init_manageability(struct e1000_adapter *adapter)
{
  if (adapter->en_mng_pt) {
    uint32_t manc = E1000_READ_REG(&adapter->hw, MANC);

    /* disable hardware interception of ARP */
    manc &= ~(E1000_MANC_ARP_EN);

    /* enable receiving management packets to the host */
    /* this will probably generate destination unreachable messages
     * from the host OS, but the packets will be handled on SMBUS */
    if (adapter->hw.has_manc2h) {
      uint32_t manc2h = E1000_READ_REG(&adapter->hw, MANC2H);

      manc |= E1000_MANC_EN_MNG2HOST;
#define E1000_MNG2HOST_PORT_623 (1 << 5)
#define E1000_MNG2HOST_PORT_664 (1 << 6)
      manc2h |= E1000_MNG2HOST_PORT_623;
      manc2h |= E1000_MNG2HOST_PORT_664;
      E1000_WRITE_REG(&adapter->hw, MANC2H, manc2h);
    }

    E1000_WRITE_REG(&adapter->hw, MANC, manc);
  }
}

/**
 * e1000_configure - configure the hardware for RX and TX
 * @adapter = private board structure
 **/
static void e1000_configure(struct e1000_adapter *adapter)
{
  int i;

  /* TODO: [mt] commented: e1000_set_multi(netdev); */
  e1000_restore_vlan(adapter);
  e1000_init_manageability(adapter);

  e1000_configure_tx(adapter);
  e1000_setup_rctl(adapter);
  e1000_configure_rx(adapter);

  for (i = 0; i < adapter->num_rx_queues; i++) {
    struct e1000_rx_ring *ring = &adapter->rx_ring[i];
    adapter->alloc_rx_buf(adapter, ring,
                          E1000_TX_QUEUE_LEN/2);
  }

  adapter->tx_queue_len = adapter->rx_ring[0].count;
}

enum latency_range {
	lowest_latency = 0,
	low_latency = 1,
	bulk_latency = 2,
	latency_invalid = 255
};

/**
 * e1000_update_itr - update the dynamic ITR value based on statistics
 *      Stores a new ITR value based on packets and byte
 *      counts during the last interrupt.  The advantage of per interrupt
 *      computation is faster updates and more accurate ITR for the current
 *      traffic pattern.  Constants in this function were computed
 *      based on theoretical maximum wire speed and thresholds were set based
 *      on testing data as well as attempting to minimize response time
 *      while increasing bulk throughput.
 *      this functionality is controlled by the InterruptThrottleRate module
 *      parameter (see e1000_param.c)
 * @adapter: pointer to adapter
 * @itr_setting: current adapter->itr
 * @packets: the number of packets during this measurement interval
 * @bytes: the number of bytes during this measurement interval
 **/
static unsigned int e1000_update_itr(struct e1000_adapter *adapter,
                                     uint16_t itr_setting,int packets,int bytes)
{
  unsigned int retval = itr_setting;
  struct e1000_hw *hw = &adapter->hw;

  if (hw->mac_type < e1000_82540)
    goto update_itr_done;

  if (packets == 0)
    goto update_itr_done;

  switch (itr_setting) {
    case lowest_latency:
      /* jumbo frames get bulk treatment*/
      if (bytes/packets > 8000)
        retval = bulk_latency;
      else if ((packets < 5) && (bytes > 512))
        retval = low_latency;
      break;
    case low_latency:  /* 50 usec aka 20000 ints/s */
      if (bytes > 10000) {
        /* jumbo frames need bulk latency setting */
        if (bytes/packets > 8000)
          retval = bulk_latency;
        else if ((packets < 10) || ((bytes/packets) > 1200))
          retval = bulk_latency;
        else if ((packets > 35))
          retval = lowest_latency;
      } else if (bytes/packets > 2000)
        retval = bulk_latency;
      else if (packets <= 2 && bytes < 512)
        retval = lowest_latency;
      break;
    case bulk_latency: /* 250 usec aka 4000 ints/s */
      if (bytes > 25000) {
        if (packets > 35)
          retval = low_latency;
      } else if (bytes < 6000) {
        retval = low_latency;
      }
      break;
  }

update_itr_done:
  return retval;
}

static void e1000_set_itr(struct e1000_adapter *adapter)
{
  struct e1000_hw *hw = &adapter->hw;
  uint16_t current_itr;
  uint32_t new_itr = adapter->itr;

  if (hw->mac_type < e1000_82540)
    return;

  /* for non-gigabit speeds, just fix the interrupt rate at 4000 */
  if (adapter->link_speed != SPEED_1000) {
    current_itr = 0;
    new_itr = 4000;
    goto set_itr_now;
  }

  adapter->tx_itr = e1000_update_itr(adapter,
                                     adapter->tx_itr,
                                     adapter->total_tx_packets,
                                     adapter->total_tx_bytes);
  /* conservative mode (itr 3) eliminates the lowest_latency setting */
  if (adapter->itr_setting == 3 && adapter->tx_itr == lowest_latency)
    adapter->tx_itr = low_latency;

  adapter->rx_itr = e1000_update_itr(adapter,
                                     adapter->rx_itr,
                                     adapter->total_rx_packets,
                                     adapter->total_rx_bytes);
  /* conservative mode (itr 3) eliminates the lowest_latency setting */
  if (adapter->itr_setting == 3 && adapter->rx_itr == lowest_latency)
    adapter->rx_itr = low_latency;

  current_itr = MAX(adapter->rx_itr, adapter->tx_itr);

  switch (current_itr) {
    /* counts and packets in update_itr are dependent on these numbers */
    case lowest_latency:
      new_itr = 70000;
      break;
    case low_latency:
      new_itr = 20000; /* aka hwitr = ~200 */
      break;
    case bulk_latency:
      new_itr = 4000;
      break;
    default:
      break;
  }

set_itr_now:
  if (new_itr != adapter->itr) {
    /* this attempts to bias the interrupt rate towards Bulk
     * by adding intermediate steps when interrupt rate is
     * increasing */
    new_itr = new_itr > adapter->itr ?
      MIN(adapter->itr + (new_itr >> 2), new_itr) :
      new_itr;
    adapter->itr = new_itr;
    E1000_WRITE_REG(hw, ITR, 1000000000 / (new_itr * 256));
  }
  return;
}

/**
 * e1000_intr - Interrupt Handler
 * @irq: interrupt number
 * @data: pointer to a network interface device structure
 **/
static bool e1000_intr(void *data)
{
  struct e1000_adapter *adapter = data;
  struct netif *netdev = adapter->netdev;
  struct e1000_hw *hw = &adapter->hw;
  uint32_t rctl, icr = E1000_READ_REG(hw, ICR);
  int i;

  //console_printf("*** IRQ !!!!!!! *** : ICR = %p, STATUS = %p, adapter->itr=%d\n",
  //               icr,E1000_READ_REG(hw, STATUS),
  //               adapter->itr_setting);
  if( !icr ) {
    return false;
  }

  if( icr & (E1000_ICR_RXSEQ | E1000_ICR_LSC) ) {
    hw->get_link_status = 1;
    /* 80003ES2LAN workaround--
     * For packet buffer work-around on link down event;
     * disable receives here in the ISR and
     * reset adapter in watchdog
     */
    // if( netif_carrier_ok(netdev) &&
    if( adapter->hw.mac_type == e1000_80003es2lan ) {
      /* disable receives */
      rctl = E1000_READ_REG(hw, RCTL);
      E1000_WRITE_REG(hw, RCTL, rctl & ~E1000_RCTL_EN);
    }

    /* guard against interrupt when we're going down */
    /* TODO:   */
    //if (!test_bit(__E1000_DOWN, &adapter->flags))
    //  mod_timer(&adapter->watchdog_timer, jiffies + 1);
  }

  /* Writing IMC and IMS is needed for 82547.
   * Due to Hub Link bus being occupied, an interrupt
   * de-assertion message is not able to be sent.
   * When an interrupt assertion message is generated later,
   * two messages are re-ordered and sent out.
   * That causes APIC to think 82547 is in de-assertion
   * state, while 82547 is in assertion state, resulting
   * in dead lock. Writing IMC forces 82547 into
   * de-assertion state.
   */
  if( hw->mac_type == e1000_82547 || hw->mac_type == e1000_82547_rev_2 ) {
    atomic_inc(&adapter->irq_sem);
    E1000_WRITE_REG(hw, IMC, ~0);
  }

  for( i = 0; i < E1000_MAX_INTR; i++ ) {
    if( !adapter->clean_rx(adapter, adapter->rx_ring) &
        !e1000_clean_tx_irq(adapter, adapter->tx_ring) ) {
      break;
    }
  }

  if( adapter->itr_setting & 3 ) {
    e1000_set_itr(adapter);
  }

  if (hw->mac_type == e1000_82547 || hw->mac_type == e1000_82547_rev_2) {
    e1000_irq_enable(adapter);
  }

  //console_printf("        ** Finished processing IRQ.\n");
  return true;
}

/* TODO: allocate RX ring properly. */
static int __e1000_alloc_queues(struct e1000_adapter *adapter)
{
  int i;

  adapter->tx_ring = (struct e1000_tx_ring*)calloc(adapter->num_tx_queues,
                                                   sizeof(struct e1000_tx_ring));
  if( !adapter->tx_ring ) {
    return -ENOMEM;
  }
  memset(adapter->tx_ring,0,sizeof(struct e1000_tx_ring));

  adapter->rx_ring = (struct e1000_rx_ring*)calloc(adapter->num_rx_queues,
                                                   sizeof(struct e1000_rx_ring));
  if( !adapter->rx_ring ) {
    free(adapter->tx_ring);
    return -ENOMEM;
  }
  memset(adapter->rx_ring,0,sizeof(struct e1000_tx_ring));

  /* Initially setup Rx/Tx rings. */
  for(i=0;i<adapter->num_rx_queues;i++) {
    adapter->rx_ring[i].count=E1000_DEFAULT_RXD;
  }

  for(i=0;i<adapter->num_tx_queues;i++) {
    adapter->tx_ring[i].count=E1000_DEFAULT_TXD;
  }

  return 0;
}

static int __e1000_sw_init(struct e1000_adapter *adapter)
{
  struct e1000_hw *hw=&adapter->hw;

  /* pci_read_config_word(adapter,PCI_COMMAND,&hw->pci_cmd_word) */

  /* TODO: [mt] setup adapter->rx_buffer_len properly. */
  adapter->rx_buffer_len=E1000_RXBUFFER_2048; //MAXIMUM_ETHERNET_VLAN_SIZE;
  adapter->rx_ps_bsize0=E1000_RXBUFFER_2048;

  /* TODO: [mt] setup hw->max_frame_size properly. */
  hw->max_frame_size= ETH_DATA_LEN + ENET_HEADER_SIZE + ETHERNET_FCS_SIZE;
  hw->min_frame_size = MINIMUM_ETHERNET_FRAME_SIZE;

  e1000_set_mac_type(&adapter->hw);
  e1000_set_media_type(&adapter->hw);

  console_printf("MAC type: %d, media type: %d, firmware: %d\n",
                 adapter->hw.mac_type,
                 adapter->hw.media_type,
                 adapter->hw.asf_firmware_present);

  hw->wait_autoneg_complete=false;
  hw->tbi_compatibility_en=true;
  hw->adaptive_ifs=true;

  /* Copper options. */
  if( hw->media_type == e1000_media_type_copper ) {
    hw->mdix=AUTO_ALL_MODES;
    hw->disable_polarity_correction=false;
    hw->master_slave=E1000_MASTER_SLAVE;
  }

  adapter->num_tx_queues=1;
  adapter->num_rx_queues=1;

  if( __e1000_alloc_queues(adapter) ) {
    console_printf("[e1000] Failed to allocate Rx/Tx queues !\n");
    return -ENOMEM;
  }

  /* Explicitly disable IRQ since the NIC can be in any state. */
  atomic_set(&adapter->irq_sem, 0);
  e1000_irq_disable(adapter);

  spin_lock_init(&adapter->stats_lock);
  arch_bit_set(&adapter->flags,__E1000_DOWN);

  return 0;
}

static int __e1000_check_phy_reset_block(struct e1000_hw *hw)
{
  uint32_t manc = 0;
  uint32_t fwsm = 0;

  if (hw->mac_type == e1000_ich8lan) {
    fwsm = E1000_READ_REG(hw, FWSM);
    return (fwsm & E1000_FWSM_RSPCIPHY) ? 0 : E1000_BLK_PHY_RESET;
  }

  if (hw->mac_type > e1000_82547_rev_2) {
    manc = E1000_READ_REG(hw, MANC);
    console_printf("==> 2: MANC=0x%X, E1000_MANC_BLK_PHY_RST_ON_IDE=0x%X\n",
                   manc,E1000_MANC_BLK_PHY_RST_ON_IDE);
  }
  return (manc & E1000_MANC_BLK_PHY_RST_ON_IDE) ? E1000_BLK_PHY_RESET : 0;
}

int e1000_read_mac_addr(struct e1000_hw * hw)
{
  uint16_t offset;
  uint16_t eeprom_data, i;

  for (i = 0; i < NODE_ADDRESS_SIZE; i += 2) {
    offset = i >> 1;
    if (e1000_read_eeprom(hw, offset, 1, &eeprom_data) < 0) {
      console_printf("[e1000] EEPROM Read Error\n");
      return -E1000_ERR_EEPROM;
    }
    hw->perm_mac_addr[i] = (uint8_t) (eeprom_data & 0x00FF);
    hw->perm_mac_addr[i+1] = (uint8_t) (eeprom_data >> 8);
  }

  switch (hw->mac_type) {
    default:
        break;
    case e1000_82546:
    case e1000_82546_rev_3:
    case e1000_82571:
    case e1000_80003es2lan:
      if (E1000_READ_REG(hw, STATUS) & E1000_STATUS_FUNC_1) {
        hw->perm_mac_addr[5] ^= 0x01;
      }
      break;
  }

  for (i = 0; i < NODE_ADDRESS_SIZE; i++) {
    hw->mac_addr[i] = hw->perm_mac_addr[i];
  }

  return 0;
}

static uint8_t __e1000_arc_subsystem_valid(struct e1000_hw *hw)
{
  uint32_t fwsm;

  /* On 8257x silicon, registers in the range of 0x8800 - 0x8FFC
   * may not be provided a DMA clock when no manageability features are
   * enabled.  We do not want to perform any reads/writes to these registers
   * if this is the case.  We read FWSM to determine the manageability mode.
   */
  switch (hw->mac_type) {
    case e1000_82571:
    case e1000_82572:
    case e1000_82573:
    case e1000_80003es2lan:
      fwsm = E1000_READ_REG(hw, FWSM);
      if ((fwsm & E1000_FWSM_MODE_MASK) != 0)
        return true;
      break;
    case e1000_ich8lan:
        return true;
    default:
      break;
  }
  return true;
}

static uint32_t __e1000_enable_mng_pass_thru(struct e1000_hw *hw)
{
  uint32_t manc;
  uint32_t fwsm, factps;

  if (hw->asf_firmware_present) {
    manc = E1000_READ_REG(hw, MANC);

    if (!(manc & E1000_MANC_RCV_TCO_EN) ||
        !(manc & E1000_MANC_EN_MAC_ADDR_FILTER))
      return false;
    if (__e1000_arc_subsystem_valid(hw) == true) {
      fwsm = E1000_READ_REG(hw, FWSM);
      factps = E1000_READ_REG(hw, FACTPS);

      if ((((fwsm & E1000_FWSM_MODE_MASK) >> E1000_FWSM_MODE_SHIFT) ==
           e1000_mng_mode_pt) && !(factps & E1000_FACTPS_MNGCG))
        return true;
    } else
      if ((manc & E1000_MANC_SMBUS_EN) && !(manc & E1000_MANC_ASF_EN))
        return true;
  }
  return false;
}

static void __e1000_get_hw_control(struct e1000_adapter *adapter)
{
  uint32_t ctrl_ext;
  uint32_t swsm;

  /* Let firmware know the driver has taken over */
  switch (adapter->hw.mac_type) {
    case e1000_82573:
      swsm = E1000_READ_REG(&adapter->hw, SWSM);
      E1000_WRITE_REG(&adapter->hw, SWSM,
                                swsm | E1000_SWSM_DRV_LOAD);
      break;
    case e1000_82571:
    case e1000_82572:
    case e1000_80003es2lan:
    case e1000_ich8lan:
      ctrl_ext = E1000_READ_REG(&adapter->hw, CTRL_EXT);
      E1000_WRITE_REG(&adapter->hw, CTRL_EXT,
                      ctrl_ext | E1000_CTRL_EXT_DRV_LOAD);
      break;
    default:
      break;
  }
}

static void *__e1000_intr_thread(void *data)
{
  struct e1000_adapter *adapter=data;
  bool done=false;

  console_printf("[e1000] __e1000_intr_thread(): Starting to service IRQs on card N %d.\n",
                 adapter->bd_number);

  /* From here on the code is the same as e1000_up() */
  arch_bit_clear(&adapter->flags,__E1000_DOWN);

  e1000_irq_enable(adapter);

  /* fire a link status change interrupt to start the watchdog */
  E1000_WRITE_REG(&adapter->hw, ICS, E1000_ICS_LSC);

  while( !done ) {
    //console_printf("[%d:%d][e1000] Waiting for interrupts on '%s'\n",
    //               getpid(),gettid(),adapter->netdev->name);
    interrupt_event_wait(adapter->hw.ievent);
  }
out:
  return NULL;
}

static void __e1000_netdev_status_callback(struct netif *nif)
{
  struct e1000_adapter *adapter=nif->state;
  void (*handler) = &e1000_intr;
  int err;

  if (arch_bit_test(&adapter->flags,__E1000_TESTING)) {
    console_printf("[e1000] __e1000_netdev_status_callback(): Interface %s is under testing !\n",
                   adapter->netdev->name);
    return;
  }

  if( !(nif->flags & NETIF_FLAG_UP) ) {
    console_printf("[e1000] __e1000_netdev_status_callback(): Shutting down packet receiving on %s.\n",
                   adapter->netdev->name);
    return;
  }

  /* Code for adapter activation was mostly taken from '__e1000_activate_adapter()' */
  console_printf("[e1000] __e1000_netdev_status_callback(): Starting up packet receiving on %s (%d.\n",
                 adapter->netdev->name,adapter->bd_number);

  /* allocate transmit descriptors */
  err = e1000_setup_all_tx_resources(adapter);
  if( err ) {
    console_printf("[e1000] __e1000_netdev_status_callback(): Failed to allocate Tx resources for %s (%d).\n",
                   adapter->netdev->name,adapter->bd_number);
    goto err_setup_tx;
  }

  /* allocate receive descriptors */
  err = e1000_setup_all_rx_resources(adapter);
  if( err ) {
    console_printf("[e1000] __e1000_netdev_status_callback(): Failed to allocate Rx resources for %s (%d).\n",
                   adapter->netdev->name,adapter->bd_number);
    goto err_setup_rx;
  }

  e1000_power_up_phy(adapter);

  adapter->mng_vlan_id = E1000_MNG_VLAN_NONE;
  if( (adapter->hw.mng_cookie.status & E1000_MNG_DHCP_COOKIE_STATUS_VLAN_SUPPORT) ) {
    e1000_update_mng_vlan(adapter);
  }

  /* If AMT is enabled, let the firmware know that the network
   * interface is now open */
  if( adapter->hw.mac_type == e1000_82573 && e1000_check_mng_mode(&adapter->hw) ) {
    __e1000_get_hw_control(adapter);
  }

  /* before we allocate an interrupt, we must be ready to handle it.
   * Setting DEBUG_SHIRQ in the kernel makes it fire an interrupt
   * as soon as we call pci_request_irq, so we have to setup our
   * clean_rx handler before we do so.  */
  e1000_configure(adapter);

  /* Now its time to set up irq.
     if (adapter->hw.mac_type >= e1000_82571) {
     adapter->have_msi = !pci_enable_msi(adapter->pdev);
     if (adapter->have_msi) {
      handler = &e1000_intr_msi;
      }
      }
      TODO: [mt] MSI support in interrupts disabled.
  */

  console_printf("======> [3]: adding IRQ handler for line %d\n",
                 adapter->irq_line);
  if( interrupt_event_add_handler(adapter->hw.ievent,adapter->irq_line,
                                  e1000_intr,adapter) ) {
    console_printf("[e1000] __e1000_intr_thread(): Can't setup IRQ handler !\n");
    //if( adapter->have_msi ) {
    //  pci_disable_msi(adapter->pdev);
    //}
    /* TODO: [mt] MSI support in interrupts disabled. */
    //goto err_req_irq;
  }

  __e1000_intr_thread(adapter);
  console_printf("Entering eternal loop ...\n");
  for(;;);

  /* finally, launch interrupt thread which will server interrupts. */
  if( !pthread_create(&__intr_threads[adapter->bd_number],NULL,__e1000_intr_thread,adapter) ) {
    console_printf("[e1000] __e1000_netdev_status_callback(): ISR thread created !\n");
    /* OK, device was configured. Our thread will perform the rest of configuraton. */
    return;
  }
  console_printf("[e1000] __e1000_netdev_status_callback(): Can't create ISR thread !\n");
  return;
err_req_irq:
  e1000_release_hw_control(adapter);
  __e1000_power_down_phy(adapter);
  e1000_free_all_rx_resources(adapter);
err_setup_rx:
  e1000_free_all_tx_resources(adapter);
err_setup_tx:
  e1000_reset(adapter);
  return;
}

static err_t __e1000_netdev_init(struct netif *nif)
{
  if( get_network_host()->eth_ops->etherif_init_device(nif) ) {
    console_printf("[e1000] __e1000_netdev_init(): Can't get device name !\n");
    return -1;
  }
  nif->linkoutput=e1000_netdev_linkoutput;
  nif->status_callback=__e1000_netdev_status_callback;
  nif->mtu=ETH_DATA_LEN;

  return ERR_OK;
}

static int e1000_link_test(struct e1000_adapter *adapter, uint64_t *data)
{
  *data = 0;

  if (adapter->hw.media_type == e1000_media_type_internal_serdes) {
    int i = 0;
    adapter->hw.serdes_link_down = true;

    /* On some blade server designs, link establishment
     * could take as long as 2-3 minutes */
    do {
      e1000_check_for_link(&adapter->hw);
      if (adapter->hw.serdes_link_down == false)
        return *data;
      msleep(20);
    } while (i++ < 3750);

    *data = 1;
  } else {
    e1000_check_for_link(&adapter->hw);
    console_printf("* L1: STATUS reg=0x%X, FLAG=0x%X\n",
                   E1000_READ_REG(&adapter->hw, STATUS),
                   E1000_STATUS_LU);
    if (adapter->hw.autoneg)  /* if auto_neg is set wait for it */
      msleep(4000);

    console_printf("* L2: STATUS reg=0x%X, FLAG=0x%X\n",
                   E1000_READ_REG(&adapter->hw, STATUS),
                   E1000_STATUS_LU);
    if (!(E1000_READ_REG(&adapter->hw, STATUS) & E1000_STATUS_LU)) {
      *data = 1;
    }
  }
  return *data;
}

static bool __e1000_setup(pci_dev_t *pdev)
{
  struct e1000_adapter *adapter;
  struct netif *nif;
  int i;
  irq_t irqs[1];
  uint16_t eeprom_data = 0;
  uint16_t eeprom_apme_mask = E1000_EEPROM_APME;
  static int global_quad_port_a = 0; /* global ksp3 port a indication */
  long mmio_start,mmio_len;
  netif_creds_t creds;

  console_printf("[e1000] __e1000_setup(): Setting up an Intel e1000 adapter.\n");
  adapter=(struct e1000_adapter*)malloc(sizeof(*adapter));
  if( !adapter ) {
    console_printf("[e1000] __e1000_setup(): Can't allocate adapter !\n");
    return false;
  }

  memset(adapter,0,sizeof(*adapter));
  adapter->hw.back=adapter;
  adapter->irq_line=pdev->irq_line;

  irqs[0]=pdev->irq_line;
  if( interrupt_event_create(&adapter->hw.ievent,irqs,1) ) {
    console_printf("[e1000] __e1000_setup(): Can't allocate IRQ line: %d\n",
                   irqs[0]);
    goto free_adapter;
  }

  adapter->hw.device_id=pdev->ident.device_id;
  console_printf(">>>>>>>>> DEV ID: 0x%X\n",pdev->ident.device_id);

  mmio_start=pdev->resources[0].start;
  mmio_len=pdev->resources[0].end-pdev->resources[0].start+1;

  console_printf("[e1000] __e1000_setup(): found MMIO region at %p of %d bytes (E=%d)\n",
                 mmio_start,
                 mmio_len,
                 128*1024);
  adapter->hw.hw_addr=mmap(NULL,mmio_len,
                           PROT_READ|PROT_WRITE,MAP_PHYS|MAP_PRIVATE,NOFD,
                           mmio_start);
  if( adapter->hw.hw_addr == MAP_FAILED ) {
    console_printf("[e1000] __e1000_setup(): Failed to map device memory region ! (%d)\n",
                   errno);
    goto free_pci_resources;
  }

  console_printf("[e1000] __e1000_setup(): BAR0 mapped at %p\n",adapter->hw.hw_addr);

  if( __e1000_sw_init(adapter) ) {
    console_printf("[e1000]: __e1000_setup(): Softinit failed !\n");
    goto free_pci_resources;
  }

  if( __e1000_check_phy_reset_block(&adapter->hw) ) {
    console_printf("[e1000]: __e1000_setup(): PHY reset is blocked due to SOL/IDER session.\n");
    goto free_pci_resources;
  }

  adapter->en_mng_pt = __e1000_enable_mng_pass_thru(&adapter->hw);
  if( e1000_init_eeprom_params(&adapter->hw) ) {
    console_printf("[e1000] __e1000_setup(): failed to initialize EEPROM parameters !\n");
    goto free_pci_resources;
  }

  e1000_reset_hw(&adapter->hw);

  /* make sure the EEPROM is good */
  if( e1000_validate_eeprom_checksum(&adapter->hw) < 0 ) {
    console_printf("[e1000] __e1000_setup(): The EEPROM Checksum Is Not Valid.\n");
    goto err_eeprom;
  }

  if( e1000_read_mac_addr(&adapter->hw) ) {
    console_printf("[e1000] __e1000_setup(): Failed to read MAC address from EEPROM.\n");
  }

  console_printf("[e1000] MAC address: %x:%x:%x:%x:%x:%x\n",
                 adapter->hw.mac_addr[0],adapter->hw.mac_addr[1],
                 adapter->hw.mac_addr[2],adapter->hw.mac_addr[3],
                 adapter->hw.mac_addr[4],adapter->hw.mac_addr[5]);

  e1000_get_bus_info(&adapter->hw);

  switch (adapter->hw.mac_type) {
    case e1000_82542_rev2_0:
    case e1000_82542_rev2_1:
    case e1000_82543:
      break;
    case e1000_82544:
      e1000_read_eeprom(&adapter->hw,
                        EEPROM_INIT_CONTROL2_REG, 1, &eeprom_data);
      eeprom_apme_mask = E1000_EEPROM_82544_APM;
      break;
    case e1000_ich8lan:
      e1000_read_eeprom(&adapter->hw,
                        EEPROM_INIT_CONTROL1_REG, 1, &eeprom_data);
      eeprom_apme_mask = E1000_EEPROM_ICH8_APME;
      break;
    case e1000_82546:
    case e1000_82546_rev_3:
    case e1000_82571:
    case e1000_80003es2lan:
      if (E1000_READ_REG(&adapter->hw, STATUS) & E1000_STATUS_FUNC_1){
        e1000_read_eeprom(&adapter->hw,
                          EEPROM_INIT_CONTROL3_PORT_B, 1, &eeprom_data);
        break;
      }
      /* Fall Through */
    default:
      e1000_read_eeprom(&adapter->hw,
                        EEPROM_INIT_CONTROL3_PORT_A, 1, &eeprom_data);
      break;
  }

  if (eeprom_data & eeprom_apme_mask)
    adapter->eeprom_wol |= E1000_WUFC_MAG;

  /* now that we have the eeprom settings, apply the special cases
   * where the eeprom may be wrong or the board simply won't support
   * wake on lan on a particular port */
  switch (pdev->ident.device_id) {
    case E1000_DEV_ID_82546GB_PCIE:
      adapter->eeprom_wol = 0;
      break;
    case E1000_DEV_ID_82546EB_FIBER:
    case E1000_DEV_ID_82546GB_FIBER:
    case E1000_DEV_ID_82571EB_FIBER:
      /* Wake events only supported on port A for dual fiber
       * regardless of eeprom setting */
      if (E1000_READ_REG(&adapter->hw, STATUS) & E1000_STATUS_FUNC_1)
        adapter->eeprom_wol = 0;
      break;
    case E1000_DEV_ID_82546GB_QUAD_COPPER_KSP3:
    case E1000_DEV_ID_82571EB_QUAD_COPPER:
    case E1000_DEV_ID_82571EB_QUAD_FIBER:
    case E1000_DEV_ID_82571EB_QUAD_COPPER_LOWPROFILE:
    case E1000_DEV_ID_82571PT_QUAD_COPPER:
      /* if quad port adapter, disable WoL on all but port A */
      if (global_quad_port_a != 0)
        adapter->eeprom_wol = 0;
      else
        adapter->quad_port_a = 1;
      /* Reset for multiple quad port adapters */
      if (++global_quad_port_a == 4)
        global_quad_port_a = 0;
      break;
  }

  /* initialize the wol settings based on the eeprom settings */
  adapter->wol = adapter->eeprom_wol;
  adapter->bd_number = __cards_found;

  /* print bus type/speed/width info */
  {
    struct e1000_hw *hw = &adapter->hw;
    console_printf("(PCI%s:%s:%s) ",
                   ((hw->bus_type == e1000_bus_type_pcix) ? "-X" :
                    (hw->bus_type == e1000_bus_type_pci_express ? " Express":"")),
                   ((hw->bus_speed == e1000_bus_speed_2500) ? "2.5Gb/s" :
                    (hw->bus_speed == e1000_bus_speed_133) ? "133MHz" :
                    (hw->bus_speed == e1000_bus_speed_120) ? "120MHz" :
                    (hw->bus_speed == e1000_bus_speed_100) ? "100MHz" :
                    (hw->bus_speed == e1000_bus_speed_66) ? "66MHz" : "33MHz"),
                   ((hw->bus_width == e1000_bus_width_64) ? "64-bit" :
                    (hw->bus_width == e1000_bus_width_pciex_4) ? "Width x4" :
                    (hw->bus_width == e1000_bus_width_pciex_1) ? "Width x1" :
                    "32-bit"));
  }

  /* reset the hardware with the new settings */
  e1000_reset(adapter);

  /* If the controller is 82573 and f/w is AMT, do not set
   * DRV_LOAD until the interface is up.  For all other cases,
   * let the f/w know that the h/w is now under the control
   * of the driver. */
  if (adapter->hw.mac_type != e1000_82573 ||
      !e1000_check_mng_mode(&adapter->hw))
    __e1000_get_hw_control(adapter);

  /* Now it is time to register network interface itself. */
  nif=(struct netif *)malloc(sizeof(*nif));
  if( !nif ) {
    console_printf("[e1000] __e1000_setup(): Can't allocate network device !\n");    
    goto free_ievent;
  }

  creds.init = __e1000_netdev_init;
  if( get_network_host()->register_interface(nif,&creds) ) {
    console_printf("[e1000] __e1000_setup(): Can't register network device !\n");    
    goto free_netdev;
  }

  console_printf("[e1000] __e1000_setup(): Registered network interface '%s'\n",
                 nif->name);
  adapter->netdev=nif;
  nif->state=adapter;

  for (i = 0; i < NODE_ADDRESS_SIZE; i++) {
    nif->hwaddr[i]=adapter->hw.mac_addr[i];
  }

  console_printf(">>>>>>>>>>>>> Interface registered !\n");

  console_printf("################################ device initialized !\n");
  {
    uint64_t ld;
    console_printf(" LINK TEST: %d\n",e1000_link_test(adapter,&ld));
    console_printf(" ISR: 0x%X\n",E1000_READ_REG(&adapter->hw, ICR));
  }

  __cards_found++;
  __adapter=adapter;
  return true;
free_netdev:
  free(nif);
free_ievent:
  interrupt_event_destroy(adapter->hw.ievent);
free_pci_resources:
err_eeprom:
free_adapter:
  free(adapter);
  return false;
}

static int32_t __e1000_validate_mdi_setting(struct e1000_hw *hw)
{
  if (!hw->autoneg && (hw->mdix == 0 || hw->mdix == 3)) {
    console_printf("[e1000] Invalid MDI setting detected\n");
    hw->mdix = 1;
    return -E1000_ERR_CONFIG;
  }
  return E1000_SUCCESS;
}


static void __e1000_check_copper_options(struct e1000_adapter *adapter)
{
  adapter->hw.autoneg = adapter->fc_autoneg = 1;
  adapter->hw.autoneg_advertised = ADVERTISE_100_HALF | ADVERTISE_100_FULL;
  adapter->hw.forced_speed_duplex = e1000_100_half;

  /* Speed, AutoNeg and MDI/MDI-X must all play nice */
  __e1000_validate_mdi_setting(&(adapter->hw));
}

static void __e1000_check_options(struct e1000_adapter *adapter)
{
  switch (adapter->hw.media_type) {
    case e1000_media_type_copper:
      __e1000_check_copper_options(adapter);
      break;
    default:
      console_printf("[e1000] __e1000_check_option(): only copper media is supported !\n");
      break;
  }
}

int e1000i_probe(void)
{
  int i,r,nd;
  pci_addr_t dev[MAXCARDS];

  if( pci_init() ) {
    console_printf("[e1000] e1000i_probe(): Can't access PCI subsystem !\n");
    return -1;
  }

  nd=pci_lookup_prod_vend(VENDOR_ID_INTEL,E1000_DEV_ID_82571EB_COPPER,
                      dev,MAXCARDS);
  if( nd <= 0 ) {
    console_printf("[e1000] e1000i_probe(): No devices found.\n");
    return -1;
  }

  console_printf("[e1000] e1000i_probe(): %d adapters found.\n",nd);
  for(i=0;i<1;i++) {
    pci_dev_t pd;

    console_printf("[e1000] e1000i_probe():setting up adapter N %d\n",i);
    if( pci_get_dev(&dev[i],&pd) ) {
      console_printf("[e1000] e1000i_probe(): failed to get PCI device for adapter N %d\n",i);
      goto release_devices;
    }

    if( pd.num_resources != 3 ) {
      console_printf("[e1000] e1000i_probe(): Insufficient number of PCI resources for adapter N %d : %d\n",
                     i,pd.num_resources);
      goto release_devices;
    }

    if( !__e1000_setup(&pd) ) {
      goto release_devices;
    }
  }

  return 0;
release_devices:
  return -1;
}
