/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * Copyright (C) 2006,2007,2008,2009 Jari OS Core Team <http://jarios.org>
 * Copyright (C) 2009 Paul Graphov <graphov@jarios.org>
 *
 * /dev/null and /dev/zero devices
 *
 */

#include <sys/types.h>
#include <unistd.h>
#include <sys/stat.h>
#include <fcntl.h>

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>

#include <chardev.h>
#include <dircons.h>

#define DRVNAME "zero.d"
#define ZERONAME "/dev/zero"
#define NULLNAME "/dev/null"

#define ZERONUM 0
#define NULLNUM 1


static status_t zero_read(int devno, void *buf, size_t size, size_t offset, void **interim)
{
  if(devno == ZERONUM) {
    memset(buf, 0, size);
    return size;
  } else {
    return 0;
  }
}

static status_t zero_write(int devno, void *buf, size_t size, size_t offset, void **interim)
{
  return size;
}

static struct device_ops zero_ops = {
  .read  = zero_read,
  .write = zero_write,
  .ioctl = NULL,
  .log = console_output,
};

device_server_t server;

int main(int argc, char **argv)
{
  char *mesg;
  int r;

#ifndef CONFIG_ZERO_GRUB
  start_service();
#endif

  if((mesg=malloc(1024))==NULL) return -ENOMEM;

  open_console();

  r = init_multi_device_server(&server, 512, 512, DRVNAME, &zero_ops, 1);
  if (r != 0) {
    console_printf(mesg, "rtc.d: failed to initialize server (%d)\n", r);
    abort();
  }

  if(add_device(0, "/dev/zero") != 0 || add_device(1, "/dev/null")) {
    console_printf(mesg, "rtc.d: failed to register device.\n");
    abort();
  }

  console_output("[zero.d]\n");
  device_server_start();

  return 0;
}
