/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 * 
 * (c) Copyright 2006,2007-2009 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2005,2008,2009 MadTirra <madtirra@jarios.org>
 *
 * Jari OS dev i/o file system,
 * internals implementation
 */

#ifndef __DEVIOFS_DEVIOFS_H__
#define __DEVIOFS_DEVIOFS_H__

#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include <datalib/usrtc.h>

#include <vfs/session.h>

#ifdef CONFIG_DEVIOFS_DEBUG
#define DEVIOFS_DEBUG  1
#endif

#define BLOCK_SIZE  512

#define NAME_SIZE    100
#define UNAME_SIZE   32
#define PREFIX_SIZE  155

#define DEV_ASSIGN  0x1f

#define TFS_UREAD   (1 << 0)
#define TFS_UWRITE  (1 << 1) 
#define TFS_UEXEC   (1 << 2)
#define TFS_GREAD   (1 << 3)
#define TFS_GWRITE  (1 << 4)
#define TFS_GEXEC   (1 << 5)
#define TFS_OREAD   (1 << 6)
#define TFS_OWRITE  (1 << 7)
#define TFS_OEXEC   (1 << 8)

typedef struct __deviofs_node_t {
  int type;      /* device/file type */
  char *name;    /* file name */
  uint64_t hash; /* name hash */
  time_t ctime;  /* created time */
  time_t mtime;  /* modified time */
  time_t atime;  /* access time */
  /* device rel */
  pid_t drv_pid; /* pid of device driver */
  int drv_port;  /* driver port */
  int dev_id;    /* driver id number */
  chid_t dev_ch; /* channel to the device */
  /* access structure */
  uid_t uid; /* owner */
  gid_t gid;
  int mask;
  /* namespace structure */
  struct __deviofs_node_t *parent;
  usrtc_node_t *node;
  /* link rel */
  struct __deviofs_node_t *hard_link; /* real */
  int num_links;
  char *symlink_path; /* path to the real file */
  /* directory related */
  usrtc_t childs;
  ulong_t num_childs;
  /* cached dir entries */
  struct dirent **entries;
} deviofs_node_t;

typedef struct __deviofs_super_t {
  int state;
  deviofs_node_t *root;
} deviofs_super_t;

/* file system specific functions */
int deviofs_ioctl(vnode_t *node,uni_rpc_t *rpc);
int deviofs_mount(vnode_t *node,char *path,int *pps,int pp,int flags);
int deviofs_umount(vnode_t *node);
int deviofs_lookup(vnode_t *node,char *path,int *pps,int pp);
int deviofs_create(vnode_t *node,char *path,int *pps,int pp,int mode,uid_t uid,gid_t gid);
int deviofs_opendir(vnode_t *node,char *path,int *pps,int pp);
int deviofs_close(vnode_t *node);
int deviofs_readdir(vnode_t *node,size_t size,off_t off,struct dirent *dentry);
int deviofs_createdir(vnode_t *node,char *path,int *pps,int pp,int mode,uid_t uid,gid_t gid);
int deviofs_unlink(vnode_t *node,uid_t uid,gid_t gid);
int deviofs_unlinkdir(vnode_t *node,uid_t uid,gid_t gid);

/* hooks */
int deviofs_addhook(pidsession_t *sess,fessid_t *nid);
int deviofs_closehook(pidsession_t *sess,fessid_t *nid);
int deviofs_killhook(pidsession_t *sess);
int deviofs_clonehook(pidsession_t *sess,pid_t pid);

#endif /* __DEVIOFS_DEVIOFS_H__ */
