/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 * 
 * (c) Copyright 2006,2007-2009 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2005,2008,2009 MadTirra <madtirra@jarios.org>
 *
 * Jari OS devices i/o file system
 * main entry
 */

#include <sys/types.h>
#include <stdint.h>
#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <fcntl.h>
#include <pthread.h>
#include <unistd.h>
#include <ipc/channel.h>
#include <ipc/rpc.h>
#include <ipc/iov.h>
#include <sys/v2.h>
#include <sys/kcontrol.h>
#include <sys/mman.h>
#include <ns/ns.h>
#include <errno.h>
#include <datalib/usrtc.h>

#include <vfs/vfsdef.h>
#include <vfs/super.h>
#include <vfs/vnode.h>

#include "deviofs.h"

#ifdef DEVIOFS_DEBUG
#include <dircons.h>
#endif

static vfs_info_t devio_fs = { 
  "deviofs",
  "Alfeiks Kaanoken",
  "Devices i/o file system",
};

static int devio_fs_specflags = V2FS_SYS;

static prtid_t fs_port;

/* static initialization */

static struct fs_funct deviofs_tbl = {
  deviofs_mount,
  deviofs_umount,
  deviofs_lookup,
  NULL,
  deviofs_ioctl, /* ioctl */
  NULL,
  deviofs_opendir,
  deviofs_close,
  deviofs_create,
  NULL,
  NULL,
  deviofs_createdir,
  deviofs_unlink,
  deviofs_unlinkdir,
  NULL,
  NULL,
  deviofs_readdir,
  NULL,
  NULL,
  NULL,
  NULL, /* readlink */
};

/* special hooks on resource management */
static struct fss_hooks devio_hooks = {
  deviofs_addhook,
  deviofs_closehook,
  deviofs_killhook,
  deviofs_clonehook,
};

static super_block_t deviofs_super;

#define IFS_MAX_LISTENERS  8

static filesystem_proc_t deviofs_proc;

int main(int argc,char **argv)
{
  int r;

#ifndef CONFIG_DEVIOFS_GRUB
  start_service();
#endif

  fs_port=ipc_port_open(IPC_BLOCKED_ACCESS,512);

  deviofs_super.info=&devio_fs;
  deviofs_super.block_size=0;
  deviofs_super.slice_size=0;
  deviofs_super.flags=devio_fs_specflags;
  deviofs_super.fs_tbl=&deviofs_tbl;
  deviofs_super.path=NULL;
  deviofs_super.pps=NULL;
  deviofs_super.pp=0;
  deviofs_super.backend=NULL;
  deviofs_super.pagecache=NULL;

  deviofs_proc.super=&deviofs_super;
  deviofs_proc.listen_port=fs_port;
  deviofs_proc.listeners=IFS_MAX_LISTENERS;

#ifdef DEVIOFS_DEBUG
  open_console();
#endif

#ifdef DEVIOFS_DEBUG
  console_printf("[deviofs] opened port %d.\n",fs_port);
#endif

  r=vfs_register(fs_port,devio_fs_specflags,&devio_fs);

#ifdef DEVIOFS_DEBUG
  console_printf("[deviofs] registered (%d).\n",r);
  console_printf("[deviofs] Waiting for mount event.\n");
#endif

  r=fsproc_init(&deviofs_proc);
  /* set hooks */
  fs_session_sethooks(&devio_hooks);

#ifdef TMPFS_DEBUG
  console_printf("[deviofs] mounted (%d).\n",r);
#endif

  v2fs_thread(&deviofs_proc);

  return 0;
}
