/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * (c) Copyright 2006,2007-2009 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2008 Dmitriy Balakshin <balakshin@jarios.org> (basic definions)
 * (c) Copyright 2005,2008,2009 MadTirra <madtirra@jarios.org>
 *
 * Jari OS init file system,
 * internals implementation
 */

#ifndef __INITFS_INITFS_H__
#define __INITFS_INITFS_H__

#include <sys/types.h>
#include <dirent.h>

#define INITFS_DEBUG  1

#define BLOCK_SIZE  512

#define NAME_SIZE    100
#define UNAME_SIZE   32
#define PREFIX_SIZE  155

enum tar_mode {
  TSUID   = 04000,
  TSGID   = 02000,
  TSVTX   = 01000,
  TUREAD  = 00400,
  TUWRITE = 00200,
  TUEXEC  = 00100,
  TGREAD  = 00040,
  TGWRITE = 00020,
  TGEXEC  = 00010,
  TOREAD  = 00004,
  TOWRITE = 00002,
  TOEXEC  = 00001
};

enum tar_link_type {
  REGTYPE  = '0',     /* Regular file (preferred code) */
  AREGTYPE = '\0',    /* Regular file (alternate code) */
  LNKTYPE  = '1',     /* Hard link */
  SYMTYPE  = '2',     /* Symbolic link (hard if not supported) */
  CHRTYPE  = '3',     /* Character special */
  BLKTYPE  = '4',     /* Block special */
  DIRTYPE  = '5',     /* Directory */
  FIFOTYPE = '6',     /* Named pipe */
  CONTTYPE = '7'      /* Contiguous file */
  /* regular file if not supported */
};

struct tar_header {
  char name[NAME_SIZE];
  char mode[8];
  char uid[8];
  char gid[8];
  char size[12];
  char mtime[12];
  char chksum[8];
  char typeflag;
  char linkname[NAME_SIZE];
  char magic[6];
  char version[2];
  char uname[UNAME_SIZE];
  char gname[UNAME_SIZE];
  char devmajor[8];
  char devminor[8];
  char prefix[PREFIX_SIZE];
} __attribute__ ((packed));

struct initfs_shead_t { /* initfs super head */
  void *start;
  size_t size;
};

struct initfs_node_t {
  int dir_entries;
  struct tar_header head;
  struct dirent **dentries; /* on directory is not null */
  void *data; /* on file or link */
};

int initfs_mount(vnode_t *node,char *path,int *pps,int pp,int flags);
int initfs_umount(vnode_t *node);
int initfs_lookup(vnode_t *node,char *path,int *pps,int pp);
void *initfs_read(vnode_t *node,void *buf,size_t *size,off_t offset);
int initfs_readlink(vnode_t *node,char *linkname,size_t *lnksize);
int initfs_opendir(vnode_t *node,char *path,int *pps,int pp);
int initfs_close(vnode_t *node);
int initfs_readdir(vnode_t *node,size_t size,off_t off,struct dirent *dentry);

#endif /* __INITFS_INITFS_H__ */
