/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 * 
 * (c) Copyright 2006,2007-2009 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2005,2008,2009 MadTirra <madtirra@jarios.org>
 *
 * Jari OS init file system,
 * main entry
 */

#include <sys/types.h>
#include <stdint.h>
#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <fcntl.h>
#include <pthread.h>
#include <unistd.h>
#include <ipc/channel.h>
#include <ipc/rpc.h>
#include <ipc/iov.h>
#include <sys/v2.h>
#include <sys/kcontrol.h>
#include <sys/mman.h>
#include <ns/ns.h>
#include <errno.h>
#include <datalib/usrtc.h>

#include <vfs/vfsdef.h>
#include <vfs/super.h>
#include <vfs/vnode.h>

#include "initfs.h"

#ifdef INITFS_DEBUG
#include <dircons.h>
#endif

static vfs_info_t init_fs = { 
  "initfs",
  "Alfeiks Kaanoken",
  "File system used for second bootstrapping",
};

static int init_fs_specflags = V2FS_SYS | V2FS_RO;

static prtid_t fs_port;

/* static initialization */

static struct fs_funct initfs_tbl = {
  initfs_mount,
  initfs_umount,
  initfs_lookup,
  initfs_read,
  NULL,
  NULL,
  initfs_opendir,
  initfs_close,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  initfs_readdir,
  NULL,
  NULL,
  NULL,
  initfs_readlink,
};

static super_block_t initfs_super;

#define IFS_MAX_LISTENERS  8

static filesystem_proc_t initfs_proc;

int main(int argc,char **argv)
{
  int r;

  fs_port=ipc_port_open(IPC_BLOCKED_ACCESS,512);

  initfs_super.info=&init_fs;
  initfs_super.block_size=0;
  initfs_super.slice_size=0;
  initfs_super.flags=init_fs_specflags;
  initfs_super.fs_tbl=&initfs_tbl;
  initfs_super.path=NULL;
  initfs_super.pps=NULL;
  initfs_super.pp=0;
  initfs_super.backend=NULL;
  initfs_super.pagecache=NULL;

  initfs_proc.super=&initfs_super;
  initfs_proc.listen_port=fs_port;
  initfs_proc.listeners=IFS_MAX_LISTENERS;

#ifdef INITFS_DEBUG
  open_console();
#endif

#ifdef INITFS_DEBUG
  console_printf("[initfs] opened port %d.\n",fs_port);
#endif

  r=vfs_register(fs_port,init_fs_specflags,&init_fs);

#ifdef INITFS_DEBUG
  console_printf("[initfs] registered (%d).\n",r);
  console_printf("[initfs] Waiting for mount event.\n");
#endif

  r=fsproc_init(&initfs_proc);

  /* start generic mapper */
  r=fsproc_set_generic_mmap(&initfs_proc);
  if(r!=0) abort();

#ifdef INITFS_DEBUG
  console_printf("[initfs] mounted (%d).\n",r);
#endif

  v2fs_thread(&initfs_proc);

  return 0;
}

