#include <sys/types.h>

#include <stdint.h>
#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <unistd.h>

#include <string.h>

#include <fcntl.h>

#include <pthread.h>

#include <ipc/channel.h>
#include <ipc/rpc.h>
#include <ipc/iov.h>

#include <sys/v2.h>
#include <sys/kcontrol.h>
#include <sys/mman.h>

#include <ns/ns.h>

#include <vfs/vfsdef.h>
#include <vfs/super.h>
#include <vfs/vnode.h>

#include <errno.h>

#include "debugfs.h"

#define IFS_MAX_LISTENERS  8

static vfs_info_t debugfs_info = {
  "debugfs",
  "Stanislav M. Ivankin",
  "Debugfs for Jari OS",
};

static int debugfs_specflags = V2FS_RW | V2FS_RO;
static prtid_t fs_port;

static struct fs_funct debugfs_tbl = {
  NULL, /* mount     */
  NULL, /* unmount   */
  NULL, /* loopup    */
  NULL, /* read      */
  NULL, /* ioctl     */
  NULL, /* write     */
  NULL, /* opendir   */
  NULL, /* close     */
  NULL, /* create    */
  NULL, /* setattr   */
  NULL, /* getattr   */
  NULL, /* createdir */
  NULL, /* unlink    */
  NULL, /* unlinkdir */
  NULL, /* link      */
  NULL, /* symlink   */
  NULL, /* readdir   */
};

static super_block_t debugfs_super;
static filesystem_proc_t debugfs_proc;

int main(int argc, char *argv[])
{
  int ret;

  open_console_d();
  console_dprintf("Initializing...");

  fs_port=ipc_port_open(IPC_BLOCKED_ACCESS,512);
  console_dprintf("Port %d opened.", fs_port);

  debugfs_super.info       = &debugfs_info;
  debugfs_super.block_size = 0;
  debugfs_super.slice_size = 0;
  debugfs_super.flags      = debugfs_specflags;
  debugfs_super.fs_tbl     = &debugfs_tbl;
  debugfs_super.path       = NULL;
  debugfs_super.pps        = NULL;
  debugfs_super.pp         = 0;
  debugfs_super.backend    = NULL;
  debugfs_super.pagecache  = NULL;

  debugfs_proc.super       = &debugfs_super;
  debugfs_proc.listen_port = fs_port;
  debugfs_proc.listeners   = IFS_MAX_LISTENERS;

  ret = vfs_register(fs_port, debugfs_specflags, &debugfs_info);
  console_dprintf("Registered on VFS (%d).", ret);

  ret = fsproc_init(&debugfs_proc);

  while(1) {
    sleep(20);
  }

  return 0;
}
