/*
 * bitops.c --- Bitmap frobbing code.  See bitops.h for the inlined
 * 	routines.
 *
 * Copyright (C) 1993, 1994, 1995, 1996 Theodore Ts'o.
 *
 * %Begin-Header%
 * This file may be redistributed under the terms of the GNU Public
 * License.
 * %End-Header%
 */

#include <sys/types.h>
#include <stdio.h>

#include "ext2_fs.h"
#include "ext2fs.h"

/*
 * For the benefit of those who are trying to port Linux to another
 * architecture, here are some C-language equivalents.  You should
 * recode these in the native assmebly language, if at all possible.
 *
 * C language equivalents written by Theodore Ts'o, 9/26/92.
 * Modified by Pete A. Zaitcev 7/14/95 to be portable to big endian
 * systems, as well as non-32 bit systems.
 */

int ext2fs_set_bit(unsigned int nr,void * addr)
{
  int mask, retval;
  unsigned char	*ADDR = (unsigned char *) addr;

  ADDR += nr >> 3;
  mask = 1 << (nr & 0x07);
  retval = mask & *ADDR;
  *ADDR |= mask;

  return retval;
}

int ext2fs_clear_bit(unsigned int nr, void * addr)
{
  int mask, retval;
  unsigned char	*ADDR = (unsigned char *) addr;

  ADDR += nr >> 3;
  mask = 1 << (nr & 0x07);
  retval = mask & *ADDR;
  *ADDR &= ~mask;

  return retval;
}

int ext2fs_test_bit(unsigned int nr, const void * addr)
{
  int mask;
  const unsigned char *ADDR = (const unsigned char *) addr;

  ADDR += nr >> 3;
  mask = 1 << (nr & 0x07);

  return (mask & *ADDR);
}

inline unsigned int ext2fs_power_of(int val)
{
  int s, i;

  if (val < 0)
    val = NEGATE(val);
  i = 0, s = val;
  do {
    s >>= 1; i ++;
  } while (s);

  return i - 1;
}

inline void ext2fs_pprint_bitmap(const char * msg, const void *bitmap,
				 unsigned int bs, unsigned int offset,
				 unsigned range)
{
  unsigned char *bm;

  if (offset + range > bs)
    return;
  if (!range) range = bs;

  bs = bs - offset;
  bm = (unsigned char *)bitmap + bs - 1;

  do {
    int i = 8;
    do {
      klog("%d", TEST_BIT(*bm, -- i) ? 1 : 0);
    } while (i);

    klog(" | ");

    bm = bm - 1;
    range --;
  } while (range != 0);

  klog("\n");
}
