/*
 * bitops.h --- Bitmap frobbing code.  The byte swapping routines are
 * 	also included here.
 *
 * Copyright (C) 1993, 1994, 1995, 1996 Theodore Ts'o.
 *
 * %Begin-Header%
 * This file may be redistributed under the terms of the GNU Public
 * License.
 * %End-Header%
 *
 * i386 bitops operations taken from <asm/bitops.h>, Copyright 1992,
 * Linus Torvalds.
 */


/* This macroses and functions are only for debugging */
#define TEST_BIT(b,n) ((b) & (1 << (n)))
#define IS_POWER2(b)  ((b) & (-(b)) == (b))
#define NEGATE(b)     ~(b) + 1

inline unsigned int ext2fs_power_of(int val);
//inline void ext2fs_pprint_bitmap(const char * msg, const void *bitmap,
				 //				 unsigned int bs);
inline void ext2fs_pprint_bitmap(const char * msg, const void *bitmap,
				 unsigned int bs, unsigned int offset,
				 unsigned range);

/* ---- */

extern int ext2fs_set_bit(unsigned int nr,void * addr);
extern int ext2fs_clear_bit(unsigned int nr, void * addr);
extern int ext2fs_test_bit(unsigned int nr, const void * addr);

static inline __u16 ext2fs_swab16(__u16 val)
{
  return (val >> 8) | (val << 8);
}

static inline __u32 ext2fs_swab32(__u32 val)
{
  return ((val>>24) | ((val>>8)&0xFF00) | ((val<<8)&0xFF0000) | (val<<24));
}

static inline __u64 ext2fs_swab64(__u64 val)
{
  return (ext2fs_swab32(val >> 32) |
	  (((__u64)ext2fs_swab32(val & 0xFFFFFFFFUL)) << 32));
}

#ifdef WORDS_BIGENDIAN
#define ext2fs_cpu_to_le64(x) ext2fs_swab64((x))
#define ext2fs_le64_to_cpu(x) ext2fs_swab64((x))
#define ext2fs_cpu_to_le32(x) ext2fs_swab32((x))
#define ext2fs_le32_to_cpu(x) ext2fs_swab32((x))
#define ext2fs_cpu_to_le16(x) ext2fs_swab16((x))
#define ext2fs_le16_to_cpu(x) ext2fs_swab16((x))
#define ext2fs_cpu_to_be32(x) ((__u32)(x))
#define ext2fs_be32_to_cpu(x) ((__u32)(x))
#define ext2fs_cpu_to_be16(x) ((__u16)(x))
#define ext2fs_be16_to_cpu(x) ((__u16)(x))
#else
#define ext2fs_cpu_to_le64(x) ((__u64)(x))
#define ext2fs_le64_to_cpu(x) ((__u64)(x))
#define ext2fs_cpu_to_le32(x) ((__u32)(x))
#define ext2fs_le32_to_cpu(x) ((__u32)(x))
#define ext2fs_cpu_to_le16(x) ((__u16)(x))
#define ext2fs_le16_to_cpu(x) ((__u16)(x))
#define ext2fs_cpu_to_be32(x) ext2fs_swab32((x))
#define ext2fs_be32_to_cpu(x) ext2fs_swab32((x))
#define ext2fs_cpu_to_be16(x) ext2fs_swab16((x))
#define ext2fs_be16_to_cpu(x) ext2fs_swab16((x))
#endif

/*
 * EXT2FS bitmap manipulation routines.
 */

/* Support for sending warning messages from the inline subroutines */
extern const char *ext2fs_block_string;
extern const char *ext2fs_inode_string;
extern const char *ext2fs_mark_string;
extern const char *ext2fs_unmark_string;
extern const char *ext2fs_test_string;

extern void ext2fs_mark_block_bitmap_range(ext2fs_block_bitmap bitmap,
					   blk_t block, int num);
extern void ext2fs_unmark_block_bitmap_range(ext2fs_block_bitmap bitmap,
					     blk_t block, int num);
extern int ext2fs_test_block_bitmap_range(ext2fs_block_bitmap bitmap,
					  blk_t block, int num);
extern void ext2fs_set_bitmap_padding(ext2fs_generic_bitmap map);

/* These routines moved to gen_bitmap.c */
extern int ext2fs_mark_generic_bitmap(ext2fs_generic_bitmap bitmap,
					 __u32 bitno);
extern int ext2fs_unmark_generic_bitmap(ext2fs_generic_bitmap bitmap,
					   blk_t bitno);
extern int ext2fs_test_generic_bitmap(ext2fs_generic_bitmap bitmap,
				      blk_t bitno);
extern int ext2fs_test_block_bitmap_range(ext2fs_block_bitmap bitmap,
					  blk_t block, int num);
extern __u32 ext2fs_get_generic_bitmap_start(ext2fs_generic_bitmap bitmap);
extern __u32 ext2fs_get_generic_bitmap_end(ext2fs_generic_bitmap bitmap);

static inline void ext2fs_fast_set_bit(unsigned int nr,void * addr)
{
  unsigned char	*ADDR = (unsigned char *) addr;

  ADDR += nr >> 3;
  *ADDR |= (1 << (nr & 0x07));
}

static inline void ext2fs_fast_clear_bit(unsigned int nr, void * addr)
{
  unsigned char	*ADDR = (unsigned char *) addr;

  ADDR += nr >> 3;
  *ADDR &= ~(1 << (nr & 0x07));
}

static inline int ext2fs_mark_block_bitmap(ext2fs_block_bitmap bitmap,
				       blk_t block)
{
  return ext2fs_mark_generic_bitmap((ext2fs_generic_bitmap) bitmap, block);
}

static inline int ext2fs_unmark_block_bitmap(ext2fs_block_bitmap bitmap,
					 blk_t block)
{
  return ext2fs_unmark_generic_bitmap((ext2fs_generic_bitmap) bitmap, block);
}

static inline int ext2fs_test_block_bitmap(ext2fs_block_bitmap bitmap,
				       blk_t block)
{
  return ext2fs_test_generic_bitmap((ext2fs_generic_bitmap) bitmap, block);
}

static inline int ext2fs_mark_inode_bitmap(ext2fs_inode_bitmap bitmap,
				       ext2_ino_t inode)
{
  return ext2fs_mark_generic_bitmap((ext2fs_generic_bitmap) bitmap, inode);
}

static inline int ext2fs_unmark_inode_bitmap(ext2fs_inode_bitmap bitmap,
					 ext2_ino_t inode)
{
  return ext2fs_unmark_generic_bitmap((ext2fs_generic_bitmap) bitmap, inode);
}

static inline int ext2fs_test_inode_bitmap(ext2fs_inode_bitmap bitmap,
				       ext2_ino_t inode)
{
  return ext2fs_test_generic_bitmap((ext2fs_generic_bitmap) bitmap, inode);
}

static inline void ext2fs_fast_mark_block_bitmap(ext2fs_block_bitmap bitmap,
					    blk_t block)
{
  ext2fs_mark_generic_bitmap((ext2fs_generic_bitmap) bitmap, block);
}

static inline void ext2fs_fast_unmark_block_bitmap(ext2fs_block_bitmap bitmap,
					      blk_t block)
{
  ext2fs_unmark_generic_bitmap((ext2fs_generic_bitmap) bitmap, block);
}

static inline int ext2fs_fast_test_block_bitmap(ext2fs_block_bitmap bitmap,
					    blk_t block)
{
  return ext2fs_test_generic_bitmap((ext2fs_generic_bitmap) bitmap, block);
}

static inline void ext2fs_fast_mark_inode_bitmap(ext2fs_inode_bitmap bitmap,
					    ext2_ino_t inode)
{
  ext2fs_mark_generic_bitmap((ext2fs_generic_bitmap) bitmap, inode);
}

static inline void ext2fs_fast_unmark_inode_bitmap(ext2fs_inode_bitmap bitmap,
					      ext2_ino_t inode)
{
  ext2fs_unmark_generic_bitmap((ext2fs_generic_bitmap) bitmap, inode);
}

static inline int ext2fs_fast_test_inode_bitmap(ext2fs_inode_bitmap bitmap,
					   ext2_ino_t inode)
{
  return ext2fs_test_generic_bitmap((ext2fs_generic_bitmap) bitmap, inode);
}

static inline blk_t ext2fs_get_block_bitmap_start(ext2fs_block_bitmap bitmap)
{
  return ext2fs_get_generic_bitmap_start((ext2fs_generic_bitmap) bitmap);
}

static inline ext2_ino_t ext2fs_get_inode_bitmap_start(ext2fs_inode_bitmap bitmap)
{
  return ext2fs_get_generic_bitmap_start((ext2fs_generic_bitmap) bitmap);
}

static inline blk_t ext2fs_get_block_bitmap_end(ext2fs_block_bitmap bitmap)
{
  return ext2fs_get_generic_bitmap_end((ext2fs_generic_bitmap) bitmap);
}

static inline ext2_ino_t ext2fs_get_inode_bitmap_end(ext2fs_inode_bitmap bitmap)
{
  return ext2fs_get_generic_bitmap_end((ext2fs_generic_bitmap) bitmap);
}

static inline int ext2fs_fast_test_block_bitmap_range(ext2fs_block_bitmap bitmap,
						 blk_t block, int num)
{
  return ext2fs_test_block_bitmap_range(bitmap, block, num);
}

static inline void ext2fs_fast_mark_block_bitmap_range(ext2fs_block_bitmap bitmap,
						  blk_t block, int num)
{
  ext2fs_mark_block_bitmap_range(bitmap, block, num);
}

static inline void ext2fs_fast_unmark_block_bitmap_range(ext2fs_block_bitmap bitmap,
						    blk_t block, int num)
{
  ext2fs_unmark_block_bitmap_range(bitmap, block, num);
}
