#ifndef BLOCK_CACHE_H
#define BLOCK_CACHE_H

#include <sys/types.h>
#include <stdint.h>
#include <datalib/list.h>

#define BCACHE_GET_BUF(entry) (entry)->buffer

typedef enum {
  BLOCK_NEW,
  BLOCK_USED,
  BLOCK_UNUSED,
} bentry_type_t;

typedef struct __bcache bcache_t;
typedef struct __bcache_entry bcache_entry_t;

/* TODO:
 * 1. We can tag blocks, so similar tagged blocks will live in cache
 * together(by some algorithm, median usage?), whether or not their
 * least refcount is 0. So we will have probability area with neat fit.
 *
 * 2. Exchange functionallity of tail and head. We begin to search
 * _from_ the head. :)
 */

struct __bcache
{
  list_head_t used_blocks;
  list_head_t unused_blocks;

  unsigned int block_size;
  /* Maximum values for used and unused items */
  uint32_t used_watermark;
  uint32_t unused_watermark;
  /* Used and unused items currently allocated in cache */
  uint32_t used;
  uint32_t unused;
  /* Last accessed block number */
  long block_nbr_last;

  pthread_mutex_t *cache_lock;
};

struct __bcache_entry
{
  list_node_t bentry_node;
  bentry_type_t type;
  void *buffer;
  long block_nbr;
  uint32_t refcount;
  unsigned dirty:1;
};

void block_cache_init(bcache_t *bcache,
                      unsigned int block_size,
                      unsigned int used_watermark,
                      unsigned int unused_watermark);
void block_cache_flush(bcache_t *bcache);
void *block_cache_add_block(bcache_t *bcache,
                             unsigned int block_nbr);
bcache_entry_t *block_cache_get_entry(bcache_t *bcache, unsigned int block_nbr);


#endif
