#ifndef CONTAINERS_H
#define CONTAINERS_H

#include <stdint.h>
#include <stddef.h>

#define INIT_LIST_HEAD(ptr) do {		\
    (ptr)->next = (ptr); (ptr)->prev = (ptr);	\
  } while (0)

#define loop_for(ptr, list)			\
  for (ptr = (list)->next;			\
       ptr != (list); ptr = ptr->next)

#define safe_loop_for(ptr, tmp, list)				\
  for (ptr = (list)->next, tmp = ptr->next; ptr != (list);	\
       ptr = tmp, tmp = ptr->next)

typedef struct __list_node list_node_t;
typedef struct __list_node * list_node_ptr;

struct __list_node
{
  struct __list_node * prev, * next;
};

static inline void
list_push_between (list_node_ptr node_new,
		   list_node_ptr prev, list_node_ptr next)
{
  next->prev = node_new;
  node_new->next = next, node_new->prev = prev;
  prev->next = node_new;
}

static inline void
list_push (list_node_ptr node_new, list_node_ptr node_cur)
{
  list_push_between (node_new, node_cur, node_cur->next);
}

static inline void
list_delete_between (list_node_ptr prev, list_node_ptr next)
{
  next->prev = prev, prev->next = next;
}

static inline void
list_delete (list_node_ptr node)
{
  list_delete_between(node->prev, node->next);
  node = (void *)0;
}

#endif
