#include <sys/types.h>
#include <fcntl.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdint.h>
#include <errno.h>

#include "dev_io.h"

off_t ext2fs_devio_llseek(dev_io_t *io, off_t offset, int flag)
{
  if (offset >= (/*(off_t) 1*/1UL << ((sizeof(off_t)*8) - 1))) {
    errno = EINVAL;
    return -1;
  }

  return lseek (io->io_fd, offset, flag);
}

dev_io_t *ext2fs_devio_init_io(char *device, int len,
                               uint32_t block_size, off_t init_off, int mask)
{
  dev_io_t *io;

  if (len > DEV_NAME_LEN) {
    errno = EINVAL;
    return NULL;
  }

  io = malloc(sizeof(*io));
  if (!io)
    return NULL;

  io->io_block_size = block_size ? block_size : EXT2FS_INITIAL_BLOCKSIZE;
  io->io_init_ofset = init_off;
  io->io_mask = mask;
  strncpy(io->device_name, device, len);

  io->io_fd = open(io->device_name, io->io_mask);

  if (io->io_fd < 0)
    return NULL;
  else if (io->io_fd == 0)
    /* Strange case */
    abort();

  block_cache_init(&io->bcache, io->io_block_size,
                   CACHE_USED_SIZE, CACHE_UNUSED_SIZE);

  return io;
}

ssize_t ext2fs_devio_read_blk(dev_io_t *io, uint32_t start_blk, uint32_t count_blk, void *data)
{
  ssize_t size, rd;
  off_t off, off_ret;

  size = count_blk * io->io_block_size;
  io->read += size;
  off = (start_blk * io->io_block_size) + io->io_offset;

  off_ret = ext2fs_devio_llseek(io, off, SEEK_SET);
  if (off_ret != off)
    return 0;

  rd = read(io->io_fd, data, size);
  if (rd != size)
    return -rd;

  return rd;
}

ssize_t ext2fs_devio_write_blk(dev_io_t *io, uint32_t start_blk, uint32_t count_blk, const void *data)
{
  ssize_t size, wr;
  off_t off, off_ret;

  size = count_blk * io->io_block_size;
  io->written += size;
  off = (start_blk * io->io_block_size) + io->io_offset;

  off_ret = ext2fs_devio_llseek(io, off, SEEK_SET);
  if (off_ret != off)
    return 0;

  wr = write(io->io_fd, data, size);
  if (wr != size)
    return -wr;

  return wr;
}
