/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * (c) Copyright 2006,2007-2009 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2009 Stanislav M. Ivankin <grepz@jarios.org>
 *
 * Jari OS ext2 file system port,
 * main entry
 */

#include <sys/types.h>
#include <stdint.h>
#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <unistd.h>
#include <string.h>
#include <fcntl.h>
#include <pthread.h>
#include <ipc/channel.h>
#include <ipc/rpc.h>
#include <ipc/iov.h>
#include <sys/v2.h>
#include <sys/mman.h>
#include <ns/ns.h>
#include <errno.h>
#include <limits.h>

#include <vfs/vfsdef.h>
#include <vfs/super.h>
#include <vfs/vnode.h>

#include "v2ext2.h"
#include "ext2fs.h"

FILE *ext2_log;

static vfs_info_t ext2_info = {
  "ext2.fs",
  "Stanislav M. Ivankin",
  "Ext2 filesystem port to Jari OS",
};

static int ext2_specflags = V2FS_RW | V2FS_RO;
static prtid_t fs_port;

static const int ext2_max_listeners = 8;

static super_block_t     ext2_super;
static filesystem_proc_t ext2_proc;

static struct fs_funct ext2_ops = {
  .mount     = ext2_mount,
  .umount    = NULL,
  .lookup    = ext2_lookup,
  .read      = ext2_read,
  .ioctl     = NULL,
  .write     = ext2_write,
  .opendir   = ext2_opendir,
  .close     = NULL,
  .create    = NULL,
  .setattr   = NULL,
  .getattr   = NULL,
  .createdir = ext2_createdir,
  .unlink    = NULL,
  .unlinkdir = NULL,
  .link      = NULL,
  .symlink   = NULL,
  .readdir   = ext2_readdir,
  .mmap      = NULL,
  .truncate  = NULL,
  .lockf     = NULL,
  .readlink  = NULL,
};

int main(int argc, char *argv[])
{
  int res;

#ifndef CONFIG_EXT2_GRUB
  start_service();
#endif

  ext2_log = fopen("/tmp/ext2.out", "w");
  fprintf(ext2_log, "Starting ext2 filesystem.\n");

  fs_port = ipc_port_open(IPC_BLOCKED_ACCESS, 512);

  ext2_super.info       = &ext2_info;
  ext2_super.block_size = 1024;
  ext2_super.slice_size = 0;
  ext2_super.flags      = ext2_specflags;
  ext2_super.fs_tbl     = &ext2_ops;
  ext2_super.path       = NULL;
  ext2_super.pps        = NULL;
  ext2_super.pp         = 0;
  ext2_super.backend    = NULL;
  ext2_super.pagecache  = NULL;

  ext2_proc.super       = &ext2_super;
  ext2_proc.listen_port = fs_port;
  ext2_proc.listeners   = ext2_max_listeners;

  fprintf(ext2_log, "Port %d opened.\n", fs_port);

  res = vfs_register(fs_port, ext2_specflags, &ext2_info);

  fprintf(ext2_log, "Registered (%d).\n", res);
  fprintf(ext2_log, "Waiting for mount event.\n");

  res = fsproc_init(&ext2_proc);

  fprintf(ext2_log, "Mount request processed: (%d).\n", res);

  v2fs_thread(&ext2_proc);

  fprintf(ext2_log, "Failed to loop in thread.\n");

  return 0;
}
