/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 * 
 * (c) Copyright 2006,2007-2009 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2005,2008,2009 MadTirra <madtirra@jarios.org>
 *
 * Jari OS tmp file system,
 * internals implementation
 */

#ifndef __TMPFS_TMPFS_H__
#define __TMPFS_TMPFS_H__

#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include <datalib/usrtc.h>

#ifdef CONFIG_TMPFS_DEBUG
#define TMPFS_DEBUG  1
#endif

#define BLOCK_SIZE  4096

#define NAME_SIZE    100
#define UNAME_SIZE   32
#define PREFIX_SIZE  155

#define TFS_UREAD   (1 << 0)
#define TFS_UWRITE  (1 << 1) 
#define TFS_UEXEC   (1 << 2)
#define TFS_GREAD   (1 << 3)
#define TFS_GWRITE  (1 << 4)
#define TFS_GEXEC   (1 << 5)
#define TFS_OREAD   (1 << 6)
#define TFS_OWRITE  (1 << 7)
#define TFS_OEXEC   (1 << 8)

enum tmpfs_type_t {
  REGTYPE  = 0,     /* Regular file */
  LNKTYPE  = 1,     /* Hard link */
  SYMTYPE  = 2,     /* Symbolic link */
  CHRTYPE  = 3,     /* Character special */
  BLKTYPE  = 4,     /* Block special */
  DIRTYPE  = 5,     /* Directory */
  FIFOTYPE = 6      /* Named pipe */
};

enum tmpfs_pgstate_t {
  PG_NORM = 0 ,
  PG_NOTFILL = 1,
  PG_HOLE = 2
};

enum tmpfs_imode_t {
  TN_SELF = 0,
  TN_PARENT = 1,
  TN_CHILD = 2
};

typedef struct __tmpfs_page_t {
  int state;
  void *page;
  struct __tmpfs_page_t *prev;
  struct __tmpfs_page_t *next;
} tmpfs_page_t;

typedef struct __tmpfs_node_t {
  uid_t owner;                  /* owner uid */
  gid_t gowner;                 /* owner gid */
  uint16_t mask;                /* access mask */
  time_t ctime;                 /* created time */
  time_t atime;                 /* access time */
  time_t mtime;                 /* modified time */
  size_t size;                  /* size of file */
  int type;                     /* type of a file */
  int num_links;                /* number of hard links */
  uint64_t hash;                /* hash */
  char *name;                   /* name */
  usrtc_node_t *node;           /* link to node */
  usrtc_t *childs;              /* childs if directory */
  tmpfs_page_t *pages;          /* data of the file, should always points to the first page */
  struct __tmpfs_node_t *parent;  /* parent */
  struct dirent **entries; /* should be null if not cached i.e. not opened */
} tmpfs_node_t;

typedef struct __tmpfs_super_t {
  size_t max_pages;
  size_t alloc_pages;
  tmpfs_node_t *root;
} tmpfs_super_t;

int tmpfs_mount(vnode_t *node,char *path,int *pps,int pp,int flags);
int tmpfs_umount(vnode_t *node);
int tmpfs_lookup(vnode_t *node,char *path,int *pps,int pp);
int tmpfs_create(vnode_t *node,char *path,int *pps,int pp,int mode,uid_t uid,gid_t gid);
void *tmpfs_read(vnode_t *node,void *buf,size_t *size,off_t offset);
int tmpfs_write(vnode_t *node,void *buf,size_t *size,off_t offset);
int tmpfs_opendir(vnode_t *node,char *path,int *pps,int pp);
int tmpfs_close(vnode_t *node);
int tmpfs_readdir(vnode_t *node,size_t size,off_t off,struct dirent *dentry);
int tmpfs_createdir(vnode_t *node,char *path,int *pps,int pp,int mode,uid_t uid,gid_t gid);
int tmpfs_unlink(vnode_t *node,uid_t uid,gid_t gid);
int tmpfs_unlinkdir(vnode_t *node,uid_t uid,gid_t gid);
int tmpfs_truncate(vnode_t *node,off_t offset);

#endif /* __TMPFS_TMPFS_H__ */
