/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 * 
 * (c) Copyright 2006,2007-2009 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2005,2008,2009 MadTirra <madtirra@jarios.org>
 *
 * Jari OS tmp file system,
 * main entry
 */

#include <sys/types.h>
#include <stdint.h>
#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <fcntl.h>
#include <pthread.h>
#include <unistd.h>
#include <ipc/channel.h>
#include <ipc/rpc.h>
#include <ipc/iov.h>
#include <sys/v2.h>
#include <sys/kcontrol.h>
#include <sys/mman.h>
#include <ns/ns.h>
#include <errno.h>
#include <datalib/usrtc.h>
#include <service.h>

#include <vfs/vfsdef.h>
#include <vfs/super.h>
#include <vfs/vnode.h>

#include "tmpfs.h"

#ifdef TMPFS_DEBUG
#include <dircons.h>
#endif

static vfs_info_t tmp_fs = { 
  "tmpfs",
  "Alfeiks Kaanoken",
  "Temproary file system",
};

static int tmp_fs_specflags = V2FS_SYS;

static prtid_t fs_port;

/* static initialization */

static struct fs_funct tmpfs_tbl = {
  tmpfs_mount,
  tmpfs_umount,
  tmpfs_lookup,
  tmpfs_read,
  NULL,
  tmpfs_write,
  tmpfs_opendir,
  tmpfs_close,
  tmpfs_create,
  NULL,
  NULL,
  tmpfs_createdir,
  tmpfs_unlink,
  tmpfs_unlinkdir,
  NULL,
  NULL,
  tmpfs_readdir,
  tmpfs_truncate,
  NULL,
  NULL,
  NULL, /* readlink */
};

static super_block_t tmpfs_super;

#define IFS_MAX_LISTENERS  8

static filesystem_proc_t tmpfs_proc;

int main(int argc,char **argv)
{
  int r;

#ifndef CONFIG_TMPFS_GRUB
  start_service();
#endif

  fs_port=ipc_port_open(IPC_BLOCKED_ACCESS,512);

  tmpfs_super.info=&tmp_fs;
  tmpfs_super.block_size=0;
  tmpfs_super.slice_size=0;
  tmpfs_super.flags=tmp_fs_specflags;
  tmpfs_super.fs_tbl=&tmpfs_tbl;
  tmpfs_super.path=NULL;
  tmpfs_super.pps=NULL;
  tmpfs_super.pp=0;
  tmpfs_super.backend=NULL;
  tmpfs_super.pagecache=NULL;

  tmpfs_proc.super=&tmpfs_super;
  tmpfs_proc.listen_port=fs_port;
  tmpfs_proc.listeners=IFS_MAX_LISTENERS;
 
#ifdef TMPFS_DEBUG
  open_console();
#endif
    
#ifdef TMPFS_DEBUG
  console_printf("[tmpfs] opened port %d.\n",fs_port);
#endif
    
  r=vfs_register(fs_port,tmp_fs_specflags,&tmp_fs);
    
#ifdef TMPFS_DEBUG
  console_printf("[tmpfs] registered (%d).\n",r);
  console_printf("[tmpfs] Waiting for mount event.\n");
#endif
    
  r=fsproc_init(&tmpfs_proc);
    
#ifdef TMPFS_DEBUG
  console_printf("[tmpfs] mounted (%d).\n",r);
#endif

  v2fs_thread(&tmpfs_proc);

  return 0;
}
