/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * (c) Copyright 2006,2007,2008 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2009 Dmitry Gromada <gromada@jarios.org>
 *
 * Block layer trace structure and functions
 */

#ifndef __BLK_TRACE_H__
#define __BLK_TRACE_H__

#include <stdbool.h>
#include <sys/types.h>
#include <sys/blkdev.h>
#include <datalib/list.h>
#include <pthread.h>
#include <errno.h>
#include <block/request.h>
#include "defs.h"

#define TRACE_HASH_BITS 6
#define TRACE_HASH_TABLE_WIDTH (1 << TRACE_HASH_BITS)
#define NR_TRACE_TYPES 2
#define NR_TRACE_LAYERS 2

struct __trace_operations;

typedef struct __blk_rq_trace {
  /*
   * hash tables of trace objects for search by sector
   * and by ID
   */
  list_head_t hash[2][TRACE_HASH_TABLE_WIDTH];
  list_head_t rq_fifo;
  pthread_mutex_t mutex;
  uint32_t nr_traced_rqs;
  uint32_t nr_traced_events;
  uint32_t last_id;
  struct __trace_operations *ops;
} blk_rq_trace_t;

typedef struct __trace_event {
  trace_event_data_t data;
  void *data_buf;
  list_node_t node;
} trace_event_t;

typedef struct __trace_entry_t {
  trace_entry_data_t data;
  blk_request_t *rq;
  list_head_t events;
  list_node_t hash_node[2];
  list_node_t fifo_node;
} trace_entry_t;

#ifdef CONFIG_BLK_TRACE

int init_blk_rq_trace(blkreq_queue_t *q);
void destroy_blk_rq_trace(blkreq_queue_t *q);
ssize_t get_trace_events(blkreq_queue_t *q, uint64_t key, skey_type_t key_type,
                         void *buf, size_t buf_size);
int clear_blk_rq_trace(blkreq_queue_t *q);

void add_trace_rq(blk_request_t *rq);
void end_trace_rq(blk_request_t *rq);
void trace_rq_iosched_add(blk_request_t *rq);
void trace_rq_merge_rq(blk_request_t *rq_front, blk_request_t *rq_tail);
void trace_rq_merge_blkio(blk_request_t *rq, blkio_t *blkio, short m_prio,
                          merge_type_t where);
void trace_rq_dispatch(blk_request_t *rq, trace_layer_t layer);
void trace_rq_starve(blk_request_t *rq, trace_layer_t layer);
void trace_rq_schedule(blk_request_t *rq);
void trace_rq_start_proc(blk_request_t *rq);
void trace_rq_end_proc(blk_request_t *rq);

#else

#define add_trace_rq(rq) { }
#define end_trace_rq(rq) { }
#define trace_rq_iosched_add(rq) { }
#define trace_rq_merge_rq(rq_front, rq_tail) { }
#define trace_rq_merge_blkio(rq, blkio, m_prio, where) { }
#define trace_rq_dispatch(rq, layer) { }
#define trace_rq_starve(rq, layer) { }
#define trace_rq_schedule(rq) { }
#define trace_rq_start_proc(rq) { }
#define trace_rq_end_proc(rq) { }

static inline int init_blk_rq_trace(blkreq_queue_t *q)
{
  return 0;
}

static inline void destroy_blk_rq_trace(blkreq_queue_t *q)
{
  return 0;
}

static inline ssize_t get_trace_events(blkreq_queue_t *q, uint64_t key, skey_type_t key_type,
                                       void *buf, size_t buf_size)
{
  return -ENOSYS;
}

static inline int clear_blk_rq_trace(blkreq_queue_t *q)
{
  return -ENOSYS;
}

#endif /* CONFIG_BLK_TRACE */

#endif /* __BLK_TRACE_H__ */
