/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * (c) Copyright 2006,2007,2008 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2009 Dmitry Gromada <gromada@jarios.org>
 *
 * include/block/iosched.h: block I/O scheduler interface
 */

#ifndef __IOSCHED_H__
#define __IOSCHED_H__

#include <stdbool.h>
#include <sys/types.h>
#include <datalib/list.h>
#include <arch/atomic.h>
#include <sys/blkdev.h>
#include "request.h"
#include "defs.h"

typedef struct __iosched_ops {
  int (*init)(struct __iosched*);
  void (*release)(struct __iosched*);
  int (*insert)(struct __iosched*, blk_request_t*);
  int (*remove)(struct __iosched*, blk_request_t*);
  int (*dispatch_rq)(struct __iosched*, blk_request_t *rq);
  int (*flush_queue)(struct __iosched*, bool force);
  void (*find_lock_mergeable)(struct __iosched*, blkio_t*,
                              blk_request_t **rq_front, blk_request_t **rq_tail);
  void (*post_merge_blkio_fn)(struct __iosched*, blk_request_t*,
                              merge_type_t where);
  bool (*pre_merge_rqs_fn)(struct __iosched*, blk_request_t *rq_front,
                           blk_request_t *rq_tail);
  void (*post_merge_rqs_fn)(struct __iosched*, blk_request_t *rq_front,
                            blk_request_t *rq_tail);
  /*
   * recalculate priority of merged request
   * @m_prio: priority of second merged request or blkio
   */
  short (*calc_prio)(struct __iosched*, blk_request_t*, short m_prio);
  void (*set_prio)(struct __iosched*, blk_request_t*, short prio);
} iosched_ops_t;

typedef struct __iosched {
  const char *name;
  iosched_ops_t *ops;
  blkreq_queue_t *queue;
  struct timer_desc tmr;
  /*
   * show whether requests distributed
   * between 2 physical device heads are
   * allowed
   */
  bool head_dist_reqs;
  atomic_t refcnt;
  void *private;
} iosched_t;

typedef struct __iosched_type {
  const char *name;
  iosched_ops_t *ops;
  list_node_t node;
} iosched_type_t;

int initialize_blk_schedulers(void);
int register_blk_iosched(const char *name, iosched_ops_t *ops);
int set_blk_default_iosched(blkreq_queue_t *q);
int set_blk_iosched(blkreq_queue_t *q, const char *name);
int disable_blk_iosched(blkreq_queue_t *q);
bool iosched_merge(iosched_t *sched, blkio_t *blkio, short m_prio);
bool requests_are_mergeable(blk_request_t *rq_front, blk_request_t *rq_tail);
bool merge_requests(blk_request_t *rq_front, blk_request_t *rq_tail);
bool blk_rq_is_mergeable(blk_request_t *rq);

static void inline get_iosched(iosched_t *sched)
{
  atomic_inc(&sched->refcnt);
}

static void inline put_iosched(iosched_t *sched)
{
  atomic_dec(&sched->refcnt);
}

#endif
