/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *	
 * (c) Copyright 2006,2007,2008 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2008 Dmitry Gromada <gromada@jarios.org>
 *
 * include/block/partition.h: partition scanning
 */

#ifndef __PARTITION_H__
#define __PARTITION_H__

#include <sys/types.h>

#define PART_TBL_MAGIC 0xAA55

#define DOS_EXTENDED_PARTITION    0x5
#define WIN95_EXTENDED_PARTITION  0xF
#define LINUX_EXTENDED_PARTITION  0x85

typedef struct __part_cs_sector {
  uint32_t cyl : 6,
    sect : 10;
} __attribute__ ((packed)) part_cs_sector_t;

typedef struct __partition {
  uint8_t boot_ind;        /* boot identificator: 0x80 active */
  uint8_t start_head;    
  part_cs_sector_t start_cs_sect;
  uint8_t fsid;            /* file system identifier */
  uint8_t end_head;
  part_cs_sector_t end_sect;
  uint32_t start_sector;  /* logical number of the first partition sector */
  uint32_t nr_sectors;    /* partition size in sectors */
} __attribute__ ((packed)) partition_t;

/* master boot record */
struct mbr {
  uint8_t __res[0x01BE];  /* boot code and free area */
  partition_t parts[4];
  uint16_t signature;
};

#endif
