/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * (c) Copyright 2006,2007,2008 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2008 Dmitry Gromada <gromada@jarios.org>
 *
 * Declarations for block server
 */

#ifndef __BLOCK_SERVER_H__
#define __BLOCK_SERVER_H__

#include <sys/types.h>
#include <stdbool.h>
#include <completion.h>
#include "request.h"

#define MAX_BLKSERV_THREADS 16

typedef struct __bio_rpc {
  int rw;
  ulong_t offset;
  size_t size;
  short prio;
  bd_part_t *part;
} bio_rpc_t;

/*
 * registration data for threads starting
 * a storage server
 */
struct serv_reg_data {
  genstor_t *stor;
  uint32_t iobuf_size;
  int nr_cmd_threads;
  int nr_io_threads;
  struct completion compl;
};


/*
 * @storage: storage object provided access for data
 * @iobuf_size - I/O buffer size; if it is less of a predefined
 *    minimum I/O buffer size, it's increased up to this value
 *
 * @nr_cmd_threads: number of threads processing service requests
 * @nr_io_threads:  number of threads processing user I/O requests
 * @start_compl: object for notification about server initialization
 *               completion
 *
 * Return value:
 *    0 on success, negated error value otherwise
 */
int start_storage_server(genstor_t *storage, uint32_t iobuf_size, int nr_cmd_threads,
                         int nr_io_threads, struct completion *start_compl);

/*
 * Does the same as the previous function, but start the
 * server in a new thread
 */
int start_storage_serv_thread(struct serv_reg_data *reg_data);
int copy_biorq_data(biorq_t *biorq, uint32_t offset, uint32_t len, void *buf);
int copy_biorq_data_sectors(biorq_t *biorq, uint32_t start_sector,
                            uint32_t nr_sectors, int sect_shift, void *buf);
void reply_blkio(blkio_t *blkio);
bool biorq_session_is_active(biorq_t *biorq);
int server_prepare_biorqs(biorq_t **biorq, int nr, biorq_t *serv_biorq);
void put_server_biorq(biorq_t *biorq);
#endif

