/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * (c) Copyright 2006,2007,2008,2009 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2009 Paul Graphov <graphov@jarios.org>
 *:
 * datastructs.c: useful (for me at least) data structures
 *
 */

#ifndef __DATASTRUCTS_H__
#define __DATASTRUCTS_H__

#include <arch/atomic.h>
#include <datalib/list.h>

#define ADDMOD(x, y, z) x = ((x + y) % z)

// Deque (with stack and fifo via it)

typedef enum {
  DEQUE_FRONT = 0,
  DEQUE_BACK = 1
} deque_end_t;

typedef struct __deque_node_t {
  struct __deque_node_t *sides[2];
  void *data;
} deque_node_t;

typedef struct __deque_t {
  deque_node_t *ends[2];
} deque_t;

void deque_init(deque_t *deque);
void deque_destroy(deque_t *deque, void (*dispose)(void *));
status_t deque_push(deque_t *deque, void *data, deque_end_t end);
void *deque_pop(deque_t *deque, deque_end_t end);
status_t deque_push_node(deque_t *deque, deque_node_t *node, deque_end_t end);
deque_node_t *deque_pop_node(deque_t *deque, deque_end_t end);

static inline void *deque_get(deque_t *deque, deque_end_t end)
{
  if(deque->ends[end]) return deque->ends[end]->data;
  return NULL;
}

static inline status_t stack_push(deque_t *deque, void *data)
{
  return deque_push(deque, data, DEQUE_FRONT);
}

static inline void *stack_pop(deque_t *deque)
{
  return deque_pop(deque, DEQUE_FRONT);
}

static inline status_t fifo_push(deque_t *deque, void *data)
{
  return deque_push(deque, data, DEQUE_FRONT);
}

static inline void *fifo_pop(deque_t *deque)
{
  return deque_pop(deque, DEQUE_BACK);
}

static inline status_t stack_push_node(deque_t *deque, deque_node_t *node)
{
  return deque_push_node(deque, node, DEQUE_FRONT);
}

static inline deque_node_t *stack_pop_node(deque_t *deque)
{
  return deque_pop_node(deque, DEQUE_FRONT);
}

static inline status_t fifo_push_node(deque_t *deque, deque_node_t *node)
{
  return deque_push_node(deque, node, DEQUE_FRONT);
}

static inline deque_node_t *fifo_pop_node(deque_t *deque)
{
  return deque_pop_node(deque, DEQUE_BACK);
}


#endif
