/*
 * This library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * (c) Copyright 2006,2007,2008 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2008 Dmitry Gromada <gromada@jarios.org>
 *
 * Debug features
 */

#ifndef __DEBUG_H__
#define __DEBUG_H__

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/kcontrol.h>

#ifndef DEBUG_BUFFER_LENGTH
#define DEBUG_BUFFER_LENGTH 1024
#endif

#define debug_print_line()                                                        \
{                                                                                 \
  klog("%s(), line %d\n", __FUNCTION__, __LINE__);                                \
}

#define debug_print_text(msg...)                                                  \
{                                                                                 \
  char *__buf = malloc(DEBUG_BUFFER_LENGTH);                                      \
  char *__p;                                                                      \
                                                                                  \
  sprintf(__buf, "[%s(), line %d]: ", __FUNCTION__, __LINE__);                    \
  __p = (void*)((ulong_t)__buf + strlen(__buf));                                  \
  sprintf(__p, msg);                                                              \
  klog(__buf);                                                                    \
  free(__buf);                                                                    \
}

#define ASSERT(condition, msg...)                                                 \
{                                                                                 \
  char *__buf;                                                                    \
  char *__p;                                                                      \
                                                                                  \
  if (!(condition)) {                                                             \
    __buf = malloc(DEBUG_BUFFER_LENGTH);                                          \
    sprintf(__buf, "!!! BUG in %s(), line %d: ", __FUNCTION__, __LINE__);         \
    __p = (void*)((ulong_t)__buf + strlen(__buf));                                \
    sprintf(__p, msg);                                                            \
    if (!strlen(__p)) {                                                           \
      *(__p - 2) = '\n';                                                          \
      *(__p - 1) = '\0';                                                          \
    }                                                                             \
    klog(__buf);                                                                  \
    abort();                                                                      \
  }                                                                               \
}

#define SPINLOCK() for (;;)

#define mark_trace_point()                                                        \
{                                                                                 \
  char __buf[64];                                                                 \
                                                                                  \
  sprintf(__buf, "%s, line %d\n", __FUNCTION__, __LINE__);                        \
  save_trace_point(__buf);                                                        \
}

#define mark_trace_point_mesg(msg...)                                             \
{                                                                                 \
  char __buf[256], *__p;                                                          \
                                                                                  \
  sprintf(__buf, "[%s, line %d]: ", __FUNCTION__, __LINE__);                      \
  __p = __buf + strlen(__buf);                                                    \
  sprintf(__p, msg);                                                              \
  save_trace_point(__buf);                                                        \
}

int init_trace(const char *addr_file, uint32_t size);
void free_trace(void);
void save_trace_point(char *location);

#endif
