/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * (c) Copyright 2006,2007,2008,2009 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2009 Dmitry Gromada <gromada@jarios.org>
 *
 * dma controller declarations
 */

#ifndef __DMA_H__
#define __DMA_H__

#include <sys/types.h>
#include <stdint.h>

/* DMA channel status constants */
enum {
  DMA_CHAN_STAT_WAIT,
  DMA_CHAN_STAT_RUN,
  DMA_CHAN_STAT_ERR,
  DMA_CHAN_STAT_INTR
};

/* dma flags */
enum {
  DMA_CHAN_SG = 0x01,
  DMA_CHAN_2D_TRANSF = 0x02,
  DMA_CHAN_NO_IRQ = 0x04
};

enum {
  DMA_READ,
  DMA_WRITE
};

struct __dma_channel;

struct sg_entry {
  uint64_t start;
  uint32_t len;
};

/*
 * 2d transfer descriptor
 *
 * Some dma controllers support
 * 2d transfer mode.
 */
typedef struct __dma_2d_desc {
  uint32_t start;       /* start address */
  uint32_t length;      /* length of each transfer */
  uint32_t end_offset;  /* offset done at end of each transfer */
  uint16_t count;       /* number of transfers */
} dma_2d_desc_t;

typedef struct __dma_channel_ops {
  bool (*load_sg_list)(struct __dma_channel*, struct sg_entry *sg_list,
                       uint16_t nt_entries);
  bool (*init_2d_transfer)(struct __dma_channel*, dma_2d_desc_t);
  bool (*start)(struct __dma_channel*);
  bool (*stop)(struct __dma_channel*);
  int (*set_prio)(struct __dma_channel*);
  uint32_t (*remain_count)(struct __dma_channel*);
  void (*set_direction)(struct __dma_channel*, int dir);
  int (*get_status)(struct __dma_channel*);
  int (*set_irq)(struct __dma_channel*, uint16_t irq);
  void (*reset)(struct __dma_channel*);
} dma_channel_ops_t;

typedef struct __dma_channel {
  int chan;
  int dir;
  uint32_t flags;
  uint16_t prio;
  uint16_t max_prio;
  int addr_width;  /* width of available address space */
  int data_width;
  uint16_t align_mask;
  uintptr_t phy_desc_tbl_addr;
  void *desc_tbl;
  uint16_t desc_tbl_len;
  uint16_t max_descs;
  uint16_t desc_size;
  uintptr_t phy_region_start;
  uint32_t phy_region_len;
  void *vdma_buf;
  void *regs_map;  /* base address of registers mapping */
  uint32_t boundary;
  /* maximimum size of bytes per single transfer */
  uint32_t max_single_tsize;
  /* maximum total transfer size */
  uint32_t max_total_tsize;
  dma_channel_ops_t *ops;
  void *private;
} dma_channel_t;

int alloc_dma_channel(dma_channel_t *dma_chan);
int release_dma_channel(dma_channel_t *dma_chan);

#endif
