/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * (c) Copyright 2006,2007,2008 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2008 Dmitry Gromada <gromada82@gmail.com>
 *
 * I/O irq controller interface
 */

/*
 * TO DO: design irq and controller flags
 */

#ifndef __IRQCON_H__
#define __IRQCON_H__

#include <sys/types.h>
#include <datalib/list.h>
#include <pthread.h>
#include "balance.h"

#define irqcon_lock(gencon)   pthread_mutex_lock(&(gencon)->mutex)
#define irqcon_unlock(gencon) pthread_mutex_unlock(&(gencon)->mutex)

/* private irq flags */
enum {
  IRQ_MASKED = 0x10000
};

typedef struct __intr_line {
  int  chip;
  int  pin;
} intr_line_t;

typedef struct __irq_listener {
  tid_t tid;
  list_node_t l;
} irq_listener_t;

typedef struct __irq_desc {
  intr_line_t line;       /* I/O interrupt line */
  irq_mode_t  mode;
  int          vector;      /* vector assigned to this irq */
  int          targ_cpu;    /* logical cpu ID to which this irq is routed */
  uint32_t    flags;
  int          count;      /* how much times this irq was handled */
  int         nr_lis;      /* listeners number */
  list_head_t listeners;
} irq_desc_t;


/* info about capacity of cpu to handle irqs */
typedef struct __irq_capacity {
  int limit;        /* limit of handled irqs */
  int curr_num;      /* current number of handled irqs */
} irq_capacity_t;

/* I/O interrupt controller hardware oprations */
typedef struct __irqcon_hw_operations {
  int (*mask_irq)(int irq, int mask);
  int (*route_irq)(int irq, int cpu);
} irqcon_hw_operations_t;

/* abstract input/output interrupt controller */
typedef struct __generic_irq_controller {
  const char *name;
  int minvec;    /* minimum interrupt vector */
  int maxvec;    /* maximum interrupt vector */
  int nr_irqs;
  irq_desc_t *irqs;
  int ncpus;    /* number of CPUs to which the controller route irqs */
  bitmap_t vecmap;    /* bitmap of unused vectors */
  irq_capacity_t *caps; 
  irqcon_hw_operations_t *ops;
  pthread_mutex_t mutex; /* balance speenlock needed in case of dynamic balancing */
  list_head_t balancers;
} generic_irq_controller_t;


static void inline set_irq_flags(irq_desc_t *idesc, uint32_t flag)
{
  idesc->flags |= flag;
}

static void inline clear_irq_flags(irq_desc_t *idesc, uint32_t flag)
{
  idesc->flags &= ~flag;
}

static int inline test_irq_flags(irq_desc_t *idesc, uint32_t flag)
{
  return (idesc->flags & flag);
}

#endif
