/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 * 
 * (c) Copyright 2006,2007-2009 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2005,2008,2009 MadTirra <madtirra@jarios.org>
 *
 * Jari OS virtual file system v2,
 * file system page cache
 */

#ifndef __V2_PAGECACHE_H__
#define __V2_PAGECACHE_H__

#include <sys/types.h>
#include <sys/stat.h>
#include <stdint.h>
#include <stdarg.h>
#include <ipc/port.h>
#include <ipc/channel.h>
#include <dirent.h>
#include <pthread.h>
#include <datalib/usrtc.h>

#include <vfs/vfsdef.h>
#include <vfs/vnode.h>

typedef struct __pagecache_lim pagecache_lim_t;
typedef struct __page_descr page_descr_t;
typedef struct __fs_pagecache fs_pagecache_t;

struct pagecache_funct
{
  int (*write)(page_descr_t *page);
  int (*read)(page_descr_t *page);
  void (*flush)(page_descr_t *page);
  void (*flushall)(fs_pagecache_t *page_cache);
  void (*map)(page_descr_t *page);
  void (*unmap)(page_descr_t *page);
};

struct __pagecache_lim
{
  size_t upper_bound;
  void *spec_lim; /* Special FS defined limits */
};

struct __page_descr
{
  vnode_t *vnode;
  off_t node_offset; /* Offset to file */
};

struct __fs_pagecache {
  pagecache_lim_t page_lim;
  struct pagecache_funct *pagecache_ops;
  usrtc_t *page_nodes; /* TODO: make as radix tree */
};

page_descr_t *page_lookup(fs_pagecache_t *cache, page_descr_t *page);

#endif /* __V2_PAGECACHE_H__ */
