/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 * 
 * (c) Copyright 2006,2007-2009 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2005,2008,2009 MadTirra <madtirra@jarios.org>
 *
 * Jari OS virtual file system v2,
 * super block
 */

#ifndef __V2_SUPER_H__
#define __V2_SUPER_H__

#include <sys/types.h>
#include <sys/stat.h>
#include <stdint.h>
#include <stdarg.h>
#include <dirent.h>
#include <ipc/port.h>
#include <ipc/channel.h>
#include <ipc/rpc.h>
#include <pthread.h>
#include <dirent.h>
#include <datalib/usrtc.h>

#include <vfs/vfsdef.h>
#include <vfs/vnode.h>
#include <vfs/backend.h>
#include <vfs/pagecache.h>

struct fs_funct {
  int (*mount)(vnode_t *node,char *path,int *pps,int pp,int flags);
  int (*umount)(vnode_t *node);
  int (*lookup)(vnode_t *node,char *path,int *pps,int pp);
  void*(*read)(vnode_t *node,void *buf,size_t *size,off_t offset);
  int (*ioctl)(vnode_t *node,uni_rpc_t *rpc);
  int (*write)(vnode_t *node,void *buf,size_t *size,off_t offset);
  int (*opendir)(vnode_t *node,char *path,int *pps,int pp);
  int (*close)(vnode_t *node);
  int (*create)(vnode_t *node,char *path,int *pps,int pp,int mode,uid_t uid,gid_t gid);
  int (*setattr)(vnode_t *node,uint16_t cmd,...); 
  int (*getattr)(vnode_t *node,uint16_t cmd,...); 
  int (*createdir)(vnode_t *node,char *path,int *pps,int pp,int mode,uid_t uid,gid_t gid);
  int (*unlink)(vnode_t *node,uid_t uid,gid_t gid);
  int (*unlinkdir)(vnode_t *node,uid_t uid,gid_t gid);
  int (*link)(vnode_t *node,char *path,int *pps,int pp);
  int (*symlink)(vnode_t *node,char *path,int *pps,int pp);
  int (*readdir)(vnode_t *node,size_t size,off_t off,struct dirent *dentry);
  int (*truncate)(vnode_t *node,off_t offset);
  int (*mmap)(vnode_t *node,int *mm_obj,size_t size,off_t offset,int flags,int prot);
  int (*lockf)(vnode_t *node,pid_t aff_pid,int func,size_t size,off_t offset);
  int (*readlink)(vnode_t *node,char *link,size_t *size);
};

typedef struct __super_block_t {
  /* information */
  vfs_info_t *info;
  /* size information */
  size_t block_size;
  size_t slice_size;
  void *spec_data;
  /* misc information */
  int flags;
  /* file system specific functions table */
  struct fs_funct *fs_tbl;
  /* path of source, if exist */
  char *path;
  int *pps;
  int pp;
  /* mappable feature */
  int is_mappable;
  prtid_t serve_port;
  /* back end of the source */
  fsbackend_t *backend;
  /* page cache if used */
  fs_pagecache_t *pagecache;
} super_block_t;


#endif /* __V2_SUPER_H__ */
