/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * (c) Copyright 2006,2007,2008 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2008 Dmitry Gromada <gromada@jarios.org>
 *
 * server.h: server part receiving client messages
 */

/*
 * TO DO [dg]: associate read ahead with file descriptors
 */

#ifndef __SERVER_H__
#define __SERVER_H__

#include <pthread.h>
#include <queue.h>
#include <block/genstor.h>
#include <block/server.h>
#include <sys/swks.h>

#define MAX_CLIENTS 128
#define CMDBUF_SIZE 512
#define MIN_IOBUF_SIZE 1024 + sizeof(uni_rpc_t)
#define ASYNC_PORT_QUEUE_SIZE 8192
#define CLIENT_HASH_BITS 5
#define CLIENT_HASH_TBL_WIDTH (1 << CLIENT_HASH_BITS)
#define CLIENT_HASH_MASK (CLIENT_HASH_TBL_WIDTH - 1)

#define BLKSERV_BUSY      (1 << __BLKSERV_BUSY)
#define BLKSERV_SHUTDOWN  (1 << __BLKSERV_SHUTDOWN)
#define BLKSERV_ERROR     (1 << __BLKSERV_ERROR)

#define set_server_busy(srv)      atomic_test_and_set_bit(&(srv)->state, 1 << __BLKSERV_BUSY)
#define clear_server_busy(srv)    atomic_test_and_reset_bit(&(srv)->state, 1 << __BLKSERV_BUSY)
#define set_server_shutdown(srv)  atomic_test_and_set_bit(&(srv)->state, 1 << __BLKSERV_SHUTDOWN)
#define set_server_error(srv)     atomic_test_and_set_bit(&(srv)->state, 1 << __BLKSERV_ERROR)

#define clients_lock(client_list)   pthread_mutex_lock(&(client_list)->mutex)
#define clients_unlock(client_list) pthread_mutex_unlock(&(client_list)->mutex)

/* server state bits */
enum {
  __BLKSERV_BUSY,
  __BLKSERV_SHUTDOWN,
  /* critical error at which server terminate it's work */
  __BLKSERV_ERROR
};

/* blk file descriptor private flags */
enum {
  BLKFD_CLOSED = 0x1000000
};

struct __blkdev_server;

typedef struct __blk_thread {
  pthread_t id;
  void *buf;
  ipc_port_msg_info_t msg_info;
  struct __blkdev_server *srv;
} blk_thread_t;

typedef struct __blkserv_ioops {
  int (*init) (struct __blkdev_server *srv);
  int (*receive_imsg) (blk_thread_t *thread);
  int (*get_imsg_data)(blk_thread_t *thread, uint32_t offset, uint32_t size, void *buf);
  int (*send_non_cached_data) (struct __blkdev_server *srv, biorq_t *biorq);
  int (*send_cached_data) (struct __blkdev_server *srv, biorq_t *biorq);
} blkserv_ioops_t;

struct client_list {
  usrtc_t clients;
  pthread_mutex_t mutex;
};

typedef struct __blkdev_server {
  pid_t devfs_pid;
  prtid_t cmd_port;
  prtid_t io_port;
  uint32_t iobuf_size;
  ulong_t state;
  genstor_t *stor;
  struct client_list client_table[CLIENT_HASH_TBL_WIDTH];
  evl_queue_t done_srv_bios;    /* done service block I/O operations queue */
  list_head_t done_user_bios;   /* done user block I/O operations list */
  list_head_t ctl_reqs;         /* active user control requests */
  pthread_mutex_t rq_mutex;     /* request mutex */
  event_t event;
  blk_thread_t *cmd_threads;
  blk_thread_t *io_threads;
  int nr_cmd_threads;
  int nr_io_threads;
  pthread_t reply_threads[2];
  blkserv_ioops_t ioops;
  bool mesgs_are_persistent;
} blkdev_server_t;

struct client {
  pid_t pid;
  usrtc_t files;
  chid_t async_chan;
  blkdev_server_t *server;
  atomic_t refcnt;
};

struct blk_fd {
  struct client *client;
  uint16_t devno;
  uint16_t id;
  uint16_t mode;
  uint32_t flags;
  atomic_t refcnt;
} blk_fd;

#endif
