#include <pthread.h>
#include <sys/uevent.h>

#include <datastructs.h>
#include <datalib/list.h>

#define SRV_QUEUE_SIZE  512
#define LISTEN_PORTS    3
#define MAX_DEVNO       256

#define WORK_LISTEN (MAX_DEVNO + 1000)

#define DEFAULT_LOGLEVEL 0

#define MKWORK(dev,dir) (dev*2+dir+1)
#define WDIR(x) ((x-1)%2)
#define WDEV(x) ((x-1)/2)

#define MSGINFO_SLOT 1

#define DEVNO(x) devicemap[(x)->arg2]
#define DIR(x) (((x)->rpc_call == RPC_WRITE)?IN:OUT)

typedef struct __msg_data {                                                                            
  ipc_port_msg_info_t msginfo;
  uni_rpc_t rpc;
  list_node_t node;
  void *data;
  void *interim;
} msg_data_t;

typedef struct __client {
  pid_t pid;
  usrtc_t files;
} client_t;

typedef struct __superio_t {
  uint16_t unique;
  uint16_t flags;
  uint16_t mode;
  uint16_t devno;
} superio_t;

static device_server_t *server = NULL;

/* local functions prototypes */
static void process(uni_rpc_t *rpc, void *data, ipc_port_msg_info_t *msginfo);
static void do_messages(int devno, uni_rpc_t *rpc, void *data, ipc_port_msg_info_t *msginfo, msg_data_t *msg);
static inline status_t do_process(int devno, uni_rpc_t *rpc, void *data, ipc_port_msg_info_t *msginfo, void **interim);
static inline msg_data_t *make_msg(uni_rpc_t *rpc, void *data, ipc_port_msg_info_t *msginfo, void *interim);
static void process_cmd(ipc_port_msg_info_t msginfo);

static status_t process_write(int devno, uni_rpc_t *rpc, void *data, ipc_port_msg_info_t *msginfo, void **interim);
static status_t process_read(int devno, uni_rpc_t *rpc, void *data, ipc_port_msg_info_t *msginfo, void **interim);
static status_t process_ioctl(int devno, int dir, int type, int nr, void *data, size_t size,
                              void *odata, size_t osize, ipc_port_msg_info_t *msginfo);
void *worker(void *thread_arg);

static void reply_with_error(int id, int err, int msg_id);
static void reply_with_replcode(int id, ulong_t len, int msg_id);
static void reply_with_data(int id, void *buf, ulong_t len, int msg_id);

static int add_file(pid_t pid, uint16_t unique, uint16_t devno, uint16_t flags, uint16_t mode);
static int rm_file(pid_t pid, uint16_t unique);
static int clone_client(pid_t src, pid_t dest);
static int kill_client(pid_t pid);
static superio_t *get_superio(pid_t pid, uint16_t unique);


typedef struct __device {
  int devno;
  list_head_t ioqueue[2];
  msg_data_t *waiting[2];
  spinlock_t lock[2];
  event_t event;
  int num_working[2];
  int pending_wake[2];
  struct stat statbuf;
} device_t;

