/*
 * This library is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU Lesser General Public License as 
 * published by the Free Software Foundation; either version 2.1 of 
 * the License.
 *
 * This library is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public 
 * License along with this library; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 *
 * (c) Copyright 2006,2007,2008 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2005,2008 MadTirra <madtirra@jarios.org> 
 *
 * file system service specific functions 
 * 
 */

#include <sys/types.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <wchar.h>
#include <datalib/usrtc.h>
#include <ipc/port.h>
#include <ipc/channel.h>
#include <ipc/rpc.h>
#include <ns/ns.h>
#include <pthread.h>

#include <fsio/vfs_call.h>
#include <fsio/vnode.h>
#include <fsio/fs_resources.h>
#include <fsio/fs_serv.h>

int poll_service_port(prtid_t port,void *msgbuf,size_t len,fs_raw_task_t *task)
{
  int r;
  ipc_port_msg_info_t info;
  uni_rpc_t *rpc;

  r=ipc_port_rcvmsg(port,len,msgbuf);
  if(r<0)
    return r;
  else {
    info=ipc_port_getinfo(port);
    task->pid=info.sender_pid;
    task->msg_id=info.msg_id;
    if(info.msg_len < sizeof(*rpc)) {
      return -EINVAL;
    }
    if(task->pid != vfs_ackpid()) 
      return -EPERM;
    task->len=info.msg_len;
    task->uid=0;
    task->gid=0;
    rpc=(uni_rpc_t*)msgbuf;
    task->fs_cmd=rpc->rpc_call;
  }

  return 0;
}

int poll_general_port(prtid_t port,void *msgbuf,size_t len,fs_raw_task_t *task)
{
  int r;
  ipc_port_msg_info_t info;
  uni_rpc_t *rpc;

  r=ipc_port_rcvmsg(port,len,msgbuf);
  if(r<0)
    return r;
  else {
    info=ipc_port_getinfo(port);
    task->pid=info.sender_pid;
    task->msg_id=info.msg_id;
    if(info.msg_len < sizeof(*rpc)) {
      return -EINVAL;
    }
    task->len=info.msg_len;
    task->uid=0;
    task->gid=0;
    rpc=(uni_rpc_t*)msgbuf;
    task->fs_cmd=rpc->rpc_call;
  }

  return 0;
}

