/*
 * This library is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU Lesser General Public License as 
 * published by the Free Software Foundation; either version 2.1 of 
 * the License.
 *
 * This library is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public 
 * License along with this library; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 *
 * (c) Copyright 2006,2007,2008 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2005,2008 MadTirra <madtirra@jarios.org> 
 *
 * vfs calls
 * 
 */

#include <sys/types.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <wchar.h>
#include <ipc/port.h>
#include <ipc/channel.h>
#include <ipc/rpc.h>
#include <ns/ns.h>

#include <fsio/vfs_call.h>

chid_t vfs_channel;
static pid_t ackpid;

int vfsio_init(void)
{
  pid_t vfs_pid=ns_getvfspid();

  if(!vfs_pid)
    return -1;

  vfs_channel=ipc_channel_open(vfs_pid,0,IPC_BLOCKED_ACCESS);
  if(vfs_channel<0)
    return -1;

  ackpid=vfs_pid;

  return 0;
}

pid_t vfs_ackpid(void)
{
  return ackpid;
}

int vfs_register(wchar_t *fs_name,prtid_t w_port,prtid_t s_port)
{
  size_t nm_len=wcslen(fs_name);
  size_t bf_len;
  void *msg,*p;
  vfs_reg_head_t *req_head;
  uni_rpc_t *rpc;
  int r;

  bf_len=nm_len*sizeof(wchar_t)+sizeof(*req_head)+sizeof(*rpc);
  msg=malloc(bf_len);
  if(!msg)
    return -ENOMEM;

  /* fill out rpc */
  rpc=(uni_rpc_t *)msg;
  memset(rpc,0,sizeof(*rpc));
  rpc->rpc_call=RPC_REGISTER;
  rpc->arg0=bf_len-sizeof(*rpc);
  /* take a header */
  req_head=(vfs_reg_head_t*)((char*)msg + sizeof(*rpc));
  req_head->pid=getpid();
  req_head->w_port=w_port;
  req_head->s_port=s_port;
  p=(void *)((char *)msg+sizeof(*rpc)+sizeof(*req_head));
  memcpy(p,fs_name,nm_len*sizeof(wchar_t));

  r=ipc_channel_msgsnd(vfs_channel,bf_len,msg,sizeof(*rpc),rpc);
  if(r<0)
    goto __err_close;
  else
    r=rpc->rpc_call;

 __err_close:
  free(msg);

  return r;
}
