/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * (c) Copyright 2006,2007,2008,2009 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2009 Dmitry Gromada <gromada@jarios.org>
 *
 * Functions for core servers debugging
 */

/*
 * TO DO [dg]: add synchronization (latches)
 */

#include <debug.h>
#include <sys/dma.h>
#include <sys/mman.h>
#include <errno.h>

static void *trace_phys_addr;
static char *trace_table;
static char *curr_addr;
static uint32_t trace_size, real_size;

int init_trace(const char *addr_file, uint32_t size)
{
  uint32_t n;
  FILE *file = 0;
  int r;

  if (trace_table) {
    if (size > real_size) {
      free_trace();
    } else {
      curr_addr = trace_table;
      trace_size = size;
      memset(trace_table, 0, trace_size + 1);
      return 0;
    }
  }

  n = (size >> PAGE_WIDTH) + ((size & PAGE_MASK) != 0);
  trace_phys_addr = dma_alloc_pages(n);
  if (trace_phys_addr == MAP_FAILED)
    return -1;

  trace_size = real_size = size;
  file = fopen(addr_file, "w+b");
  if (file == NULL)
    goto out;

  r = fwrite(&trace_phys_addr, sizeof(trace_phys_addr), 1, file);
  r += fwrite(&real_size, sizeof(real_size), 1, file);
  fclose(file);
  if (r < 2)
    goto out;

  trace_table = mmap(NULL, real_size, PROT_READ | PROT_WRITE,
          MAP_ANONYMOUS | MAP_PRIVATE | MAP_PHYS, 0, (ulong_t)trace_phys_addr);
  if (trace_table == MAP_FAILED)
    goto out;

  memset(trace_table, 0, real_size);
  curr_addr = trace_table;
  return 0;

out:
  dma_free_pages(trace_phys_addr, real_size >> PAGE_WIDTH);
  trace_phys_addr = 0;
  curr_addr = trace_table = NULL;
  trace_size = real_size = 0;

  return -1;
}

void save_trace_point(char *location)
{
  int n = strlen(location) + 1;

  if (curr_addr + n - trace_table <= trace_size) {
    memcpy(curr_addr, location, n);
    curr_addr += n;
  }
}

void free_trace(void)
{
  if (trace_table) {
    munmap(trace_table, real_size);
    dma_free_pages(trace_phys_addr, real_size >> PAGE_WIDTH);
    curr_addr = trace_table = NULL;
    trace_size = real_size = 0;
  }
}
