/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * (c) Copyright 2006,2007,2008,2009 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2009 Paul Graphov <graphov@jarios.org>
 *
 * deque.c: deque implementation.
 *
 */

#include <sys/types.h>

#include <datastructs.h>
#include <stdlib.h>

void deque_init(deque_t *deque)
{
  deque->ends[DEQUE_FRONT] = NULL;
  deque->ends[DEQUE_BACK] = NULL;
}

void deque_destroy(deque_t *deque, void (*dispose)(void *))
{
  void *data;
  while((data = deque_pop(deque, DEQUE_BACK))) {
    if(dispose) {
      dispose(data);
    }
  }
}

status_t deque_push(deque_t *deque, void *data, deque_end_t end)
{
  deque_node_t *node;
  node = malloc(sizeof(*node));
  if(!node) return -1;
  node->data = data;
  if(deque->ends[end]) {
    node->sides[1-end] = deque->ends[end];
    node->sides[end] = NULL;
    deque->ends[end]->sides[end] = node;
    deque->ends[end] = node;
  } else {
    node->sides[DEQUE_FRONT] = NULL;
    node->sides[DEQUE_BACK] = NULL;
    deque->ends[DEQUE_FRONT] = node;
    deque->ends[DEQUE_BACK] = node;
  }
  return 0;
}

void *deque_pop(deque_t *deque, deque_end_t end)
{
  deque_node_t *node;
  void *data;
  if(deque->ends[DEQUE_FRONT] == NULL) return NULL;
  if(deque->ends[DEQUE_FRONT] == deque->ends[DEQUE_BACK]) {
    node = deque->ends[DEQUE_FRONT];
    data = node->data;
    free(node);
    deque->ends[DEQUE_FRONT] = NULL;
    deque->ends[DEQUE_BACK] = NULL;
  } else {
    node = deque->ends[end];
    data = node->data;
    node->sides[1-end]->sides[end] = NULL;
    deque->ends[end] = node->sides[1-end];
    free(node);
  }
  return data;
}

status_t deque_push_node(deque_t *deque, deque_node_t *node, deque_end_t end)
{
  if(deque->ends[end]) {
    node->sides[1-end] = deque->ends[end];
    node->sides[end] = NULL;
    deque->ends[end]->sides[end] = node;
    deque->ends[end] = node;
  } else {
    node->sides[DEQUE_FRONT] = NULL;
    node->sides[DEQUE_BACK] = NULL;
    deque->ends[DEQUE_FRONT] = node;
    deque->ends[DEQUE_BACK] = node;
  }
  return 0;
}

deque_node_t *deque_pop_node(deque_t *deque, deque_end_t end)
{
  deque_node_t *node;
  if(deque->ends[DEQUE_FRONT] == NULL) return NULL;
  if(deque->ends[DEQUE_FRONT] == deque->ends[DEQUE_BACK]) {
    node = deque->ends[DEQUE_FRONT];
    deque->ends[DEQUE_FRONT] = NULL;
    deque->ends[DEQUE_BACK] = NULL;
  } else {
    node = deque->ends[end];
    node->sides[1-end]->sides[end] = NULL;
    deque->ends[end] = node->sides[1-end];
  }
  return node;
}

