/*
 * This library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * (c) Copyright 2006,2007,2008 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2008 Dmitry Gromada <gromada@jarios.org>
 * (c) Copyright 2008 Paul Graphov <graphov@jarios.org>
 *
 * Direct output to console or serial port implementation
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <time.h>
#include <sys/console.h>
#include <dircons.h>
#include <stdio.h>
#include <unistd.h>

typedef struct __console {
//  hw_direct_io_t superio;
  int fd;
  atomic_t refcnt;
} console_t;

static int do_open(const char *devname);

static console_t cons;

int open_console(void)
{
  return do_open("/dev/console");
}

int open_serial(void)
{
  return do_open("/dev/uart0");
}

static int do_open(const char *devname)
{
  int r = 0;

  if (!atomic_get(&cons.refcnt)) {
    r = open(devname, O_WRONLY);
    if(!r) {
      atomic_inc(&cons.refcnt);
      cons.fd = r;
    }
  }
  return r;
}

void close_console(void)
{
  if (atomic_dec_and_test(&cons.refcnt)) {
    close(cons.fd);
  }
}

void console_printf(const char *fmt,... )
{
  char tmp[MAX_CONSOLE_STRLEN];
  cschar_t tbuf[MAX_CONSOLE_STRLEN];
  va_list ap;
  int i, len;

  va_start(ap, fmt);
  len = vsnprintf(tmp, MAX_CONSOLE_STRLEN, fmt, ap);
  va_end(ap);

  if (len < MAX_CONSOLE_STRLEN)
    tmp[len] = '\0';

  for(i=0; i<len; i++) {
    tbuf[i].ch=(wchar_t)((unsigned)tmp[i]);
    tbuf[i].cattr=0x07;
    tbuf[i].xattr=0;
  }

  write(cons.fd, tbuf,len*sizeof(cschar_t));
}

void console_output(const char *msg)
{
  console_printf(msg);
}
