/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * (c) Copyright 2006,2007-2009 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2005,2008,2009 MadTirra <madtirra@jarios.org>
 *
 * Jari OS virtual file system v2,
 * generic file mapper
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/mman.h>
#include <stdint.h>
#include <stdarg.h>
#include <string.h>
#include <stdlib.h>
#include <fcntl.h>
#include <ipc/port.h>
#include <ipc/channel.h>
#include <ipc/iov.h>
#include <ipc/rpc.h>
#include <pthread.h>
#include <sys/v2.h>
#include <ns/ns.h>
#include <errno.h>
#include <unistd.h>
#include <datalib/usrtc.h>

#include <vfs/vfsdef.h>
#include <vfs/vnode.h>
#include <vfs/super.h>
#include <vfs/session.h>

#include "vfsint.h"

#define GENERIC_MAPMSG_SIZE  1024

#include <sys/syscall.h>

void *v2fs_generic_mapper(void *fs_proc)
{
  filesystem_proc_t *fpr=(filesystem_proc_t *)fs_proc;
  void *msg;
  vnode_t *vnode;
  struct mmev_hdr *evhdr;
  struct mmev_fault *fault;
  ipc_port_msg_info_t msgi;
  int r,err;
  void *naddr,*cpbuf;
  size_t nsize;

  if((msg=malloc(GENERIC_MAPMSG_SIZE))==NULL)
    abort();

  /* generic mapper shouldn't be used on pagecache enable fs! */
  for(;;) {
    r=ipc_port_rcvmsg_mi(fpr->super->serve_port,GENERIC_MAPMSG_SIZE,msg,&msgi);
    if(r<0) continue;
    else {
      evhdr=(struct mmev_hdr*)msg;
      vnode=(vnode_t *)evhdr->private;
      if(evhdr->event==MMEV_PAGE_FAULT) {
        fault=(struct mmev_fault*)msg;
        vnode_lock(vnode);
        if(fault->offset*PAGE_SIZE>=vnode->vstat.st_size) {
          err=-EOVERFLOW;
          r=ipc_port_reply_nmsg(fpr->super->serve_port,sizeof(err),&err,msgi.msg_id);
          goto __end_fault;
        }
        nsize=PAGE_SIZE;
        cpbuf=vnode->super->fs_tbl->read(vnode,NULL,&nsize,
                                         fault->offset*PAGE_SIZE);
        if(!cpbuf) {
          err=-EIO;
          r=ipc_port_reply_nmsg(fpr->super->serve_port,sizeof(err),&err,msgi.msg_id);
          goto __end_fault;
        }
        naddr=mk_mmap(NULL,PAGE_SIZE, PROT_READ | PROT_WRITE,
                      MAP_SHARED | MAP_POPULATE,evhdr->memobj_id,
                      fault->offset*PAGE_SIZE);
        if(naddr==MAP_FAILED) {
          err=-ENOMEM;
          r=ipc_port_reply_nmsg(fpr->super->serve_port,sizeof(err),&err,msgi.msg_id);
          goto __end_fault;
        }
        memcpy(naddr,cpbuf,nsize);
        /*        r=ipc_port_reply_nmsg(fpr->super->serve_port,sizeof(uintptr_t),
                  (uintptr_t)naddr,msgi.msg_id);*/
        r=__syscall4(SC_PORT_REPLY,fpr->super->serve_port,
                     msgi.msg_id, (uintptr_t)&naddr,
                     sizeof(uintptr_t));
      __end_fault:
        vnode_unlock(vnode);
      } else {
        err=-ENOSYS;
        r=ipc_port_reply_nmsg(fpr->super->serve_port,sizeof(err),&err,msgi.msg_id);
      }
    }
  }

  return NULL;
}

