/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * Copyright (C) 2006,2007,2008,2009 Jari OS Core Team <http://jarios.org>
 * Copyright (C) 2009 Dmitry Balakshin <balakshin@jarios.org>
 *
 * Jari OS virtual file system v2:
 *     generic UNIX access permissions.
 */

#include <stdint.h>
#include <sys/types.h>
#include <sys/stat.h>

#include <vfs/vfsdef.h>

int generic_permission(vnode_t *vnode, uid_t uid, gid_t gid)
{
  int st_mode = vnode->vstat.st_mode;
  int perm = 0;

  if (vnode->vstat.st_uid == uid) {
    perm |= ((st_mode & S_IRUSR) ? MAY_READ : 0);
    perm |= ((st_mode & S_IWUSR) ? MAY_WRITE : 0);
    perm |= ((st_mode & S_IXUSR) ? MAY_EXEC : 0);
  }

  if (vnode->vstat.st_gid == gid) {
    perm |= ((st_mode & S_IRGRP) ? MAY_READ : 0);
    perm |= ((st_mode & S_IWGRP) ? MAY_WRITE : 0);
    perm |= ((st_mode & S_IXGRP) ? MAY_EXEC : 0);
  }

  perm |= ((st_mode & S_IROTH) ? MAY_READ : 0);
  perm |= ((st_mode & S_IWOTH) ? MAY_WRITE : 0);
  perm |= ((st_mode & S_IXOTH) ? MAY_EXEC : 0);

  return perm;
}

int vnode_permission(vnode_t *vnode, uid_t uid, gid_t gid)
{
  int perm;

  perm = generic_permission(vnode, uid, gid);

  /* Check here for read-only mounted filesystems, etc. */

  return perm;
}
