/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 * 
 * (c) Copyright 2006,2007-2009 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2005,2008,2009 MadTirra <madtirra@jarios.org>
 *
 * Jari OS virtual file system v2,
 * generic stuff
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/mman.h>
#include <stdint.h>
#include <stdarg.h>
#include <string.h>
#include <stdlib.h>
#include <fcntl.h>
#include <ipc/port.h>
#include <ipc/channel.h>
#include <ipc/iov.h>
#include <ipc/rpc.h>
#include <pthread.h>
#include <sys/v2.h>
#include <ns/ns.h>
#include <errno.h>
#include <unistd.h>
#include <datalib/usrtc.h>

#include <vfs/vfsdef.h>
#include <vfs/vnode.h>
#include <vfs/super.h>
#include <vfs/session.h>

#include "vfsint.h"

/* local functions prototypes */
/* read standard parsed path - IPC */
static inline char *__read_ipc_path(prtid_t pprt,uni_rpc_t *rpc,int *pps,int msgid);

/* piece by piece read for writing - IPC */
static inline int __rpc_write(prtid_t pprt,pid_t c_pid,uid_t c_uid,int msgid,
                              gid_t c_gid,uni_rpc_t *rpc,void *wrb);

/* TODO: works with mcache ... */
void *v2fs_thread(filesystem_proc_t *fs_proc)
{
  char *msg,*wrb;
  uni_rpc_t rpc;
  ipc_port_msg_info_t msgi;
  char *path;
  int *pps;
  void *buf;
  iovec_t r_iov[2];
  size_t rrsize;
  pid_t c_pid;
  uid_t c_uid;
  gid_t c_gid;
  int msgid;
  int r,i;
  uint16_t ec;
  struct stat tstat;
  int prot_flags[2];
  int mm_obj;

  if((msg=malloc(PAGE_SIZE*2))==NULL) return NULL;
  if((wrb=malloc(PAGE_SIZE))==NULL) return NULL;

  while(1) {
    r=ipc_port_rcvmsg_mi(fs_proc->listen_port,0,NULL,&msgi);
    if(r<0) {
      sleep(1);
      continue;
    } else if(r<sizeof(rpc)) { /* fucking deal */
      rpc.rpc_call=EINVAL;
      r=ipc_port_reply_nmsg(fs_proc->listen_port,sizeof(rpc),&rpc,msgi.msg_id);
    } else {
      c_pid=msgi.sender_pid;
      msgid=msgi.msg_id;
      c_uid=msgi.sender_uid;
      c_gid=msgi.sender_gid;
      ipc_port_read_msg(fs_proc->listen_port,msgid,0,&rpc,sizeof(rpc));
      /* ok now we're freely to handle calls */
      switch(rpc.rpc_call) {
      case RPC_SYNC: /* TODO: should flush all caches */
	if(c_pid!=v2pid)
	  rpc.rpc_call=EPERM;
	else {
	  /* do some magic */
	  rpc.rpc_call=0;
	}
	r=ipc_port_reply_nmsg(fs_proc->listen_port,sizeof(rpc),&rpc,msgi.msg_id);
	break;
      case RPC_PIDFDCLONE: 
	if(c_pid!=v2pid) {
	  rpc.rpc_call=EPERM;
	} else {
	  rpc.rpc_call=pidsession_clone(rpc.arg0,rpc.arg1);
	}
	r=ipc_port_reply_nmsg(fs_proc->listen_port,sizeof(rpc),&rpc,msgi.msg_id);
	break;
      case RPC_PIDFDRESET: 
	if(c_pid!=v2pid) {
	  rpc.rpc_call=EPERM;
	} else {
	  /* do magic */
	  rpc.rpc_call=pidsession_reset(rpc.arg0);
	}
	r=ipc_port_reply_nmsg(fs_proc->listen_port,sizeof(rpc),&rpc,msgi.msg_id);
	break;
      case RPC_PIDFDKILL:
	if(c_pid!=v2pid) {
	  rpc.rpc_call=EPERM;
	} else {
	  /* do magic */
	  rpc.rpc_call=pidsession_close(rpc.arg0);
	}
	r=ipc_port_reply_nmsg(fs_proc->listen_port,sizeof(rpc),&rpc,msgi.msg_id);
	break;
      case RPC_OPEN:
	if(c_pid!=v2pid) {
	  rpc.rpc_call=EPERM;
	} else {
	  pps=(int *)msg;
          path=__read_ipc_path(fs_proc->listen_port,&rpc,pps,msgid);
	  r=v2_open(fs_proc->super,path,pps,rpc.arg5/*pp*/,rpc.offset/*offset*/,
		    rpc.arg6/*plen*/,rpc.arg2/*pid*/,rpc.arg3/*uid*/,rpc.arg4/*gid*/,
		    rpc.arg1/*mode*/,rpc.arg0/*flags*/,&tstat);
	  if(r<0)
	    rpc.rpc_call=errno;
	  else {
	    /* specific stuff for special filesystems */
	    if((S_ISCHR(tstat.st_mode) || S_ISBLK(tstat.st_mode)) && (tstat.st_rdev!=0)) {
	      rpc.arg2=tstat.st_rdev;
	      rpc.arg3=tstat.st_dev;
	    } else {
	      rpc.arg2=fs_proc->f_pid;
	      rpc.arg3=fs_proc->listen_port;
	    }
	    rpc.rpc_call=0;
	    rpc.arg4=r;
	  }
	}
	r=ipc_port_reply_nmsg(fs_proc->listen_port,sizeof(rpc),&rpc,msgi.msg_id);
	break;
      case RPC_STAT:
	if(c_pid!=v2pid) {
	  rpc.rpc_call=EPERM;
	} else {
	  pps=(int *)msg;
          path=__read_ipc_path(fs_proc->listen_port,&rpc,pps,msgid);
	  r=v2_stat(fs_proc->super,path,pps,rpc.arg5/*pp*/,rpc.arg6,rpc.offset/*offset*/,&tstat);
	  if(r<0) {
	    rpc.rpc_call=errno;
	    goto __stat_err;
	  } else {
	    rpc.rpc_call=0;
	    r_iov[0].iov_base=&rpc;
	    r_iov[0].iov_len=sizeof(rpc);
	    r_iov[1].iov_base=&tstat;
	    r_iov[1].iov_len=sizeof(struct stat);
	    r=ipc_port_reply_nmsg_iov(fs_proc->listen_port,r_iov,2,msgid);
	  }
	}
      __stat_err:
	r=ipc_port_reply_nmsg(fs_proc->listen_port,sizeof(rpc),&rpc,msgi.msg_id);
	break;
      case RPC_READLINK:
	if(c_pid!=v2pid) {
	  rpc.rpc_call=EPERM;
	} else {
	  pps=(int *)msg;
          path=__read_ipc_path(fs_proc->listen_port,&rpc,pps,msgid);
	  r=v2_readlink(fs_proc->super,path,pps,rpc.arg5/*pp*/,rpc.arg6,rpc.offset/*offset*/);
	  if(r<0) {
	    rpc.rpc_call=errno;
	    goto __readlink_err;
	  } else {
	    rpc.rpc_call=0;
	    r_iov[0].iov_base=&rpc;
	    r_iov[0].iov_len=sizeof(rpc);
	    r_iov[1].iov_base=path;
	    r_iov[1].iov_len=strlen(path)+1;
	    r=ipc_port_reply_nmsg_iov(fs_proc->listen_port,r_iov,2,msgid);
	  }
	}
      __readlink_err:
	r=ipc_port_reply_nmsg(fs_proc->listen_port,sizeof(rpc),&rpc,msgi.msg_id);
	break;
      case RPC_MMAP:
        if(r!=(sizeof(rpc)+(sizeof(int)*2))) {
          rpc.rpc_call=EINVAL;
        } else {
          ipc_port_read_msg(fs_proc->listen_port,msgid,sizeof(rpc),prot_flags,2*sizeof(int));
          r=v2_mmap(c_pid,c_uid,c_gid,rpc.arg0,rpc.size,rpc.offset,
                    prot_flags[1],prot_flags[0],&mm_obj);
          if(r!=0) {
            rpc.rpc_call=r;
          } else {
            rpc.rpc_call=0;
            rpc.size=mm_obj;
          }
        }
        r=ipc_port_reply_nmsg(fs_proc->listen_port,sizeof(rpc),&rpc,msgi.msg_id);
        break;
      case RPC_CREATEDIR:
	if(c_pid!=v2pid) {
	  rpc.rpc_call=EPERM;
	} else {
	  pps=(int *)msg;
          path=__read_ipc_path(fs_proc->listen_port,&rpc,pps,msgid);
	  r=v2_mkdir(fs_proc->super,path,pps,rpc.arg5/*pp*/,rpc.arg6/*pathlen*/,
		     rpc.offset/*offset*/,/*uid*/rpc.arg3,/*gid*/rpc.arg4,/*mode*/rpc.arg0);
	  rpc.rpc_call=r;
	}
	r=ipc_port_reply_nmsg(fs_proc->listen_port,sizeof(rpc),&rpc,msgi.msg_id);
	break;
      case RPC_UNLINK:
	if(c_pid!=v2pid) {
	  rpc.rpc_call=EPERM;
	} else {
	  pps=(int *)msg;
	  ipc_port_read_msg(fs_proc->listen_port,msgid,sizeof(rpc),pps,rpc.arg5*sizeof(int));
	  path=msg+rpc.arg5*sizeof(int);
	  ipc_port_read_msg(fs_proc->listen_port,msgid,sizeof(rpc)+rpc.arg5*sizeof(int),path,rpc.arg6);
	  path[rpc.arg6]='\0';
	  r=v2_unlink(fs_proc->super,path,pps,rpc.arg5/*pp*/,rpc.arg6/*pathlen*/,
		      rpc.offset/*offset*/,/*uid*/rpc.arg3,/*gid*/rpc.arg4);
	  rpc.rpc_call=r;
	}
	r=ipc_port_reply_nmsg(fs_proc->listen_port,sizeof(rpc),&rpc,msgi.msg_id);
	break;
      case RPC_UNLINKDIR:
	if(c_pid!=v2pid) {
	  rpc.rpc_call=EPERM;
	} else {
	  pps=(int *)msg;
          path=__read_ipc_path(fs_proc->listen_port,&rpc,pps,msgid);
	  r=v2_unlinkdir(fs_proc->super,path,pps,rpc.arg5/*pp*/,rpc.arg6/*pathlen*/,
		      rpc.offset/*offset*/,/*uid*/rpc.arg3,/*gid*/rpc.arg4);
	  rpc.rpc_call=r;
	}
	r=ipc_port_reply_nmsg(fs_proc->listen_port,sizeof(rpc),&rpc,msgi.msg_id);
	break;
      case RPC_CLOSE:
	if(c_pid!=v2pid) {
	  rpc.rpc_call=EPERM;
	} else {
	  rpc.rpc_call=v2_close(rpc.arg2/*pid*/,rpc.arg1/*unique*/);
	}
	r=ipc_port_reply_nmsg(fs_proc->listen_port,sizeof(rpc),&rpc,msgi.msg_id);
	break;
	/* functions that going directly */
      case RPC_READ:
	buf=v2_read(c_pid,c_uid,c_gid,rpc.arg1/*unique session*/,&rpc.size,rpc.offset,NULL);
	if(!buf) {
	  rpc.rpc_call=errno;
	  r=ipc_port_reply_nmsg(fs_proc->listen_port,sizeof(rpc),&rpc,msgi.msg_id);
	} else {
	  rpc.rpc_call=0;
	  r_iov[0].iov_base=&rpc;
	  r_iov[0].iov_len=sizeof(rpc);
	  r_iov[1].iov_base=buf;
	  r_iov[1].iov_len=rpc.size;
	  r=ipc_port_reply_nmsg_iov(fs_proc->listen_port,r_iov,2,msgid);
	}
	break;
      case RPC_FSTAT:
	r=v2_fstat(c_pid,c_uid,c_gid,rpc.arg1/*unique*/,&tstat);
	if(r!=0) {
	  rpc.rpc_call=r;
	  r=ipc_port_reply_nmsg(fs_proc->listen_port,sizeof(rpc),&rpc,msgid);
	} else {
	  rpc.rpc_call=0;
	  r_iov[0].iov_base=&rpc;
	  r_iov[0].iov_len=sizeof(rpc);
	  r_iov[1].iov_base=&tstat;
	  r_iov[1].iov_len=sizeof(struct stat);
	  r=ipc_port_reply_nmsg_iov(fs_proc->listen_port,r_iov,2,msgid);
	}
	break;
      case RPC_IOCTL: /* *NOTE:* file system handles only rpc sized iotcls, other stuff goes to each device */
	r=v2_ioctl(c_pid,c_uid,c_gid,rpc.arg1/*unique*/,&rpc); /* I don't care about other shitty stuff */
	rpc.rpc_call=r;
	r=ipc_port_reply_nmsg(fs_proc->listen_port,sizeof(rpc),&rpc,msgid);
	break;
      case RPC_FTRUNCATE:
	r=v2_ftruncate(c_pid,c_uid,c_gid,rpc.arg1/*unique*/,rpc.offset);
	rpc.rpc_call=r;
	r=ipc_port_reply_nmsg(fs_proc->listen_port,sizeof(rpc),&rpc,msgid);
	break;
      case RPC_WRITE:
        i=__rpc_write(fs_proc->listen_port,c_pid,c_uid,msgid,c_gid,&rpc,wrb);
        if(i!=0) rpc.rpc_call=EIO;
        r=ipc_port_reply_nmsg(fs_proc->listen_port,sizeof(rpc),&rpc,msgi.msg_id);
	break;
      case RPC_READDIR:
	ec=0; rrsize=PAGE_SIZE;
	r=v2_readdir(c_pid,c_uid,c_gid,rpc.arg0/*unique*/,msg,&rrsize,&rpc.size,rpc.offset,&ec/* spec code */);
	rpc.arg1=ec;
	if(!r) {
	  rpc.rpc_call=0;
	  if(rpc.size==0) {
	    r=ipc_port_reply_nmsg(fs_proc->listen_port,sizeof(rpc),&rpc,msgid);
	  } else {
	    r_iov[0].iov_base=&rpc;
	    r_iov[0].iov_len=sizeof(rpc);
	    r_iov[1].iov_base=msg;
	    r_iov[1].iov_len=rrsize;
	    r=ipc_port_reply_nmsg_iov(fs_proc->listen_port,r_iov,2,msgid);
	  }
	} else {
	  rpc.rpc_call=errno;
	  r=ipc_port_reply_nmsg(fs_proc->listen_port,sizeof(rpc),&rpc,msgid);
	}
	break;
      default:
	rpc.rpc_call=EINVAL;
	r=ipc_port_reply_nmsg(fs_proc->listen_port,sizeof(rpc),&rpc,msgi.msg_id);
	break;
      }
    }
  }

  return NULL;
}

static inline char *__read_ipc_path(prtid_t pprt,uni_rpc_t *rpc,int *pps,int msgid)
{
  char *path;

  ipc_port_read_msg(pprt,msgid,sizeof(*rpc),
                    pps,rpc->arg5*sizeof(int)); /* points first */

  path=(char *)pps+rpc->arg5*sizeof(int);
  ipc_port_read_msg(pprt,msgid, /* path second */
                    sizeof(*rpc)+rpc->arg5*sizeof(int),
                    path,rpc->arg6);

  path[rpc->arg6]='\0';

  return path;
}

/* local functions implementation - keep it bottom, I think it's better */

static inline int __rpc_write(prtid_t pprt,pid_t c_pid,uid_t c_uid,int msgid,
                              gid_t c_gid,uni_rpc_t *rpc,void *wrb)
{
  int r,i;
  size_t pg_a,pg_lb,a_write;
  size_t ac_write=0,ipc_read;
  int err;

  /* do the check */
  r=v2_checkwr(c_pid,c_uid,c_gid,rpc->arg1/*unique session*/,
               rpc->offset);
  if(!r) {
    /* take amount of pages to read */
    pg_a=rpc->size/PAGE_SIZE;
    if((pg_lb=rpc->size%PAGE_SIZE)>0) pg_a++;

    /* well - starts */
    for(i=0;i<pg_a;i++) {
      ipc_read=PAGE_SIZE;
      if(ac_write+ipc_read>rpc->size) ipc_read=pg_lb;
      r=ipc_port_read_msg(pprt,msgid,sizeof(*rpc)+ac_write,wrb,ipc_read);
      if(r<0)         return -1; /* oops, IPC error */
      a_write=ipc_read;
      r=v2_write(c_pid,c_uid,c_gid,rpc->arg1,wrb,(size_t *)&a_write,(off_t)(rpc->offset+ac_write));
      if(r!=0) {
        err=r;
        goto __rpc_write_end;
      } else if(a_write!=ipc_read) {
        err=0;
        rpc->size=ac_write+a_write;
        goto __rpc_write_end;
      }
      ac_write+=ipc_read;
    }
    err=0;
    rpc->size=ac_write;
  } else err=r;

 __rpc_write_end:
  rpc->rpc_call=err;

  return 0; /* Always returns 0, if -1 then it's a critical error */
}

