/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 * 
 * (c) Copyright 2006,2007-2009 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2005,2008,2009 MadTirra <madtirra@jarios.org>
 *
 * Jari OS virtual file system v2,
 * service process related functions 
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <stdint.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <ipc/port.h>
#include <ipc/channel.h>
#include <ipc/iov.h>
#include <ipc/rpc.h>
#include <pthread.h>
#include <sys/v2.h>
#include <ns/ns.h>
#include <errno.h>
#include <unistd.h>
#include <datalib/usrtc.h>

#include <vfs/vfsdef.h>
#include <vfs/vnode.h>
#include <vfs/super.h>
#include <vfs/session.h>

#include "vfsint.h"

#define MAX_MEM_MSG  512

/**
 * @fn fsproc_init waits for mount event and do mount 
 *     with fs specific call
 */
int fsproc_init(filesystem_proc_t *fs_proc)
{
  uni_rpc_t rpc;
  ipc_port_msg_info_t msgi;
  pid_t c_pid;
  int msgid;
  vnode_t vnode;
  int ec=0,r;

  /* initialize sessions */
  r=fs_session_init();
  if(r) return r;

  /* initialize vnode cache */
  r=vnode_cache_initialize();
  if(r) return r;

  /* ok let's do misc job */
  fs_proc->f_pid=getpid();

  /* will waits for mount event */
  while(1) { 
    r=ipc_port_rcvmsg_mi(fs_proc->listen_port,0,NULL,&msgi);
    if(r<0)
      continue;
    else  {
      c_pid=msgi.sender_pid;
      msgid=msgi.msg_id;
      if(r<sizeof(uni_rpc_t)) {
	rpc.rpc_call=EINVAL;
	r=ipc_port_reply_nmsg(fs_proc->listen_port,sizeof(rpc),&rpc,msgid);
      } else if(c_pid!=v2pid) {
	rpc.rpc_call=EPERM;
	r=ipc_port_reply_nmsg(fs_proc->listen_port,sizeof(rpc),&rpc,msgid);
      } else {
	ipc_port_read_msg(fs_proc->listen_port,msgid,0,&rpc,sizeof(rpc));
	switch(rpc.rpc_call) {
	case RPC_MOUNT:
	  if(fs_proc->super->flags & V2FS_SYS) {
	    /* arg0 spec_flags */
	    vnode.super=fs_proc->super;
	    ec=fs_proc->super->fs_tbl->mount(&vnode,NULL,NULL,0,rpc.arg0);
	    rpc.rpc_call=ec;
	    r=ipc_port_reply_nmsg(fs_proc->listen_port,sizeof(rpc),&rpc,msgid);
	    goto __mount;
	  } else { 
	    if((fs_proc->super->path=malloc(rpc.arg3+sizeof(char)))==NULL) {
	      ec=ENOMEM;
	      goto __mount;
	    } else memset(fs_proc->super->path,'\0',rpc.arg3+sizeof(char));
	    if((fs_proc->super->pps=malloc(rpc.arg2*sizeof(int)))==NULL) {
	      ec=ENOMEM;
	      goto __mount;
	    } else memset(fs_proc->super->pps,0,rpc.arg2*sizeof(int));
	    /* read pps */
	    ipc_port_read_msg(fs_proc->listen_port,msgid,sizeof(rpc),
			      fs_proc->super->pps,rpc.arg2*sizeof(int));
	    /* read path */
	    ipc_port_read_msg(fs_proc->listen_port,msgid,sizeof(rpc)+rpc.arg2*sizeof(int),
			      fs_proc->super->path,rpc.arg3);
	    fs_proc->super->pp=rpc.arg2;
	    /* do actual mount */
	    vnode.super=fs_proc->super;
	    ec=fs_proc->super->fs_tbl->mount(&vnode,fs_proc->super->path,
					     fs_proc->super->pps,fs_proc->super->pp,
					     rpc.arg0);
	    rpc.rpc_call=ec;
	    r=ipc_port_reply_nmsg(fs_proc->listen_port,sizeof(rpc),&rpc,msgid);
	    ec=0;
	    goto __mount;
	  }
	  break;
	default:
	  rpc.rpc_call=EINVAL;
	  r=ipc_port_reply_nmsg(fs_proc->listen_port,sizeof(rpc),&rpc,msgid);
	  break;
	}
      }
    }
  }

 __mount:

  /* by default we're cannot be mappable */
  fs_proc->super->is_mappable=0;

  return ec;
}

int fsproc_set_generic_mmap(filesystem_proc_t *fs_proc)
{
  int r;
  pthread_t mthr;

  fs_proc->super->serve_port=ipc_port_open(IPC_BLOCKED_ACCESS,MAX_MEM_MSG);
  if(fs_proc->super->serve_port<0) return -1;

  r=pthread_create(&mthr,NULL,v2fs_generic_mapper,(void *)fs_proc);
  if(r<0) {
    ipc_port_close(fs_proc->super->serve_port);
    return -1;
  }

  fs_proc->super->is_mappable=1;

  return 0;
}
