/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 * 
 * (c) Copyright 2006,2007-2009 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2005,2008,2009 MadTirra <madtirra@jarios.org>
 *
 * Jari OS virtual file system v2,
 * general functions to operate with VFS v2
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <stdint.h>
#include <stdarg.h>
#include <string.h>
#include <ipc/port.h>
#include <ipc/channel.h>
#include <ipc/iov.h>
#include <ipc/rpc.h>
#include <pthread.h>
#include <ns/ns.h>
#include <errno.h>
#include <unistd.h>
#include <datalib/usrtc.h>

#include <vfs/vfsdef.h>

#include "vfsint.h"

/* global io vars */
pid_t v2pid;
chid_t v2channel;

/* local functions prototypes */
static int __init_v2IO(void);

int vfs_register(prtid_t id,int spec_flags,vfs_info_t *fsinf)
{
  int r;
  int ilen=2;
  uni_rpc_t rpc;
  iovec_t iov[4];

  if(!fsinf)    return EINVAL;
  if(!fsinf->fstype)    return EINVAL;

  if((r=__init_v2IO())!=0) /* init vfs io */
    return r;

  /* now let's complete a message */
  rpc.rpc_call=RPC_REGISTER;
  rpc.arg0=id;
  rpc.arg1=spec_flags;
  rpc.arg2=strlen(fsinf->fstype);
  iov[0].iov_base=&rpc;
  iov[0].iov_len=sizeof(rpc);
  iov[1].iov_base=fsinf->fstype;
  iov[1].iov_len=rpc.arg2;
  /* check for author value */
  if(!fsinf->author)
    rpc.arg3=0;
  else {
    rpc.arg3=strlen(fsinf->author);
    iov[ilen].iov_base=fsinf->author;
    iov[ilen].iov_len=rpc.arg3;
    ilen++;
  }
  if(!fsinf->info)
    rpc.arg4=0;
  else {
    rpc.arg4=strlen(fsinf->info);
    iov[ilen].iov_base=fsinf->info;
    iov[ilen].iov_len=rpc.arg4;
    ilen++;
  }

  r=ipc_iov_send(v2channel,iov,ilen,&rpc,sizeof(rpc));
  if(r<0) {
    errno=-r;
    return -1;
  } else {
    if(rpc.rpc_call!=0) {
      errno=rpc.rpc_call;
      return -1;
    }
    r=0;
  }

  return 0;
}

/* some local functions */

static int __init_v2IO(void)
{
  v2pid=ns_getvfspid();
  if(!v2pid)
    return ENOSYS; /* no vfs system */

  v2channel=ipc_channel_open(v2pid,v2port,IPC_BLOCKED_ACCESS);
  if(v2channel<0)
    return -v2channel;

  return 0;
}
