/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * Copyright (C) 2006,2007,2008,2009 Jari OS Core Team <http://jarios.org>
 * Copyright (C) 2009 Dmitry Balakshin <balakshin@jarios.org>
 */

%option noyywrap

%{
#include <string.h>

#include "sconfig.tab.h"

#define YY_EXIT_FAILURE 2

extern int lineno;
void warning(const char *fmt, ...);
void error(int status, const char *fmt, ...);
%}

qstring    \"[^\"\n]*[\"\n]
id         [a-zA-Z0-9_]+

%%
[ \t]+     /* skip whitespaces */ ;
#.*        /* ignore comments */ ;

{qstring}  {
             /* skip open quote */
             yylval.string = strdup(yytext + 1);
             if (yylval.string[yyleng - 2] != '"')
               error(YY_EXIT_FAILURE, "unterminated character string");
             else /* remove close quote */
               yylval.string[yyleng - 2] = '\0';
             return T_QSTRING;
           }

menu       { return T_MENU; }
bool       { return T_BOOL; }
string     { return T_STRING; }
desc       { return T_DESC; }
switch     { return T_SWITCH; }
case       { return T_CASE; }

{id}       {
             yylval.string = strdup(yytext);
             return T_ID;
           }

\n         { lineno++; }
.          { return yytext[0]; }

%%
