/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * (c) Copyright 2006,2007,2008 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2008 MadTirra <madtirra@jarios.org>
 *
 * hwavx: (hardware and virtual devices server)
 * src/hwdirio.c: directory operations
 *
 */

#include <sys/types.h>
#include <sys/mman.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <ipc/port.h>
#include <errno.h>
#include <time.h>
#include <datalib/usrtc.h>
#include <ipc/rpc.h>
#include <ns/ns.h>

#include "namespace.h"

static int __hash_compare0(const void *a,const void *b);

iodir_t *get_root_dir(usrtc_t *dirs)
{
  iodir_t *dir;
  usrtc_node_t *node;
  ulong_t hash=ionode_hash(NSSEP);

  node=usrtc_lookup(dirs,&hash);
  if(!node) {
    return NULL;
  }
  dir=(iodir_t*)usrtc_node_getdata(node);
  if(!dir) {
    return NULL;
  }
  else
    return dir;
}

iodir_t *nshw_dirfind(usrtc_t *dirs,const char *dir)
{
  iodir_t *iodir=NULL;
  usrtc_node_t *node;
  ulong_t hash=ionode_hash(dir);

  if(!dirs || !dir)
    return NULL;

  node=usrtc_lookup(dirs,&hash);
  if(!node)
    return NULL;
  iodir=(iodir_t *)usrtc_node_getdata(node);

  return iodir;
}

int dir_root_create(usrtc_t *nodes,usrtc_t *dirs)
{
  usrtc_node_t *node;
  ionode_t *ionode;
  char *rootd_name;
  iodir_t *iodir;
  usrtc_t *ionodes;

  node=malloc(sizeof(*node));
  ionode=malloc(sizeof(*ionode));
  if(!node || !ionode)
    return -1;

  /* fill up ionode */
  rootd_name=malloc(strlen(NSSEP)+1);
  if(!rootd_name)
    return -1;
  else
    memset(rootd_name,'\0',strlen(NSSEP)+1);

  rootd_name=strcpy(rootd_name,NSSEP);

  ionode->symbolic_name=rootd_name;
  ionode->hash=ionode_hash(rootd_name);
  ionode->ioops=NULL;

  usrtc_node_init(node,ionode);
  usrtc_insert(nodes,node,&ionode->hash);

  iodir=malloc(sizeof(*iodir));
  ionodes=malloc(sizeof(*ionodes));
  if(!iodir || !ionodes)
    return -1;

  /* init iodir */
  iodir->node=ionode;
  usrtc_init(ionodes,USRTC_SPLAY,USRTC_COUNT_T_MAX,__hash_compare0);
  iodir->ionodes=ionodes;

  node=NULL;
  node=malloc(sizeof(*node));
  if(!node)
    return -1;

  usrtc_node_init(node,iodir);
  usrtc_insert(dirs,node,&iodir->node->hash);

  return 0;
}

int create_hwdir(usrtc_t *nodes,usrtc_t *dirs,const char *hwdir_name)
{
  iodir_t *iorootdir=NULL;
  usrtc_node_t *node=NULL;
  usrtc_t *ionodes;
  ionode_t *ionode;
  iodir_t *iodir;
  char *symb_name;
  ulong_t hash;
  int len,i;
  /* we're need to find root of the requested dir */

  iorootdir=nshw_find_rootdir(hwdir_name);
  if(!iorootdir)
    return ENOENT; /* failed, not found */

  /* ok, let's check if it's exist yet */
  hash=ionode_hash(hwdir_name);
  node=usrtc_lookup(dirs,&hash);
  if(node!=NULL)
    return EEXIST;
  node=usrtc_lookup(nodes,&hash);
  if(node!=NULL)
    return EEXIST;

  /* now we're need to add this */
  node=malloc(sizeof(*node)); /* adding to the global ionode tree */
  ionode=malloc(sizeof(*ionode));
  if(!node || !ionode)
    return ENOMEM;
  /* parse symbolic name */
  len=strlen(hwdir_name);
  for(i=0;i<len;i++) {
    if(hwdir_name[i]==CCNSSEP) {
      symb_name=(char*)hwdir_name+sizeof(char)*(i+1);
    }
  }
  ionode->symbolic_name=malloc(strlen(symb_name)+1);
  if(!ionode->symbolic_name)
    return ENOMEM;
  else
    strcpy(ionode->symbolic_name,symb_name);

  ionode->hash=ionode_hash(hwdir_name);
  ionode->ioops=NULL;

  usrtc_node_init(node,ionode);
  usrtc_insert(nodes,node,&ionode->hash);

  node=NULL;
  node=malloc(sizeof(*node)); /* adding to root iodir tree */
  if(!node)
    return ENOMEM;

  usrtc_node_init(node,ionode); 
  usrtc_insert(iorootdir->ionodes,node,&ionode->hash);

  /* now let's init iodir */
  iodir=malloc(sizeof(*iodir));
  ionodes=malloc(sizeof(*ionodes));
  if(!iodir || !ionodes)
    return ENOMEM;

  iodir->node=ionode;
  usrtc_init(ionodes,USRTC_SPLAY,USRTC_COUNT_T_MAX,__hash_compare0);
  iodir->ionodes=ionodes;

  node=NULL; /* let's add to the iodirs */
  node=malloc(sizeof(*node));
  if(!node)
    return ENOMEM;

  usrtc_node_init(node,iodir);
  usrtc_insert(dirs,node,&iodir->node->hash);

  return 0;
}

int iodir_create(const char *dir_name)
{
  usrtc_t *nodes=nshw_ionodes();
  usrtc_t *dirs=nshw_iodirs();

  return create_hwdir(nodes,dirs,dir_name);
}

static int __hash_compare0(const void *a,const void *b)
{
  ulong_t *h1=(ulong_t*)a;
  ulong_t *h2=(ulong_t*)b;
  ulong_t hh1=*h1,hh2=*h2;

  return hh1-hh2;
}
