/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * (c) Copyright 2006,2007,2008 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2008 MadTirra <madtirra@jarios.org>
 * (c) Copyright 2008 Dmitry Gromada <gromada@jarios.org>
 *
 * hwavx: (hardware and virtual devices server)
 * src/namespace.c: high level functions for operating with name space 
 *
 */

#ifndef __NAMESPACE_H__
#define __NAMESPACE_H__

#include <sys/types.h>
#include <datalib/usrtc.h>
#include <ipc/channel.h>
#include <devio/devio.h>

#define NSSEP    "/"
#define CCNSSEP  '/'

typedef struct __driverio_t {
  pid_t pid;
  ulong_t cmd_id;
  ulong_t input_id;
  ulong_t output_id;
  chid_t service;
  chid_t input;
  chid_t output;
  char *name;
} driverio_t;

typedef struct __hw_resio_t {
  int nirq;
  int niomem;
  int ntio_ports;
  int *irqs;
  int *io_ports;
  iomem_area_t *iomem;
} hw_resio_t;

typedef struct __ionode_t {
  char *symbolic_name;
  ulong_t hash;
  uint16_t cap;
  devid_t devno;
  driverio_t *ioops;
  hw_resio_t *hwres;
} ionode_t;

typedef struct __iodir_t {
  ionode_t *node;
  usrtc_t *ionodes; 
} iodir_t;

typedef struct __hwfd_t {
  int fd;
  ionode_t *node;
  size_t rd_set;
  size_t wr_set;
  uint8_t flags;
} hwfd_t;

typedef struct __hwdirfd_t {
  int fd;
  iodir_t *iodir;
  usrtc_node_t *current_point;
  hw_dirent_t *dentry;
} hwdirfd_t;

typedef struct __pid_session_t {
  pid_t pid;
  int lfd;
  int ldfd;
  usrtc_t *hw_fds;
  usrtc_t *hw_dirfds;
} pid_session_t;

/* namespace ops */
usrtc_t *nshw_ionodes(void);
usrtc_t *nshw_iodirs(void);
usrtc_t *nshw_pidsessions(void);
int init_namespace(void);
iodir_t *nshw_find_rootdir(const char *hwdir_name);
int nshw_open_node(pid_t pid,const char *devn,int *fd,int flags);
int nshw_close_node(pid_t pid,int fd);
int nshw_open_dir(pid_t pid,const char *dirn,int *fd);
int nshw_close_dir(pid_t pid,int fd);
size_t nshw_read(pid_t pid,int fd,size_t size);
size_t nshw_write(pid_t pid,int fd,size_t size);
int nshw_seek(pid_t pid,int fd,size_t size);
hw_dirent_t *nshw_readdir(pid_t pid,int fd,int *res);

/* directory ops */
iodir_t *nshw_dirfind(usrtc_t *dirs,const char *dir);
int dir_root_create(usrtc_t *nodes,usrtc_t *dirs);
iodir_t *get_root_dir(usrtc_t *dirs);
int create_hwdir(usrtc_t *nodes,usrtc_t *dirs,const char *hwdir_name);
int iodir_create(const char *dir_name);
/* node ops */
int initial_devices_set(usrtc_t *nodes,usrtc_t *dirs);
ulong_t ionode_hash(const char *name);
int ionode_add(iodir_t *dir,usrtc_t *nodes,
	       char *rawname,char *symbolic_name);
int ionode_rm(iodir_t *dir, usrtc_t *nodes, char *rawname);
int ionode_create(const char *msg);
int ionode_delete(const char *msg);
int ionode_register_driverio(usrtc_t *nodes,char *rawname,char *drvname,pid_t pid,
			     ulong_t cmd_id,ulong_t input_id,ulong_t output_id,
			     int cap,devid_t devno,chid_t service,chid_t input,
			     chid_t output);
int ionode_unregister_driverio(usrtc_t *nodes, char *rawname);
ionode_t *ionode_nodefind(ulong_t hash,int *err);

#endif /* __NAMESPACE_H__ */
