/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * (c) Copyright 2006,2007,2008 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2008 MadTirra <madtirra@jarios.org>
 *
 * hwavx: (hardware and virtual devices server)
 * src/server.c: main entry and IPC internals
 *
 */

#include <stdbool.h>
#include <sys/types.h>
#include <sys/mman.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <ipc/port.h>
#include <ipc/channel.h>
#include <ipc/rpc.h>
#include <errno.h>
#include <time.h>
#include <datalib/usrtc.h>
#include <ns/ns.h>
#include <devio/devio.h>
#include <devio/server.h>

#include "namespace.h"

static char *info="Jari OS: Hardware and virtual devices server.\nVersion 0.0.1a\n";

#define CMDINLEN   1024
#define CMDOUTLEN  1024

static void register_hwavx(void);
static void process_rpc(uni_rpc_t rpc,char *msg,
			prtid_t port);

static char *__income;

int main(int argc,char **argv)
{
  int r;
  prtid_t do_p;
  char *cmd_in;
  uni_rpc_t rpc;

  register_hwavx();

  /* init IPC internals */
  do_p=ipc_port_open(IPC_BLOCKED_ACCESS,512);
  if(do_p<0)
    abort();

  cmd_in=malloc(CMDINLEN);
  if(!cmd_in) {
    abort();
  } else {
    memset(cmd_in,'\0',CMDINLEN);
  }

  if(init_namespace()!=0) /* init namespace */
    abort();

  __income=cmd_in;

  while(true) {
    r=ipc_port_rcvmsg(do_p,CMDINLEN,__income);
    if(r < 0) {
      /* fuck, TODO: do something while error */
    } else {
      memcpy(&rpc,__income,sizeof(rpc));
      process_rpc(rpc,__income+sizeof(rpc),do_p);
    }

  }
  /* never reached */

  return 0;
}

/* generic reply rpc functions */
static void reply_alive(prtid_t port);
static void reply_info(prtid_t port);
static void reply_error(int err,prtid_t port);
/* specific implementation */
static void reply_open(uni_rpc_t rpc,char *msg,prtid_t port);
static void reply_close(uni_rpc_t rpc,char *msg,prtid_t port);
static void reply_read(uni_rpc_t rpc,char *msg,prtid_t port);
static void reply_write(uni_rpc_t rpc,char *msg,prtid_t port);
static void reply_exec(uni_rpc_t rpc,char *msg,prtid_t port);
static void reply_ioctl(uni_rpc_t rpc,char *msg,prtid_t port);
static void reply_register(uni_rpc_t rpc,char *msg,prtid_t port);
static void reply_unregister(uni_rpc_t rpc,char *msg,prtid_t port);
static void reply_create_node(uni_rpc_t rpc,char *msg,prtid_t port);
static void reply_unlink_node(uni_rpc_t rpc,char *msg,prtid_t port);
static void reply_create_dir(uni_rpc_t rpc,char *msg,prtid_t port);
static void reply_opendir(uni_rpc_t rpc,char *msg,prtid_t port,pid_t caller);
static void reply_closedir(uni_rpc_t rpc,prtid_t port,pid_t caller);
static void reply_readdir(uni_rpc_t rpc,prtid_t port,pid_t caller);
static void reply_directio_open(uni_rpc_t rpc,char *msg,prtid_t port);
static void reply_regevents(uni_rpc_t rpc,char *msg,prtid_t port);
static void reply_unregevents(uni_rpc_t rpc,char *msg,prtid_t port);

static void process_rpc(uni_rpc_t rpc,char *msg,prtid_t port)
{
  pid_t caller;
  ipc_port_msg_info_t minfo=ipc_port_getinfo(port);

  caller=minfo.sender_pid;

  switch(rpc.rpc_call) {
  case RPC_ALIVE:
    reply_alive(port);
    break;
  case RPC_INFO:
    reply_info(port);
    break;
  case RPC_OPEN:
    reply_open(rpc,msg,port);
    break;
  case RPC_CLOSE:
    reply_close(rpc,msg,port);
    break;
  case RPC_READ:
    reply_read(rpc,msg,port);
    break;
  case RPC_WRITE:
    reply_write(rpc,msg,port);
    break;
  case RPC_EXEC:
    reply_exec(rpc,msg,port);
    break;
  case RPC_IOCTL:
    reply_ioctl(rpc,msg,port);
    break;
  case RPC_REGISTER:
    reply_register(rpc,msg,port);
    break;
  case RPC_UNREGISTER:
    reply_unregister(rpc,msg,port);
    break;
  case RPC_CREATE:
    reply_create_node(rpc,msg,port);
    break;
  case RPC_UNLINK:
    reply_unlink_node(rpc,msg,port);
    break;
  case RPC_CREATEDIR:
    reply_create_dir(rpc,msg,port);
    break;
  case RPC_OPENDIR:
    reply_opendir(rpc,msg,port,caller);
    break;
  case RPC_READDIR:
    reply_readdir(rpc,port,caller);
    break;
  case RPC_CLOSEDIR:
    reply_closedir(rpc,port,caller);
    break;
  case RPC_OPENDIO:
    reply_directio_open(rpc,msg,port);
    break;
  case RPC_CLOSEDIO:
    reply_error(ENOSYS,port);
    break;
  case RPC_REGEVENTS:
    reply_regevents(rpc,msg,port);
    break;
  case RPC_UNREGEVENTS:
    reply_unregevents(rpc,msg,port);
    break;
  case RPC_SETCAP:
    reply_error(ENOSYS,port);
    break;
  case RPC_READDIO:
    reply_error(ENOSYS,port);
    break;
  case RPC_WRITEDIO:
    reply_error(ENOSYS,port);
    break;
  default:
    reply_error(EINVAL,port);
    break;
  }

  return;
}

static void __repl_msg(prtid_t port,int err,char *msg,int len)
{
  uni_rpc_t rpc;
  int r;

  rpc.rpc_call=err;
  memcpy(__income,&rpc,sizeof(rpc));
  if(msg!=NULL) {
    memcpy((char *)__income+sizeof(rpc),msg,len);
  }

  r=ipc_port_reply(port,sizeof(rpc)+len,__income);

  return;
}

static void __repl_rpcmsg(prtid_t port,uni_rpc_t rpc,char *msg,int len)
{
  int r;

  if(msg!=NULL) {
    memset(__income,'\0',sizeof(rpc)+len+1);
    memcpy((char *)__income+sizeof(rpc),msg,len);
  }
  memcpy(__income,&rpc,sizeof(rpc));

  r=ipc_port_reply(port,sizeof(rpc)+len,__income);

  return;
}

static void reply_alive(prtid_t port)
{
  reply_error(0,port);
  return;
}

static void reply_info(prtid_t port)
{
  __repl_msg(port,0,info,strlen(info)+1);
  return;
}

static void reply_error(int err,prtid_t port)
{
  uni_rpc_t rpc;
  int r;

  rpc.rpc_call=err;
  r=ipc_port_reply(port,sizeof(rpc),&rpc);

  return;
}

/* specific implementation */
static void reply_open(uni_rpc_t rpc,char *msg,prtid_t port)
{
  usrtc_node_t *node;
  ionode_t *ionode;
  driverio_t *driverio;
  ulong_t hash=ionode_hash(msg);
  uni_rpc_t drpc;
  int r;

  node=usrtc_lookup(nshw_ionodes(),&hash);
  if(!node) {
  __nosuchdev:
    reply_error(ENODEV,port);
    return;
  }
  ionode=usrtc_node_getdata(node);
  if(!ionode)
    goto __nosuchdev;
  driverio=ionode->ioops;
  if(!driverio)
    goto __nosuchdev;

  if(r<0)
    reply_error(r,port);
  if(rpc.rpc_call!=0)
    reply_error(rpc.rpc_call,port);

  drpc.rpc_call=0; /*it's ok */
  drpc.arg0=getpid();
  drpc.arg1=port;
  drpc.arg2=port;
  drpc.arg3=port;
  __repl_rpcmsg(port,drpc,NULL,0);

  return;
}

static void reply_directio_open(uni_rpc_t rpc,char *msg,prtid_t port)
{
  usrtc_node_t *node;
  ionode_t *ionode;
  driverio_t *driverio;
  ulong_t hash=ionode_hash(msg);
  uni_rpc_t drpc;
  ipc_port_msg_info_t minfo;
  int r;

  node=usrtc_lookup(nshw_ionodes(),&hash);
  if(!node) {
  __nosuchdev:
    reply_error(ENODEV,port);
    return;
  }
  ionode=usrtc_node_getdata(node);
  if(!ionode)
    goto __nosuchdev;
  driverio=ionode->ioops;
  if(!driverio)
    goto __nosuchdev;

  minfo=ipc_port_getinfo(port);
  rpc.arg0=minfo.sender_pid;
  rpc.arg2=ionode->devno;

  r=ipc_channel_msgsnd(driverio->service,sizeof(rpc),&rpc,sizeof(rpc),&rpc);
  if(r<0) {
    reply_error(r,port);
    return;
  }
  if(rpc.rpc_call!=0) {
    reply_error(rpc.rpc_call,port);
  } else {
    drpc.rpc_call=0;
    drpc.arg0=driverio->pid;
    drpc.arg1=driverio->cmd_id;
    drpc.arg2=driverio->input_id;
    drpc.arg3=driverio->output_id;
    drpc.arg4=ionode->devno;
    __repl_rpcmsg(port,drpc,NULL,0);
  }

  return;
}

static void reply_regevents(uni_rpc_t rpc,char *msg,prtid_t port)
{
  ionode_t *ionode;
  usrtc_node_t *node;
  driverio_t *driverio;
  ulong_t hash=ionode_hash(msg);
  ipc_port_msg_info_t minfo;
  int r;

  node=usrtc_lookup(nshw_ionodes(),&hash);
  if(!node) {
  __nosuchdev:
    reply_error(ENODEV,port);
    return;
  }

  ionode=usrtc_node_getdata(node);
  if(!ionode)
    goto __nosuchdev;
  driverio=ionode->ioops;
  if(!driverio)
    goto __nosuchdev;

  if(ionode->cap & EVENTS_POLL) {
    minfo=ipc_port_getinfo(port);
    rpc.arg0=minfo.sender_pid;
    r=ipc_channel_msgsnd(driverio->service,sizeof(rpc),&rpc,sizeof(rpc),&rpc);
    if(r<0) {
      reply_error(-r,port);
      return;
    }
    reply_error(rpc.rpc_call,port);
  } else {
    reply_error(ENOSYS,port);
    return;
  }

  return;
}

static void reply_unregevents(uni_rpc_t rpc,char *msg,prtid_t port)
{
  ionode_t *ionode;
  usrtc_node_t *node;
  driverio_t *driverio;
  ulong_t hash=ionode_hash(msg);
  ipc_port_msg_info_t minfo;
  int r;

  node=usrtc_lookup(nshw_ionodes(),&hash);
  if(!node) {
  __nosuchdev:
    reply_error(ENODEV,port);
    return;
  }
  ionode=usrtc_node_getdata(node);
  if(!ionode)
    goto __nosuchdev;
  driverio=ionode->ioops;
  if(!driverio)
    goto __nosuchdev;

  if(ionode->cap & EVENTS_POLL) {
    minfo=ipc_port_getinfo(port);
    rpc.arg0=minfo.sender_pid;
    r=ipc_channel_msgsnd(driverio->service,sizeof(rpc),&rpc,sizeof(rpc),&rpc);
    if(r<0) {
      reply_error(-r,port);
      return;
    }
    reply_error(rpc.rpc_call,port);
  } else {
    reply_error(ENOSYS,port);
    return;
  }

  return;
}

static void reply_close(uni_rpc_t rpc,char *msg,prtid_t port)
{
  reply_error(ENOSYS,port);
  return;
}

static void reply_read(uni_rpc_t rpc,char *msg,prtid_t port)
{
  reply_error(ENOSYS,port);
  return;
}

static void reply_write(uni_rpc_t rpc,char *msg,prtid_t port)
{
  reply_error(ENOSYS,port);
  return;
}

static void reply_exec(uni_rpc_t rpc,char *msg,prtid_t port)
{
  reply_error(ENOSYS,port);
  return;
}

/*
 * ioctl throught devio mean that you works with device _node_
 * not with device driver service.
 * Actually it's works like the following:
 * HWGETINFO:
 * nothing
 * HWSETINFO:
 * rpc.arg1 -> irq data offset
 * rpc.arg2 -> num irqs
 * rpc.arg3 -> ioport data offset
 * rpc.arg4 -> num ioports
 * rpc.arg5 -> iomem areas data offset
 * rpc.arg6 -> num datas 
 */
static void reply_ioctl(uni_rpc_t rpc,char *msg,prtid_t port)
{
  ulong_t hash=ionode_hash(msg);
  ionode_t *ionode=NULL;
  char *rmsg=msg;
  int err;

  switch(rpc.arg0) {
  case RPC_GETHWINFO:
    ionode=ionode_nodefind(hash,&err);
    if(ionode!=NULL) {
      if(ionode->hwres==NULL) {
	err=ENOINFO;
	goto __ioctl_failed;
      }
      rpc.arg1=sizeof(rpc);
      rpc.arg2=ionode->hwres->nirq;
      rpc.arg3=rpc.arg1+sizeof(int)*rpc.arg2;
      rpc.arg4=ionode->hwres->ntio_ports;
      rpc.arg5=rpc.arg3+sizeof(int)*rpc.arg4;
      rpc.arg6=ionode->hwres->niomem;
      /* complete data*/
      if(rpc.arg2!=0) {
	memcpy(rmsg,ionode->hwres->irqs,sizeof(int)*rpc.arg2);
	rmsg+=sizeof(int)*rpc.arg2;
      }
      if(rpc.arg4!=0) {
	memcpy(rmsg,ionode->hwres->io_ports,sizeof(int)*rpc.arg4);
	rmsg+=sizeof(int)*rpc.arg4;
      }
      if(rpc.arg6!=0) {
	memcpy(rmsg,ionode->hwres->iomem,sizeof(*ionode->hwres->iomem)*rpc.arg6);
	rmsg+=sizeof(*ionode->hwres->iomem)*rpc.arg6;
      }

      __repl_rpcmsg(port,rpc,msg,rmsg-msg);
    } else {
    __ioctl_failed:
      reply_error(err,port);
      return;
    }
    break;
  case RPC_SETHWINFO:
    ionode=ionode_nodefind(hash,&err);
    if(ionode) {
      err=0;
      if(ionode->hwres!=NULL) {
	/* TODO: fix up and clean */
      } else {
	ionode->hwres=malloc(sizeof(*ionode->hwres));
      }
      if(rpc.arg2!=0) {
	rmsg+=rpc.arg1-sizeof(rpc);
	ionode->hwres->irqs=malloc(sizeof(int)*rpc.arg2);
	if(!ionode->hwres->irqs)
	  err=ENOMEM;
	else
	  memcpy(ionode->hwres->irqs,rmsg,sizeof(int)*rpc.arg2);
      }
      if(rpc.arg4!=0) {
	rmsg=msg;
	rmsg+=rpc.arg3-sizeof(rpc);
	ionode->hwres->io_ports=malloc(sizeof(int)*rpc.arg4);
	if(!ionode->hwres->io_ports)
	  err=ENOMEM;
	else
	  memcpy(ionode->hwres->io_ports,rmsg,sizeof(int)*rpc.arg4);
      }
      if(rpc.arg6!=0) {
	rmsg=msg;
	rmsg+=rpc.arg5-sizeof(rpc);
	ionode->hwres->iomem=malloc(sizeof(*ionode->hwres->iomem)*rpc.arg6);
	if(!ionode->hwres->iomem)
	  err=ENOMEM;
	else
	  memcpy(ionode->hwres->iomem,rmsg,sizeof(*ionode->hwres->iomem)*rpc.arg6);
      }

      reply_error(err,port);
    } else 
      goto __ioctl_failed;
    break;
  default:
    reply_error(ENOSYS,port);
    break;
  }

  return;
}

static void reply_register(uni_rpc_t rpc,char *msg,prtid_t port)
{
  int r;
  pid_t pid=rpc.arg0;
  ulong_t cmd_id=rpc.arg1,
    input_id=rpc.arg2,
    output_id=rpc.arg3;
  int cap=rpc.arg4;
  char *rmsg=msg;
  devid_t devno=rpc.arg5;
  char *drvname;
  chid_t service,input,output;

  drvname=rmsg;

  service=ipc_channel_open(pid,cmd_id,IPC_BLOCKED_ACCESS);
  input=ipc_channel_open(pid,input_id,IPC_BLOCKED_ACCESS);
  if(input_id!=output_id)
    output=ipc_channel_open(pid,output_id,IPC_BLOCKED_ACCESS);
  else
    output=input;

  r=ionode_register_driverio(nshw_ionodes(),msg,drvname,
			     pid,cmd_id,input_id,output_id,cap,
			     devno,service,input,output);

  if(r!=0) {
    ipc_channel_close(service);
    ipc_channel_close(input);
    if(input!=output)
      ipc_channel_close(output);
  }

  reply_error(r,port);

  return;
}

static void reply_unregister(uni_rpc_t rpc,char *msg,prtid_t port)
{
  int r;

  r = ionode_unregister_driverio(nshw_ionodes(),msg);
  reply_error(r,port);

  return;
}

static void reply_opendir(uni_rpc_t rpc,char *msg,
			  prtid_t port,pid_t caller)
{
  int r;
  int fd;
  uni_rpc_t drpc;

  r=nshw_open_dir(caller,msg,&fd);

  drpc.rpc_call=r;
  drpc.arg0=fd;

  __repl_rpcmsg(port,drpc,NULL,0);

  return;
}

static void reply_readdir(uni_rpc_t rpc,prtid_t port,pid_t caller)
{
  int fd=rpc.arg0;
  int r,len;
  hw_dirent_t *dentry;
  uni_rpc_t drpc;

  dentry=nshw_readdir(caller,fd,&r);

  drpc.rpc_call=r;

  if(dentry!=NULL) {
    len=sizeof(*dentry);
  }

  __repl_rpcmsg(port,drpc,(char *)dentry,len);

  return;
}

static void reply_closedir(uni_rpc_t rpc,prtid_t port,pid_t caller)
{
  int r;
  int fd=rpc.arg0;

  r=nshw_close_dir(caller,fd);

  reply_error(r,port);

  return;
}

static void reply_create_node(uni_rpc_t rpc,char *msg,prtid_t port)
{
  int r;

  r=ionode_create(msg);

  reply_error(r,port);

  return;
}

static void reply_unlink_node(uni_rpc_t rpc,char *msg,prtid_t port)
{
  int r;

  r = ionode_delete(msg);
  reply_error(r,port);

  return;
}

static void reply_create_dir(uni_rpc_t rpc,char *msg,prtid_t port)
{
  int r;

  r=iodir_create(msg);

  reply_error(r,port);

  return;
}

static void register_hwavx(void)
{
  char name[]="hwavx";

  /* make nameserver acknowneledge */
  ns_register(getpid(),0,0,(V2T_TRUSTED|V2T_MKSERVICE),SYS_DEVIO,name,NULL,0);

  return;
}
