/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 * 
 * (c) Copyright 2006,2007,2008,2009 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2005,2008,2009 MadTirra <madtirra@jarios.org>
 *
 * Nameserver V2
 *
 */

#ifndef __V2NSX_H__
#define __V2NSX_H__

#include <sys/types.h>
#include <datalib/usrtc.h>

#ifdef CONFIG_NSDEBUG
#define NSDEBUG  1
#endif

#ifdef CONFIG_NSENVFS
typedef struct __envfs_node_t {
  char *name;
  struct __envfs_node_t *parent;
  int type;
  size_t size;
} envfs_node_t;
#endif

typedef struct __v2task_t {
  pid_t pid; /* pid */
  uid_t uid; /* user id */
  gid_t gid; /* group id */
  int type; /* type of a task */
  uint8_t subsystem; /* subsystem id , 0 if it's a regular task */
  /* ports information */
  usrtc_t *ports;
  /* path */
  char *path;
  /* environ variables i.e. NAME -> VARIABLE VALUE , ... */
  char **environ;
  int env_amount;
#ifdef CONFIG_NSENVFS
  time_t c_time; /* changed time */
  envfs_node_t *fsnod; /* <-- points to directory */
#endif
  /* lock */
  pthread_mutex_t lock;
  /* internal structure */
  usrtc_node_t *node;
} v2task_t;

#define v2task_lock(n)  pthread_mutex_lock(&n->lock)
#define v2task_unlock(n)  pthread_mutex_unlock(&n->lock)

/*
 * NOTE: nameservice don't care about port -> name validation
 * if you will add named port with incorrect id - you are 
 * idiot - servers and system will not affected, but your client
 * will.
 */
typedef struct __v2regport_t {
  prtid_t id;
  char *name;
  usrtc_node_t *node;
#ifdef CONFIG_NSENVFS
  envfs_node_t *fsnod; /* <-- points to file */
#endif
  v2task_t *parent;
} v2regport_t;

/* special trees */
extern usrtc_t *t_list;    /* general task list */
extern usrtc_t *sys_list;  /* subsystems lists */

#ifdef CONFIG_NSENVFS
extern time_t glc_time; /* changed time of root directory */
extern envfs_node_t *fsroot; /* <-- points to directory */
#endif


#endif /* __V2NSX_H__ */
