/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 * 
 * (c) Copyright 2006,2007,2008 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2009 Graphov Paul <graphov@jarios.org>
 * 
 * Russian ytzuken (winkeys) layout for at/xt keyboard.
 *
 */

#include <sys/types.h>
#include <ctype.h>
#include <sys/keyboard.h>

#include "keyb.h"
#include "ru_layout.h"

char ru_codes[128][2] = {
      {0, 0},    {27, 0},   {'1', 0},   {'2', 0},   {'3', 0},   {'4', 0},   {'5', 0},   {'6', 0},
    {'7', 0},   {'8', 0},   {'9', 0},   {'0', 0},   {'-', 0},   {'=', 0},  {'\b', 0},  {'\t', 0},

  {208, 185}, {209, 134}, {209, 131}, {208, 186}, {208, 181}, {208, 189}, {208, 179}, {209, 136},
  {209, 137}, {208, 183}, {209, 133}, {209, 138},    {10, 0},     {0, 0}, {209, 132}, {209, 139},

  {208, 178}, {208, 176}, {208, 191}, {209, 128}, {208, 190}, {208, 187}, {208, 180}, {208, 182},
  {209, 141}, {209, 145},     {0, 0},  {'\\', 0}, {209, 143}, {209, 135}, {209, 129}, {208, 188},

  {208, 184}, {209, 130}, {209, 140}, {208, 177}, {209, 142},    {46, 0},     {0, 0},     {0, 0},
      {0, 0},   {' ', 0}};


char ru_s_codes[128][2] = {
     {0, 0},    {27, 0},    {33, 0},    {34, 0}, {226, 132},  {150, 59},    {37, 0},    {58, 0},
    {63, 0},    {42, 0},    {40, 0},    {41, 0},   {'_', 0},   {'+', 0},  {'\b', 0},  {'\t', 0},
      
 {208, 153}, {208, 166}, {208, 163}, {208, 154}, {208, 149}, {208, 157}, {208, 147}, {208, 168},
 {208, 169}, {208, 151}, {208, 165}, {208, 170},    {10, 0},     {0, 0}, {208, 164}, {208, 171},

 {208, 146}, {208, 144}, {208, 159}, {208, 160}, {208, 158}, {208, 155}, {208, 148}, {208, 150},
 {208, 173}, {208, 129},     {0, 0},   {'|', 0}, {208, 175}, {208, 167}, {208, 161}, {208, 156},

 {208, 152}, {208, 162}, {208, 172}, {208, 145}, {208, 174}, { 44,   0},     {0, 0},     {0, 0},
     {0, 0},   {' ', 0}};


int ru_scan_to_ascii(char *buf, uint8_t scancode, uint8_t comb_flags, uint8_t state_flags) {

  int n = 0;
  char (*tbl)[2];
  
  tbl = (comb_flags & SHIFT_MASK) ? ru_s_codes : ru_codes;
  
  if(state_flags & CAPS_MASK) {
    if((scancode >= KC_Q && scancode <= KC_RBRACE) ||
       (scancode >= KC_A && scancode <= KC_QUOTE) ||
       (scancode >= KC_Z && scancode <= KC_SLASH) ||
       scancode == KC_BQUOTE)
    {
      tbl = (comb_flags & SHIFT_MASK) ? ru_codes : ru_s_codes;
    }
  }
 
  if(tbl[scancode][0]) {
    buf[0] = tbl[scancode][0];
    ++n;
    if(tbl[scancode][1]) {
      buf[1] = tbl[scancode][1];
      ++n;
    }
  }

  return n;
}

