/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * (c) Copyright 2006,2007,2008 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2008 MadTirra <madtirra@jarios.org>
 *
 * termio.x:
 * termio.c: main entry and IPC internals
 *
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/mman.h>
#include <sys/poll.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <unistd.h>
#include <errno.h>
#include <wchar.h>
#include <datalib/usrtc.h>
#include <sys/scheduler.h>
#include <sys/thread.h>
#include <ns/ns.h>
#include <ipc/rpc.h>
#include <devio/devio.h>
#include <pthread.h>
#include <sys/ioctl.h>

#include <sys/console.h>
#include <chardev.h>

#include "debug.h"
#include "termio.h"
#include "ptm.h"
#include "keyb.h"

int consolefd;

static pthread_t keyb_thread_id;

char tbuf[512];

int main(int argc,char **argv)
{
  int r;
  timeval_t tval;
  coord_t coords;

#ifndef CONFIG_TERMIO_GRUB
  start_service();
#endif

  while ((consolefd = open("/dev/console", O_RDWR)) < 0)
    usleep(100);

  debug_init();

  errno = 0;
  r = ioctl(consolefd, CIOC_GETSIZE, &coords);
  if(r < 0) {
    ascii_printf(L"[%s] failed to get console size %u (%d %d).\n", SERVICE_NAME, _IOC_SIZE(CIOC_GETSIZE), r, errno);
    abort();
  }

  if(coords.x < 2 || coords.y < 2) {
    ascii_printf(L"Console size too small: x: %d, y: %d\n", coords.x, coords.y);
    abort();
  };

  r = ptm_init(coords.x, coords.y);

  if(r != 0) {
    ascii_printf(L" Fail (%d)\n[%s] aborting\n", r, SERVICE_NAME);
    abort();
  }
  
  ptm_print("[termio] DONE\n");

  ptm_switch_active(0);

  /* create separate thread */

  init_keyb();

  ptm_print("[termio] starting keyb_thread.\n");
  r = pthread_create(&keyb_thread_id, NULL, keyb_thread, NULL);
  if(r < 0) {
    ptm_print("[termio] failed to create keyb thread.\n");
  }

  tval.tv_sec=600;

  device_server_start();

  // Unreached in normal execution
  abort();
  return 0;
}

