/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 * 
 * (c) Copyright 2006,2007,2008 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2005,2008 MadTirra <madtirra@jarios.org>
 *
 * Jari OS virtual file system
 *
 */

#ifndef __CRS_VFS_GLOB_H__
#define __CRS_VFS_GLOB_H__

#include <sys/types.h>
#include <ipc/rpc.h>
#include <ipc/port.h>
#include <wchar.h>
#include <datalib/usrtc.h>

typedef struct __vfs_sys_mount_t {
  char *device;
  pid_t dev_pid;
  ulong_t cid;
  ulong_t iid;
  ulong_t oid;
} vfs_sys_mount_t;

typedef struct __vfs_fs_info_t {
  wchar_t *fs_name;
  ulong_t size;
  ulong_t free;
} vfs_fs_info_t;

typedef struct __vfs_mount_point_t {
  ulong_t hash;
  pid_t fspid; /* fs pid */
  prtid_t w_port; /* worker port */
  prtid_t s_port; /* service port */
  chid_t w_channel;
  chid_t s_channel;
  ulong_t use_count;
  int mount_flags;
  int state_flags;
  vfs_sys_mount_t *sys_arrangel;
  vfs_fs_info_t *info;
  char *mount_path;
  usrtc_t *over_off;
} vfs_mount_point_t;

typedef struct __vfs_mount_request_t {
  uint16_t flags;
  char *devno;
  char *point;
  char *fstype;
} vfs_mount_request_t;

typedef struct __vfs_fd_t {
  int fd; /* file descriptor */
  int flags; /* flags */
  uint64_t offset; /* current offset position */
  vfs_mount_point_t *owner;
} vfs_fd_t;

typedef struct __task_res_t {
  usrtc_t *fds;
  char *cwd;
  pid_t pid;
  uid_t uid;
  gid_t gid;
  int state;
  int ref_count;
  pthread_mutex_t lock;
} task_res_t;

extern usrtc_t *res_list;

extern prtid_t serv_port;

#endif /* __CRS_VFS_GLOB_H__ */
