/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 * 
 * (c) Copyright 2006,2007,2008 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2005,2008 MadTirra <madtirra@jarios.org>
 *
 * Jari OS virtual file system
 * requests
 *
 */

#ifndef __CRS_VFS_RPC_H__
#define __CRS_VFS_RPC_H__

#include <sys/types.h>
#include <ipc/rpc.h>
#include <wchar.h>

typedef struct __vfs_path_loc_t {
  int nelems;
  wchar_t **elems;
} vfs_path_loc_t;

typedef struct __vfs_task_t {
  uint16_t vfs_cmd;
  vfs_path_loc_t *loc;
  int fd;
  int kmsgid;
  uid_t uid;
  gid_t gid;
  pid_t pid;
  size_t data_len;
  void *data; /* specific data to send */
} vfs_task_t;

typedef struct __serv_task_t {
  uint16_t vfs_cmd;
  vfs_path_loc_t *loc;
  int fd;
  int kmsgid;
  uid_t uid;
  gid_t gid;
  pid_t pid;
  size_t data_len;
  void *data; /* specific data to send */
} serv_task_t;

/* storage resources */
extern int init_resources(void);
/*
 * lookup_task_resource: try to find existing pid resource 
 * It returns NULL, or pointer to pid resource, 
 * take care *it increments reference counter!*
 */
extern task_res_t *lookup_task_resource(pid_t pid);
/*
 * new_task_resource: add a new resource entry for pid
 * make sure that this pid doesn't exist, there are no 
 * checking.
 * It returns pointer to new resource and increments reference count.
 * Take care! if CWD is NULL *ROOT* directory will be set as CWD, also
 * this function *freeing cwd* on any error.
 */
extern task_res_t *new_task_resource(char *cwd,pid_t pid,uid_t uid,gid_t gid);
/*
 * locking functions
 */
extern void lock_task_resource(task_res_t *res);
extern void unlock_task_resource(task_res_t *res);
/*
 * add file descriptor to task resource 
 */
extern int add_descriptor_task_resource(task_res_t *res,int fd,
					vfs_mount_point_t *dest_fs,
					int flags);
/* lookup */
extern vfs_fd_t *task_resource_lookup(task_res_t *res,int fd);
/* delete, TODO: optimize this one  */
extern int task_resource_delete(task_res_t *res,vfs_fd_t *tvfd);

/* queue functions */
extern int init_queue(void);
extern int add_task(int vfs_cmd,vfs_path_loc_t *loc,
		    uid_t uid,gid_t gid,pid_t pid,
		    int kmsgid,void *data,size_t data_len);
extern vfs_task_t *take_task(void);
extern int queue_atom_run(void);

/* mount table ops */
extern int init_mounts(void);
extern int add_mount_task(void *msg,size_t len,int kmsgid);
extern int register_fs(void *msg,size_t len,int kmsgid);
extern int vfs_mount(vfs_mount_request_t *mount_req);
/* this functions lookups mount point depends on name */
extern vfs_mount_point_t *vfs_lookup_end_point(char *path,int *mt_off,
					       int amount,int *markers);

/* garbage collector */
extern void gc_cleanup_pid(pid_t pid);

/* ops with service queue */
extern int init_service(void);
extern void service_atom_run(void);
extern void service_queue_lock(void);
extern void service_queue_unlock(void);
extern int add_service_job(serv_task_t *job);

/* support routines */
/* hashes ops */
ulong_t hash_w(const wchar_t *name);
ulong_t hash_s(const char *name);

#endif /* __CRS_VFS_RPC_H__ */
