/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 * 
 * (c) Copyright 2006,2007-2009 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2005,2008,2009 MadTirra <madtirra@jarios.org>
 *
 * Jari OS virtual file system v2,
 * filesystem operations for iterate 
 * with file system, includes proto and
 * internal point lookup
 */

#ifndef __V2_FSOPS_H__
#define __V2_FSOPS_H__

#include <sys/types.h>
#include <sys/stat.h>
#include <stdint.h>
#include <stdarg.h>
#include <ipc/port.h>
#include <ipc/channel.h>
#include <pthread.h>
#include <datalib/usrtc.h>

#include "resource.h"

/* operations without fds */
/**
 * @fn int vfs_lookup(vfs_path_t *path,struct stat *buf);
 * @brief Lookup (stat) given vfs path
 * @desc This function lookup a root of a requested object
 * and fill out it's stat buffer.
 * Proto for this iteraction the following:
 *  - vfs -> fs:
 *    c -> RPC_LOOKUP
 *    r0 -> amount of points
 *    r1 -> path name length in bytes
 *    offset -> offset within points of the root
 *    <parsed points>
 *    <pathname>
 *  - fs -> vfs:
 *    c -> errno or 0
 *    <struct stat>
 */
int vfs_lookup(vfs_path_t *path,struct stat *buf);
int vfs_lookup_resolve(vfs_path_t *path,struct stat *buf);
int vfs_readlink(vfs_path_t *path);
int vfs_readlink_solve(vfs_path_t *path);
int vfs_mkdir(vfs_path_t *path,int mode,uid_t uid,gid_t gid);
int vfs_unlink(vfs_path_t *path,uid_t uid,gid_t gid);
int vfs_unlinkdir(vfs_path_t *path,uid_t uid,gid_t gid);
/* operations fd related */
int vfs_open(user_link_t *ulink,link_point_t *fslink,vfs_path_t *mould,
	     int aff_am,uid_t uid,gid_t gid,pid_t pid); /* there are maybe both - file and dir */
/* close file descriptor */
int vfs_close(user_gate_t *sgate,user_link_t *slink);
/* system operations */
int vfs_mount(link_point_t *ulink,vfs_path_t *d_loc,int spec_flags);
/* sync all io */
void vfs_sync(link_point_t *ulink);
/* clone resources */
void vfs_pidfdclone(link_point_t *ulink,pid_t ppid,pid_t npid);
/* reset resources */
void vfs_pidfdreset(link_point_t *ulink,pid_t ppid);
/* kill resources */
void vfs_pidfdkill(link_point_t *ulink,pid_t ppid);


#endif /* __V2_FSOPS_H__ */
