/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 * 
 * (c) Copyright 2006,2007,2008,2009 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2005,2008,2009 MadTirra <madtirra@jarios.org>
 *
 * Jari OS virtual file system v2,
 * user gate interface (it doesn't implement any protocol, 
 * intercommunication features)
 *
 */

#include <sys/types.h>
#include <stdint.h>
#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <fcntl.h>
#include <errno.h>
#include <pthread.h>
#include <unistd.h>
#include <datalib/usrtc.h>

#include "resource.h"

/* store user link */
int user_link_store(user_gate_t *ugate,user_link_t *ulink)
{
  int n_mex,i;

  if(!ulink)
    return EINVAL;
  if(!ugate)
    return EINVAL;

  ugate_lock(ugate);

  if(user_link_max(ugate)<ulink->u_fd) { /* should realloc this */
    n_mex=ulink->u_fd/MAGIC_ULI;
    if(ulink->u_fd%MAGIC_ULI!=0)      n_mex++;
    n_mex*=MAGIC_ULI;
    ugate->u_links=realloc(ugate->u_links,n_mex*sizeof(uintptr_t));
    for(i=ugate->mex+1;i<n_mex;i++) 
      ugate->u_links[i]=NULL;
    ugate->mex=n_mex;
  }

  ugate->u_links[ulink->u_fd]=ulink;
  ugate->rcon++;

  ugate_unlock(ugate);

  return 0;
}

/* remove user link */
int user_link_pop(user_gate_t *ugate,user_link_t *ulink)
{
  if(!ulink)
    return EINVAL;
  if(!ugate)
    return EINVAL;

  ugate_lock(ugate);

  if(user_link_max(ugate)<ulink->u_fd) { /* try to pop not available resource */
    ugate_unlock(ugate);
    return EINVAL;
  }

  /* TODO: split areas, i.e. remove it if needs */

  ugate->u_links[ulink->u_fd]=NULL;
  ugate->rcon--;

  ugate_unlock(ugate);

  return 0;
}

/* int user_link shake */
int user_link_shake(user_gate_t *ugate,user_link_t *ulink,int cmd,...)
{
  user_link_t *alink;
  int aff_lnk;
  int *md;
  va_list vp;

  switch(cmd) {
  case ULNK_CLONE:
    va_start(vp,cmd);
    aff_lnk=va_arg(vp,int);
    va_end(vp);
    if(user_link_max(ugate) < aff_lnk)
      return EINVAL;
    if(user_link_lookup(ugate,aff_lnk)!=NULL)
      return EINVAL;
    if((alink=malloc(sizeof(*alink)))==NULL)
      return ENOMEM;
    /* clone user link */
    user_link_init(alink,aff_lnk,ulink->point,
                   ulink->mode,ulink->flags);
    /* assign additional user link parameters */
    alink->ufsid=ulink->ufsid;
    alink->port_id=ulink->port_id;
    alink->pid=ulink->pid;
    ulink->point->ref_count++;
    ugate->rcon++;
    ugate->u_links[alink->u_fd]=alink;
    break;
  case ULNK_SETMD:
    return ENOSYS;
    break;
  case ULNK_GETMD:
    va_start(vp,cmd);
    md=va_arg(vp,int*);
    va_end(vp);
    *md=ulink->mode;
    break;
  default:
    return EINVAL;
    break;
  }

  return 0;
}

