/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * (c) Copyright 2006,2007,2008,2009 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2009 Dmitry Gromada <gromada@jarios.org>
 *
 * Block device subsystem server part
 */

#include "server.h"
#include <sys/scheduler.h>
#include <unistd.h>
#include <pthread.h>
#include <bitwise.h>

#define bdev_subsys_lock()    pthread_mutex_lock(&subsys_mutex)
#define bdev_subsys_unlock()  pthread_mutex_unlock(&subsys_mutex)

static pthread_mutex_t subsys_mutex;
static struct iface_type reg_iface_types[NR_BDEV_IFACES];

static int __compare_id(const void *a, const void *b)
{
  return *(int*)a - *(int*)b;
}

int start_subsystem_server(void)
{
  /* yet dummy */
  sleep(1000000);
}

int start_interface_server(struct bd_iface *iface, int index)
{
  /* yet dummy */
  sleep(1000000);
}

int init_subsys_reginfo(void)
{
  int i, ret;

  ret = -pthread_mutex_init(&subsys_mutex, NULL);
  if (!ret) {
    for (i = 0; i < NR_BDEV_IFACES; i++) {
      usrtc_init(&reg_iface_types[i].ifaces, USRTC_LIST,
                 USRTC_COUNT_T_MAX, __compare_id);
      memset(&reg_iface_types[i].free_indexes, 0xFF, sizeof(ulong_t));
    }
  }

  return ret;
}

int find_reserve_iface_index(int iface_type)
{
  int r;

  bdev_subsys_lock();
  r = bit_find_lsf(reg_iface_types[iface_type].free_indexes);
  if (r >= 0) {
    bit_clear(&reg_iface_types[iface_type].free_indexes, r);
  }
  bdev_subsys_unlock();

  return r;
}

void free_iface_index(int iface_type, int index)
{
  bdev_subsys_lock();
  bit_set(&reg_iface_types[iface_type].free_indexes, index);
  bdev_subsys_unlock();
}

void get_iface_ndevs(int *ndevs)
{

}
