/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * (c) Copyright 2006,2007,2008,2009 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2009 Stanislav M. Ivankin <grepz@jarios.org>
 *
 * bootstrap service
 *
 */

#ifndef SYNTAX_H
#define SYNTAX_H

#include <sexpr/sexp.h>
#include <sexpr/sexp_ops.h>
#include <datalib/usrtc.h>

#include "syntax_defaults.h"
#include "btsd.h"

typedef enum {
  BTSD_SYNTAX_OK = 0,
  BTSD_SYNTAX_SEXP_TYPE_ERR,
  BTSD_SYNTAX_SEXP_INCOMPLETE_ERR,
  BTSD_SYNTAX_SEXP_NOTFOUND_ERR,
  BTSD_SYNTAX_SEXP_LENGTH_ERR,
  BTSD_SYNTAX_VAL_ERR,
  BTSD_SYNTAX_MEM_ERR,
  BTSD_SYNTAX_UNKNOWN_SEXP_ERR,
} btsd_syntax_err_t;

enum {
  BTSD_TOKEN_MODULE_BIN_VAL,
} btsd_module_token_t;

typedef enum {
  BTSD_TOKEN_RUNLEVEL_MODLIST,
} btsd_runlevel_token_t;

typedef struct __btsd_syntax_table btsd_syntax_table_t;
struct __btsd_syntax_table
{
  int token_id;
  btsd_syntax_err_t (*token_parser)(sexp_t *expr, void *data);
};

/* Generic functions */

static inline btsd_syntax_err_t is_valid_dquote(sexp_t *expr)
{
  if (expr->ty != SEXP_VALUE || expr->aty != SEXP_DQUOTE)
    return BTSD_SYNTAX_SEXP_TYPE_ERR;
  if (strlen(expr->val) > DQUOTE_LENGTH)
    return BTSD_SYNTAX_SEXP_LENGTH_ERR;

  return BTSD_SYNTAX_OK;
}

inline static btsd_syntax_err_t is_valid_token(sexp_t *expr)
{
  if (expr->ty != SEXP_VALUE)
    return BTSD_SYNTAX_SEXP_TYPE_ERR;
  if (strlen(expr->val) > TOKEN_LENGTH)
    return BTSD_SYNTAX_SEXP_LENGTH_ERR;

  return BTSD_SYNTAX_OK;
}

btsd_syntax_err_t list_get_token(sexp_t *expr, sexp_t **sx, int n);
btsd_syntax_err_t token_get_value(sexp_t *expr, atom_t type,
				  const char *token_name, char *data,
				  unsigned int maxlen);

btsd_syntax_err_t eval_modules(sexp_t *expr, usrtc_t *mlist);
btsd_syntax_err_t eval_runlevel_conf(sexp_t *expr, btsd_runlevel_conf_t **rconf);


#endif
