/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * (c) Copyright 2006,2007-2009 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2005,2008,2009 MadTirra <madtirra@jarios.org>
 * (c) Copyright 2009 Stanislav M. Ivankin <grepz@jarios.org>
 *
 * Jari OS file system linker daemon,
 * main routine
 */

#include <sys/types.h>
#include <sys/scheduler.h>
#include <sys/thread.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/v2.h>
#include <pthread.h>
#include <ctype.h>
#include <fcntl.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <ctype.h>
#include <time.h>
#include <sys/mman.h>
#include <dirent.h>
#include <ns/ns.h>
#include <sys/utsname.h>
#include <wchar.h>
#include <ipc/rpc.h>
#include <stdbool.h>
#include <limits.h>

#include <dirent.h>
#include <devio/devio.h>
#include <sys/utsname.h>

#include <dircons.h>

#include "fslinker_int.h"

    //    fprintf(serr, text"\n");

#define FSL_REPLY_FAIL(text,rply)					\
  do {									\
    rpc.rpc_call = rply;						\
    ipc_port_reply_nmsg(sport, sizeof(rpc), &(rpc), msg.msg_id);	\
    fflush(serr);							\
  } while (0)

FILE *sout;
FILE *serr;

static prtid_t   sport;
static pid_t     vfs_pid;
static chid_t    vfs_chan;
static pthread_t fslnk_thread_id;

static const fsl_modules_t fsmodules[] = {
  { "ext2.fs", "/system/fs/ext2.fs",
    FSLINK_HANDLE_CONF|FSLINK_HANDLE_RECONF},
  { "iso9660", "/system/fs/iso9660.fs",
    FSLINK_HANDLE_CONF|FSLINK_HANDLE_RECONF},
  { "init.fs", NULL, FSLINK_HANDLE_NOT},
  { "tmp.fs",  NULL, FSLINK_HANDLE_NOT},
  { "dev.fs",  NULL, FSLINK_HANDLE_NOT},
  { NULL, NULL, 0},
};

static int __process_req_options(fsl_requested_t *request,
				 int opt_n, int str_len,
				 const char *opt_str, const int *opt_len);
static inline int __get_request_val(int mid, void **val, size_t *off, size_t sz);
static inline int __get_request_str(int mid, char **str, size_t *off, size_t sz);
static inline int __validate_user_request(const uni_rpc_t *rpc, size_t msglen);
static inline int __check_fs_handle(const char *module, fsl_module_handle_t handle);

static int fsl_get_message(int message_id, const uni_rpc_t *rpc,
			   fsl_requested_t *request);
static void *fsl_thread(void *data);
static int fsl_vfs_mount_request(const fsl_requested_t *request);
static int fsl_execute_fs(fsl_requested_t *request);

int main(int argc,char **argv)
{
  int ret;

#ifndef CONFIG_FSLINKER_GRUB
  start_service();
#endif

  sout = fopen("/tmp/fslinker.out", "a");
  serr = fopen("/tmp/fslinker.err", "a");
  if (!sout || !serr)
    return EXIT_FAILURE;

  sport = ipc_port_open(IPC_BLOCKED_ACCESS, 512);
  if (sport < 0) {
    //    fprintf(serr, "Can't open port.\n");
    return EXIT_FAILURE;
  }

#ifndef CONFIG_FSLINKER_GRUB
  ret = ns_setattrs(V2T_TRUSTED|V2T_SYSDAEMON, SYS_FSL);
#else
  ret = ns_register(getpid(), 0, 0, (V2T_TRUSTED|V2T_SYSDAEMON), SYS_FSL,
		    "fslinker.x", NULL, 0);
#endif
  if (ret < 0) {
    //    fprintf(serr, "Can't register on NS server.\n");
    return EXIT_FAILURE;
  }

  vfs_pid = ns_getvfspid();
  if (!vfs_pid)
    return EXIT_FAILURE;

  vfs_chan = ipc_channel_open(vfs_pid, 0, IPC_BLOCKED_ACCESS);
  if (vfs_chan <= 0)
    return EXIT_FAILURE;

  //  fprintf(sout, "Fslinker started.\n");
  pthread_create(&fslnk_thread_id, NULL, fsl_thread, NULL);

  while (1) {
    sleep(20);
  }

  return EXIT_SUCCESS;
}

static void *fsl_thread(void *data)
{
  uni_rpc_t rpc;
  ipc_port_msg_info_t msg;
  fsl_requested_t request;
  int ret, ret_msg;

  while (1) {
    /* Receiving initial user message */
    ret_msg = ipc_port_rcvmsg_mi(sport, 0, NULL, &msg);
    if (ret_msg < sizeof(uni_rpc_t)) {
      FSL_REPLY_FAIL("Malformed user request received.", EBADMSG);
      continue;
    }

    /* Reading RPC header */
    memset(&rpc, 0, sizeof(uni_rpc_t));
    ipc_port_read_msg(sport, msg.msg_id, 0, &rpc, sizeof(rpc));
    ret = __validate_user_request(&rpc, ret_msg);
    if (ret) {
      FSL_REPLY_FAIL("User request validation failed.", ret);
      continue;
    }

    /* Getting actual request data. */
    ret = fsl_get_message(msg.msg_id, &rpc, &request);
    if (ret) {
      FSL_REPLY_FAIL("Can't read actual user message.", ret);
      continue;
    }

    /* TODO: Execute FS/link/unlink/relink */

    /*
      switch (rpc.rpc_call) {
      case RPC_MOUNT:
      ret = fsl_vfs_mount_request(&request);
      break;
      }

      if (ret) {
      FSL_REPLY_FAIL("Failed processing user request.", ret);
      continue;
      }
    */

    ret = fsl_execute_fs(&request);
    if (ret) {
      FSL_REPLY_FAIL("Executing fs failed.", ret);
      continue;
    }

    ret = fsl_vfs_mount_request(&request);
    if (ret) {
      FSL_REPLY_FAIL("Mount request failed.", ret);
      continue;
    }

    rpc.rpc_call = 0;
    ipc_port_reply_nmsg(sport, sizeof(rpc), &rpc, msg.msg_id);
  }
}

static int fsl_get_message(int mid, const uni_rpc_t *rpc,
			   fsl_requested_t *request)
{
  char *optstr;
  int *opt_len;
  size_t off;
  int ret;

  off = sizeof(*rpc);
  memset(request, 0, sizeof(*request));

  /* FS type */
  ret = __get_request_str(mid, &request->fstype, &off, rpc->arg0);
  if (ret)
    return ret;
  if (!__check_fs_handle(request->fstype, FSLINK_HANDLE_CONF)) {
    free(request->fstype);
    return EINVAL;
  }

  /* First path */
  ret = __get_request_str(mid, &request->mountp.path, &off, rpc->arg1);
  if (ret)
    goto __mounts_fail;
  /* Points of first path */
  request->mountp.p_num = rpc->arg2;
  ret = __get_request_val(mid, (void **)&request->mountp.pp, &off,
			  rpc->arg2*sizeof(int));
  if (ret) {
    free(request->mountp.path);
    goto __mounts_fail;
  }

  /* Second path */
  ret = __get_request_str(mid, &request->mounts.path, &off, rpc->arg3);
  if (ret)
    goto __mountp_fail;
  /* Points of second path */
  request->mounts.p_num = rpc->arg4;
  ret = __get_request_val(mid, (void **)&request->mounts.pp, &off,
			  rpc->arg4*sizeof(int));
  if (ret) {
    free(request->mounts.path);
    goto __mountp_fail;
  }

  /* Options */
  if (rpc->arg5 > 0) {
    /* BUG: rpc args size */
    ret = __get_request_val(mid, (void **)&optstr, &off, rpc->arg5);
    if (ret) goto __opts_fail;
    ret = __get_request_val(mid, (void **)&opt_len, &off, rpc->arg6);
    if (ret) {
      free(optstr);
      goto __opts_fail;
    }

    ret = __process_req_options(request, rpc->arg6/sizeof(int),
				rpc->arg5, optstr, opt_len);
    if (ret) {
      free(optstr);
      free(opt_len);
      goto __opts_fail;
    }
  }

  fflush(sout);

  return 0;

  __opts_fail:
  free(request->mountp.path);
  free(request->mountp.pp);

 __mountp_fail:
  free(request->mounts.path);
  free(request->mounts.pp);

 __mounts_fail:
  free(request->fstype);

  return ret;
}

static int fsl_execute_fs(fsl_requested_t *request)
{
  int status;
  pid_t pid;
  char *binary;
  const fsl_modules_t *ptr;

  binary = NULL;
  ptr = fsmodules;

  // TODO: Move this to get_message
  while (ptr->module) {
    if (!strcmp(request->fstype, ptr->module)){
      binary = ptr->binary;
      break;
    }
    ++ ptr;
  }

  if (binary == NULL || (access(binary, R_OK|X_OK) != 0))
    return 1;

  fprintf(sout, "Executing (%s) FS, options (%d).\n",
  	  binary, request->options ? 1 : 0);

  pid = fork();

  switch (pid) {
  case -1:
    abort();
    break;
  case 0:
    if (execve(binary, request->options, NULL) < 0)
      abort();
  default:
    pid = waitpid(pid, &status, 0);
    usleep(1000);
  }

  sleep(1);

  return 0;
}

static int fsl_vfs_mount_request(const fsl_requested_t *request)
{
  int ret;
  uni_rpc_t rpc;
  size_t link_s_len, link_p_len, rpcmsg_len;
  void *msg, *msg_ptr;

  memset(&rpc, 0, sizeof(uni_rpc_t));

  /* arg0 - FS flags
   * arg1 - fstype length
   * arg2 - link point points amount
   * arg3 - link point path len
   * arg4 - link subject points amount
   * arg5 - link subject path len
   * Actual message must looks like: <rpc><fstype><points><path><points><path>
   */

  rpc.arg0 = V2FS_RW;
  rpc.arg1 = strlen(request->fstype);
  rpc.arg2 = request->mountp.p_num;
  rpc.arg3 = strlen(request->mountp.path);
  rpc.arg4 = request->mounts.p_num;
  rpc.arg5 = strlen(request->mounts.path);
  rpc.rpc_call = RPC_MOUNT;

  link_p_len = rpc.arg2*sizeof(int);
  link_s_len = rpc.arg4*sizeof(int);

  rpcmsg_len = sizeof(uni_rpc_t) + rpc.arg1 +
    sizeof(int)*rpc.arg2 + rpc.arg3 +
    sizeof(int)*rpc.arg4 + rpc.arg5;

  fprintf(sout, "Message length: %d\n"
	  "FS = %s:%d\n"
	  "Link point path = (%s:%d:%d)\n"
	  "Link subject path = (%s:%d:%d)\n",
	  rpcmsg_len,
	  request->fstype, rpc.arg1,
	  request->mountp.path, rpc.arg2, rpc.arg3,
	  request->mounts.path, rpc.arg4, rpc.arg5);

  int i;
  for (i = 0; i < rpc.arg2; i++)
    fprintf(sout, "\t%d\n", request->mountp.pp[i]);
  fprintf(sout, "-----------\n");
  for (i = 0; i < rpc.arg4; i++)
    fprintf(sout, "\t%d\n", request->mounts.pp[i]);

  msg = malloc(rpcmsg_len);
  if (!msg)
    return ENOMEM;

  memcpy(msg, &rpc, sizeof(uni_rpc_t));

  /* Filesystem type */
  msg_ptr = (void *)((char *)msg + sizeof(uni_rpc_t));
  memcpy(msg_ptr, request->fstype, rpc.arg1);
  /* link point path */
  msg_ptr = (void *)((char *)msg_ptr + rpc.arg1);
  memcpy(msg_ptr, request->mountp.pp, link_p_len);
  msg_ptr = (void *)((char *)msg_ptr + link_p_len);
  memcpy(msg_ptr, request->mountp.path, rpc.arg3);
  /* Link subject path */
  msg_ptr = (void *)((char *)msg_ptr + rpc.arg3);
  memcpy(msg_ptr, request->mounts.path, link_s_len);
  msg_ptr = (void *)((char *)msg_ptr + link_s_len);
  memcpy(msg_ptr, request->mounts.path, rpc.arg5);

  ret = ipc_channel_msgsnd(vfs_chan, rpcmsg_len, msg, sizeof(uni_rpc_t), &rpc);
  if (ret < 0)
    fprintf(serr, "Error (%d) occured while sending request to VFS.\n", ret);
  else
    ret = rpc.rpc_call;

  free(msg);

  return ret;
}


static inline int __check_fs_handle(const char *module, fsl_module_handle_t handle)
{
  const fsl_modules_t *ptr;

  ptr = fsmodules;

  while (ptr->module) {
    if (!strcmp(module, ptr->module)) {
      if (ptr->fs_handle & handle) {
	return 1;
      } else
	return 0;
    }

    ++ ptr;
  }

  return 0;
}

/* TODO: Check msg sum */
static inline int __validate_user_request(const uni_rpc_t *rpc, size_t msglen)
{
  size_t args_len;

  /* TODO: Dummy */
  if (rpc->rpc_call != RPC_MOUNT)
    return ENOSYS;

  args_len = sizeof(*rpc) + rpc->arg0 + rpc->arg1 + rpc->arg2*sizeof(int) +
    rpc->arg3 + rpc->arg4*sizeof(int) + rpc->arg5 + rpc->arg6*sizeof(int);
  if (msglen < args_len)
    return EBADMSG;

  /* These arguments can't be zeroed */
  if (!rpc->arg0 || !rpc->arg1 || !rpc->arg2)
    return EINVAL;

  if (rpc->arg0 > FSTYPE_LEN_MAX  ||
      rpc->arg1 > PATH_MAX        ||
      rpc->arg3 > PATH_MAX        ||
      rpc->arg5 > LINE_MAX)
    return ENAMETOOLONG;

  if (rpc->arg2 > PATH_POINTS_MAX ||
      rpc->arg4 > PATH_POINTS_MAX ||
      rpc->arg6 > FS_OPTS_MAX)
    return EINVAL;

  return 0;
}

static inline int __get_request_str(int mid, char **str, size_t *off, size_t sz)
{
  /* Allocating with respect to \0 termination */
  *str = malloc(sz + 1);
  if (!*str)
    return ENOMEM;

  if (ipc_port_read_msg(sport, mid, *off, *str, sz) != 0) {
    free(*str);
    return EINVAL;
  }

  (*str)[sz] = '\0';
  *off += sz;

  return 0;

}

static inline int __get_request_val(int mid, void **val, size_t *off, size_t sz)
{
  *val = malloc(sz);
  if (!*val)
    return ENOMEM;

  if (ipc_port_read_msg(sport, mid, *off, *val, sz) != 0) {
    free(*val);
    return EINVAL;
  }

  *off = *off + sz;

  return 0;
}

static int __process_req_options(fsl_requested_t *request,
				 int opt_n, int str_len,
				 const char *opt_str, const int *opt_len)
{
  const char *opt_ptr;
  char **options;
  int i, len;

  /* First of all, check if options length is correct */
  for (i = 0, len = 0; i < opt_n; i ++)
    len += opt_len[i];

  if (len != str_len)
    return EINVAL;

  /* We must end argv of execve with NULL pointer,
   * so allocate n + 1 rows
   */
  options = malloc(sizeof(uintptr_t) * (opt_n + 1));
  if (!options)
    return ENOMEM;

  options[0] = malloc(str_len + opt_n);
  if (!options[0]) {
    free(options);
    return ENOMEM;
  }

  memset(options[0], '\0', str_len + opt_n);

  for (len = 0, i = 1, opt_ptr = opt_str; i < opt_n; i++) {
    len     += opt_len[i-1] + 1;
    opt_ptr += opt_len[i-1];
    strncpy(options[0] + len, opt_ptr, opt_len[i]);
    options[i] = options[0] + len;
  }

  strncpy(options[0], opt_str, opt_len[0]);
  options[opt_n] = NULL;

  request->options = options;

  return 0;
}
