#ifndef __PM_ELF_H__
#define __PM_ELF_H__

#include <sys/types.h>
#include <arch/types.h>
#include <stdbool.h>
#include <sys/kcontrol.h>
#include "elf_e.h"
#include "pm.h"

#define MAX_LIBS_PER_EXECUTABLE  32
#define MAX_LIBS_RESOLVE_DEPTH   64

typedef struct __dynamic_elf_stat {
  int dynamic_id,dynstr_id,dynsym_id,hash_id;
} dynamic_elf_stat_t;

typedef struct __shl_descr_data {
  int num_sections;
  void *load_addr;
  elf64_sh_t *sections;
  elf64_sym_t *dynsyms;
  uint32_t *hash;
  char *symnames;
  int num_dynsyms,hash_size;
} shl_descr_data_t;

typedef struct __weaksym_descr {
  struct __weaksym_descr *next;
  const char *name;
  char *dynstr;
  elf64_sym_t sym;
  elf64_rela_t rsym;
  long obj_base;
  int cache_id;
} weaksym_descr_t;

struct __vma_range {
  struct __vma_range *next;
  void *start,*grant_addr;
  long size;
};

typedef struct __pm_elf_ctx {
  pm_exec_type_t type;
  elf_header_t elf_header;
  elf64_sh_t *sections;
  int num_sections;

  /* Stuff related to dynamic ELFes */
  dynamic_elf_stat_t dyn_stats;
  char *dynstr;
  bool dynamic,rt_launch;
  int lib_cache_ids[MAX_LIBS_PER_EXECUTABLE];
  shl_descr_data_t *descr_data[MAX_LIBS_PER_EXECUTABLE];
  int num_libs;
  void *libs_map_base;

  /* Weak aliases related stuff. */
  weaksym_descr_t *weak_symbols;

  struct __vma_range *mapped_vmas;
} pm_elf_ctx_t;

typedef struct shared_lib_descr {
  const char *filename;
  int fd,cache_id;
} shared_lib_descr_t;

#define ELF_MIN_SIZE      4096
#define ELF_MAX_SECTIONS  512

int check_library(char *filename);
int lookup_library(char *filename,bool new);
shared_lib_descr_t *get_cached_library(int id);

#define SHLIB_RANGE_START     0x50000000
#define DYNIMAGE_RANGE_START  0x40000000

#endif
