/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * (c) Copyright 2006,2007,2008,2009 MString Core Team <http://mstring.jarios.org>
 * (c) Copyright 2009 Michael Tsymbalyuk <mtzaurus@gmail.com>
 *
 * services/system/procmgr/execfmt.c: Executabel format API.
 */

#include <sys/types.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>
#include <sys/mman.h>
#include <sys/thread.h>
#include <arch/system.h>
#include <sys/ptd.h>
#include <sys/syscall.h>
#include <dircons.h>
#include <errno.h>
#include "pm.h"

static executable_format_t *__exec_formats[]={
  &elf_exec_fmt,
  NULL,
};

/* TODO: [mt] Move it to another location. */
int ioctx_read_offt(int fd,io_ctx_t *ctx,int size,unsigned long offset)
{
  if( size > ctx->iobuf.size ) {
    void *new_addr=malloc(size);

    if( !new_addr ) {
      return -ENOMEM;
    }

    free(ctx->iobuf.buf);
    ctx->iobuf.buf=new_addr;
    ctx->iobuf.size=size;
  }

  return pread(fd,ctx->iobuf.buf,size,offset);
}

long setup_process_exec_attrs(pmp_exec_action_t *ea,exec_attrs_t *exec_attrs)
{
  char *our_vma,*stack_top,*t;
  per_task_data_t *ptd;
  int argv_delta=ea->argsize+ea->numargs*sizeof(long)+2*sizeof(long)+sizeof(*ptd);
  int r,stack_size=PAGE_ALIGN((STACK_PAGES<<PAGE_WIDTH)+argv_delta);
  int our_vma_size=PAGE_ALIGN(argv_delta);
  long *argv;

  /* Stack layout of a newly reincarnated executable is as follows:
   * ==================> USER_MAX_VIRT
   *    <PTD>
   *    <Null-terminated command line arguments, one by one>
   *    <array of pointers for every command line argument, (i.e. 'argv')>
   *    <Zero return address>   <==
   *                              ==== Stack pointer upon unvoking entry point.
   */

  stack_top=mmap_pid(ea->target,(void*)(USER_MAX_VIRT-stack_size),
                     stack_size,PROT_READ | PROT_WRITE,
                     MAP_ANON | MAP_PRIVATE | MAP_STACK | MAP_FIXED | MAP_CANRECPAGES,
                     0,0);
  if( (long)stack_top & PAGE_MASK ) {
#ifdef CONFIG_PM_DEBUG
    console_printf("[ProcMgr] Failed to allocate %d bytes for target stack (PID=%d)!\n",
                   stack_size,ea->target);
#endif
    return -ENOMEM;
  }

  our_vma=mmap(NULL,our_vma_size,PROT_READ | PROT_WRITE,
               MAP_ANON | MAP_PRIVATE | MAP_POPULATE,0,0);
  if( (long)our_vma & PAGE_MASK ) {
#ifdef CONFIG_PM_DEBUG
    console_printf("[ELF] __map_one_vma(): Failed to allocate %d bytes for stack buffer !\n",
                   our_vma_size);
#endif
    return -ENOMEM;
  }

  memset(exec_attrs,0,sizeof(*exec_attrs));

  ptd=(per_task_data_t*)(our_vma+our_vma_size-sizeof(*ptd));
  ptd->ptd_base=(uintptr_t)((per_task_data_t *)(stack_top-sizeof(*ptd)));

  /* Copy program arguments to our VMA. */
  t=(char *)(ptd)-ea->argsize;
  memcpy(t,ea->args,ea->argsize);

  /* Build array of arguments and make it be properly aligned in our VMA.
   * Note that 1 extra long (here and later during the building of 'argv')
   * is taken into account to hold NULL that must terminate 'argv' according
   * to POSIX.
   */
  t-=((ea->numargs+1)*sizeof(long));
  argv=(long *)((long)t & ~(long)(sizeof(long)-1));

  /* Now calculate address of the first argument in target address space. */
  t=(char *)((long)ptd->ptd_base-(long)ea->argsize);

  /* Fill target command line arguments array in our VMA. */
  for(r=0;r<ea->numargs;r++) {
    argv[r]=(long)t+ea->arg_offsets[r];
  }
  argv[r]=(long)NULL; /* argv must be NULL-terminated. */

  /* Finally, calculate the address of argument array in target address space
   * and make it be long-aligned.
   */
  t=t-(ea->numargs+1)*sizeof(long);
  t=(char *)((long)t & ~(long)(sizeof(long)-1));

  exec_attrs->arg1=ea->numargs;
  exec_attrs->arg2=(uintptr_t)t;    /* Address of argument array. */
  exec_attrs->stack=(uintptr_t)t-8; /* Guard return address. */
  exec_attrs->per_task_data=(uintptr_t)ptd->ptd_base;

  r=grant_pages(our_vma,our_vma_size,ea->target,(uintptr_t)t & ~PAGE_MASK);
#ifdef CONFIG_PM_DEBUG
  if( r ) {
    console_printf("[ProcMgr] Target VMA: [%p:%p), Our VMA: [%p:%p), Stack size=0x%X)\n",
                   stack_top-stack_size,stack_top,
                   our_vma,our_vma+our_vma_size,stack_size);
    console_printf("[ProcMgr] Failed to grant stack to PID %d ! %p ==> %p, errno=%d\n",
                   ea->target,our_vma,(uintptr_t)t & ~PAGE_MASK,
                   errno);
    console_printf("[ProcMgr] Our VMA size: 0x%X, Last used address: %p\n",
                   our_vma_size,(long)argv & ~PAGE_MASK);
  }
#endif

  munmap(our_vma,our_vma_size);
  return r;
}

int pm_locate_executable_format(int fd,io_ctx_t *iobuf,executable_format_t **fmt,
                                void **ctx,pm_exec_type_t type)
{
  executable_format_t *f=NULL;
  int i,t,r=-EINVAL;

  for(i=0;__exec_formats[i];i++) {
    t=__exec_formats[i]->lookup(fd,iobuf,ctx,type);
    if( !t ) {
      f=__exec_formats[i];
      r=0;
      break;
    } else if( t < 0 ) {
      r=t;
      break;
    }
  }

  *fmt=f;
  return r;
}
