/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * (c) Copyright 2006,2007,2008,2009 MString Core Team <http://mstring.jarios.org>
 * (c) Copyright 2009 Michael Tsymbalyuk <mtzaurus@gmail.com>
 *
 * services/system/procmgr/libcache.c: here ProcMgr keeps cached shared libraries.
 */

#include <sys/types.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <sys/mman.h>
#include <errno.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <limits.h>
#include <stdbool.h>

#include "pm.h"
#include "elf.h"
#include "elf_e.h"

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#define MAX_LIBS_IN_CACHE  32

static int __num_cached_libs=0;
static shared_lib_descr_t __cached_libs[MAX_LIBS_IN_CACHE];

static const char * library_roots[] = {
  "/lib/",
  "/usr/lib/",
  NULL,
};

shared_lib_descr_t *get_cached_library(int id)
{
  if( id < __num_cached_libs ) {
    return &__cached_libs[id];
  }
  return NULL;
}

int lookup_library(char *filename,bool new)
{
  int i,id=-1;

  for( i=0; i<__num_cached_libs; i++ ) {
    if( !strcmp(filename,__cached_libs[i].filename) ) {
      id=i;
      break;
    }
  }

  if( id == -1 && new ) {
    char *b = malloc(strlen(filename)+1);

    if( b ) {
      id=open(filename,O_RDONLY);
      if( id >=0 ) {
        strcpy(b,filename);
        __log(" * Caching new library '%s', fd=%d\n",filename,id);
        __cached_libs[__num_cached_libs].filename = b;
        __cached_libs[__num_cached_libs].fd=id;
        __cached_libs[__num_cached_libs].cache_id=__num_cached_libs;
        id=__num_cached_libs;
        __num_cached_libs++;
      } else {
        free(b);
      }
    }
  }

  return id;
}

int check_library(char *filename)
{
  char b[2*PATH_MAX];
  int id,i;

  for( i=0,id=-1; id < 0 && library_roots[i]; i++ ) {
    strcpy(b,library_roots[i]);
    strcat(b,filename);

    id=lookup_library(b,1);
  }

  return id;
}
