#ifndef __PM_H__
#define  __PM_H__

#include <sys/types.h>
#include <datalib/usrtc.h>
#include <sys/procmgr.h>
#include <sys/thread.h>
#include <datalib/usrtc.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <limits.h>
#include <unistd.h>

#define PMP_INIT_ACTION(a,t)  do {              \
    usrtc_node_init(&(a)->rbnode,(a));          \
    (a)->type=(t);                              \
  } while(0)

typedef struct __pmp_action {
  usrtc_node_t rbnode; /* Must be first */
  int type;
} pmp_action_t;

typedef struct __pmp_fork_action {
  pmp_action_t a; /* Must be first. */
  pid_t parent;
  pm_fork_request_t rq;
} pmp_fork_action_t;

typedef struct __io_buf {
  void *buf;
  size_t size;
} io_buf_t;

typedef struct __io_ctx {
  io_buf_t iobuf;
  struct stat stats;
} io_ctx_t;

typedef struct __pmp_exec_action {
  pmp_action_t a; /* Must be first. */
  pid_t target;
  int fd;
  void *fmtctx;
  int num_argv,num_envp;
  char *envp;
  struct __executable_format *fmt;
  io_ctx_t ioctx;
  char args[PATH_MAX+PM_ARGV_SIZE];
  int numargs,argsize;
  uint16_t arg_offsets[ARG_MAX];
} pmp_exec_action_t;

#define PM_MAX_MAGIC_SIZE  4096
#define ELF_MIN_SIZE      4096
#define ELF_MAX_SECTIONS  512

typedef enum __pm_exec_type {
  PM_EXECUTABLE=0,
  PM_SHARED_LIBRARY=1,
} pm_exec_type_t;

typedef struct __exec_fmt_task_ctx {
  uintptr_t entrypoint;
} exec_fmt_task_ctx_t;

typedef struct __executable_format {
  const char *id;
  int (*lookup)(int fd,io_ctx_t *ioctx,void **ctx,pm_exec_type_t type);
  int (*pre_load)(pid_t target,int fd,io_ctx_t *ioctx,void *ctx);
  int (*load_image)(pid_t target,int fd,io_ctx_t *ioctx,void *ctx,
                    exec_fmt_task_ctx_t* task_ctx);
  void (*cleanup)(void *ctx);
} executable_format_t;

int pm_locate_executable_format(int fd,io_ctx_t *iobuf,executable_format_t **fmt,
                                void **ctx,pm_exec_type_t type);

extern executable_format_t elf_exec_fmt;

int ioctx_read_offt(int fd,io_ctx_t *ctx,int size,unsigned long offset);
long setup_process_exec_attrs(pmp_exec_action_t *ea,exec_attrs_t *exec_attrs);

#define __log(x ...)
#define __log_err klog

#endif
