/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * (c) Copyright 2006,2007,2008,2009 MString Core Team <http://mstring.jarios.org>
 * (c) Copyright 2009 Michael Tsymbalyuk <mtzaurus@gmail.com>
 *
 * services/system/procmgr/procmgr.c: General logic of the Process Manager.
 */

#include <stdbool.h>
#include <sys/types.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <datalib/usrtc.h>
#include <errno.h>
#include <ipc/port.h>
#include <ipc/channel.h>
#include <ipc/rpc.h>
#include <ns/ns.h>
#include <errno.h>
#include <devio/devio.h>
#include <sys/poll.h>
#include <sys/thread.h>
#include <debug.h>
#include <ns/ns.h>
#include <sys/procmgr.h>
#include <ipc/rpc.h>
#include <sys/proc.h>
#include <sys/syscall.h>
#include <sys/scheduler.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <dircons.h>
#include <sys/kcontrol.h>

#include "pm.h"
#include "procmgr.h"

#define lock_action_tree()
#define unlock_action_tree()
#define signal_new_task()

static usrtc_t __action_tree;
static prtid_t __client_port,__disint_port;

#define PM_NUMPORTS 2
#define PM_MSG_DATA_OFFT  sizeof(pm_cmd_header_t)
#define PM_FORK_REQUEST_SIZE  (PM_MSG_DATA_OFFT+sizeof(pm_fork_request_t))
#define PM_EXEC_REQUEST_SIZE  (PM_MSG_DATA_OFFT+sizeof(pm_exec_request_t))

char *ddd="12345";

#define PMP_INIT_EXEC_ACTION(ea) do {                   \
    PMP_INIT_ACTION(&(ea)->a,__PROCMGR_CMD_EXEC);       \
    memset(&(ea)->ioctx,0,sizeof(io_ctx_t));            \
    ea->fd=-1;                                          \
  } while(0)

static int __compare_tasks(const void *a,const void *b)
{
  return (long)a-(long)b;
}

static void __initialize_procmgr(void)
{
  __client_port=ipc_port_open(IPC_BLOCKED_ACCESS,DQUEUE_SIZE);
  if( __client_port < 0 ) {
    abort();
  }

  __disint_port=ipc_port_open(0,DQUEUE_SIZE);
  if( __disint_port < 0 ) {
    abort();
  }

  if( ns_register(getpid(),0,0,(V2T_TRUSTED|V2T_MKSERVICE),SYS_PROC,PROCMGR_NAME,NULL,0) ) {
    abort();
  }

  if( ns_register_port(__client_port,PROCMGR_CLIENT_PORT_NAME) ) {
    abort();
  }

  usrtc_init(&__action_tree,USRTC_REDBLACK,USRTC_COUNT_T_MAX,__compare_tasks);
}

static void __remove_action(pid_t key)
{
  usrtc_node_t *node;
  pmp_action_t *action;
  pmp_exec_action_t *ea;

  lock_action_tree();
  node=usrtc_lookup(&__action_tree,(void *)(long)key);
  if( node ) {
    usrtc_delete(&__action_tree,node);
  }
  unlock_action_tree();

  if( node ) {
    action=(pmp_action_t*)node;

    switch (action->type) {
      case __PROCMGR_CMD_EXEC:
        ea=(pmp_exec_action_t *)action;
        if( ea->ioctx.iobuf.buf ) {
          free(ea->ioctx.iobuf.buf);
        }

        if( ea->fd >= 0 ) {
          //close(ea->fd);
          ea->fd=-1;
        }
        break;
    }
    free(node);
  }
}

static int __replicate_task_resources(pmp_fork_action_t *fa)
{
  pid_t vfs;
  int r=-ENOSYS,vfs_ch,ns_ch;
  uni_rpc_t ur;

  if((ns_ch=ipc_channel_open(1,0,IPC_BLOCKED_ACCESS))>=0) {
    ur.rpc_call=RPC_IOCTL;
    ur.arg0=NX_CLONE;
    ur.arg1=fa->parent;
    ur.arg2=fa->rq.child_pid;
    r=ipc_channel_msgsnd(ns_ch,sizeof(ur),&ur,sizeof(ur),&ur);
    ipc_channel_close(ns_ch);
  }
  vfs=ns_getvfspid();
  if( vfs ) {
    vfs_ch=ipc_channel_open(vfs,0,IPC_BLOCKED_ACCESS);
    if( vfs_ch >= 0 ) {
      uni_rpc_t reply;

      ur.rpc_call=RPC_PIDFDCLONE;
      ur.arg0=fa->parent;
      ur.arg1=fa->rq.child_pid;

#ifdef CONFIG_PM_DEBUG
      console_printf("[ProcMgr] Sending RPC_PIDFDCLONE ...\n");
#endif
      r=ipc_channel_msgsnd(vfs_ch,sizeof(ur),&ur,sizeof(reply),&reply);
#ifdef CONFIG_PM_DEBUG
      console_printf("[ProcMgr] VFS reply: %d\n",r);
#endif
      if( r == sizeof(reply) ) {
        r=0;
      } else {
        r=-ENOSYS;
      }
      ipc_channel_close(vfs_ch);
    }
  }
  return r;
}

static int __process_fork_request(pm_fork_request_t *rq,ipc_port_msg_info_t *msginfo)
{
  int r,res=-EINVAL;
  task_details_t taskinfo;
  usrtc_node_t *node;
  pmp_fork_action_t *fa=NULL;

#ifdef CONFIG_PM_DEBUG
  console_printf("[ProcMgr] Process %d is cloning PID %d,\n",msginfo->sender_pid,rq->child_pid);
#endif

  r=get_task_details(rq->child_pid,&taskinfo);
  if( !r ) {
    if( taskinfo.state == TASK_STATE_JUST_BORN /* && taskinfo.ppid == msginfo->sender_pid */ ) {

      lock_action_tree();
      node=usrtc_lookup(&__action_tree,(void*)(long)rq->child_pid);
      unlock_action_tree();

      if( !node ) {
        if( (fa=malloc(sizeof(*fa)) ) ) {
          PMP_INIT_ACTION(&fa->a,__PROCMGR_CMD_FORK);
          fa->parent=msginfo->sender_pid;
          fa->rq=*rq;

          lock_action_tree();
          node=usrtc_lookup(&__action_tree,(void *)(long)rq->child_pid);
          if( !node ) {
            usrtc_insert(&__action_tree,&fa->a.rbnode,(void *)(long)rq->child_pid);
            res=0;
          } else {
            res=-EBUSY;
          }
          unlock_action_tree();

          if( !res ) {
            res=__replicate_task_resources(fa);
            if( res ) {
              __remove_action(rq->child_pid);
            }
          } else {
            free(fa);
          }
        } else {
          res=-ENOMEM;
        }
      } else {
        res=-EBUSY;
      }
    }
  }

  return res;
}

static int __setup_environment(pmp_exec_action_t *ea,pm_exec_request_t *rq)
{
  return 0;
}

static int __run_executable_tophalf(pmp_exec_action_t *ea,pm_exec_request_t *rq,
                                    pid_t target)
{
  int fd,r;
  executable_format_t *fmt;
  void *fmtctx;

#ifdef CONFIG_PM_DEBUG
  __log("[ProcMgr] Running executable: \"%s\"<\n",rq->args);
#endif

  ea->envp=NULL;
  r=__setup_environment(ea,rq);
  if( r ) {
    goto free_memory;
  }

  fd=open(rq->args,O_RDONLY);
  if( fd < 0 ) {
    return -errno;
  }

  ea->ioctx.iobuf.size=PM_MAX_MAGIC_SIZE;
  ea->ioctx.iobuf.buf=malloc(PM_MAX_MAGIC_SIZE);
  if( !ea->ioctx.iobuf.buf ) {
    r=-ENOMEM;
    goto close_fd;
  }

  r=fstat(fd,&ea->ioctx.stats);
  if( r ) {
    r=-errno;
    goto free_ctx;
  }  

  r=pread(fd,ea->ioctx.iobuf.buf,PM_MAX_MAGIC_SIZE,0);
  if( r <= 0 ) {
    r=-EIO;
    goto free_ctx;
  }

  r=pm_locate_executable_format(fd,&ea->ioctx,&fmt,&fmtctx,PM_EXECUTABLE);
  if( !r ) {
#ifdef CONFIG_PM_DEBUG
    __log("[ProcMgr] Detected format: %s\n",fmt->id);
#endif
    r=fmt->pre_load(target,fd,&ea->ioctx,fmtctx);
    if( r ) {
      fmt->cleanup(fmtctx);
      goto free_ctx;
    }

    ea->fmtctx=fmtctx;
    ea->fmt=fmt;
    ea->target=target;
    ea->fd=fd;
  }

  return r;
free_ctx:
  free(ea->ioctx.iobuf.buf);
  ea->ioctx.iobuf.buf=NULL;
close_fd:
  //close(fd);
free_memory:
  if( ea->envp ) {
    free(ea->envp);
  }
  return r;
}

static bool __validate_exec_request(pm_exec_request_t *rq)
{
  int i;

  if( rq->numargs > ARG_MAX || !rq->numargs ||
      rq->argsize > PATH_MAX+PM_ARGV_SIZE ) {
    return false;
  }

  for(i=0;i<rq->numargs;i++) {
    if( rq->arg_offsets[i] > PATH_MAX+PM_ARGV_SIZE ) {
      return false;
    }
  }

  if( rq->numargs == 1 ) {
    rq->args[PATH_MAX-1]='\0';
  } else {
    rq->args[rq->arg_offsets[1]-1]='\0';
  }
  return true;
}

static int __process_exec_request(pm_exec_request_t *rq,ipc_port_msg_info_t *msginfo)
{
  int res=-EBUSY;
  usrtc_node_t *node;
  pmp_exec_action_t *ea=NULL;

#ifdef CONFIG_PM_DEBUG
  __log("[ProcMgr] %d is executing '%s'.\n",msginfo->sender_pid,
        rq->args);
#endif

  if( !__validate_exec_request(rq) ) {
    return -EINVAL;
  }

  lock_action_tree();
  node=usrtc_lookup(&__action_tree,&msginfo->sender_pid);
  unlock_action_tree();

  if( !node ) {
    if( (ea=malloc(sizeof(*ea)) ) ) {
      int i;

      PMP_INIT_EXEC_ACTION(ea);

      ea->numargs=rq->numargs;
      ea->argsize=rq->argsize;
      memcpy(ea->args,rq->args,ea->argsize);
      for(i=0;i<rq->numargs;i++) {
        ea->arg_offsets[i]=rq->arg_offsets[i];
      }
    } else {
      return -ENOMEM;
    }

    lock_action_tree();
    node=usrtc_lookup(&__action_tree,(void *)(long)msginfo->sender_pid);
    if( !node ) {
      usrtc_insert(&__action_tree,(usrtc_node_t *)ea,(void *)(long)msginfo->sender_pid);
      res=0;
    }
    unlock_action_tree();

    if( !res ) {
      res=__run_executable_tophalf(ea,rq,msginfo->sender_pid);
      if( !res ) {
        res=task_ctl(msginfo->sender_pid,0,SYS_PR_CTL_DISINTEGRATE_TASK,__disint_port);
#ifdef CONFIG_PM_DEBUG
        console_printf("[ProcMgr] Disintegrating task %d. res=%d\n",
                       msginfo->sender_pid,res);
#endif
      }

      if( res ) {
        __remove_action(msginfo->sender_pid);
      }
    } else {
      free(ea);
    }
  }

  return res;
}

static void __process_client_request(void)
{
  pm_cmd_header_t header;
  ipc_port_msg_info_t msginfo;
  int r;
  pid_t rp=-EINVAL;
  pm_fork_request_t fork_rq;
  pm_exec_request_t exec_rq;
  pid_t key_pid=0;

  r=__syscall5(SC_PORT_RECEIVE,__client_port,0,(syscall_arg_t)&header,
               sizeof(header),(syscall_arg_t)&msginfo);
  if( !r ) {
    if( msginfo.msg_len >= sizeof(header) ) {
      if( header.cmd == __PROCMGR_CMD_FORK ) {
        if( msginfo.msg_len == PM_FORK_REQUEST_SIZE ) {
          r=ipc_port_read_msg(__client_port,msginfo.msg_id,PM_MSG_DATA_OFFT,
                              &fork_rq,sizeof(fork_rq));
          if( !r ) {
            rp=__process_fork_request(&fork_rq,&msginfo);
            if( !rp ) {
              rp=task_scheduler_control(fork_rq.child_pid,0,
                                        SYS_SCHED_CTL_SET_STATE,TASK_STATE_RUNNABLE);
              if( !rp ) {
                rp=fork_rq.child_pid;
              }
              key_pid=fork_rq.child_pid;
#ifdef CONFIG_PM_DEBUG
              console_printf("[ProcMgr] Starting new task: %d, KEY: %d\n",rp,fork_rq.child_pid);
#endif
            }
          }
        }
      } else if( header.cmd == __PROCMGR_CMD_EXEC ) {
        if( msginfo.msg_len == PM_EXEC_REQUEST_SIZE ) {
          r=ipc_port_read_msg(__client_port,msginfo.msg_id,PM_MSG_DATA_OFFT,
                              &exec_rq,sizeof(exec_rq));
          if( !r ) {
            rp=__process_exec_request(&exec_rq,&msginfo);
            __log("[ProcMgr] __process_exec_request(): %d\n",rp);

            /* In case we managed to initiate a top-half logic (which means that) target
             * executable was validated and a disintegration request was initiated
             * against the calling process), we must reply anyway to remove the message
             * from our port and dereference client's PID.
             */
          }
        }
      } else if( header.cmd == __PROCMGR_CMD_LOAD_LIBRARY ) {
        rp=-ENOSYS;
      }
    }
    ipc_port_reply_nmsg(__client_port,sizeof(rp),&rp,msginfo.msg_id);

    if( key_pid ) {
      __remove_action(key_pid);
    }
  }
}

static int __change_task_path(pid_t pid,char *path)
{
  char *buf,*nm;
  uni_rpc_t *rpc;
  chid_t nsch;
  int len,e=0;

  if(!path) {
    return EINVAL;
  }

  len=strlen(path)+sizeof(*rpc);
  if((buf=malloc(len))==NULL) 
    return ENOMEM;
  else memset(buf,0,len);

  if((nsch=ipc_channel_open(1,0,IPC_BLOCKED_ACCESS))>=0) {
    nm=buf+sizeof(*rpc);
    rpc=(uni_rpc_t*)buf;
    memcpy(nm,path,len-sizeof(uni_rpc_t));
    rpc->rpc_call=RPC_IOCTL;
    rpc->arg0=NX_CHGPATH;
    rpc->arg1=len-sizeof(uni_rpc_t);
    rpc->arg2=pid;
    e=ipc_channel_msgsnd(nsch,len,buf,sizeof(*rpc),rpc);
    if(e<0) e=-e;
    else e=rpc->rpc_call;
    ipc_channel_close(nsch);
  } else e=EIO;

  free(buf);

  return e;
}

static void __process_kernel_ack(void)
{
  disintegration_req_packet_t drp;
  ipc_port_msg_info_t msginfo;
  int r,len;
  usrtc_node_t *node;
  exec_attrs_t exec_attrs;
  exec_fmt_task_ctx_t task_ctx;
  char shortname[TASK_SHORTNAME_LEN];
  char *p,*d;

  r=__syscall5(SC_PORT_RECEIVE,__disint_port,0,(syscall_arg_t)&drp,
               sizeof(drp),(syscall_arg_t)&msginfo);
  if( !r ) {
    lock_action_tree();
    node=usrtc_lookup(&__action_tree,(void *)(long)drp.pid);
    lock_action_tree();

    if( node ) {
      pmp_exec_action_t *ea=(pmp_exec_action_t *)node;

 #ifdef CONFIG_PM_DEBUG
      __log("[ProcMgr] Continuing execution of PID %d (FMT: %s)\n",
            ea->target,ea->fmt->id);
#endif

      r=ea->fmt->load_image(ea->target,ea->fd,&ea->ioctx,ea->fmtctx,&task_ctx);
      ea->fmt->cleanup(ea->fmtctx);

      if( !r && !(r=setup_process_exec_attrs(ea,&exec_attrs)) ) {
        exec_attrs.entrypoint=task_ctx.entrypoint;

        /* Prepare shortname. */
        r=TASK_SHORTNAME_LEN-1;
        len=strlen(ea->args);
        p=ea->args+len-1;
        d=&shortname[TASK_SHORTNAME_LEN-1];
        *d='\0';
        d--;
        while( r && len && *p != '/' ) {
          *d--=*p--;
          r--;
          len--;
        }
        if( *p == '/' ) {
          d++;
        }

        /* Set shortname. */
        task_ctl(ea->target,0,SYS_PR_CTL_SET_SHORTNAME,(long)d);

#ifdef CONFIG_PM_DEBUG
        __log("[ProcMgr] New image for %d was successfully built.\n",
              ea->target);
        __log("          Entry: %p, Stack: %p, PTD: %p-%p\n",
              exec_attrs.entrypoint,exec_attrs.stack,
              exec_attrs.per_task_data,
              exec_attrs.per_task_data+sizeof(per_task_data_t));
#endif

	r=__change_task_path(ea->target/* pid */,ea->args/* new name */);
        r=task_ctl(ea->target,0,SYS_PR_CTL_REINCARNATE_TASK,(long)&exec_attrs);
      }
      if( r ) {
        __log_err("[ProcMgr] Failed to reincarnate %d. Terminating it.\n",
                  ea->target);
        kill(ea->target,SIGKILL);
      }
      __remove_action(ea->target);
    }
  }
}

static void __procmgr_main_loop(void)
{
  while(1) {
    pollfd_t pfds[PM_NUMPORTS];
    int r;

    pfds[0].fd=__client_port;
    pfds[0].events=POLLIN | POLLRDNORM;
    pfds[1].fd=__disint_port;
    pfds[1].events=POLLIN | POLLRDNORM;

#ifdef CONFIG_PM_DEBUG
    console_printf("[ProcMgr/0x%X] Entering main loop.\n",gettid());
#endif
    r=ipc_port_poll(pfds,PM_NUMPORTS,NULL);
#ifdef CONFIG_PM_DEBUG
    console_printf("[ProcMgr/0x%X] Got events: %d\n",gettid(),r);
#endif
    if( r < 0 ) {
      continue;
    }

    for(r=0;r<PM_NUMPORTS;r++) {
      if( pfds[r].revents ) {
        if( pfds[r].fd == __client_port ) {
          __process_client_request();
        } else if(pfds[r].fd == __disint_port ) {
          __process_kernel_ack();
        }
      }
    }
  }
}

int main(void)
{
  open_console();
  klog("[ProcMgr] Starting with PID = %d\n",getpid());

  __initialize_procmgr();
  __procmgr_main_loop();
  return 0;
}
