#include <sys/types.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <datalib/usrtc.h>
#include <errno.h>
#include <ipc/port.h>
#include <ipc/channel.h>
#include <ipc/rpc.h>
#include <ns/ns.h>
#include <errno.h>
#include <devio/devio.h>
#include <sys/poll.h>
#include <sys/thread.h>
#include <debug.h>
#include <ns/ns.h>
#include <sys/procmgr.h>
#include <ipc/rpc.h>
#include <sys/proc.h>
#include <sys/syscall.h>
#include <sys/scheduler.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <net/network.h>
#include <pthread.h>

#include "lwip/netif.h"
#include "lwip/init.h"
#include "lwip/ip_addr.h"
#include "lwip/sockets.h"
#include "netif/etherif.h"

struct __hose_priv {
  pthread_t intr_thread;
};

int __port,__peer_channel=-1;
int __peer_pid;
struct ip_addr __peer_addr,__myaddr;

static err_t __hose_output(struct netif *nif,struct pbuf *p,
                           struct ip_addr *ipaddr)
{
  int r,ret;
  int port=0;

//  console_printf("[%d] __hose_output(): Sending packet.\n",getpid());
//  console_printf("[%d] __hose_output(): packet size=%d bytes to 0x%X\n",
//                getpid(),p->len,ipaddr->addr);

  if( __peer_channel < 0 ) {
//    console_printf("[%d] __hose_output(): Trying to connect to %d\n",
//                   getpid(),__peer_pid);
    __peer_channel=__syscall3(SC_OPEN_CHANNEL,__peer_pid,port,IPC_BLOCKED_ACCESS);
    if( __peer_channel < 0 ) {
      console_printf("[%d] Can't connect to %d:%d ! r=%d\n",
                     getpid(),__peer_pid,port,__peer_channel);
      for(;;);
    }
//    console_printf("[%d] Channel opened. %d\n",getpid(),
//                   __peer_channel);
  }

  r=__syscall5(SC_PORT_SEND,__peer_channel,(syscall_arg_t)p->payload,p->len,
               (syscall_arg_t)&ret,sizeof(ret));
//  console_printf("[%d] __hose_output(): Packet was sent (%d)\n",
//                 getpid(),r);
  if( r != sizeof(ret) || ret != 0 ) {
    console_printf("[%d] Can't send a packet ! r=%d\n",
                   getpid(),r);
    for(;;);
  }
  return ERR_OK;
}

static void *__hose_in_logic(void *data)
{
  struct netif *nif=(struct netif *)data;
  ipc_port_msg_info_t msginfo;
  char buf[4096];
  int r,len;
  struct pbuf *p,*q;
  char *in_data;

  while(1) {
//    console_printf("[%d] __hose_in_logic(): Receiving packets on %s (port=%d)\n",
//                 getpid(),nif->name,__port);
    r=__syscall5(SC_PORT_RECEIVE,__port,IPC_BLOCKED_ACCESS,(syscall_arg_t)buf,sizeof(buf),
                 (syscall_arg_t)&msginfo);
//    console_printf("[%d] __hose_in_logic(): IP packet (%d bytes).\n",
//                   getpid(),msginfo.msg_len);
    if( r < 0 ) {
      console_printf("[%d] ERROR during packet receiving ! %d\n",getpid(),r);
      for(;;);
    }

    r=0;
    __syscall4(SC_PORT_REPLY,__port,msginfo.msg_id,&r,sizeof(r));
//    console_printf("nif->input: %p\n",nif->input);

    p=pbuf_alloc(PBUF_RAW,msginfo.msg_len,PBUF_POOL);
    if( !p ) {
      console_printf("[%d] ERROR during packet buffer allocation !\n",getpid());
      for(;;);
    }

    for(q=p,len=msginfo.msg_len,in_data=buf; q != NULL; q=q->next) {
//      console_printf("[%d] filling in pbuf with %d bytes of data.\n",
//                     getpid(),q->len);
      memcpy(q->payload,in_data,q->len);
      in_data+=q->len;
    }

    r=nif->input(p,nif);
    if( r ) {
      console_printf("[%d] ERROR during packet processing ! %d\n",getpid(),r);
    }
  }
//  console_printf("__hose_in_logic(): Exiting.\n");
}

static void __hose_status_callback(struct netif *nif)
{
  if( nif->flags & NETIF_FLAG_UP ) {
    struct __hose_priv *priv=(struct __hose_priv *)nif->state;

    console_printf("[%d] __hose_status_callback(): Starting up packet receiving.\n",
                   getpid());
    if( pthread_create(&priv->intr_thread,NULL,__hose_in_logic,nif) ) {
      console_printf("Can't create a thread !\n");
    }
    console_printf("Thread created.\n");
  } else {
    console_printf("[%d] __hose_status_callback(): Shutting down packet receiving.\n",
                   getpid());
  }
}

static err_t __hose_linkoutput(struct netif *nif,struct pbuf *p)
{
  console_printf("[%d] __hose_link_output(): Shutting down packet receiving.\n",
                 getpid());
  return 0;
}

static err_t __hose_init(struct netif *nif)
{
  __port=ipc_port_open(IPC_BLOCKED_ACCESS,32);
  if( __port != 0 ) {
    console_printf("Can't allocate IPC port ! %d\n",__port);
    return -1;
  }

  if( etherif_assign_name(nif) ) {
    return -1;
  }

  nif->output=__hose_output;
  nif->linkoutput=__hose_linkoutput;
  nif->status_callback=__hose_status_callback;

  nif->state=malloc(sizeof(struct __hose_priv));
  if( !nif->state ) {
    return -1;
  }
  memset(nif->state,0,sizeof(struct __hose_priv));

  return ERR_OK;
}

static int __hose_probe(network_host_t *nethost)
{
  struct netif *nif=(struct netif *)malloc(sizeof(*nif));
  netif_creds_t creds;
  int r=ENOMEM;

  if( nif ) {
    creds.init=__hose_init;

    r=nethost->register_interface(nif,&creds);
    if( r ) {
      free(nif);
    }
  }

  return r;
}

struct __network_driver hose_drv = {
  .probe=__hose_probe,
};
