#include <sys/types.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <datalib/usrtc.h>
#include <errno.h>
#include <ipc/port.h>
#include <ipc/channel.h>
#include <ipc/rpc.h>
#include <ns/ns.h>
#include <errno.h>
#include <devio/devio.h>
#include <sys/poll.h>
#include <sys/thread.h>
#include <debug.h>
#include <ns/ns.h>
#include <sys/procmgr.h>
#include <ipc/rpc.h>
#include <sys/proc.h>
#include <sys/syscall.h>
#include <sys/scheduler.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <net/network.h>
#include <sys/uevent.h>
#include <datalib/list.h>
#include <lwip/tcp.h>
#include <pthread.h>
#include <time.h>

#include "lwip/netif.h"
#include "lwip/init.h"
#include "lwip/ip_addr.h"
#include "lwip/sockets.h"
#include "ipv4/lwip/ip.h"

extern int __peer_pid;
extern struct ip_addr __peer_addr,__myaddr;
static bool __initiator;
struct netif *nif;

#define spinlock_t int

#define NUM_INPUT_THREADS  1

static int __num_input_threads=NUM_INPUT_THREADS;
static event_t __input_events[NUM_INPUT_THREADS];
static pthread_t __input_threads[NUM_INPUT_THREADS];
static spinlock_t __ithread_locks[NUM_INPUT_THREADS];
static list_head_t __ithread_lists[NUM_INPUT_THREADS];
static pthread_t __tcp_tmr_thread;

#define spinlock_lock_irq(s)
#define spinlock_unlock_irq(s)

#define spinlock_lock(s)
#define spinlock_unlock(s)

#define lock_ithreads_array()
#define unlock_ithreads_array()

char *test_pattern="TEST_PATTERN_123456";

static void __test(void)
{
  struct sockaddr_in server_addr;
  int s1,r;

  s1=lwip_socket(PF_INET,SOCK_STREAM,AF_INET);
  if( s1 < 0 ) {
    console_printf("Failed to create a socket !\n");
    return;
  }
  console_printf("[%d] Opened socket1: %d\n",
                 getpid(),s1);

  server_addr.sin_addr.s_addr = __peer_addr.addr;
  server_addr.sin_family = AF_INET;
  server_addr.sin_port = htons(22);

  console_printf("[%d] Trying to connect to 0x%X:%d\n",
                 getpid(),server_addr.sin_addr.s_addr,
                 ntohs(server_addr.sin_port));
  r=lwip_connect(s1,(struct sockaddr *)&server_addr,sizeof(server_addr));
  sleep(1);
  console_printf("[%d] Connect status: %d. Sending data over the socket (%d bytes).\n",
                 getpid(),r,strlen(test_pattern)+1);
  console_printf("[%d] Data to be send: '%s'\n",getpid(),test_pattern);
  r=send(s1,test_pattern,strlen(test_pattern)+1,0);
  console_printf("[%d] Send status: %d.\n",getpid(),r);
  sleep(100);
}

#define FIRST_PID 11

/**********************************************************************
*                Logic related to Network host (inner)
***********************************************************************/
static err_t __nhi_ip_input(struct pbuf *p, struct netif *inp)
{
  int id=0;

  p->stack_priv=inp;

  spinlock_lock_irq(&__ithread_locks[id]);
  list_add2tail(&__ithread_lists[id],&p->node);
  spinlock_unlock_irq(&__ithread_locks[id]);

  event_signal(&__input_events[id]);
  return 0;
}

static int __nhi_register_interface(struct netif *nif,netif_creds_t *creds)
{
  int r;
  struct ip_addr mask={INADDR_NONE};

  r=(netif_add(nif,IP_ADDR_ANY,&mask,IP_ADDR_ANY,NULL,creds->init,
               __nhi_ip_input) == nif ? 0 : -1);
  return r;
}

static int __nhi_unregister_interface(struct netif *nif)
{
  return 0;
}

static struct __network_host nethost_inner = {
  .register_interface=__nhi_register_interface,
  .unregister_interface=__nhi_unregister_interface
};

/**********************************************************************
*                Logic related to input threads (inner)
***********************************************************************/
static void *__nhi_input_thread(void *d)
{
  int id=(long)d;

  while( 1 ) {
    list_node_t *ln;
    struct pbuf *pb;
    struct netif *nif;

    event_wait(&__input_events[id]);

    spinlock_lock_irq(&__ithread_locks[id]);
    ln=list_node_first(&__ithread_lists[id]);
    list_del(ln);
    spinlock_unlock_irq(&__ithread_locks[id]);

    pb=(struct pbuf *)ln;
    nif=(struct netif *)pb->stack_priv;

    console_printf("[%d:%d] input_thread(): packet status: %d\n",
                   getpid(),gettid(),ip_input(pb,nif));
  }
}

static void *__tcp_timer_thread(void *d)
{
  struct timespec tv={0,TCP_FAST_INTERVAL*1000000};

  while(1) {
    nanosleep(&tv,NULL);
    tcp_tmr();
  }
  return NULL;
}

/**************************************************************************************/



extern struct __network_driver hose_drv;

static int __register_interface(void)
{
  char *ifname="eth0";
  struct ip_addr mask={0xffffff00};

  if( getpid() == FIRST_PID ) {
    __initiator=true;
    __peer_pid=FIRST_PID+1;
    __peer_addr.addr=0xC0A80002;   /* 192.168.0.2 */
    __myaddr.addr=0xC0A80001;     /* 192.168.0.1 */
  } else {
    __initiator=false;
    __peer_pid=FIRST_PID;
    __peer_addr.addr=0xC0A80001;   /* 192.168.0.1 */
    __myaddr.addr=0xC0A80002;     /* 192.168.0.2 */
  }
  console_printf("[%d] Peer PID: %d, Initiator: %d\n",
                 getpid(),__peer_pid,__initiator);

  if( hose_drv.probe(&nethost_inner) ) {
    console_printf("[%d] Can't probe device !\n",getpid());
    return -1;
  }

  nif=netif_find(ifname);
  if( !nif ) {
    console_printf("Can't find interface '%s' !\n",ifname);
  }
  console_printf("[%d] Located interface: %s\n",getpid(),
                 nif->name);
  netif_set_ipaddr(nif,&__myaddr);
  netif_set_gw(nif,&__myaddr);
  netif_set_netmask(nif,&mask);
  netif_set_default(nif);
  netif_set_up(nif);
  console_printf("[%d] Interface configured.\n",
                 getpid());
  return 0;
}

static void *__server_thread(void *d)
{
  int s1,r,sc;
  struct sockaddr_in server_addr,client_addr;
  socklen_t slen;

  s1=lwip_socket(PF_INET,SOCK_STREAM,AF_INET);
  if( s1 < 0 ) {
    console_printf("Failed to start the server !\n");
    for(;;);
  }

  server_addr.sin_addr.s_addr = __myaddr.addr;
  server_addr.sin_family = AF_INET;
  server_addr.sin_port = htons(22);

  r=bind(s1,(struct sockaddr *)&server_addr,sizeof(server_addr));
  if( r < 0 ) {
    console_printf("bind(%d) failed ! %d\n",s1,r);
    for(;;);
  }

  r=listen(s1,5);
  if( r < 0 ) {
    console_printf("listen(%d) failed ! %d\n",s1,r);
    for(;;);
  }

  console_printf("[%d:%d] calling accept()..\n",
                 getpid(),gettid());
  slen=sizeof(server_addr);
  r=accept(s1,(struct sockaddr *)&client_addr,&slen);
  if( r < 0 ) {
    console_printf("accept(%d) failed ! %d\n",s1,r);
    for(;;);
  }
  console_printf("[%d:%d] Accepted connection from 0x%X! %d\n",
                 getpid(),gettid(),client_addr.sin_addr.s_addr,r);
  sc=r;
  while(1) {
    char buf[64];

    memset(buf,0,sizeof(buf));
    console_printf("[%d:%d] Waiting for incoming data on socket %d ...\n",
                   getpid(),gettid(),sc);
    r=recv(sc,buf,sizeof(buf),0);

    if( r <= 0 ) {
      console_printf("[%d:%d] Error during 'recv()': %d ...\n",
                     getpid(),gettid(),r);
      for(;;);
    }
    console_printf("[%d:%d] received %d bytes from peer.\n",
                   getpid(),gettid(),r);
    console_printf("[%d:%d] Received data: '%s'\n",
                   getpid(),gettid(),buf);
  }
  return NULL;
}

err_t ip_input(struct pbuf *p, struct netif *inp)
{
  return lwip_ip_input(p,inp);
}

static int __initialize(void)
{
  int i,r;

  lwip_init();

  /* Start TCP timer threads. */
  if( pthread_create(&__tcp_tmr_thread,NULL,__tcp_timer_thread,NULL) ) {
    console_printf("[NetMgr] Can't start TCP timer thread !\n");
    return -1;
  }

  /* Initialize input threads. */
  for(i=0;i<NUM_INPUT_THREADS;i++) {
    list_init_head(&__ithread_lists[i]);

    r=event_init(&__input_events[i]);
    if( r ) {
      console_printf("[NetMgr] Can't create event for input thread !\n");
      return r;
    }

    if( pthread_create(&__input_threads[i],NULL,__nhi_input_thread,
                       (void *)(long)i) ) {
      console_printf("[NetMgr] Can't create input thread !\n");
      return -1;
    }
  }
  return 0;
}

int main(void)
{
  open_console();
  console_printf("** LWIP test starting with PID %d.\n",getpid());

  if( __initialize() ) {
    console_printf("Initialization failed !\n");
    for(;;);
  }

  if( !__register_interface() ) {
    if( __initiator ) {
      console_printf("[%d] Waiting for the peer to start (3 seconds)..\n",getpid());
      sleep(3);
      console_printf("[%d] Calling tests.\n",getpid());
      __test();
    } else {
      pthread_t t;

      console_printf("[%d] Launching a server thread.\n",getpid());
      if( pthread_create(&t,NULL,__server_thread,NULL) ) {
        console_printf("Failed to create server thread !\n");
        for(;;);
      }
    }
  } else {
    console_printf("Failed to register interface !\n");
    for(;;);
  }

  console_printf("[%d] Entering command loop ...\n",getpid());
  sleep(100);
  console_printf("LWIP test finished.\n");
  for(;;);
  return 0;
}
