/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * Based on kconfig from Linux kernel:
 *   Copyright (C) 2002 Roman Zippel <zippel@linux-m68k.org>
 *   Released under the terms of the GNU GPL v2.0.
 *
 * Copyright (C) 2006,2007,2008,2009 Jari OS Core Team <http://jarios.org>
 * Copyright (C) 2009 Dmitry Balakshin <balakshin@jarios.org>
 */

#ifndef SYMBOL_H
#define SYMBOL_H

#ifndef __INSIDE_PARSER
#include "sconfig.tab.h"
#endif

struct symbol;

struct menu
{
  char *name;
  struct symbol *list;
};

/* string, case */
struct simple
{
  char *id;
  char *desc;
};

/* switch, bool */
struct expandable
{
  char *id;
  char *desc;
  struct symbol *list;
};

struct symbol
{
  int type;
  union
  {
    struct menu menu;
    struct simple simple;
    struct expandable exp;
  } t;
  union
  {
    bool yesno;                 /* T_BOOL */
    char *str;                  /* T_STRING */
    struct symbol *choice;      /* T_SWITCH */
  } v;
  struct symbol *parent;
  struct symbol *prev, *next;
};

void add_symbol(int type);
bool set_menu_name(char *name);
bool set_simple_id(char *id);
bool set_simple_desc(char *desc);
bool set_exp_id(char *id);
bool set_exp_desc(char *desc);

bool free_symbol(struct symbol *sym);
struct symbol *get_root_symbol(void);
bool is_empty_symbol(const struct symbol *sym);
struct symbol *find_symbol(struct symbol *first, const char *id);

#endif  /* SYMBOL_H */
