/* This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * (c) Copyright 2006,2007,2008,2009 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2005,2008,2009 MadTirra <madtirra@jarios.org>
 * Copyright (C) 2009 Dmitry Balakshin <balakhsin@jarios.org>
 *
 * Jari OS login program.
 *
 */

#include <limits.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/utsname.h>
#include <sys/wait.h>
#include <errno.h>
#include <fcntl.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <unistd.h>
#include <pwd.h>

#include <ns/ns.h>
#include <termios.h>

static const char DEFAULT_PTM[] = "/dev/ptm0";

int main(int argc, char *argv[])
{
  const char *ptm;
  struct utsname uts;
  struct termios t;
  char login[LOGIN_NAME_MAX];
  char passwd[256];
  pid_t pid;
  int status;
  struct passwd *pwent;

  pid = fork();
  if (pid == -1)
    exit(EXIT_FAILURE);
  if (pid != 0)
    exit(EXIT_SUCCESS);

  ptm = DEFAULT_PTM;
  if (argc >= 2)
    ptm = argv[1];

  /* Setup terminal */
  if (open(ptm, O_RDONLY) == -1)
    exit(EXIT_FAILURE);
  if (open(ptm, O_WRONLY) == -1)
    exit(EXIT_FAILURE);
  if (open(ptm, O_WRONLY) == -1)
    exit(EXIT_FAILURE);

  ioctl(0, TIOC_GETATTR, &t);
  t.c_lflag |= ICANON;
  ioctl(0, TIOC_SETATTR, &t);

start_session:

  if (uname(&uts) != -1)
    printf("\nThis is %s (%s %s %s)\n",uts.sysname, uts.nodename,
           uts.machine, uts.version);
  printf("\n");

  for (;;) {
    /* ask login name */
    memset(login, 0, sizeof(login));
    printf("login: ");
    fflush(stdout);
    if (fgets(login, sizeof(login), stdin) == NULL)
      continue;

    if (login[strlen(login) - 1] == '\n')
      login[strlen(login) - 1] = '\0';

    /* ask password */
    memset(passwd, 0, sizeof(passwd));
    printf("password: ");
    fflush(stdout);

    /* turn echo off */
    t.c_lflag &= ~ECHO;
    ioctl(0, TIOC_SETATTR, &t);

    status = (fgets(passwd, sizeof(passwd), stdin) != NULL);

    t.c_lflag |= ECHO;
    ioctl(0, TIOC_SETATTR, &t);
    printf("\n\n");

    if (!status)
      continue;
    if (passwd[strlen(passwd) - 1] == '\n')
      passwd[strlen(passwd) - 1] = '\0';

    /* check login/password */
    if (strlen(login) == 0)
      continue;
    pwent = getpwnam(login);

    pid = fork();
    if (pid == -1) {
      fprintf(stderr, "fork() failed: %d\n", errno);
      continue;
    }
    if (pid == 0) {
      if (pwent == NULL) {
        setenv("USER", login, 1);
        execve("/bin/josh", NULL, NULL);
      } else {
        setenv("USER", login, 1);
	setgid(pwent->pw_gid);
	setuid(pwent->pw_uid);
	execve(pwent->pw_shell, NULL, NULL);
      }
      fprintf(stderr, "execve() failed: %d\n", errno);
      exit(EXIT_FAILURE);
    }

    waitpid(pid, &status, 0);
    printf("\nlogout\n");
    goto start_session;
  }

  return 0;
}
