#include <stdlib.h>
#include <string.h>
#include <test/framework.h>
#include <unistd.h>
#include <sys/mman.h>
#include <sys/syscall.h>
#include <unistd.h>
#include <pthread.h>
#include <sys/thread.h>
#include <ns/ns.h>
#include <ipc/channel.h>

#define CHECK_MMAP_ADDR(addr, fn)                               \
  do {                                                          \
    (addr) = (fn);                                              \
    if ((addr) == MAP_FAILED) {                                 \
      printf("Failed to map (%s): [%d]\n", #fn, errno); \
      abort();                                                  \
    }                                                           \
  } while (0)


#define MAX_MSG_SIZE 128
#define SHARED_OBJ_PAGES 10
static void TC_blablabla(long *unsued)
{
  int chpid, ret;
  struct memobj_info mmo_info;
  long memobj_id;
  prtid_t port;

  port = ipc_port_open(IPC_BLOCKED_ACCESS, 32);
  if (port < 0) {
    printf("Failed to create port %d\n", port);
    abort();
  }

  memset(&mmo_info, 0, sizeof(mmo_info));
  mmo_info.size = SHARED_OBJ_PAGES * PAGE_SIZE;
  mmo_info.nature = MMO_NTR_PROXY;
  mmo_info.backend_port = port;
  mmo_info.lifetype = MMO_LIFE_LEECH;
  mmo_info.flags = MEMOBJ_SHARED;
  ret = __syscall1(SC_MEMOBJ_CREATE, (uintptr_t)&mmo_info);
  if (ret < 0) {
    printf("Failed to create memobj. %d\n", ret);
    abort();
  }

  memobj_id = mmo_info.id;
  printf("Create memory object with id %d\n", memobj_id);
  chpid = fork();
  if (chpid < 0) {
    printf("FORK failed [%d] (errno = %d)\n", chpid, errno);
    abort();
  }
  else if (chpid > 0) {
    char msg[MAX_MSG_SIZE];
    struct mmev_hdr evhdr;
    ipc_port_msg_info_t msg_info;
    void *shared_obj[SHARED_OBJ_PAGES];

    memset(shared_obj, 0, sizeof(void *) * SHARED_OBJ_PAGES);
    printf("Create child %d\n", chpid);
    for (;;) {
      ret = __syscall5(SC_PORT_RECEIVE, port, IPC_BLOCKED_ACCESS, (uintptr_t)msg,
                       MAX_MSG_SIZE, (uintptr_t)&msg_info);
      if (ret) {
        printf("Failed to receive a message! %d\n", ret);
        abort();
      }

      evhdr = *(struct mmev_hdr *)msg;
      if (evhdr.event == MMEV_PAGE_FAULT) {
        struct mmev_fault flt;
        void *myaddr;
        static long testcounter = 0;
        
        printf("Got page fault from memobj %d\n", evhdr.memobj_id);
        flt = *(struct mmev_fault *)msg;
        printf("PFMASK = %#x, offset = %#x\n", flt.pfmask, flt.offset);
        if (shared_obj[flt.offset]) {
          printf("OFFSET %#x found in a cache!\n", flt.offset);
          myaddr = shared_obj[flt.offset];          
        }
        else {
          CHECK_MMAP_ADDR(myaddr, mmap(NULL,PAGE_SIZE, PROT_READ | PROT_WRITE, MAP_SHARED | MAP_POPULATE,
                                       memobj_id, flt.offset * PAGE_SIZE));
        }
        *(long *)myaddr = testcounter++;
        shared_obj[flt.offset] = myaddr;
        ret = __syscall4(SC_PORT_REPLY, port, msg_info.msg_id, (uintptr_t)&myaddr, sizeof(uintptr_t));
        if (ret) {
          printf("Failed to reply: %d\n", ret);
          abort();
        }
      }
      else if (evhdr.event == MMEV_MSYNC) {
        struct mmev_msync msync;
        int repl = 0;

        printf("Got msync event from memobj %d\n", evhdr.memobj_id);
        msync = *(struct mmev_msync *)msg;
        printf(".... by offset %d\n", msync.offset);
        if (shared_obj[msync.offset]) {
          void *addr = shared_obj[msync.offset];

          printf("===============================\n");
          printf("[%#x] %s\n", msync.offset, (char *)addr);
          printf("===============================\n");
        }

        ret = __syscall4(SC_PORT_REPLY, port, msg_info.msg_id, (uintptr_t)&repl, sizeof(int));
        if (ret) {
          printf("Failed to reply: %d\n", ret);
          abort();
        }
      }
      else {
        printf("WTF? unknown event type %d\n", evhdr.event);
        abort();
      }
    }

    printf("I'm old and ill, it's time to die... Let the heaven open its foors :(\n");
    __syscall1(SC_EXIT, 0);

  }
  else {
    void *addr, *addr2;
    int i, ret, srv_pid, repl;
    prtid_t srv_port;
    chid_t channel;
    
    printf("I'm a child of %d and now I'm going to sleep...\n", getppid());
    sleep(2);    
    CHECK_MMAP_ADDR(addr, mmap(NULL, PAGE_SIZE * 3, PROT_READ | PROT_WRITE,
                               MAP_SHARED, memobj_id, PAGE_SIZE * 2));
    for (i = 0; i < 3; i++) {
      printf("OFFSET %#x, got %ld\n",
                     PAGE_SIZE * 2 + i * PAGE_SIZE, *(long *)((char *)addr + i * PAGE_SIZE));
      *(long *)((char *)addr + i * PAGE_SIZE) = 0xDEADBEEF + i;
    }

    CHECK_MMAP_ADDR(addr2, mmap(NULL, PAGE_SIZE * 2, PROT_READ | PROT_WRITE,
                               MAP_SHARED, memobj_id, 0));
    for (i = 0; i < 2; i++) {
      printf("OFFSET %#x, got %ld\n", i * PAGE_SIZE,
                     *(long *)((char *)addr2 + i * PAGE_SIZE));
      *(long *)((char *)addr2 + i * PAGE_SIZE) = 0xBEEF + i;
    }

    srv_pid = ns_getgixpid();
    printf("SERVER PID = %d\n", srv_pid);
    if (!srv_pid) {
      printf("Failed to get gix pid! %d [errno = %d]\n", srv_pid, errno);
      abort();
    }

    srv_port = 0;
    printf("SERVER PORT = %d\n", srv_port);
    channel = __syscall3(SC_OPEN_CHANNEL, srv_pid, srv_port, IPC_BLOCKED_ACCESS);
    if (channel < 0) {
      printf("Failed to open channel to port %d of server %d\n [RET = %d]\n",
                     srv_port, srv_pid, channel);
      abort();
    }

    ret = __syscall5(SC_PORT_SEND, channel, (uintptr_t)&memobj_id, sizeof(long),
                     (uintptr_t)&repl, sizeof(int));
    if (ret < 0) {
      printf("SC_PORT_SEND failed: %d\n", ret);
      abort();
    }

    printf("Good, server has just responded us.\n");
    for (;;);
    //sleep(5);
    //__syscall1(SC_EXIT, 0);
  }
}
  
test_struct_t test_struct = {
  .pause_seconds = 1,
  .cases = {
    {
      .name = "simple_memobj_test",
      .function = TC_blablabla,
      .description = "Very simple memobj test...",
      .nolaunch_on_fails = false,
      .abort_on_fail = true,
      .arg_list = ARG_LIST { "fake", ARG_LIST_END },
      .args = ARGS_CHAIN {
        ARGS { 0 },
        ARGS_END,
      },
    },

    CASE_LIST_END,
  },
};
