#include <sys/types.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ipc/port.h>
#include <ipc/channel.h>
#include <ipc/rpc.h>
#include <errno.h>
#include <sys/syscall.h>
#include <time.h>

#include <sys/proc.h>
#include <pthread.h>
#include <sys/thread.h>
#include <signal.h>
#include <time.h>
#include <pthread.h>
#include <sys/swks.h>

#define SPIN() for(;;){}

static char *t="12345";

void __sighandler(int sig)
{
  printf("**** SIG: %d\n",sig);
}

#define FAULT_GRANULARITY  100000

static inline long __get_cs(void)
{
  long __cs;

  __asm__ __volatile__("mov %%cs,%%eax":"=r"(__cs):);
  return __cs;
}

void __sa_sighandler(int signum,siginfo_t *siginfo,void *data)
{
  char b[128];
  static int __sigcount=0;

  if( !(__sigcount % FAULT_GRANULARITY) ) {
    sprintf(b,"COUNT: %d, __sa_sighandler(): SIG=%d, >si_signo=%d, >si_code=%d, ADDR=%p\n",
          __sigcount,signum,siginfo->si_signo,siginfo->si_code,siginfo->si_addr);
    printf(b);

  }

  if( __get_cs() != 0x23 ) {
     *(char *)0x333=0x55;
  }

  __sigcount++;
}

void __signal_test(void)
{
  struct sigaction sact,oact;
  int r;
  char b[256];

  memset(&sact,0,sizeof(sact));
  sact.sa_sigaction=__sa_sighandler;
  sact.sa_flags=SA_SIGINFO;

  r=sigaction(SIGSEGV,&sact,&oact);
  if( r ) {
    printf("Can't install signal handler !\n");
    SPIN();
  }

  sprintf(b,"OLD HANDLER: sa_handler=%p, sa_sigaction=%p, FAULT GRANULARITY: %d\n",
          oact.sa_handler,oact.sa_sigaction,FAULT_GRANULARITY);
  printf(b);
  while(1) {
    *(char *)0x200=0x55;
  }
  printf("Hmmmmm .... Skipped fault ?\n");
}

/*
#define sigwait(a, b) { \
  struct timespec ts;   \
                        \
  ts.tv_sec = 0;        \
  ts.tv_nsec = 50000000; \
  nanosleep(&ts, NULL); \
}
*/

#define CITEM 10

#define MTICK  100

void __swks_test(void)
{
  swks_t *swks=get_swks();
  long t,count=0;

  while(1) {
    char b[128];

    if( !(swks->system_clock_ticks % MTICK) ) {
      sprintf(b,"ticks: %d\n",swks->system_clock_ticks);
      printf(b);
    }
  }
}


void __bd_test(void)
{
  int i;
  char b[128];
  swks_t *swks=get_swks();

  for(i=0;i<10000;i++) {
    if(!(i % CITEM)) {
      sprintf(b,"i=%d, SYSTEM TICKS: %d, GRANULARITY: %d\n",
              i,swks->system_clock_ticks,CITEM);
      printf(b);
    }
    sigwait(1,2);
  }
}

#define NUMTIMERS 5
#define TIMER_USEC_STEP  0 /* 1200000 */
#define TIMER_SEC_STEP   0

#define TIMERS_TO_DISARM 3

static void __timer_tests(void)
{
  struct sigevent ev;
  timer_t timers[NUMTIMERS];
  int i,r,rounds;
  sigset_t set;
  long t;
  struct itimerspec v1,v2;
  char b[128];
  int timer_sig=SIGUSR1;
  int __counter;
  swks_t *swks=get_swks();
  long nsec;
  timer_t to_disarm[TIMERS_TO_DISARM]={0,1,2,3,4};
  timeval_t tv={0,100000000};
  int test_rounds;
  siginfo_t info;
  int use_siginfo=1;

   sprintf(b,"[1] errno=%d\n",errno);
  printf(b);

  memset(&ev,0,sizeof(ev));
  ev.sigev_notify=SIGEV_SIGNAL_THREAD;
  ev.sigev_signo=timer_sig;
  ev.tid=gettid();

  for(i=0;i<NUMTIMERS;i++) {
    ev.sigev_value.sival_int=500+i;
    if( timer_create(CLOCK_REALTIME,&ev,&timers[i] ) ) {
      sprintf(b,"* Failed to create timer N %d !\n",i);
      printf(b);
      SPIN();
    }
  }

  sigemptyset(&set);
  sigaddset(&set,SIGUSR1);

//  for(i=0;i<100;i++) {
//    printf("SLEEP !\n");
//    nanosleep(&tv,NULL);
// }

  if( sigprocmask(SIG_BLOCK,&set,NULL) ) {
    printf("Can't block the signal !\n");
  }

  for(test_rounds=0;test_rounds<10;test_rounds++) {
    memset(&v1,0,sizeof(v1));
    t=(swks->system_clock_ticks+2250)/1000;

    sprintf(b,"! system ticks: %d, t=%d\n",
            swks->system_clock_ticks,t);
    printf(b);
    nsec=0;
    v1.it_interval.tv_sec=0;
    v1.it_interval.tv_nsec=450000000;

    for(i=0;i<NUMTIMERS;i++) {
      v1.it_value.tv_sec=t;
      v1.it_value.tv_nsec=nsec;

      r=timer_settime(timers[i],TIMER_ABSTIME,&v1,NULL);
      if( r ) {
        sprintf(b,"* Failed to arm timer N %d ! r=%d, errno=%d\n",i,r,errno);
        printf(b);
        SPIN();
      }
      t+=TIMER_SEC_STEP;
      nsec+=TIMER_USEC_STEP;
    }

    printf("Disarming timers.\n");
   for(i=0;i<TIMERS_TO_DISARM;i++) {
      v1.it_value.tv_sec=0;
      v1.it_value.tv_nsec=0;
      v1.it_interval.tv_sec=0;
      v1.it_interval.tv_nsec=0;

//      sprintf(b,"Disarming timer N %d\n",timers[i]);
//      printf(b);

      r=timer_settime(to_disarm[i],TIMER_ABSTIME,&v1,NULL);
      if( r ) {
        sprintf(b,"* Failed to disarm timer N %d !\n",to_disarm[i]);
        printf(b);
        SPIN();
      }
    }
//    printf("Timers disarmed.\n");
//    for(;;);

#if 0 /* Immediately deactivate some timers. */
    for(i=0;i<TIMERS_TO_DISARM;i++) {

#if 1 /* Rearm timers immediately. */
      v1.it_value.tv_sec=i*1000+1000;
      v1.it_value.tv_nsec=0;
      v1.it_interval.tv_sec=0;
      v1.it_interval.tv_nsec=230000000;

      r=timer_settime(to_disarm[i],TIMER_ABSTIME,&v1,NULL);
      if( r ) {
        sprintf(b,"* Failed to modify timer N %d !\n",to_disarm[i]);
        printf(b);
        SPIN();
      }
#endif
      v1.it_value.tv_sec=0;
      v1.it_value.tv_nsec=0;
      v1.it_interval.tv_sec=0;
      v1.it_interval.tv_nsec=0;

      r=timer_settime(timers[i],TIMER_ABSTIME,&v1,NULL);
      if( r ) {
        sprintf(b,"* Failed to disarm timer N %d !\n",to_disarm[i]);
        printf(b);
        SPIN();
      }
  }

#endif

    sprintf(b,"Waiting for the signal to arrive. errno=%d\n",errno);
    printf(b);

    errno=0;
    rounds=0;
    while( rounds < 100 ) {
      int sn=-1;

      errno=0;

      if( use_siginfo ) {
        r=sigwaitinfo(&set,&info);
      } else {
        r=sigwait(&set,&sn);
      }

      if( r < 0 ) {
        sprintf(b,"[!!] FAILED !!! (use_siginfo=%d) r=%d\n",
                use_siginfo,r);
        printf(b);
        SPIN();
      }

      if( !(rounds % 25) ) {
        if( use_siginfo ) {
          sprintf(b,"sigwaitinfo(): TICKS: %d, Beeeee ! signal received: %d (si_value=%d)\n",
                  swks->system_clock_ticks,info.si_signo,info.si_value);
        } else {
          sprintf(b,"sigwait(): TICKS: %d, Beeeee ! signal received: %d\n",
                  swks->system_clock_ticks,sn);
        }
        printf(b);
      }

    if( rounds <= 14 ) {
      if( use_siginfo ) {
        sprintf(b, "[%d] * Got a signal at %d ! r=%d, si_signo=%d, si_value=%d\n",
                swks->system_clock_ticks,r,info.si_signo,info.si_value);
      } else {
        sprintf(b, "[%d] * Got a signal at %d ! r=%d, sn=%d, errno=%d\n",
                swks->system_clock_ticks,r,sn,errno);
      }
      printf(b);
    }

    if( ++rounds == 14 ) {
      v1.it_value.tv_sec=0;
      v1.it_value.tv_nsec=1200000;
      v1.it_interval.tv_sec=0;
      v1.it_interval.tv_nsec=1200000;

      printf( ">>>>> Rearming timer.\n" );
      r=timer_settime(timers[0],0,&v1,NULL);
      sprintf(b,"r=%d\n",r);
      printf(b);
     }
    }

    sprintf(b,"ROUNDS=%d, TICKS=%d.\n",rounds,swks->system_clock_ticks);
    printf(b);

    for(i=0;i<TIMERS_TO_DISARM;i++) {
      v1.it_value.tv_sec=0;
      v1.it_value.tv_nsec=0;
      v1.it_interval.tv_sec=0;
      v1.it_interval.tv_nsec=0;

      sprintf(b,"Disarming timer N %d . ",timers[i]);
      printf(b);
     errno=0;
      r=timer_settime(timers[i],0,&v1,NULL);
      if( r ) {
        sprintf(b,"* Failed to deactivate timer N %d ! r=%d, errno=%d\n",timers[i],
	        r,errno);
        printf(b);
        SPIN();
      }
      printf("Done !\n");
    }

    sprintf(b,"** Timers deactivated. TICKS=%d\n",
            swks->system_clock_ticks);
    printf(b);

#if 1 /* Immediately deactivate some timers. */
    for(i=0;i<TIMERS_TO_DISARM;i++) {

#if 1 /* Rearm timers immediately. */
      v1.it_value.tv_sec=30;
      v1.it_value.tv_nsec=0;
      v1.it_interval.tv_sec=0;
      v1.it_interval.tv_nsec=230000000;

      r=timer_settime(to_disarm[i],0,&v1,NULL);
      if( r ) {
        sprintf(b,"* Failed to modify timer N %d !\n",to_disarm[i]);
        printf(b);
        SPIN();
      }
#endif
      v1.it_value.tv_sec=0;
      v1.it_value.tv_nsec=00000000;
      v1.it_interval.tv_sec=0;
      v1.it_interval.tv_nsec=000000000;

      r=timer_settime(to_disarm[i],TIMER_ABSTIME,&v1,NULL);
      if( r ) {
        sprintf(b,"* Failed to disarm timer N %d !\n",to_disarm[i]);
        printf(b);
        SPIN();
      }
    }
#endif
    sprintf(b,">>>>>> ROUND %d IS OVER <<<<<<<\n",test_rounds);
    printf(b);
    for(;;);
    nanosleep(&tv,NULL);
  }
}

static void *__timer_test_thread(void *arg)
{
  struct sigevent ev;
  timer_t timer1;
  int r,rounds;
  sigset_t set;
  struct itimerspec v1;
  char b[128];
  int timer_sig=SIGUSR1;
  int __counter;
  siginfo_t info;

  sprintf(b,"[++] My TID: 0x%X\n",gettid());
  printf(b);

  memset(&ev,0,sizeof(ev));
  ev.sigev_notify=SIGEV_SIGNAL_THREAD;
  ev.sigev_signo=timer_sig;
  ev.sigev_value.sival_int=500;
  ev.tid=gettid();

  r=timer_create(CLOCK_REALTIME,&ev,&timer1);
  if( r ) {
    sprintf(b,"[!!] Failed to create timer N1 ! %d, errno=%d\n",r,errno);
    printf(b);
    SPIN();
  }

  printf("[++] Timer created.\n");
  sigemptyset(&set);
  sigaddset(&set,SIGUSR1);

  if( sigprocmask(SIG_BLOCK,&set,NULL) ) {
    printf("Can't block the signal !\n");
  }

  memset(&v1,0,sizeof(v1));
  v1.it_value.tv_sec=0;
  v1.it_value.tv_nsec=450000000;
  v1.it_interval.tv_sec=0;
  v1.it_interval.tv_nsec=450000000;

  r=timer_settime(timer1,0,&v1,NULL);
  if( r ) {
   printf("Failed to arm timer 1\n");
   SPIN();
  }

  sprintf(b,"[++] Waiting for the signal to arrive. errno=%d\n",errno);
  printf(b);
  errno=0;
  rounds=0;
  while( rounds < 100 ) {
    int sn=-1;

    errno=0;
    r=sigwait(&set,&sn);

    if( r < 0 ) {
      sprintf(b,"[!!] r=%d\n",r);
      printf(b);
      SPIN();
    }

    if( !(rounds % 25) ) {
      sprintf(b,"[++] Beeeee ! got a signal %d\n",sn);
      printf(b);
    }

    sprintf(b, "[++] Got a signal ! r=%d, sn=%d, errno=%d\n",
    r,sn,errno);
    printf(b);

    if( ++rounds == 3 ) {
      v1.it_value.tv_sec=0;
      v1.it_value.tv_nsec=1200000;
      v1.it_interval.tv_sec=0;
      v1.it_interval.tv_nsec=1200000;

      printf( "[++]  Rearming timer.\n" );
      r=timer_settime(timer1,0,&v1,NULL);
      sprintf(b,"    r=%d\n",r);
      printf(b);
    }
  }

   sprintf(b,"[++] ROUNDS=%d\n",rounds);
   printf(b);
   return 0x1122334466554433;
}

void __timer_thread_tests(void)
{
  pthread_t t;
  void *retval;
  char b[128];
  int r;
  timeval_t tv={3,0};

  if( pthread_create(&t,NULL,__timer_test_thread,NULL) ) {
    printf("[!!] Can't launch a thread !\n");
    SPIN();
  }
  printf("[**] Sleeping for 3 seconds...\n");
  nanosleep(&tv,NULL);
  printf("[**] Joining target thread...\n");
  r=pthread_join(t,&retval);
  sprintf(b,"[**] r=%d, retval=%p\n",r,retval);
  printf(b);
  printf("[**] Joining target thread yet again ...\n");
  retval=NULL;
  r=pthread_join(t,&retval);
  sprintf(b,"[**] r=%d, retval=%p\n",r,retval);
  printf(b);
}

void __fork_test(void)
{
  int a=10;
  int c=90000;
  char t[74];
  int pid;
  char b[128];

  //pid=fork();
  switch( pid ) {
    case 0:
      for(;;);
    case -1:
      printf("** Can't fork() !\n");
      break;
    default:
      sprintf(b,"Child PID: %d\n",pid);
      printf(b);
      break;
  }
}

#define NUM_EXIT_THREADS  5

void *__exit_thread(void *arg)
{
  char b[128];
  int i;

 // if( (gettid() & 0xFF) == 5 ) {
//    __syscall1(SC_EXIT,0);
 // }

  sprintf(b,"Thread 0x%X starting.\n",gettid());
  printf(b);

//  if( gettid() == 0x80005 ) {
//    *(char *)0x100=10;
//    __syscall1(SC_EXIT,0);
//    sprintf(b,"Thread 0x%X is exiting.\n",gettid());
//    printf(b);
//    pthread_exit(0x11223344);
//    printf("Hmmmm.... Not exited ???!!!\n");
//  }

/*
  if( (gettid() & 0xFF) == 2 ) {
    for(i=0;i<300000;i++) {
    }
    printf("Initiating process exit ...\n");
    __syscall1(SC_EXIT,0);
    printf("Not exited ???!!!\n");
  }
*/
  for(;;);

  return NULL;
}

void __exit_test(void)
{
  int i,r;
  pthread_t threads[NUM_EXIT_THREADS];
  char b[128];
  timeval_t tv={1,0};

  for(i=0;i<NUM_EXIT_THREADS;i++) {
    r=pthread_create(&threads[i],NULL,__exit_thread,NULL);
    if( r ) {
      printf("Can't create thread !\n");
      SPIN();
    }
  }
  printf("Sleeping for 1 second ...\n");
  nanosleep(&tv,NULL);
  printf("Killing thread.\n");
  if( thread_kill(getpid(),threads[0].__tid,SIGUSR1) ) {
    printf("Can't kill thread !\n");
    SPIN();
  }
  //printf("Joining ...\n");
  if( pthread_join(threads[0],NULL) ) {
    printf("Can't join !\n");
    SPIN();
  }
  //nanosleep(&tv,NULL);
  //printf("Exiting ...\n");
  //__syscall1(SC_EXIT,0);
}

int main(void)
{
  char b[128];

   sprintf(b,"[Zero] errno=%d\n",errno);

  printf("Starting POSIX timers test (sleeping for 1 second).\n");
  //sleep(1);
  __timer_tests();
  //__signal_test();
  //__timer_thread_tests();
  //__sigwait_test();
  //__exit_test();
  //__thread_test();
  //__swks_test();
  //__sigwait_test();
  //__bd_test();
  //__fork_test();
  printf("All POSIX timer tests finished.\n");
  SPIN();
}
