/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 * 
 * (c) Copyright 2006,2007,2008,2009 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2005,2008,2009 MadTirra <madtirra@jarios.org>
 *
 * Jari OS `ls' util, it's not ported from GNU coreutils,
 * instead of this own implementation was used.
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <errno.h>
#include <sys/mman.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <dirent.h>
#include <limits.h>
#include <string.h>
#include <unistd.h>
#include <getopt.h>

#include "config.h"
#include "system.h"

#include "lib/filemode.h"
#include "lib/filetype.h"

#define PROGRAM_NAME "ls"

#define AUTHORS  proper_name("Alfeiks Kaanoken")

void usage(int status)
{
  if(status != EXIT_SUCCESS)
    fprintf (stderr, _("Try `%s --help' for more information.\n"),
	     program_name);
  else {
    printf (_("\
Usage: %s [OPTION]... [FILE]...\n               \
"),
            program_name);
    fputs (_("\
List contents and file attributes.\n            \
\n                                              \
  -a, --show-all           list all contents\n                 \
  -l, --list-mode          listing mode\n       \
"), stdout);
    fputs (HELP_OPTION_DESCRIPTION, stdout);
    fputs (VERSION_OPTION_DESCRIPTION, stdout);
    fputs (_("\
\n                                                      \
With no FILE read current directory.\n \
"), stdout);
  }
  exit (status);
}

static char *human_access(char *modebuf, struct stat const *statbuf)
{
  memset(modebuf,0,11);
  filemodestring(statbuf, modebuf);
  modebuf[10] = 0;

  return modebuf;
}

int simple_ls_dir(char *name,bool list_all)
{
  DIR *ldir;
  struct dirent *ent;
  struct stat stbuf;
  char *fname=malloc(PATH_MAX);

  if(!fname) return EXIT_FAILURE;

  ldir=opendir(name);

  if(ldir==NULL) {
    fprintf(stderr,"Failed to read directory '%s' (%d)",name,errno);
    return -1;
  } else {
    while((ent=readdir(ldir))!=NULL) {
      if(list_all==false) {
        if(!strcmp(ent->d_name,".") || !strcmp(ent->d_name,".."))
          continue;
      }
      sprintf(fname,"%s/%s",name,ent->d_name);
      if (lstat(fname,&stbuf) == -1) {
        continue;
      }
      if(S_ISDIR(stbuf.st_mode)) {
        if(list_all==true && (!strcmp(ent->d_name,".") ||
                              !strcmp(ent->d_name,".."))) {
          printf(" \e[31m%s\e[0m ",ent->d_name);
        } else
          printf(" \e[31m%s/\e[0m ",ent->d_name);
      } else if(S_ISCHR(stbuf.st_mode)) 
        printf(" \e[33m%s\e[0m ",ent->d_name);
      else if(S_ISBLK(stbuf.st_mode)) 
        printf(" \e[35m%s\e[0m ",ent->d_name);
      else if(S_ISLNK(stbuf.st_mode))
        printf(" \e[36m@%s\e[0m ",ent->d_name);
      else if(S_ISFIFO(stbuf.st_mode))
        printf(" \e[31m%s\e[0m ",ent->d_name);
      else
        printf(" %s ",ent->d_name);
    }
    printf("\n");
  }

  free(fname);
  closedir(ldir);

  return EXIT_SUCCESS;
}

int simple_ls_file(char *name,bool list_all)
{
  struct stat stbuf;

  if (lstat(name,&stbuf) == -1) 
    return EXIT_FAILURE;

  if(S_ISCHR(stbuf.st_mode)) 
    printf("\e[33m%s\e[0m",name);
  else if(S_ISBLK(stbuf.st_mode)) 
    printf("\e[35m%s\e[0m",name);
  else
    printf("%s",name);

  printf("\n");

  return EXIT_SUCCESS;
}

int simple_stat_file(char *name)
{
  struct stat stbuf;
  char md[11];

  if (lstat(name, &stbuf) == -1)
    return EXIT_FAILURE;

  human_access(md, &stbuf);

  printf("   File: %s Type: '%s'\n"
	 "   Size: %lu\n"
	 " Blocks: %d IO Block: %d\n"
	 "  Inode: %lu   Links: %d\n"
	 "    GID: %d, UID: %d\n"
	 "   Mode: %s\n"
	 "Devtype: %d\n",
	 //	 "Access: (%04a/%10.10A)  Uid: (%5u/%8U)   Gid: (%5g/%8G)\n"
	 //	 "Access: %x\n" "Modify: %y\n" "Change: %z\n",
	 name, file_type(&stbuf),
	 stbuf.st_size, stbuf.st_blocks,
	 stbuf.st_blocks * stbuf.st_blksize,
	 stbuf.st_ino, stbuf.st_nlink,
	 stbuf.st_gid, stbuf.st_uid, md,
	 stbuf.st_rdev);

  return EXIT_SUCCESS;
}

int listed_ls_file(char *name,bool list_all)
{
  struct stat stbuf;
  char md[11];
  char *tmp;

  if (lstat(name,&stbuf) == -1)
    return EXIT_FAILURE;

  human_access(md, &stbuf);
  /*stmode_torwx(md,stbuf);*/

  if(S_ISCHR(stbuf.st_mode))
    printf("%s %d %d:%d %ld \e[33m%s\e[0m\n", md,stbuf.st_ino,
           stbuf.st_uid,stbuf.st_gid,stbuf.st_size,name);
  else if(S_ISBLK(stbuf.st_mode))
    printf("%s %d %d:%d %ld \e[35m%s\e[0m\n",md,stbuf.st_ino,
           stbuf.st_uid,stbuf.st_gid,stbuf.st_size,name);
  else if (S_ISLNK(stbuf.st_mode)) {
    printf("%s %d %d:%d %ld \e[36m@%s\e[0m ->", md, stbuf.st_ino,
        stbuf.st_uid, stbuf.st_gid, stbuf.st_size, name);
    tmp = NULL;
    tmp = calloc(1, PATH_MAX + 1);
    if ((tmp != NULL) && (readlink(name, tmp, PATH_MAX) > 0))
      printf(" %s\n", tmp);
    else
      printf("\n");
    if (tmp != NULL)
      free(tmp);
  } else if(S_ISFIFO(stbuf.st_mode)) {
    printf("%s %d %d:%d %ld \e[31m%s\e[0m\n",md,stbuf.st_ino,
           stbuf.st_uid,stbuf.st_gid,stbuf.st_size,name);
  }
  else
    printf("%s %d %d:%d %ld %s\n",md,stbuf.st_ino,
           stbuf.st_uid,stbuf.st_gid,stbuf.st_size,name);

  return EXIT_SUCCESS;
}

int listed_ls_dir(char *name,bool list_all)
{
  DIR *ldir;
  struct dirent *ent;
  struct stat stbuf;
  char *fname=malloc(PATH_MAX);
  char md[11];
  int cnt=0;
  char *tmp;

  if(!fname) return EXIT_FAILURE;

  ldir=opendir(name);

  if(ldir==NULL) {
    fprintf(stderr,"Failed to read directory '%s' (%d)",name,errno);
    return -1;
  } else {
    printf("%s:\n",name);
    while((ent=readdir(ldir))!=NULL) {
      if(list_all==false) {
        if(!strcmp(ent->d_name,".") || !strcmp(ent->d_name,".."))
          continue;
      }
      sprintf(fname,"%s/%s",name,ent->d_name);
      if (lstat(fname,&stbuf) == -1) {
        continue;
      }
      cnt++;
      /*stmode_torwx(md,stbuf);*/
      human_access(md, &stbuf);
      if(S_ISDIR(stbuf.st_mode)) {
        if(list_all==true && (!strcmp(ent->d_name,".") ||
                              !strcmp(ent->d_name,".."))) {
          printf("%s %d %d:%d %ld \e[31m%s\e[0m\t<DIR>\n", md,
                 stbuf.st_ino,stbuf.st_uid,stbuf.st_gid,stbuf.st_size,
                 ent->d_name);
        } else
          printf("%s %d %d:%d %ld \e[31m%s/\e[0m\t<DIR>\n",md,
                 stbuf.st_ino,stbuf.st_uid,stbuf.st_gid,stbuf.st_size,
                 ent->d_name);
      } else if(S_ISCHR(stbuf.st_mode)) 
        printf("%s %d %d:%d %ld \e[33m%s\e[0m\n",md,stbuf.st_ino,
               stbuf.st_uid,stbuf.st_gid,stbuf.st_size,ent->d_name);
      else if(S_ISBLK(stbuf.st_mode)) 
        printf("%s %d %d:%d %ld \e[35m%s\e[0m\n",md,stbuf.st_ino,
               stbuf.st_uid,stbuf.st_gid,stbuf.st_size,ent->d_name);
      else if (S_ISLNK(stbuf.st_mode)) {
        printf("%s %d %d:%d %ld \e[36m@%s\e[0m ->", md, stbuf.st_ino,
            stbuf.st_uid, stbuf.st_gid, stbuf.st_size, ent->d_name);
	tmp = NULL;
        tmp = calloc(1, PATH_MAX + 1);
        if ((tmp != NULL) && (readlink(fname, tmp, PATH_MAX) > 0))
          printf(" %s\n", tmp);
        else
          printf("\n");
	if (tmp != NULL)
          free(tmp);
      } else if(S_ISFIFO(stbuf.st_mode)) {
        printf("%s %d %d:%d %ld \e[31m%s\e[0m\n",md,stbuf.st_ino,
               stbuf.st_uid,stbuf.st_gid,stbuf.st_size,name);
      }
      else
        printf("%s %d %d:%d %ld %s\n",md,stbuf.st_ino,
               stbuf.st_uid,stbuf.st_gid,stbuf.st_size,ent->d_name);
    }
    printf("total %d\n",cnt);
  }

  free(fname);
  closedir(ldir);

  return EXIT_SUCCESS;
}

int main(int argc,char **argv)
{
  struct stat cstat;
  int c;
  int lent=0;
  bool list_all=false;
  bool list_mode=false;
  bool simple_stat=false;

  static struct option const long_options[] = {
    {"show-all",  no_argument, NULL, 'a'},
    {"list-mode", no_argument, NULL, 'l'},
    {"stat-mode", no_argument, NULL, 's'},
    {GETOPT_HELP_OPTION_DECL},
    {GETOPT_VERSION_OPTION_DECL},
    {NULL, 0, NULL, 0}
  };

  while ((c = getopt_long (argc, argv, "alshV", long_options, NULL))
	 != -1) {
    switch(c) {
    case 'a':
      list_all=true;
      break;
    case 'l':
      list_mode=true;
      break;
    case 's':
      simple_stat=true;
      break;
    case_GETOPT_HELP_CHAR;
    case_GETOPT_VERSION_CHAR (PROGRAM_NAME, AUTHORS);
    default:
      usage(EXIT_FAILURE);
    }
  }

  /* check all inputed stuff */
  argv++;
  while(*argv!=NULL) {
    if(*argv[0]=='-') goto __end_sub;
    else {
      lent++;
      if(!lstat(*argv,&cstat)) {
	if (simple_stat) {
	  simple_stat_file(*argv);
	} else {
	  if(S_ISDIR(cstat.st_mode)) {
	    if(list_mode==false) simple_ls_dir(*argv,list_all);
	    else listed_ls_dir(*argv,list_all);
	  } else { /* it's file */
	    if(list_mode==false) simple_ls_file(*argv,list_all);
	    else listed_ls_file(*argv,list_all);
	  }
	}
      } else
        fprintf(stdout,"%s: %s: No such file or directory\n",PROGRAM_NAME, *argv);
    }

  __end_sub:
    argv++;
  }

  if(!lent) { /* we're doesn't listed anything */
    char cdir[PATH_MAX];
    getcwd(cdir,PATH_MAX);
    if(list_mode==false) simple_ls_dir(cdir,list_all);
    else listed_ls_dir(cdir,list_all);
  }

  return 0;
}
