/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 * 
 * (c) Copyright 2006,2007,2008,2009 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2005,2008,2009 MadTirra <madtirra@jarios.org>
 *
 * Jari OS `ls' util, it's not ported from GNU coreutils,
 * instead of this own implementation was used.
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <errno.h>
#include <sys/mman.h>
#include <sys/stat.h>
#include <sys/utsname.h>
#include <fcntl.h>
#include <dirent.h>
#include <limits.h>
#include <string.h>
#include <unistd.h>
#include <getopt.h>

#include "config.h"
#include "system.h"

#define PROGRAM_NAME "ls"

#define AUTHORS  proper_name("Alfeiks Kaanoken")

void usage(int status)
{
  if(status != EXIT_SUCCESS)
    fprintf (stderr, _("Try `%s --help' for more information.\n"),
	     program_name);
  else {
    printf (_("\
Usage: %s [OPTION]\n               \
"),
            program_name);
    fputs (_("\
List contents and file attributes.\n            \
\n                                              \
  -a, --show-all           show all information about system type\n                 \
  -m,                      show hardware installed on\n       \
  -n,                      show node name\n       \
  -r,                      show current release name\n       \
  -s,                      show operating system name\n       \
  -v,                      show operating system version\n       \
"), stdout);
    fputs (HELP_OPTION_DESCRIPTION, stdout);
    fputs (VERSION_OPTION_DESCRIPTION, stdout);
    fputs (_("\
\n                                                      \
By default all be show with -s flag.\n \
"), stdout);
  }
  exit (status);
}

int main(int argc,char **argv)
{
  bool show_machine=false;
  bool show_nodename=false;
  bool show_release=false;
  bool show_sysname=true;
  bool show_version=false;
  struct utsname sysnm;
  char output[512];
  int c;

  static struct option const long_options[] = {
    {"show-all", no_argument, NULL, 'a'},
    {GETOPT_HELP_OPTION_DECL},
    //    {GETOPT_VERSION_OPTION_DECL},
    {NULL, 0, NULL, 0}
  };

  while ((c = getopt_long (argc, argv, "amnrsv", long_options, NULL))
	 != -1) {
    switch(c) {
    case 'a':
      show_machine=show_nodename=show_release=
        show_version=show_sysname=true;
      break;
    case 'm':
      show_machine=true;
      break;
    case 'n':
      show_nodename=true;
      break;
    case 'r':
      show_release=true;
      break;
    case 's':
      show_sysname=true;
      break;
    case 'v':
      show_version=true;
      break;
      case_GETOPT_HELP_CHAR;
      //      case_GETOPT_VERSION_CHAR (PROGRAM_NAME, AUTHORS);
    default:
      usage(EXIT_FAILURE);
    }
  }

  uname(&sysnm);
  memset(output,0,512);
  if(show_sysname) {
    strcat(output,sysnm.sysname);
    strcat(output," ");
  }
  if(show_version) {
    strcat(output,sysnm.version);
    strcat(output," ");
  }
  if(show_release) {
    strcat(output,sysnm.release);
    strcat(output," ");
  }
  if(show_machine) {
    strcat(output,sysnm.machine);
    strcat(output," ");
  }
  if(show_nodename) {
    strcat(output,sysnm.nodename);
    strcat(output," ");
  }

  printf("%s\n",output);

  return EXIT_SUCCESS;
}
