/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * (c) Copyright 2006,2007,2008,2009 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2009 Dmitry Gromada <gromada@jarios.org>
 *
 * Small utility to concatenate trace table of core servers to standart output
 */

#include <sys/types.h>
#include <stdio.h>
#include <string.h>
#include <sys/mman.h>
#include <sys/stat.h>
#include <unistd.h>
#include <errno.h>

enum {
  FSIZE = sizeof(void*) + sizeof(uint32_t)
};

int main(int argc, char *argv[])
{
  FILE *file;
  void *phy_addr;
  char *tbl;
  uint32_t size;
  struct stat st;
  int r;

  if (argc == 1) {
    printf("The file containing base trace physical address and table size "
           "is not provided!\n");
    printf("Usage: trace_view <file_name>\n");
    return 1;
  }

  file = fopen(argv[1], "rb");
  if (file == NULL) {
    printf("File opening failed\n");
    return 1;
  }

  stat(argv[1], &st);
  if (st.st_size != FSIZE) {
    printf("File has an invalid format\n");
    return 1;
  }

  r = fread(&phy_addr, sizeof(phy_addr), 1, file);
  r += fread(&size, sizeof(size), 1, file);
  if (r < 2) {
    printf("File read error\n");
    return 1;
  }

  tbl = mmap(NULL, size, PROT_READ | PROT_WRITE,
             MAP_ANONYMOUS | MAP_PRIVATE | MAP_PHYS, 0, (ulong_t)phy_addr);
  if (tbl == MAP_FAILED) {
    printf("Trace table mapping failed\n");
    return 1;
  }

  if (tbl[size - 1]) {
    printf("Trace table is corrupted or provided file contains invalid base "
            "address and size\n");
    return 1;
  }

  while (*tbl) {
    printf(tbl);
    tbl += strlen(tbl) + 1;
  }

  return 0;
}
