#ifndef __INT10_H__
#define __INT10_H__

#include <sys/types.h>

typedef struct __vbe_mode_t {
  uint16_t 	mode_attributes;
  uint8_t win_a_attributes;
  uint8_t win_b_attributes;
  uint16_t 	win_granularity;
  uint16_t 	win_size;
  uint16_t 	win_a_segment;
  uint16_t 	win_b_segment;
  uint32_t 	win_func;
  uint16_t 	bytes_per_scanline;

// >= VESA version 1.2
  uint16_t x_resolution;
  uint16_t y_resolution;
  uint8_t x_char_size;
  uint8_t	y_char_size;
  uint8_t number_of_planes;
  uint8_t bits_per_pixel;
  uint8_t number_of_banks;
  uint8_t memory_model;
  uint8_t bank_size;
  uint8_t number_of_image_pages;
  uint8_t reserved0;

// direct color
  uint8_t red_mask_size;
  uint8_t red_field_position;
  uint8_t green_mask_size;
  uint8_t green_field_position;
  uint8_t blue_mask_size;
  uint8_t blue_field_position;
  uint8_t reserved_mask_size;
  uint8_t reserved_field_position;
  uint8_t direct_color_mode_info;

// >= VESA version 2.0
  uint32_t phys_base;
  uint32_t reserved1;
  uint16_t reversed2;
  
//  >= VESA version 3.0
  uint16_t linear_bytes_per_scanline;
  uint8_t banked_number_of_image_pages;
  uint8_t linear_number_of_image_pages;
  uint8_t linear_red_mask_size;
  uint8_t linear_red_field_position;
  uint8_t linear_green_mask_size;
  uint8_t linear_green_field_position;
  uint8_t linear_blue_mask_size;
  uint8_t linear_blue_field_position;
  uint8_t linear_reserved_mask_size;
  uint8_t linear_reserved_field_position;
  uint32_t max_pixel_clock;
  uint8_t reserved3 [189];
} vbe_mode_t;

typedef struct __vbe_ctrl_t {
  uint8_t signature[4];
  uint16_t version;
  uint32_t oem_string;
  uint32_t capabilities;
  uint32_t video_mode;
  uint16_t total_memory;
  uint16_t oem_software_rev;
  uint32_t oem_vendor_name;
  uint32_t oem_product_name;
  uint32_t oem_product_rev;
  uint8_t reserved[222];
  uint8_t oem_data[256];
} vbe_ctrl_t;

#define VESA_MODE_640x480x8 0x101
#define VESA_MODE_640x480x16 0x111
#define VESA_MODE_640x480x32 0x112
#define VESA_MODE_720x480x8 0x171
#define VESA_MODE_720x480x16 0x173
#define VESA_MODE_720x480x32 0x175
#define VESA_MODE_720x576x8 0x17c
#define VESA_MODE_720x576x16 0x17e
#define VESA_MODE_720x576x32 0x17f
#define VESA_MODE_800x480x8 0x19c
#define VESA_MODE_800x480x16 0x19d
#define VESA_MODE_800x480x32 0x19e
#define VESA_MODE_800x600x8 0x103
#define VESA_MODE_800x600x16 0x114
#define VESA_MODE_800x600x32 0x115
#define VESA_MODE_848x480x8 0x15c
#define VESA_MODE_848x480x16 0x15d
#define VESA_MODE_848x480x32 0x15f
#define VESA_MODE_1024x768x8 0x105
#define VESA_MODE_1024x768x16 0x117
#define VESA_MODE_1024x768x32 0x118
#define VESA_MODE_1280x768x8 0x179
#define VESA_MODE_1280x768x16 0x17a
#define VESA_MODE_1280x768x32 0x17b
#define VESA_MODE_1280x1024x8 0x107
#define VESA_MODE_1280x1024x16 0x11a
#define VESA_MODE_1280x1024x32 0x11b
#define VESA_MODE_1368x768x8 0x183
#define VESA_MODE_1368x768x16 0x184
#define VESA_MODE_1368x768x32 0x185
#define VESA_MODE_1400x1050x8 0x13b
#define VESA_MODE_1400x1050x16 0x13c
#define VESA_MODE_1400x1050x32 0x13e
#define VESA_MODE_80x60x4 0x108
#define VESA_MODE_800x600x4 0x102 


/* Allocate needed structures, etc. */
int int10_init(void);

/* Free resources. */
int int10_done(void);

/* Do actual work. */
int int10_set_vbe_mode(int mode, vbe_ctrl_t **ctrl_info, vbe_mode_t **mode_info);

int int10_pan(unsigned *x, unsigned *y);

#endif
