/*
 * This library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * (c) Copyright 2006,2007,2008,2009 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2008,2009 Paul Graphov <graphov@jarios.org> 
 *
 * PCI vendors, products, etc. lookup functions and types
 *
 */

#ifndef __NAMES_H__
#define __NAMES_H__

#include <sys/types.h>

/* 
 * It's a bad idea to change these structures for two reasons:
 * huge auto generated ids database file is made according to it;
 * search operations depend on field order.
 * The latter can be fixed.
 */

typedef struct __subprod_desc_t {
    uint16_t subprod1, subprod2;
    char *name;
} subprod_desc_t;

typedef struct __product_desc_t {
    uint16_t id;
    char *name;
    uint32_t nsubprods;
    subprod_desc_t *subprods;
} product_desc_t;

typedef struct __vendor_desc_t {
    uint16_t id;
    char *name;
    uint16_t nproducts;
    product_desc_t *products;
} vendor_desc_t;

typedef struct __prog_if_desc_t {
    uint8_t id;
    char *name;
} prog_if_desc_t;

typedef struct __subclass_desc_t {
    uint8_t id;
    char *name;
    uint8_t nifs;
    prog_if_desc_t *ifs;
} subclass_desc_t;

typedef struct __class_desc_t {
    uint8_t id;
    char *name;
    uint8_t nsubclasses;
    subclass_desc_t *subclasses;
} class_desc_t;
                   
vendor_desc_t *lookup_vendor(uint16_t vendor_id);
product_desc_t *lookup_product(vendor_desc_t *vendor, uint16_t product_id);
subprod_desc_t *lookup_subprod(product_desc_t *product, uint16_t subp1, uint16_t subp2);

class_desc_t *lookup_class(uint8_t class_id);
subclass_desc_t *lookup_subclass(class_desc_t *class, uint8_t subclass_id);
prog_if_desc_t *lookup_prog_if(subclass_desc_t *subclass, uint8_t prog_if_id);

#endif
