/*
 * This library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * (c) Copyright 2006,2007,2008,2009 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2009 Paul Graphov <graphov@jarios.org>
 * (c) Copyright 2009 Dmitry Gromada <gromada@jarios.org>
 *
 * pci/pci.h: pci bus device ioctls and functions.
 *
 */

#ifndef __PCI_PCI_H__
#define __PCI_PCI_H__

#define PCI_BAR_0	0x10

// Header field offsets:
#define PCI_VENDOR_ID		0x00	/* 16 bits */
#define PCI_DEVICE_ID		0x02	/* 16 bits */
#define PCI_SUBSYSTEM_VENDOR_ID 0x2c
#define PCI_SUBSYSTEM_ID        0x2e

#define PCI_DEVICE_CLASS 0x0b
#define PCI_DEVICE_SUBCLASS 0x0a
#define PCI_DEVICE_PROG_IF 0x09

#define PCI_HEADER_TYPE		0x0e	/* 8 bits */
#define  PCI_HEADER_TYPE_NORMAL	0
#define  PCI_HEADER_TYPE_BRIDGE 1
#define  PCI_HEADER_TYPE_CARDBUS 2

#define PCI_STATUS		0x06	/* 16 bits */
#define  PCI_STATUS_CAP_LIST	0x10	/* Support Capability List */
#define  PCI_STATUS_66MHZ	0x20	/* Support 66 Mhz PCI 2.1 bus */
#define  PCI_STATUS_UDF		0x40	/* Support User Definable Features [obsolete] */
#define  PCI_STATUS_FAST_BACK	0x80	/* Accept fast-back to back */
#define  PCI_STATUS_PARITY	0x100	/* Detected parity error */
#define  PCI_STATUS_DEVSEL_MASK	0x600	/* DEVSEL timing */
#define  PCI_STATUS_DEVSEL_FAST		0x000
#define  PCI_STATUS_DEVSEL_MEDIUM	0x200
#define  PCI_STATUS_DEVSEL_SLOW		0x400
#define  PCI_STATUS_SIG_TARGET_ABORT	0x800 /* Set on target abort */
#define  PCI_STATUS_REC_TARGET_ABORT	0x1000 /* Master ack of " */
#define  PCI_STATUS_REC_MASTER_ABORT	0x2000 /* Set on master abort */
#define  PCI_STATUS_SIG_SYSTEM_ERROR	0x4000 /* Set when we drive SERR */
#define  PCI_STATUS_DETECTED_PARITY	0x8000 /* Set on parity error */

#define PCI_INTERRUPT_LINE	0x3c	/* 8 bits */
#define PCI_INTERRUPT_PIN	0x3d	/* 8 bits */
#define PCI_BASE_ADDRESS_0	0x10	/* 32 bits */
#define PCI_BASE_ADDRESS_1	0x14	/* 32 bits [htype 0,1 only] */
#define PCI_BASE_ADDRESS_2	0x18	/* 32 bits [htype 0 only] */
#define PCI_BASE_ADDRESS_3	0x1c	/* 32 bits */
#define PCI_BASE_ADDRESS_4	0x20	/* 32 bits */
#define PCI_BASE_ADDRESS_5	0x24	/* 32 bits */
#define  PCI_BASE_ADDRESS_SPACE	0x01	/* 0 = memory, 1 = I/O */
#define  PCI_BASE_ADDRESS_SPACE_IO 0x01
#define  PCI_BASE_ADDRESS_SPACE_MEMORY 0x00
#define  PCI_BASE_ADDRESS_MEM_TYPE_MASK 0x06
#define  PCI_BASE_ADDRESS_MEM_TYPE_32	0x00	/* 32 bit address */
#define  PCI_BASE_ADDRESS_MEM_TYPE_1M	0x02	/* Below 1M [obsolete] */
#define  PCI_BASE_ADDRESS_MEM_TYPE_64	0x04	/* 64 bit address */
#define  PCI_BASE_ADDRESS_MEM_PREFETCH	0x08	/* prefetchable? */
#define  PCI_BASE_ADDRESS_MEM_MASK	(~0x0fU)
#define  PCI_BASE_ADDRESS_IO_MASK	(~0x03U)

#define PCI_CAPABILITY_LIST 0x34 // capabilities pointer
#define PCI_CB_CAPABILITY_LIST 0x14 // the same for cardbus

#define  PCI_CAP_ID_PM	    0x01  /* Power Management */
#define  PCI_CAP_ID_AGP	    0x02  /* Accelerated Graphics Port */
#define  PCI_CAP_ID_VPD	    0x03  /* Vital Product Data */
#define  PCI_CAP_ID_SLOTID	0x04  /* Slot Identification */
#define  PCI_CAP_ID_MSI	    0x05  /* Message Signalled Interrupts */
#define  PCI_CAP_ID_CHSWP	  0x06  /* CompactPCI HotSwap */
#define  PCI_CAP_ID_PCIX    0x07  /* PCI-X */
#define  PCI_CAP_ID_HT      0x08  /* HyperTransport */
#define  PCI_CAP_ID_VNDR    0x09  /* Vendor specific */
#define  PCI_CAP_ID_DBG		  0x0A  /* Debug port */
#define  PCI_CAP_ID_CCRC	  0x0B  /* CompactPCI Central Resource Control */
#define  PCI_CAP_ID_SHPC 	  0x0C  /* PCI Standard Hot-Plug Controller */
#define  PCI_CAP_ID_SSVID	  0x0D  /* Bridge subsystem vendor/device ID */
#define  PCI_CAP_ID_AGP3	  0x0E  /* AGP Target PCI-PCI bridge */
#define  PCI_CAP_ID_EXP     0x10  /* PCI Express */
#define  PCI_CAP_ID_MSIX	  0x11  /* MSI-X */

#define PCI_CAP_ID_OFFSET    0x00
#define PCI_CAP_NEXT_OFFSET  0x01
#define PCI_CAP_FLAGS_OFFSET 0x02

#define PCI_SECONDARY_BUS	0x19	/* Secondary bus number */

#define MAX_SCANNED_PCI_DEVICES 256

typedef struct __pci_dev_header_t {
  
  uint16_t vendor_id;
  uint16_t device_id;

  uint16_t command; 
  uint16_t status;

  uint8_t revision_id;

  uint8_t prog_if_id;
  uint8_t subclass_id;
  uint8_t class_id;

  uint8_t cacheline_size;
  uint8_t latency_timer;
  uint8_t header_type;
  uint8_t bist;

  uint32_t base_address_registers[6];

  uint32_t cardbus_cis_pointer;
  
  uint16_t subsys_vendor_id;
  uint16_t subsys_id;

  uint32_t expansion_rom_base_address;

  uint8_t capabilities_pointer;
  uint8_t reserved[7];

  uint8_t interrupt_line;
  uint8_t interrupt_pin;
  uint8_t min_gnt;
  uint8_t max_lat;
  
  uint32_t device_specific_data[48];
  
} pci_dev_header_t;

typedef struct __pci_bridge_header_t {
  
  uint16_t vendor_id;
  uint16_t device_id;

  uint16_t command; 
  uint16_t status;

  uint8_t revision_id;
 
  uint8_t prog_if_id;
  uint8_t subclass_id;
  uint8_t class_id;

  uint8_t cacheline_size;
  uint8_t latency_timer;
  uint8_t header_type;
  uint8_t bist;

  uint32_t base_address_registers[2];

  uint8_t primary_bus_number;
  uint8_t secondary_bus_number;
  uint8_t subordinate_bus_number;
  
  uint8_t secondary_latency_timer;

  uint8_t io_base;
  uint8_t io_limit;

  uint16_t secondary_status;

  uint16_t memory_base;
  uint16_t memory_limit;
  uint16_t prefetchable_memory_base;
  uint16_t prefetchable_memory_limit;

  uint32_t prefetchable_base_upper;
  uint32_t prefetchable_limit_upper;

  uint16_t io_base_upper;
  uint16_t io_limit_upper;

  uint8_t capability_pointer;
  uint8_t reserved[3];

  uint32_t expansion_rom_base;

  uint8_t interrupt_line;
  uint8_t interrupt_pin;

  uint16_t bridge_controll;
  
  uint32_t device_specific_data[48];
  
} pci_bridge_header_t;

typedef struct __pci_addr_t {
  uint8_t bus;
  uint8_t device;
  uint8_t function;
} pci_addr_t;

typedef struct __pci_devid {
  uint16_t vendor;
  uint16_t product;
} pci_devid_t;


#define PCI_LOOKUP_ANY   0x0
#define PCI_LOOKUP_V     0x1
#define PCI_LOOKUP_P     0x2

#define PCI_LOOKUP_VP (PCI_LOOKUP_V | PCI_LOOKUP_P)

typedef struct __pci_ident_t {
  uint16_t device_id;
  uint16_t vendor_id;
  uint8_t revision_id;
  uint8_t class_id;
  uint8_t subclass_id;
  uint8_t prog_if_id;
  uint16_t subsys_id;
  uint16_t subsys_vendor_id;
  uint8_t lookup_opt;
} pci_ident_t;

typedef struct __pci_reg_op_t {
  pci_addr_t addr;
  uint16_t offset;
  uint16_t size;
  uint32_t value;
} pci_reg_op_t;

typedef uint64_t resource_size_t;

#define PCI_RESOURCE_IO		    0x01
#define PCI_RESOURCE_MEM    	0x02
#define PCI_RESOURCE_MEM64    0x04
#define PCI_RESOURCE_PREFETCH 0x08
#define PCI_RESOURCE_BELOW_1M 0x10

#define PCI_BAR_IO      0x1
#define PCI_BAR_BELOW1M 0x2
#define PCI_BAR_64      0x4

typedef struct __pci_resource_t {
  resource_size_t start;
	resource_size_t end;
	ulong_t flags;
} pci_resource_t;

typedef struct __pci_cap {
  uint8_t  cap_id;
  uint8_t  next_cap;
  uint16_t cap_flags;
} pci_cap_t;

#define PCI_MAX_CAP 20

typedef struct __pci_dev_t {
  pci_addr_t addr;
  pci_ident_t ident;
  uint8_t header_type; //specifies if the device is a bridge
  uint16_t status;
  uint8_t irq_line;
  uint8_t num_resources;
  pci_resource_t resources[6];
  int num_caps;
  pci_cap_t capabilities[PCI_MAX_CAP];
} pci_dev_t;

#define PCI_IOCTL 'P'

enum {
  PCI_LOOKUP,
  PCI_LOOKUP_CNT,
  PCI_READ_CONF,
  PCI_READ_REG,
  PCI_WRITE_REG,
  PCI_GET_DEV,
};

#define PIOC_LOOKUP _IOWRARB(PCI_IOCTL, PCI_LOOKUP, pci_ident_t)
#define PIOC_LOOKUP_CNT _IOW(PCI_IOCTL, PCI_LOOKUP_CNT, pci_ident_t)
#define PIOC_READ_CONF _IOWRARB(PCI_IOCTL, PCI_READ_CONF, pci_addr_t)
#define PIOC_READ_REG _IOWRFIX(PCI_IOCTL, PCI_READ_REG, pci_reg_op_t)
#define PIOC_WRITE_REG _IOW(PCI_IOCTL, PCI_WRITE_REG, pci_reg_op_t)
#define PIOC_GET_DEV _IOWRARB(PCI_IOCTL, PCI_GET_DEV, pci_addr_t)

// libpci functions:
int pci_init(void);
void pci_close(void);
int pci_lookup_prod_vend(uint16_t vendor, int product, pci_addr_t *buf, int size);
int pci_lookup(pci_ident_t *ident, pci_addr_t *buf, int size);
int pci_get_dev(pci_addr_t *addr, pci_dev_t *dev);

/*
 * search PCI device presented in the system from the given identification list
 *
 * @devs [out]: pointer to the buffer where the descriptors of found
 *              devices were be stored; the function allocates the buffer.
 * @ids: device ids
 * @nids: number of ids contained in the buffer pointed to by 'ids'
 *        parameter
 *
 * Returned value:
 *    Non negative number of found devices (don't exceed
 *          MAX_SCANNED_PCI_DEVICES);
 *    -1 on error with error code contained in the 'errno'
 *        variable
 */
int pci_search_devices(pci_dev_t **devs, pci_devid_t *ids, uint16_t nids);

int pci_read_config_byte(pci_dev_t *dev, uint16_t offset, uint8_t *val);
int pci_read_config_word(pci_dev_t *dev, uint16_t offset, uint16_t *val);
int pci_read_config_dword(pci_dev_t *dev, uint16_t offset, uint32_t *val);

int pci_write_config_byte(pci_dev_t *dev, uint16_t offset, uint8_t val);
int pci_write_config_word(pci_dev_t *dev, uint16_t offset, uint16_t val);
int pci_write_config_dword(pci_dev_t *dev, uint16_t offset, uint32_t val);
#endif
