/*
 * This library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * (c) Copyright 2006,2007,2008,2009 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2008 Paul Graphov <graphov@jarios.org> 
 *
 * PCI vendors, products, etc. lookup functions
 *
 */

#include <sys/types.h>
#include <names.h>
#include "pci_ids.h"

// to make comparison functions easily
#define CMPFN(x) compare_##x
#define CMPIMPL(x) int CMPFN(x)(const void *a, const void *b){          \
    int aa = *((x *)a);                                                 \
    int bb = *((x *)b);                                                 \
    if( aa > bb ) return 1;                                             \
    if( aa < bb ) return -1;                                            \
    return 0;                                                           \
  }

int compare_uint16_t(const void *a, const void *b);
int compare_uint16_t(const void *a, const void *b);

// binary search through array
void *dich_search(const void *el, void *base, int n, size_t width, int (*compar)(const void *, const void *) )
{
    int l, r;

    for(l = 0, r = n-1; l <= r;)
    {
        int mid = (l+r)/2;
        int res = compar( el, base + width * mid );
        if(res < 0) r = mid-1;
        else if(res > 0) l = mid+1;
        else return base + width * mid;
    }
    return NULL;
}

CMPIMPL(uint8_t)
CMPIMPL(uint16_t)

int compare_subprods(const void *a, const void *b)
{
  uint16_t *aa = (uint16_t *)a;
  uint16_t *bb = (uint16_t *)b;
  if(aa[0] > bb[0]) return 1;
  if(aa[0] < bb[0]) return -1;
  if(aa[1] > bb[0]) return 1;
  if(aa[1] < bb[0]) return -1;
  return 0;
}

vendor_desc_t *lookup_vendor(uint16_t vendor_id)
{
  return dich_search(&vendor_id, vendors, vendors_num, sizeof(*vendors), compare_uint16_t);
}

product_desc_t *lookup_product(vendor_desc_t *vendor, uint16_t product_id)
{
  return dich_search(&product_id, vendor->products, vendor-> nproducts,
                     sizeof(*(vendor->products)), compare_uint16_t);
}

subprod_desc_t *lookup_subprod(product_desc_t *product, uint16_t subp1, uint16_t subp2)
{
  uint16_t subp[]= {subp1, subp2};
  return dich_search(&subp, product->subprods, product->nsubprods,
                     sizeof(*(product->subprods)), compare_subprods);
}

class_desc_t *lookup_class(uint8_t class_id)
{
  return dich_search(&class_id, classes, classes_num, sizeof(*classes), compare_uint8_t);
}

subclass_desc_t *lookup_subclass(class_desc_t *class, uint8_t subclass_id)
{
  return dich_search(&subclass_id, class->subclasses, class->nsubclasses,
                     sizeof(*(class->subclasses)), compare_uint8_t);
}

prog_if_desc_t *lookup_prog_if(subclass_desc_t *subclass, uint8_t prog_if_id)
{
  return dich_search(&prog_if_id, subclass->ifs, subclass->nifs, sizeof(*(subclass->ifs)), compare_uint8_t);
}
    
