/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * (c) Copyright 2006,2007,2008 MString Core Team <http://mstring.berlios.de>
 * (c) Copyright 2008 Dmitry Gromada <gromada@jarios.org>
 *
 *  Test framework declarations
 */

#ifndef __FRAMEWORK_H__
#define __FRAMEWORK_H__

#include <sys/types.h>
#include <stdbool.h>
#include <errno.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>

#define MAX_TESTS 1024
#define MAX_MSG_LENGTH 256
#define MAX_ERRNO 4095

#define SPINLOCK() for (;;)
#define CASE_LIST_END {.name = NULL}
#define ARG_LIST (const char*[])
#define ARG_LIST_END NULL
#define ARGS_CHAIN (long*[])
#define ARGS (long[])
#define ARGS_END NULL

typedef enum __comp_type {
  COMP_LESS,    /* less */
  COMP_LEQU,    /* less or equal */
  COMP_EQUAL,
  COMP_NOT_EQU,
  COMP_GREATER,
  COMP_GREQU     /* greater or equal */
} comp_type_t;

typedef void (*test_function_t)(long *args);

typedef struct __tc_arg_chain {
  long *values;
} tc_arg_chain_t;

typedef enum __tc_status {
  test_failed,
  test_passed
} tc_status_t;

typedef struct __test_case {
  const char *name;
  void (*function)(long *args);
  const char *description;
  bool nolaunch_on_fails;
  bool abort_on_fail;
  const char **arg_list;
  long **args;
} test_case_t;

typedef struct __test_struct {
  bool (*init)(void);
  void (*complete)(void);
  bool (*must_launched)(test_function_t);
  int pause_seconds;
  test_case_t cases[];
} test_struct_t;

extern test_struct_t test_struct;
extern const char *error_strings[MAX_ERRNO + 1];
extern int cmdline_args_num;
extern char **cmdline_args;
extern char **env_vars;

int main(int argc, char *argv[], char *envp[]);
void test_fail(char *msg,...);
bool check_test_condition(long res, long exp_res, const char *err_msg,
                          bool pass_on_true);
bool check_test_condition_ext(long res, long exp_res, comp_type_t ctype,
                              int exp_errno, const char *err_msg,
                              bool pass_on_true);

static inline void test_pass(void)
{
  printf("PASS\n\n");
}

#endif
