/*
 * Realmode X86 Emulator Library
 *
 * This library is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU Lesser General Public License as 
 * published by the Free Software Foundation; either version 2.1 of 
 * the License.
 *
 * This library is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public 
 * License along with this library; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 *
 * Copyright (C) 1996-1999 SciTech Software, Inc.
 * Copyright (C) David Mosberger-Tang
 * Copyright (C) 1999 Egbert Eich
 * Copyright (C) 2008 Jari OS core team <http://jarios.org>
 * Copyright (C) 2008 Paul Graphov <graphov@jarios.org> 
 *               (porting to Jari OS)
 *
 */

#include "x86emu/x86emui.h"
#include "x86emu/debug.h"
#include "x86emu/prim_ops.h"
#ifndef NO_SYS_HEADERS
#include <string.h>
#endif                                                                                           
/*------------------------- Global Variables ------------------------------*/

X86EMU_sysEnv		_X86EMU_env;		/* Global emulator machine state */
X86EMU_intrFuncs	_X86EMU_intrTab[256];

/*----------------------------- Implementation ----------------------------*/
#if defined(__alpha__) || defined(__alpha)
/* to cope with broken egcs-1.1.2 :-(((( */

#define ALPHA_UALOADS
/*
 * inline functions to do unaligned accesses
 * from linux/include/asm-alpha/unaligned.h
 */

/*
 * EGCS 1.1 knows about arbitrary unaligned loads.  Define some
 * packed structures to talk about such things with.
 */

#if defined(__GNUC__) && ((__GNUC__ > 2) || (__GNUC_MINOR__ >= 91))
struct __una_u64 { unsigned long  x __attribute__((packed)); };
struct __una_uint32_t { unsigned int   x __attribute__((packed)); };
struct __una_uint16_t { unsigned short x __attribute__((packed)); };
#endif

static __inline__ unsigned long ldq_u(unsigned long * r11)
{
#if defined(__GNUC__) && ((__GNUC__ > 2) || (__GNUC_MINOR__ >= 91))
  const struct __una_u64 *ptr = (const struct __una_u64 *) r11;
  return ptr->x;
#else
  unsigned long r1,r2;
  __asm__("ldq_u %0,%3\n\t"
	  "ldq_u %1,%4\n\t"
	  "extql %0,%2,%0\n\t"
	  "extqh %1,%2,%1"
	  :"=&r" (r1), "=&r" (r2)
	  :"r" (r11),
	   "m" (*r11),
	   "m" (*(const unsigned long *)(7+(char *) r11)));
  return r1 | r2;
#endif
}

static __inline__ unsigned long ldl_u(unsigned int * r11)
{
#if defined(__GNUC__) && ((__GNUC__ > 2) || (__GNUC_MINOR__ >= 91))
  const struct __una_uint32_t *ptr = (const struct __una_uint32_t *) r11;
  return ptr->x;
#else
  unsigned long r1,r2;
  __asm__("ldq_u %0,%3\n\t"
	  "ldq_u %1,%4\n\t"
	  "extll %0,%2,%0\n\t"
	  "extlh %1,%2,%1"
	  :"=&r" (r1), "=&r" (r2)
	  :"r" (r11),
	   "m" (*r11),
	   "m" (*(const unsigned long *)(3+(char *) r11)));
  return r1 | r2;
#endif
}

static __inline__ unsigned long ldw_u(unsigned short * r11)
{
#if defined(__GNUC__) && ((__GNUC__ > 2) || (__GNUC_MINOR__ >= 91))
  const struct __una_uint16_t *ptr = (const struct __una_uint16_t *) r11;
  return ptr->x;
#else
  unsigned long r1,r2;
  __asm__("ldq_u %0,%3\n\t"
	  "ldq_u %1,%4\n\t"
	  "extwl %0,%2,%0\n\t"
	  "extwh %1,%2,%1"
	  :"=&r" (r1), "=&r" (r2)
	  :"r" (r11),
	   "m" (*r11),
	   "m" (*(const unsigned long *)(1+(char *) r11)));
  return r1 | r2;
#endif
}

/*
 * Elemental unaligned stores 
 */

static __inline__ void stq_u(unsigned long r5, unsigned long * r11)
{
#if defined(__GNUC__) && ((__GNUC__ > 2) || (__GNUC_MINOR__ >= 91))
  struct __una_u64 *ptr = (struct __una_u64 *) r11;
  ptr->x = r5;
#else
  unsigned long r1,r2,r3,r4;

  __asm__("ldq_u %3,%1\n\t"
	  "ldq_u %2,%0\n\t"
	  "insqh %6,%7,%5\n\t"
	  "insql %6,%7,%4\n\t"
	  "mskqh %3,%7,%3\n\t"
	  "mskql %2,%7,%2\n\t"
	  "bis %3,%5,%3\n\t"
	  "bis %2,%4,%2\n\t"
	  "stq_u %3,%1\n\t"
	  "stq_u %2,%0"
	  :"=m" (*r11),
	   "=m" (*(unsigned long *)(7+(char *) r11)),
	   "=&r" (r1), "=&r" (r2), "=&r" (r3), "=&r" (r4)
	  :"r" (r5), "r" (r11));
#endif
}

static __inline__ void stl_u(unsigned long r5, unsigned int * r11)
{
#if defined(__GNUC__) && ((__GNUC__ > 2) || (__GNUC_MINOR__ >= 91))
  struct __una_uint32_t *ptr = (struct __una_uint32_t *) r11;
  ptr->x = r5;
#else
  unsigned long r1,r2,r3,r4;

  __asm__("ldq_u %3,%1\n\t"
	  "ldq_u %2,%0\n\t"
	  "inslh %6,%7,%5\n\t"
	  "insll %6,%7,%4\n\t"
	  "msklh %3,%7,%3\n\t"
	  "mskll %2,%7,%2\n\t"
	  "bis %3,%5,%3\n\t"
	  "bis %2,%4,%2\n\t"
	  "stq_u %3,%1\n\t"
	  "stq_u %2,%0"
	  :"=m" (*r11),
	   "=m" (*(unsigned long *)(3+(char *) r11)),
	   "=&r" (r1), "=&r" (r2), "=&r" (r3), "=&r" (r4)
	  :"r" (r5), "r" (r11));
#endif
}

static __inline__ void stw_u(unsigned long r5, unsigned short * r11)
{
#if defined(__GNUC__) && ((__GNUC__ > 2) || (__GNUC_MINOR__ >= 91))
  struct __una_uint16_t *ptr = (struct __una_uint16_t *) r11;
  ptr->x = r5;
#else
  unsigned long r1,r2,r3,r4;

  __asm__("ldq_u %3,%1\n\t"
	  "ldq_u %2,%0\n\t"
	  "inswh %6,%7,%5\n\t"
	  "inswl %6,%7,%4\n\t"
	  "mskwh %3,%7,%3\n\t"
	  "mskwl %2,%7,%2\n\t"
	  "bis %3,%5,%3\n\t"
	  "bis %2,%4,%2\n\t"
	  "stq_u %3,%1\n\t"
	  "stq_u %2,%0"
	  :"=m" (*r11),
	   "=m" (*(unsigned long *)(1+(char *) r11)),
	   "=&r" (r1), "=&r" (r2), "=&r" (r3), "=&r" (r4)
	  :"r" (r5), "r" (r11));
#endif
}

#elif defined(__GNUC__) && ((__GNUC__ < 3)) &&	\
  (defined (__ia64__) || defined (ia64__))
#define IA64_UALOADS
/*
 * EGCS 1.1 knows about arbitrary unaligned loads.  Define some
 * packed structures to talk about such things with.
 */
struct __una_u64 { unsigned long  x __attribute__((packed)); };
struct __una_uint32_t { unsigned int   x __attribute__((packed)); };
struct __una_uint16_t { unsigned short x __attribute__((packed)); };

static __inline__ unsigned long
__uldq (const unsigned long * r11)
{
  const struct __una_u64 *ptr = (const struct __una_u64 *) r11;
  return ptr->x;
}

static __inline__ unsigned long
uldl (const unsigned int * r11)
{
  const struct __una_uint32_t *ptr = (const struct __una_uint32_t *) r11;
  return ptr->x;
}

static __inline__ unsigned long
uldw (const unsigned short * r11)
{
  const struct __una_uint16_t *ptr = (const struct __una_uint16_t *) r11;
  return ptr->x;
}

static __inline__ void
ustq (unsigned long r5, unsigned long * r11)
{
  struct __una_u64 *ptr = (struct __una_u64 *) r11;
  ptr->x = r5;
}

static __inline__ void
ustl (unsigned long r5, unsigned int * r11)
{
  struct __una_uint32_t *ptr = (struct __una_uint32_t *) r11;
  ptr->x = r5;
}

static __inline__ void
ustw (unsigned long r5, unsigned short * r11)
{
  struct __una_uint16_t *ptr = (struct __una_uint16_t *) r11;
  ptr->x = r5;
}

#endif

/****************************************************************************
PARAMETERS:
addr	- Emulator memory address to read

RETURNS:
Byte value read from emulator memory.

REMARKS:
Reads a byte value from the emulator memory. 
****************************************************************************/
uint8_t X86API rdb(
		   uint32_t addr)
{
  uint8_t val;

  if (addr > M.mem_size - 1) {
    DB(printk("mem_read: address %#lx out of range!\n", addr);)
      HALT_SYS();
  }
  val = *(uint8_t*)(M.mem_base + addr);
  DB(	if (DEBUG_MEM_TRACE())
	  printk("%#08x 1 -> %#x\n", addr, val);)
    return val;
}

/****************************************************************************
PARAMETERS:
addr	- Emulator memory address to read

RETURNS:
Word value read from emulator memory.

REMARKS:
Reads a word value from the emulator memory.
****************************************************************************/
uint16_t X86API rdw(
		    uint32_t addr)
{
  uint16_t val = 0;

  if (addr > M.mem_size - 2) {
    DB(printk("mem_read: address %#lx out of range!\n", addr);)
      HALT_SYS();
  }
#ifdef __BIG_ENDIAN__
  if (addr & 0x1) {
    val = (*(uint8_t*)(M.mem_base + addr) |
	   (*(uint8_t*)(M.mem_base + addr + 1) << 8));
  }
  else
#endif
#if defined(ALPHA_UALOADS)
    val = ldw_u((uint16_t*)(M.mem_base + addr));
#elif  defined(IA64_UALOADS)
  val = uldw((uint16_t*)(M.mem_base + addr));
#else
  val = *(uint16_t*)(M.mem_base + addr);
#endif
  DB(	if (DEBUG_MEM_TRACE())
	  printk("%#08x 2 -> %#x\n", addr, val);)
    return val;
}

/****************************************************************************
PARAMETERS:
addr	- Emulator memory address to read

RETURNS:
Long value read from emulator memory.
REMARKS:
Reads a long value from the emulator memory. 
****************************************************************************/
uint32_t X86API rdl(
		    uint32_t addr)
{
  uint32_t val = 0;

  if (addr > M.mem_size - 4) {
    DB(printk("mem_read: address %#lx out of range!\n", addr);)
      HALT_SYS();
  }
#ifdef __BIG_ENDIAN__
  if (addr & 0x3) {
    val = (*(uint8_t*)(M.mem_base + addr + 0) |
	   (*(uint8_t*)(M.mem_base + addr + 1) << 8) |
	   (*(uint8_t*)(M.mem_base + addr + 2) << 16) |
	   (*(uint8_t*)(M.mem_base + addr + 3) << 24));
  }
  else
#endif
#if defined(ALPHA_UALOADS)
    val = ldl_u((uint32_t*)(M.mem_base + addr));
#elif  defined(IA64_UALOADS)
  val = uldl((uint32_t*)(M.mem_base + addr));
#else
  val = *(uint32_t*)(M.mem_base + addr);
#endif
  DB(	if (DEBUG_MEM_TRACE())
	  printk("%#08x 4 -> %#x\n", addr, val);)
    return val;
}

/****************************************************************************
PARAMETERS:
addr	- Emulator memory address to read
val		- Value to store

REMARKS:
Writes a byte value to emulator memory.
****************************************************************************/
void X86API wrb(
		uint32_t addr,
		uint8_t val)
{
  DB(	if (DEBUG_MEM_TRACE())
	  printk("%#08x 1 <- %#x\n", addr, val);)
    if (addr > M.mem_size - 1) {
      DB(printk("mem_write: address %#lx out of range!\n", addr);)
	HALT_SYS();
    }
  *(uint8_t*)(M.mem_base + addr) = val;
}

/****************************************************************************
PARAMETERS:
addr	- Emulator memory address to read
val		- Value to store

REMARKS:
Writes a word value to emulator memory.
****************************************************************************/
void X86API wrw(
		uint32_t addr,
		uint16_t val)
{
  DB(	if (DEBUG_MEM_TRACE())
	  printk("%#08x 2 <- %#x\n", addr, val);)
    if (addr > M.mem_size - 2) {
      DB(printk("mem_write: address %#lx out of range!\n", addr);)
	HALT_SYS();
    }
#ifdef __BIG_ENDIAN__
  if (addr & 0x1) {
    *(uint8_t*)(M.mem_base + addr + 0) = (val >> 0) & 0xff;
    *(uint8_t*)(M.mem_base + addr + 1) = (val >> 8) & 0xff;
  }
  else
#endif
#if defined(ALPHA_UALOADS)
    stw_u(val,(uint16_t*)(M.mem_base + addr));
#elif defined(IA64_UALOADS)
  ustw(val,(uint16_t*)(M.mem_base + addr));
#else
  *(uint16_t*)(M.mem_base + addr) = val;
#endif
}

/****************************************************************************
PARAMETERS:
addr	- Emulator memory address to read
val		- Value to store

REMARKS:
Writes a long value to emulator memory. 
****************************************************************************/
void X86API wrl(
		uint32_t addr,
		uint32_t val)
{
  DB(	if (DEBUG_MEM_TRACE())
	  printk("%#08x 4 <- %#x\n", addr, val);)
    if (addr > M.mem_size - 4) {
      DB(printk("mem_write: address %#lx out of range!\n", addr);)
	HALT_SYS();
    }
#ifdef __BIG_ENDIAN__
  if (addr & 0x1) {
    *(uint8_t*)(M.mem_base + addr + 0) = (val >>  0) & 0xff;
    *(uint8_t*)(M.mem_base + addr + 1) = (val >>  8) & 0xff;
    *(uint8_t*)(M.mem_base + addr + 2) = (val >> 16) & 0xff;
    *(uint8_t*)(M.mem_base + addr + 3) = (val >> 24) & 0xff;
  }
  else
#endif
#if defined(ALPHA_UALOADS)
    stl_u(val,(uint32_t*)(M.mem_base + addr));
#elif defined(IA64_UALOADS)
  ustl(val,(uint32_t*)(M.mem_base + addr));
#else
  *(uint32_t*)(M.mem_base + addr) = val;
#endif
}

/****************************************************************************
PARAMETERS:
addr	- PIO address to read
RETURN:
0
REMARKS:
Default PIO byte read function. Doesn't perform real inb.
****************************************************************************/
static uint8_t X86API p_inb(
			    uint16_t addr)
{
  DB(	if (DEBUG_IO_TRACE())
	  printk("inb %#04x \n", addr);)
    return 0;
}

/****************************************************************************
PARAMETERS:
addr	- PIO address to read
RETURN:
0
REMARKS:
Default PIO word read function. Doesn't perform real inw.
****************************************************************************/
static uint16_t X86API p_inw(
			     uint16_t addr)
{
  DB(	if (DEBUG_IO_TRACE())
	  printk("inw %#04x \n", addr);)
    return 0;
}

/****************************************************************************
PARAMETERS:
addr	- PIO address to read
RETURN:
0
REMARKS:
Default PIO long read function. Doesn't perform real inl.
****************************************************************************/
static uint32_t X86API p_inl(
			     uint16_t addr)
{
  DB(	if (DEBUG_IO_TRACE())
	  printk("inl %#04x \n", addr);)
    return 0;
}

/****************************************************************************
PARAMETERS:
addr	- PIO address to write
val     - Value to store
REMARKS:
Default PIO byte write function. Doesn't perform real outb.
****************************************************************************/
static void X86API p_outb(
			  uint16_t addr,
			  uint8_t val)
{
  DB(	if (DEBUG_IO_TRACE())
	  printk("outb %#02x -> %#04x \n", val, addr);)
    return;
}

/****************************************************************************
PARAMETERS:
addr	- PIO address to write
val     - Value to store
REMARKS:
Default PIO word write function. Doesn't perform real outw.
****************************************************************************/
static void X86API p_outw(
			  uint16_t addr,
			  uint16_t val)
{
  DB(	if (DEBUG_IO_TRACE())
	  printk("outw %#04x -> %#04x \n", val, addr);)
    return;
}

/****************************************************************************
PARAMETERS:
addr	- PIO address to write
val     - Value to store
REMARKS:
Default PIO ;ong write function. Doesn't perform real outl.
****************************************************************************/
static void X86API p_outl(
			  uint16_t addr,
			  uint32_t val)
{
  DB(	if (DEBUG_IO_TRACE())
	  printk("outl %#08x -> %#04x \n", val, addr);)
    return;
}

/*------------------------- Global Variables ------------------------------*/

uint8_t  	(X86APIP sys_rdb)(uint32_t addr) 			            = rdb;
uint16_t 	(X86APIP sys_rdw)(uint32_t addr) 			            = rdw;
uint32_t 	(X86APIP sys_rdl)(uint32_t addr) 			            = rdl;
void 	(X86APIP sys_wrb)(uint32_t addr,uint8_t val) 		            = wrb;
void 	(X86APIP sys_wrw)(uint32_t addr,uint16_t val) 	            = wrw;
void 	(X86APIP sys_wrl)(uint32_t addr,uint32_t val) 	            = wrl;
uint8_t  	(X86APIP sys_inb)(uint16_t addr)	            = p_inb;
uint16_t 	(X86APIP sys_inw)(uint16_t addr)	            = p_inw;
uint32_t 	(X86APIP sys_inl)(uint16_t addr)              = p_inl;
void 	(X86APIP sys_outb)(uint16_t addr, uint8_t val) 	= p_outb;
void 	(X86APIP sys_outw)(uint16_t addr, uint16_t val)	= p_outw;
void 	(X86APIP sys_outl)(uint16_t addr, uint32_t val)	= p_outl;

/*----------------------------- Setup -------------------------------------*/

/****************************************************************************
PARAMETERS:
funcs	- New memory function pointers to make active

REMARKS:
This function is used to set the pointers to functions which access
memory space, allowing the user application to override these functions
and hook them out as necessary for their application.
****************************************************************************/
void X86EMU_setupMemFuncs(
			  X86EMU_memFuncs *funcs)
{
  sys_rdb = funcs->rdb;
  sys_rdw = funcs->rdw;
  sys_rdl = funcs->rdl;
  sys_wrb = funcs->wrb;
  sys_wrw = funcs->wrw;
  sys_wrl = funcs->wrl;
}

/****************************************************************************
PARAMETERS:
funcs	- New programmed I/O function pointers to make active

REMARKS:
This function is used to set the pointers to functions which access
I/O space, allowing the user application to override these functions
and hook them out as necessary for their application.
****************************************************************************/
void X86EMU_setupPioFuncs(
			  X86EMU_pioFuncs *funcs)
{
  sys_inb = funcs->inb;
  sys_inw = funcs->inw;
  sys_inl = funcs->inl;
  sys_outb = funcs->outb;
  sys_outw = funcs->outw;
  sys_outl = funcs->outl;
}

/****************************************************************************
PARAMETERS:
funcs	- New interrupt vector table to make active

REMARKS:
This function is used to set the pointers to functions which handle
interrupt processing in the emulator, allowing the user application to
hook interrupts as necessary for their application. Any interrupts that
are not hooked by the user application, and reflected and handled internally
in the emulator via the interrupt vector table. This allows the application
to get control when the code being emulated executes specific software
interrupts.
****************************************************************************/
void X86EMU_setupIntrFuncs(
			   X86EMU_intrFuncs funcs[])
{
  int i;
    
  for (i=0; i < 256; i++)
    _X86EMU_intrTab[i] = NULL;
  if (funcs) {
    for (i = 0; i < 256; i++)
      _X86EMU_intrTab[i] = funcs[i];
  }
}

/****************************************************************************
PARAMETERS:
int	- New software interrupt to prepare for

REMARKS:
This function is used to set up the emulator state to exceute a software
interrupt. This can be used by the user application code to allow an
interrupt to be hooked, examined and then reflected back to the emulator
so that the code in the emulator will continue processing the software
interrupt as per normal. This essentially allows system code to actively
hook and handle certain software interrupts as necessary.
****************************************************************************/
void X86EMU_prepareForInt(
			  int num)
{
  push_word((uint16_t)M.x86.R_FLG);
  CLEAR_FLAG(F_IF);
  CLEAR_FLAG(F_TF);
  push_word(M.x86.R_CS);
  M.x86.R_CS = mem_access_word(num * 4 + 2);
  push_word(M.x86.R_IP);
  M.x86.R_IP = mem_access_word(num * 4);
  M.x86.intr = 0;
}
