/*
 * Realmode X86 Emulator Library
 *
 * This library is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU Lesser General Public License as 
 * published by the Free Software Foundation; either version 2.1 of 
 * the License.
 *
 * This library is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public 
 * License along with this library; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 *
 * Copyright (C) 1996-1999 SciTech Software, Inc.
 * Copyright (C) David Mosberger-Tang
 * Copyright (C) 1999 Egbert Eich
 * Copyright (C) 2008 Jari OS core team <http://jarios.org>
 * Copyright (C) 2008 Paul Graphov <graphov@jarios.org> 
 *               (porting to Jari OS)
 *
 */

#ifndef __X86EMU_DECODE_H
#define __X86EMU_DECODE_H

/*---------------------- Macros and type definitions ----------------------*/

/* Instruction Decoding Stuff */

#define FETCH_DECODE_MODRM(mod,rh,rl) 	fetch_decode_modrm(&mod,&rh,&rl)
#define DECODE_RM_BYTE_REGISTER(r)    	decode_rm_byte_register(r)
#define DECODE_RM_WORD_REGISTER(r)    	decode_rm_word_register(r)
#define DECODE_RM_LONG_REGISTER(r)    	decode_rm_long_register(r)
#define DECODE_CLEAR_SEGOVR()         	M.x86.mode &= ~SYSMODE_CLRMASK

/*-------------------------- Function Prototypes --------------------------*/

#ifdef  __cplusplus
extern "C" {            			/* Use "C" linkage when in C++ mode */
#endif

void 	x86emu_intr_raise (uint8_t type);
void    fetch_decode_modrm (int *mod,int *regh,int *regl);
uint8_t      fetch_byte_imm (void);
uint16_t     fetch_word_imm (void);
uint32_t     fetch_long_imm (void);
uint8_t      fetch_data_byte (uint32_t offset);
uint8_t      fetch_data_byte_abs (uint32_t segment, uint32_t offset);
uint16_t     fetch_data_word (uint32_t offset);
uint16_t     fetch_data_word_abs (uint32_t segment, uint32_t offset);
uint32_t     fetch_data_long (uint32_t offset);
uint32_t     fetch_data_long_abs (uint32_t segment, uint32_t offset);
void    store_data_byte (uint32_t offset, uint8_t val);
void    store_data_byte_abs (uint32_t segment, uint32_t offset, uint8_t val);
void    store_data_word (uint32_t offset, uint16_t val);
void    store_data_word_abs (uint32_t segment, uint32_t offset, uint16_t val);
void    store_data_long (uint32_t offset, uint32_t val);
void    store_data_long_abs (uint32_t segment, uint32_t offset, uint32_t val);
uint8_t* 	decode_rm_byte_register(int reg);
uint16_t* 	decode_rm_word_register(int reg);
uint32_t* 	decode_rm_long_register(int reg);
uint16_t* 	decode_rm_seg_register(int reg);
uint32_t	decode_rm00_address(int rm);
uint32_t	decode_rm01_address(int rm);
uint32_t	decode_rm10_address(int rm);
uint32_t	decode_sib_address(int sib, int mod);

#ifdef  __cplusplus
}                       			/* End of "C" linkage for C++   	*/
#endif

#endif /* __X86EMU_DECODE_H */
