/*
 * This library is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU Lesser General Public License as 
 * published by the Free Software Foundation; either version 2.1 of 
 * the License.
 *
 * This library is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public 
 * License along with this library; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 *
 * (c) Copyright 2006,2007,2008 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2008 MadTirra <madtirra@jarios.org>
 *
 * LibAlloc:
 * lib/imcache.c: Simple implementation of virtual address space manager.
 *                It was used in ilix and works well on many cases, but it's 
 *                just one of the memory managers/allocators implemented on 
 *                uspace, and it's possible to choose one what you need.
 *
 */

#ifndef __ALLOC_IMCACHE_H__
#define __ALLOC_IMCACHE_H__

#include <sys/types.h>
#include <sys/mman.h>
#include <datalib/usrtc.h>

#define MAX_ER_SIZE  256
#define FULLFILL     0xff

typedef struct __mmap_area_t {
  usrtc_node_t node;
  void *addr;
  size_t pages;
} mmap_area_t;

typedef struct __mcache_t {
  usrtc_t free;  /* free areas */
  usrtc_t alloc; /* alloc areas */
  usrtc_t raw;   /* raw areas that doesn't used yet */
  struct __mcache_t *head;
  struct __mcache_t *next;
  struct __mcache_t *prev;
} mcache_t;

typedef struct __stack_node {
  usrtc_node_t node;
  ulong_t start_addr,end_addr;  
} stack_node_t;

typedef struct __stack_cache {
  usrtc_t free,active;
} stack_cache_t;

/* functions */

/* allocate pages amount , return virtual non mapped valid address */
void *alloc_virt_pages(size_t pages);

/* free virtual pages, return 0 on succeful, actually doesn't unmap pages */
status_t free_virt_pages(void *addr);

/* cache virtual addres is valid */
status_t validate_virt_addr(void *addr);

/* allocate unique memory range for a stack and retur its virtual address */
void *alloc_stack_pages(size_t pages);

#endif /* __ALLOC_IMCACHE_H__ */

