/*
 * This library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * (c) Copyright 2006,2007,2008,2009 Jari OS Core Team <http://jarios.org>
 *
 * LibC:
 * include/errno.h: error codes
 */

#ifndef __ERRNO_H__
#define __ERRNO_H__

#include <arch/errno.h>

/* ANSI C error codes */
#define EPERM            1      /* Operation not permitted */
#define ENOENT           2      /* No such file or directory */
#define ESRCH            3      /* No such process */
#define EINTR            4      /* Interrupted system call */
#define EIO              5      /* I/O error */
#define ENXIO            6      /* No such device or address */
#define E2BIG            7      /* Argument list too long */
#define ENOEXEC          8      /* Exec format error */
#define EBADF            9      /* Bad file number */
#define ECHILD          10      /* No child processes */
#define EAGAIN          11      /* Try again */
#define ENOMEM          12      /* Out of memory */
#define EACCES          13      /* Permission denied */
#define EFAULT          14      /* Bad address */
#define ENOTBLK         15      /* Block device required */
#define EBUSY           16      /* Device or resource busy */
#define EEXIST          17      /* File exists */
#define EXDEV           18      /* Cross-device link */
#define ENODEV          19      /* No such device */
#define ENOTDIR         20      /* Not a directory */
#define EISDIR          21      /* Is a directory */
#define EINVAL          22      /* Invalid argument */
#define ENFILE          23      /* File table overflow */
#define EMFILE          24      /* Too many open files */
#define ENOTTY          25      /* Not a typewriter */
#define ETXTBSY         26      /* Text file busy */
#define EFBIG           27      /* File too large */
#define ENOSPC          28      /* No space left on device */
#define ESPIPE          29      /* Illegal seek */
#define EROFS           30      /* Read-only file system */
#define EMLINK          31      /* Too many links */
#define EPIPE           32      /* Broken pipe */
#define EDOM            33      /* Math argument out of domain of func */
#define ERANGE          34      /* Math result not representable */

/* MuiString error codes */
#define ENOSYS          35      /* Not implemented system functionality. */
#define EWOULDBLOCK     36      /* Call would block. */
#define EDEADLK         37      /* Resource deadlock would occur */
#define EDEADLOCK       EDEADLK

/* Jari OS v2 error codes */
#define ENOSLOT         240     /* No free slot found to mount */
#define EPOSTPONED      241     /* Call was postponed */

/* POSIX error codes */

/* These two error codes overlap with MuiString values. Add 200 to them */
#define ENAMETOOLONG   236      /* File or device name is too long */
#define ENOLCK         237      /* No locks available */

#define ENOTEMPTY       39      /* Directory not empty */
#define ELOOP           40      /* Too many levels of symbolic links */
#define ENOMSG          42      /* No message of the desired type */
#define EIDRM           44      /* Identifier removed */

#define ENOSTR          60      /* Not a STREAM */
#define ENODATA         61      /* No message is available on the STREAM head read queue */
#define ETIME           62      /* Stream ioctl() timeout */
#define ENOSR           63      /* No STREAM resources */
#define ENOLINK         67      /* Reserved */
#define EPROTO          71      /* Protocol error */
#define EMULTIHOP       72      /* Reserved */
#define EBADMSG         74      /* Bad message */
#define EOVERFLOW       43      /* Value too large to be stored in data type */
#define EILSEQ          84      /* Bad character sequence.  */
#define ENOTSOCK        88      /* Not a socket */
#define EDESTADDRREQ    89      /* Destination address required */
#define EMSGSIZE        90      /* Message too large */
#define EPROTOTYPE      91      /* Protocol wrong type for socket */
#define ENOPROTOOPT     92      /* Protocol not available */
#define EPROTONOSUPPORT 93      /* Protocol not supported */
#define EOPNOTSUPP      95      /* Operation not supported on socket */
#define ENOTSUP         41      /* Not supported */
#define EAFNOSUPPORT    97      /* Address family not supported */
#define EADDRINUSE      98      /* Address in use */
#define EADDRNOTAVAIL   99      /* Address not available */
#define ENETDOWN       100      /* Network is down */
#define ENETUNREACH    101      /* Network unreachable */
#define ENETRESET      102      /* Connection aborted by network */
#define ECONNABORTED   103      /* Connection aborted */
#define ECONNRESET     104      /* Connection reset */
#define ENOBUFS        105      /* No buffer space available */
#define EISCONN        106      /* Socket is connected */
#define ENOTCONN       107      /* The socket is not connected */
#define ESHUTDOWN      108      /* Cannot send after endpoint shutdown */
#define ETIMEDOUT      110      /* Connection timed out */
#define ECONNREFUSED   111      /* Connection refused */
#define EHOSTUNREACH   113      /* Host is unreachable */
#define EALREADY       114      /* Connection already in progress */
#define EINPROGRESS    115      /* Operation in progress */
#define ESTALE         116      /* Reserved */
#define EDQUOT         122      /* Reserved */

#define ECANCELED      125      /* Operation canceled */

/* POSIX requires this error code also */
#define ENOINFO        200      /* No info found */

/* Declare the 'errno' variable, unless it's defined as a macro in a
 * arch-specific way.
 */
#ifndef errno
extern int errno;
#endif

#define POSIX_ERRNO_RETURN_PATH(e)  \
  if ((e) < 0) {                    \
    errno = -(e);                   \
    return -1;                      \
  } else                            \
    return 0;

#endif  /* __ERRNO_H__ */
