/*
 * This library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * (c) Copyright 2006,2007,2008,2009 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2008 Dmitry Gromada <gromada@jarios.org>
 * (c) Copyright 2009 Alfeiks Kaanoken (MadTirra) <madtirra@jarios.org>
 *
 * LibC:
 * include/fcntl.h: File opening, locking and other operations.
 */

#ifndef __FCNTL_H__
#define __FCNTL_H__

/* Commands */
enum {
  F_DUPFD,
  F_GETFD,
  F_SETFD,
  F_GETFL,
  F_SETFL,
  F_GETLK,
  F_SETLK,
  F_SETLKW,
  F_GETOWN,
  F_SETOWN
};

/* File descriptor flags */
#define FD_CLOEXEC     0x1

/* File access modes */
#define O_RDONLY       (1 << 0)
#define O_WRONLY       (1 << 1)
#define O_RDWR         (O_RDONLY | O_WRONLY)

/* Additional file access flags  */
#define O_APPEND       (1 << 3)
#define O_CREAT        (1 << 4)
#define O_DSYNC        (1 << 5)
#define O_EXCL         (1 << 6)
#define O_NOCTTY       (1 << 7)
#define O_NONBLOCK     (1 << 8)
#define O_RSYNC        (1 << 9)
#define O_SYNC         (1 << 10)
#define O_TRUNC        (1 << 11)

/* Jari OS specific */
#define O_DIRECTORY    (1 << 12)       /* JariOS: open directory */
#define O_CHARDEV      (1 << 13)       /* JariOS: char device */
#define O_BLKDEV       (1 << 14)       /* JariOS: block device */
#define O_DIRECTIO     (1 << 15)       /* JariOS: direct I/O ops with device files */

/* 
 * *NOTE:* Jari OS specific -> 
 * to create/open block device - add O_BLKDEV,
 * for char device - add O_CHARDEV, mixing there ones
 * will returns EINVAL!
 */

/* Record locking types */
enum {
  F_RDLCK,
  F_UNLCK,
  F_WRLCK
};

int open(const char *path, int oflags, ...);
int creat(const char *path, mode_t mode);
int close(int fildes);

int fcntl(int fildes, int cmd, ...);

/* JariOS specific */
pid_t getfpid(int fildes);
int getufsid(int fildes);

#endif
