/*
 * This library is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU Lesser General Public License as 
 * published by the Free Software Foundation; either version 2.1 of 
 * the License.
 *
 * This library is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public 
 * License along with this library; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 *
 * (c) Copyright 2006,2007,2008 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2008 MadTirra <madtirra@jarios.org>
 *
 * LibC:
 * include/ipc/port.h: IPC port
 *
 */

#ifndef __IPCPORT_H__
#define __IPCPORT_H__

#include <sys/types.h>
#include <time.h>
#include <sys/poll.h>
#include <datalib/usrtc.h>

#define IPC_BLOCKED_ACCESS      0x1
#define IPC_PRIORITIZED_ACCESS  0x2

/* default queue size */
#define DQUEUE_SIZE  512 
#define DTOPORTS     32

typedef ulong_t prtid_t;
typedef ulong_t ippqid_t;

typedef enum {
  FIFO = 0 ,
  STACK ,
  CUSTOM ,
} ippq_type_t;

/* reply ipc port */
typedef struct __ipc_port_msg_info {
  uint16_t msg_id;
  pid_t sender_pid;
  uint32_t msg_len;
  tid_t sender_tid;
  uid_t sender_uid;
  uid_t sender_gid;
} ipc_port_msg_info_t;

typedef struct __ipc_port_t {
  prtid_t id;
  ulong_t flags;
  ipc_port_msg_info_t msginfo;
} ipc_port_t;

/* General IPC port operations */
prtid_t ipc_port_open(ulong_t,size_t);
int ipc_port_close(prtid_t);
int ipc_port_reply(prtid_t,size_t,void *);
int ipc_port_reply_nmsg(prtid_t,size_t,void *,int);
int ipc_port_rcvmsg(prtid_t,size_t,void *);
int ipc_port_rcvmsg_mi(prtid_t,size_t,void *,ipc_port_msg_info_t *minfo);
ipc_port_msg_info_t ipc_port_getinfo(prtid_t);

/* Advanced operations */
/*
 * ipc_port_read_msg: reads a message data grom a given 
 * message offset with len.
 * return -1 on error (and set errno to error), or 0 if all is ok
 */
int ipc_port_read_msg(prtid_t id,int msg_id,size_t offset,
		      void *msg,size_t recv_len);
/* receive message with custom flags */
int ipc_port_rcvmsg_mi_cs(prtid_t id, size_t msg_size, void * msg,
			  ipc_port_msg_info_t *minfo,int flags);


/* Polling */
status_t ipc_port_poll(pollfd_t *fds,ulong_t nfds,timeval_t *timeout);

/* IPPQ = IPC port queue */

#define IPPQ_IMMEDIATE  0x1
#define IPPQ_WREPL      0x2

typedef struct __ippq_ent_t {
  ipc_port_msg_info_t msginfo;
  size_t dsize;
  void *data;
} ippq_ent_t;

typedef struct __ippq_t {
  ippqid_t id;
  prtid_t port_id;
  ippq_type_t qtype;
  int flags;
  usrtc_t *queue;
  int (*queue_put)(struct __ippq_t *,void *,size_t ,ipc_port_msg_info_t );
  ippq_ent_t* (*queue_get)(struct __ippq_t *,void *);
  int (*rep_hook)(struct __ippq_t *,void *,size_t,ipc_port_msg_info_t);
  int (*del_hook)(struct __ippq_t *,ippq_ent_t *);
  void *data;
} ippq_t;

ippqid_t ippq_create(ulong_t flags,size_t kqueue_size,ippq_type_t type,int qflags);
int ippq_destroy(ippqid_t qid);
int ippq_rcvmsg(ippqid_t qid,size_t size,void *buf);
ippq_ent_t *ippq_getmsg(ippqid_t qid);
int ippq_rplmsg(ippqid_t qid,size_t size,void *buf,ippq_ent_t *ent);

void *ippq_ent_get_data(size_t *,ippq_ent_t *);
void ippq_ent_destroy(ippq_ent_t *);

int ippq_set_rephook(ippqid_t qid,int (*rep_hook)(ippq_t *,void *,size_t,ipc_port_msg_info_t));
int ippq_set_delhook(ippqid_t qid,int (*del_hook)(ippq_t *,ippq_ent_t *));
int ippq_set_puthook(ippqid_t qid,int (*queue_put)(ippq_t *,void *,size_t,ipc_port_msg_info_t));
int ippq_set_gethook(ippqid_t qid,ippq_ent_t* (*queue_get)(ippq_t *,void *));

#endif /* __IPCPORT_H__ */

