/*
 * This library is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU Lesser General Public License as 
 * published by the Free Software Foundation; either version 2.1 of 
 * the License.
 *
 * This library is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public 
 * License along with this library; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 *
 * (c) Copyright 2006,2007,2008 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2005,2008 MadTirra <madtirra@jarios.org> 
 *
 * RPC commands and specified protocols
 * 
 */

#ifndef __LIBOS_RPC_H__
#define __LIBOS_RPC_H__

#include <sys/types.h>
#include <ipc/channel.h>
#include <ipc/iov.h>

/**
 * NOTE:
 *  in case of minimality don't add specific RPC calls
 *  it can override other things.
 */

/* generic RPC calls */
#define RPC_ALIVE   0x0
#define RPC_STATUS  0x1
#define RPC_INFO    0xaa

/* generic operational calls */
#define RPC_OPEN         0x2
#define RPC_CLOSE        0x3
#define RPC_READ         0x4
#define RPC_WRITE        0x5
#define RPC_EXEC         0x6
#define RPC_IOCTL        0x7
#define RPC_REGISTER     0x8
#define RPC_UNREGISTER   0x9
#define RPC_CREATE       0xa
#define RPC_UNLINK       0xb
#define RPC_CREATEDIR    0xc
#define RPC_UNLINKDIR    0xd
#define RPC_DIRECTIO     0xe
#define RPC_REGEVENTS    0xf
#define RPC_UNREGEVENTS  0x10
#define RPC_READDIR      0x11
#define RPC_OPENDIR      0x12
#define RPC_CLOSEDIR     0x13
#define RPC_STAT         0x14
#define RPC_OPENDIO      0x15
#define RPC_CLOSEDIO     0x16
#define RPC_READDIO      0x17
#define RPC_WRITEDIO     0x18
#define RPC_SETCAP       0x19
#define RPC_GETHWINFO    0x1a
#define RPC_SETHWINFO    0x1b
#define RPC_DUP          0x1c
#define RPC_DUP2         0x1d
#define RPC_SYNC         0x1e
#define RPC_MOUNT        0x1f
#define RPC_UMOUNT       0x20
#define RPC_PIDFDGET     0x21
#define RPC_PIDFDCLONE   0x22
#define RPC_MMAP         0x23
#define RPC_UNMAP        0x24
#define RPC_GETCWD       0x25
#define RPC_SETPWD       0x26
#define RPC_FCNTL        0x27
#define RPC_PIDFDRESET   0x28 /* behaivor on exec call */
#define RPC_PIDFDKILL    0x29 /* kill all resources */
#define RPC_LOOKUP       0x2a /* uni call */
#define RPC_FSTAT        0x2b
#define RPC_FTRUNCATE    0x2c
#define RPC_TRUNCATE     0x2d
#define RPC_SETATTRS     0x2e
#define RPC_GETENVIRON   0x2f
#define RPC_SETENVIRON   0x30
#define RPC_SETENV       0x31
#define RPC_UPDENV       0x32
#define RPC_UNAME        0x33
#define RPC_LSTAT        0x34 /* POSIX lstat() remote call */
#define RPC_SYMLINK      0x35 /* POSIX symlink() remote call */
#define RPC_READLINK     0x36 /* POSIX readlink() remote call */

typedef struct __uni_rpc_t {
  uint32_t rpc_call : 16;
  uint32_t arg0 : 16;
  uint32_t arg1 : 16;
  uint32_t arg2 : 16;
  uint32_t arg3 : 16;
  uint32_t arg4 : 16;
  uint32_t arg5 : 16;
  uint32_t arg6 : 16;
  size_t size;
  size_t offset;
} __attribute__ ((packed)) uni_rpc_t;

void *unirpc_send(chid_t ch,uni_rpc_t rpc,size_t msg_size,
                  void *msg,size_t rlen,void *rmsg);

#endif /* __LIBOS_RPC_H__ */
