/*
 * This library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * (c) Copyright 2006,2007,2008,2009 Jari OS Core Team <http://jarios.org>
 *
 * LibC:
 * include/limits.h: Define constants specifying the implementation-specific
 *                   properties of the integer types.
 */

#ifndef __LIMITS_H__
#define __LIMITS_H__

#include <arch/wordsize.h>

/* specific */
#define  POSIX2_RE_DUP_MAX  255

/* Number of bits in a byte */
#define CHAR_BIT             8

/* Min/max values for a char */
#define CHAR_MIN             (-128)
#define CHAR_MAX             (127)

/* Min/max values for a signed char */
#define SCHAR_MIN            (-128)
#define SCHAR_MAX            (127)

/* Max value for a unsigned char */
#define UCHAR_MAX            (255)

/* Min/max values for a signed short int */
#define SHRT_MIN             (-32768)
#define SHRT_MAX             (32767)

/* Max value for a unsigned short int */
#define USHRT_MAX            (65535)

/* Min/max values for a signed int */
#define INT_MIN              (-2147483648)
#define INT_MAX              (2147483647)

/* Max value for a unsigned int */
#define UINT_MAX             (4294967295U)

#if BITS_PER_LONG == 64
/* Min/max values for a signed long */
#define LONG_MIN  (-9223372036854775807L)
#define LONG_MAX  (9223372036854775807L)

/* Max value for a unsigned long */
#define ULONG_MAX            (18446744073709551615UL)
#endif

/* Min/max values for a signed long long int */
#define LLONG_MIN            (-9223372036854775808L)
#define LLONG_MAX            (9223372036854775807L)

/* Max value for a unsigned long long int */
#define ULLONG_MAX           (18446744073709551615UL)

#ifndef SSIZE_MAX
#define SSIZE_MAX            LONG_MAX
#endif

/* Runtime Invariant Values */

#define OPEN_MAX             256       /* # files that one process can have
					* open at any one time.
					* Must be order of 8
					*/

#define ARG_MAX              128       /* Maximum number of command-line arguments
                                        * to be passed for a program via 'execve()',
					* or in environ variable.
					* Must be order of 8
                                        */

#define ATEXIT_MAX           32        /* Maximum number of functions that may
					* be registered with atexit().
					*/

#define LOGIN_NAME_MAX       256       /* Maximim length of a login name.
					*/

/* Pathname Variable Values */

/* Don't forget to check dirent.d_name field from dirent.h */
#define NAME_MAX             512       /* # chars in a file name */

#define PATH_MAX             4096      /* # chars in a path name (including nul) */

/* Runtime Increasable Values */

#define LINE_MAX             2048      /* The maximum length, in bytes, of a
					* utility's input line. The length
					* includes room for the trailing
					* <newline>.
					*/

/* Jari OS specific Values */

#define IPC_PORT_MSGLEN_MAX  2097152L  /* IPC port maximum message length
					* (2 MB)
					*/

#endif  /* __LIMITS_H__ */
