/*
 * This library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * (c) Copyright 2006,2007,2008,2009 Jari OS Core Team <http://jarios.org>
 *
 * LibC:
 * include/locale.h: setlocale() and related constants.
 */

#ifndef __LOCALE_H__
#define __LOCALE_H__

#include <stddef.h>

enum {
  LC_CTYPE = 0,
  LC_NUMERIC = 1,
  LC_TIME = 2,
  LC_COLLATE = 3,
  LC_MONETARY = 4,
  LC_MESSAGES = 5,
  LC_ALL = 6
};

struct lconv
{
  /* Numeric (non-monetary) information.  */

  char *decimal_point;       /* Decimal point character */
  char *thousands_sep;       /* Thousands separator */
  /* Each element is the number of digits in each group;
   * elements with higher indices are farther left.
   * An element with value CHAR_MAX means that no further grouping is done.
   * An element with value 0 means that the previous element is used
   * for all groups farther left.
   */
  char *grouping;

  /* Monetary information.  */

  /* First three chars are a currency symbol from ISO 4217.
   * Fourth char is the separator.  Fifth char is '\0'.
   */
  char *int_curr_symbol;
  char *currency_symbol;     /* Local currency symbol */
  char *mon_decimal_point;   /* Decimal point character */
  char *mon_thousands_sep;   /* Thousands separator */
  char *mon_grouping;        /* Like `grouping' element (above) */
  char *positive_sign;       /* Sign for positive values */
  char *negative_sign;       /* Sign for negative values */
  char int_frac_digits;      /* Int'l fractional digits */
  char frac_digits;          /* Local fractional digits */
  /* 1 if currency_symbol precedes a positive value, 0 if succeeds */
  char p_cs_precedes;
  /* 1 if a space separates currency_symbol from a positive value */
  char p_sep_by_space;
  /* 1 if currency_symbol precedes a negative value, 0 if succeeds */
  char n_cs_precedes;
  /* 1 if a space separates currency_symbol from a negative value */
  char n_sep_by_space;
  /* Positive and negative sign positions:
   *   0 Parentheses surround the quantity and currency_symbol.
   *   1 The sign string precedes the quantity and currency_symbol.
   *   2 The sign string follows the quantity and currency_symbol.
   *   3 The sign string immediately precedes the currency_symbol.
   *   4 The sign string immediately follows the currency_symbol.
   */
  char p_sign_posn;
  char n_sign_posn;

  /* ISO C99 addition */
  /* 1 if int_curr_symbol precedes a positive value, 0 if succeeds */
  char int_p_cs_precedes;
  /* 1 iff a space separates int_curr_symbol from a positive value */
  char int_p_sep_by_space;
  /* 1 if int_curr_symbol precedes a negative value, 0 if succeeds */
  char int_n_cs_precedes;
  /* 1 if a space separates int_curr_symbol from a negative value */
  char int_n_sep_by_space;
  /* Positive and negative sign positions:
   *   0 Parentheses surround the quantity and int_curr_symbol.
   *   1 The sign string precedes the quantity and int_curr_symbol.
   *   2 The sign string follows the quantity and int_curr_symbol.
   *   3 The sign string immediately precedes the int_curr_symbol.
   *   4 The sign string immediately follows the int_curr_symbol.
   */
  char int_p_sign_posn;
  char int_n_sign_posn;
};

/* Return the numeric/monetary imformation for the current locale. */
struct lconv* localeconv(void);

/* Set and/or return the current locale. */
char* setlocale(int category, const char *locale);

#endif  /* __LOCALE_H__ */
