/*
 * This library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * (c) Copyright 2006,2007,2008,2009 Jari OS Core Team <http://jarios.org>
 *
 * LibC:
 * include/math.h: For computing common mathematical functions
 */

#ifndef __MATH_H__
#define __MATH_H__

typedef float          float_t;
typedef double         double_t;

#define M_E            2.7182818284590452354   /* e */
#define M_LOG2E        1.4426950408889634074   /* log_2 e */
#define M_LOG10E       0.43429448190325182765  /* log_10 e */
#define M_LN2          0.69314718055994530942  /* log_e 2 */
#define M_LN10         2.30258509299404568402  /* log_e 10 */
#define M_PI           3.14159265358979323846  /* pi */
#define M_PI_2         1.57079632679489661923  /* pi/2 */
#define M_PI_4         0.78539816339744830962  /* pi/4 */
#define M_1_PI         0.31830988618379067154  /* 1/pi */
#define M_2_PI         0.63661977236758134308  /* 2/pi */
#define M_2_SQRTPI     1.12837916709551257390  /* 2/sqrt(pi) */
#define M_SQRT2        1.41421356237309504880  /* sqrt(2) */
#define M_SQRT1_2      0.70710678118654752440  /* 1/sqrt(2) */

/* Simple decimal realization log base of 2 */
static inline uint32_t ilog2(unsigned long n)
{
  int ind;
  register unsigned int res;
  const unsigned long B[] = {0x2, 0xC,
			    0xF0, 0xFF00,
			    0xFFFF0000, 0xFFFFFFFF00000000};
  const unsigned int BS[] = {1, 2, 4, 8, 16, 32};

  res = 0;
  for (ind = 5; ind >= 0; ind--) {
    if (n & B[ind]) {
      n >>= BS[ind];
      res |= BS[ind];
    }
  }
  return res;
}

static inline unsigned int dpow_of(int val)
{
  int s, i;

  if (val < 0)
    val = ~(val) + 1;
  i = 0, s = val;
  do {
    s >>= 1; i ++;
  } while (s);

  return i - 1;
}

// TODO: next functions are GCC-dependant...
static inline double fabs(double x)
{
  return __builtin_fabs(x);
}

static inline float fabsf(float x)
{
  return __builtin_fabsf(x);
}

static inline long double fabsl(long double x)
{
  return __builtin_fabsl(x);
}

static inline double pow(double x, double y)
{
  return __builtin_pow(x, y);
}

static inline long double powl(long double x, long double y)
{
  return __builtin_powl(x, y);
}

static inline float powf(float x, float y)
{
  return __builtin_powf(x, y);
}

#endif  /* __MATH_H__ */
