/*
 * This library is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU Lesser General Public License as 
 * published by the Free Software Foundation; either version 2.1 of 
 * the License.
 *
 * This library is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public 
 * License along with this library; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 *
 * (c) Copyright 2006,2007,2008 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2005,2008 MadTirra <madtirra@jarios.org> 
 *
 * name server related structures & operations
 * 
 */

#ifndef __NS_NS_H__
#define __NS_NS_H__

#include <sys/types.h>
#include <sys/utsname.h>
#include <ipc/port.h>

/* hardcoded NX pid */
#define NX_PID  0x1

/* some ioctls for name service */
/* ports ops */
#define NX_REGPORT   0x1
#define NX_RMPORT    0x2
#define NX_SRCHPORT  0x3
/* proc ops */
#define NX_CLONE     0x4
#define NX_CHGPATH   0x6
/* envfs ops */
#define NX_ENVFSUP   0x5
/* attributes */
#define NX_SETATTR   0x7

/* Max message size name service can send/revceive */
#define NX_BUFSIZE   8192

/* functions */
pid_t ns_gettermxpid(void);
pid_t ns_gethwxpid(void);
pid_t ns_getvfspid(void);
pid_t ns_getprocpid(void);
pid_t ns_getnetpid(void);
pid_t ns_getgixpid(void);
pid_t ns_getfslpid(void);
/* register task on name service */
int ns_register(pid_t pid,uid_t uid,gid_t gid,int type,int subsystem,
		char *pathname,char **environ,int env_amount);
/* ports operations */
int ns_register_port(prtid_t port,char *name);
int ns_unregister_port(char *name);
int ns_lookup_portid(pid_t pid,char *name);

/* ns v2 functions and defines oldies are deprecated ... */
/* type flags */
#define V2T_TRUSTED    (1 << 0)
#define V2T_MKSERVICE  (1 << 1)
#define V2T_SYSDAEMON  (1 << 2)
#define V2T_BLKDEVICE  (1 << 3)
#define V2T_FSSERVICE  (1 << 4)
#define V2T_CHRDEVICE  (1 << 5)
#define V2T_NETDEVICE  (1 << 6)

/* subsystems - NOTE: there things doesn't cloned and trusted task shouldn't change it - it will confuse the system */
#define SYS_NONE      0 /* regular service or task */
#define SYS_NS        1 /* Nameservice */
#define SYS_VFS       2 /* VFS service  */
#define SYS_PROC      3 /* Process service */
#define SYS_TERM      4 /* Terminal service */
#define SYS_NET       5 /* Networking service */
#define SYS_GIX       6 /* Graphical service */
#define SYS_SCHEMEVM  7 /* Scheme VM service */
#define SYS_DEVIO     8 /* Leaved for backward compatibility ! FIXME: remove when it will be possible */
#define SYS_FSL       9 /* File system linker service */

/* Setup environment variables for process with given pid.
 * If envp is NULL, then these variables are taken from caller
 * process.
 */
int ns_set_environ(pid_t pid, char *envp[]);

/*
 * Setup system task attributes, allowed only for 
 * trusted services.
 * Use it with care while you are trusted.
 */
int ns_setattrs(int type,uint8_t subsys);

#endif /* __NS_NS_H__ */
