/*
 * This library is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU Lesser General Public License as 
 * published by the Free Software Foundation; either version 2.1 of 
 * the License.
 *
 * This library is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public 
 * License along with this library; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 * 
 * (c) Copyright 2006,2007,2008 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2008 Dmitry Gromada <gromada@jarios.org>
 *
 * data structures for processing with queueing
 */

#ifndef __QUEUE_H__
#define __QUEUE_H__

#include <stdbool.h>
#include <sys/types.h>
#include <pthread.h>
#include <datalib/list.h>
#include <sys/uevent.h>
#include <bitwise.h>

#define evl_queue_add(q, node) evl_queue_add_range(q, node, node)
#define evl_queue_get_next(q) evl_queue_get_next_prep(q, NULL)
#define prio_queue_get_next(q) prio_queue_get_next_match(q, NULL)

/* event locked queue */
typedef struct __evl_queue {
  list_head_t head;
  pthread_mutex_t mutex;
  event_t event;
} evl_queue_t;

typedef struct __prio_queue {
  bitmap_t bm;
  list_head_t *heads;
  int nr_prios;
} prio_queue_t;

/* 
 * evl_queue_add_cond - conditional add to the evl queue
 *
 * @q - queue to insert in
 * @node - node to insert in the queue
 * @fn - function which compare to entries
 *
 * The node 'node' is inserted before the node
 * at comparing with which the 'predicate'
 * function return true. If the function is not provided
 * the node is inserted at head of the queue
 */
void evl_queue_add_cond(evl_queue_t *q, list_node_t *node, 
                        bool (*fn) (list_node_t*));

void evl_queue_add_range(evl_queue_t *q, list_node_t *first, list_node_t *last);

/* fetch next entry and prepare it before return */
list_node_t *evl_queue_get_next_prep(evl_queue_t *q, void (*func)(list_node_t *node));

int init_evl_queue(evl_queue_t *q);

void destroy_evl_queue(evl_queue_t *q);

int evl_queue_is_empty(evl_queue_t *q);

int init_prio_queue(prio_queue_t *q, int nr_prios);

void prio_queue_add(prio_queue_t *q, list_node_t *node, int prio);

bool prio_queue_is_empty(prio_queue_t *q);

list_node_t *prio_queue_get_next_match(prio_queue_t *q, bool (*match)(list_node_t *node));

void destroy_prio_queue(prio_queue_t *q);

#endif
