/*
 * This library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * (c) Copyright 2006,2007,2008,2009 Jari OS Core Team <http://jarios.org>
 *
 * LibC:
 * include/stddef.h: Define several useful types and macros.
 */

#ifndef __STDDEF_H__
#define __STDDEF_H__

#include <arch/wordsize.h>

#define NULL ((void *)0)

#if BITS_PER_LONG == 64
typedef unsigned long   size_t;
typedef long            ptrdiff_t;
#endif

/* wchar_t definition */
#ifndef __WCHAR_TYPE__
#define __WCHAR_TYPE__  short int
#endif

#define _WCHAR_T_       __WCHAR_TYPE__
typedef __WCHAR_TYPE__  wchar_t;

#define offsetof(TYPE, MEMBER) ((size_t) &((TYPE *)0)->MEMBER)

/* JariOS addition */

#define container_of(PTR, TYPE, MEMBER) \
    (TYPE *)((char *)(PTR) - offsetof(TYPE, MEMBER))

#define MIN(a,b) ((a) < (b) ? (a) : (b))
#define MAX(a,b) ((a) > (b) ? (a) : (b))

#define BOUNDED_VAL(val, min, max) ({     \
  typeof(val) __val;                      \
                                          \
  __val = (val) < (min) ? (min) : (val);  \
  __val = __val > (max) ? (max) : __val;  \
  __val;})

#define ARRAY_LENGTH(x) (sizeof(x) / sizeof(x[0]))

#endif  /* __STDDEF_H__ */
