/*
 * This library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * (c) Copyright 2006,2007,2008,2009 Jari OS Core Team <http://jarios.org>
 *
 * LibC:
 * include/stdint.h: Define various integer types.
 * Standard: ANSI C
 */

#ifndef __STDINT_H__
#define __STDINT_H__

#include <arch/wordsize.h>

/* Exact-width integer types */

typedef signed char          int8_t;
typedef short int            int16_t;
typedef int                  int32_t;
#if BITS_PER_LONG == 64
typedef long int             int64_t;
#endif

typedef unsigned char        uint8_t;
typedef unsigned short int   uint16_t;
typedef unsigned int         uint32_t;
#if BITS_PER_LONG == 64
typedef unsigned long int    uint64_t;
#endif

/* Minimum-width integer types */

typedef signed char          int_least8_t;
typedef short int            int_least16_t;
typedef int                  int_least32_t;
#if BITS_PER_LONG == 64
typedef long int             int_least64_t;
#endif

typedef unsigned char        uint_least8_t;
typedef unsigned short int   uint_least16_t;
typedef unsigned int         uint_least32_t;
#if BITS_PER_LONG == 64
typedef unsigned long int    uint_least64_t;
#endif

/* Fastest minimum-width integer types */

typedef signed char          int_fast8_t;
#if BITS_PER_LONG == 64
typedef long int             int_fast16_t;
typedef long int             int_fast32_t;
typedef long int             int_fast64_t;
#endif

typedef unsigned char        uint_fast8_t;
#if BITS_PER_LONG == 64
typedef unsigned long int    uint_fast16_t;
typedef unsigned long int    uint_fast32_t;
typedef unsigned long int    uint_fast64_t;
#endif

/* Types for 'void*' pointers */

#if BITS_PER_LONG == 64
typedef long int             intptr_t;
typedef unsigned long int    uintptr_t;
#endif

/* Greatest-width integer types */

#if BITS_PER_LONG == 64
typedef long int             intmax_t;
typedef unsigned long int    uintmax_t;
#endif

/* Limits of integral types */

#define INT8_MIN             (-128)
#define INT16_MIN            (-32768)
#define INT32_MIN            (-2147483648)
#if BITS_PER_LONG == 64
#define INT64_MIN            (-9223372036854775808L)
#endif

#define INT8_MAX             (127)
#define INT16_MAX            (32767)
#define INT32_MAX            (2147483647)
#if BITS_PER_LONG == 64
#define INT64_MAX            (9223372036854775807L)
#endif

#define UINT8_MAX            (255)
#define UINT16_MAX           (65535)
#define UINT32_MAX           (4294967295U)
#if BITS_PER_LONG == 64
#define UINT64_MAX           (18446744073709551615UL)
#endif

#define INT_LEAST8_MIN       (-128)
#define INT_LEAST16_MIN      (-32768)
#define INT_LEAST32_MIN      (-2147483648)
#if BITS_PER_LONG == 64
#define INT_LEAST64_MIN      (-9223372036854775808L)
#endif

#define INT_LEAST8_MAX       (127)
#define INT_LEAST16_MAX      (32767)
#define INT_LEAST32_MAX      (2147483647)
#if BITS_PER_LONG == 64
#define INT_LEAST64_MAX      (9223372036854775807L)
#endif

#define UINT_LEAST8_MAX      (255)
#define UINT_LEAST16_MAX     (65535)
#define UINT_LEAST32_MAX     (4294967295U)
#if BITS_PER_LONG == 64
#define UINT64_MAX           (18446744073709551615UL)
#endif

#define INT_FAST8_MIN        (-128)
#if BITS_PER_LONG == 64
#define INT_FAST16_MIN       (-9223372036854775808L)
#define INT_FAST32_MIN       (-9223372036854775808L)
#define INT_FAST64_MIN       (-9223372036854775808L)
#endif

#define INT_FAST8_MAX        (127)
#if BITS_PER_LONG == 64
#define INT_FAST16_MAX       (9223372036854775807L)
#define INT_FAST32_MAX       (9223372036854775807L)
#define INT_FAST64_MAX       (9223372036854775807L)
#endif

#define UINT_FAST8_MAX       (255)
#if BITS_PER_LONG == 64
#define UINT_FAST16_MAX      (18446744073709551615UL)
#define UINT_FAST32_MAX      (18446744073709551615UL)
#define UINT_FAST64_MAX      (18446744073709551615UL)
#endif

#if BITS_PER_LONG == 64
#define INTPTR_MIN           (-9223372036854775808L)
#define INTPTR_MAX           (9223372036854775807L)
#define UINTPTR_MAX          (18446744073709551615UL)
#endif

#if BITS_PER_LONG == 64
#define INTMAX_MIN           (-9223372036854775807L)
#define INTMAX_MAX           (9223372036854775807L)
#define UINTMAX_MAX          (18446744073709551615UL)
#endif

/* Limit of 'size_t type */

#if BITS_PER_LONG == 64
#define SIZE_MAX             (18446744073709551615UL)
#endif

/* Limits of 'wchar_t' type */

#define WCHAR_MIN            (-2147483648)
#define WCHAR_MAX            (2147483647)

/* Lmimts of 'wint_t' type */

#define WINT_MIN             (0U)
#define WINT_MAX             (4294967295U)

#if 0
/* Limits of 'sig_atomic_t' type */

#define SIG_ATOMIC_MIN       (-2147483648)
#define SIG_ATOMIC_MAX       (2147483647)
#endif

/* Limits of 'ptrdiff_t' type */

#if BITS_PER_LONG == 64
#define PTRDIFF_MIN          (-9223372036854775808L)
#define PTRDIFF_MAX          (9223372036854775807L)
#endif

#endif  /* __STDINT_H__ */
