/*
 * This library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * (c) Copyright 2006,2007,2008,2009 Jari OS Core Team <http://jarios.org>
 *
 * LibC:
 * include/stdlib.h: Variety of operations, including conversion,
 *                   pseudo-random numbers, memory allocation, process control,
 *                   environment, signalling, searching, and sorting.
 */

#ifndef __STDLIB_H__
#define __STDLIB_H__

#include <stddef.h>

typedef struct {
  int quot, rem;
} div_t;

#if BIT_PER_LONG == 64
typedef struct {
  long int quot, rem;
} ldiv_t;
#endif

#define EXIT_FAILURE   1       /* Failing exit status */
#define EXIT_SUCCESS   0       /* Successfull exit status */

#define MB_CUR_MAX  2 /* default, it's a 2 bytes per multibyte */

void free(void *ptr);
void *malloc(size_t size);
void *realloc(void *ptr, size_t size);
void *calloc(size_t nelem, size_t elsize);

void qsort(void *base, size_t nmemb, size_t size,
	   int(*comp)(const void*, const void*));


/* octave number to string */
char *otoa(int octav, char *buf, size_t buflen);
/* decimal number to string */
char *dtoa(int dec, char *buf, size_t buflen);
/* hex to string */
char *htoa(int hex, char *buf, size_t buflen);

unsigned long strtoul(const char *str, char **endptr, int base);
long strtol(const char *str,char **endptr,int base);
int atoi(const char *str);

unsigned long long strtoull(const char *str, char **endptr, int base);

long atol(const char *str);
long long atoll(const char *str);

char *getenv(const char *name);
int setenv(const char *envname, const char *envval, int overwrite);
int unsetenv(const char *name);
int putenv(char *string);

void abort(void);
void exit(int status);
int atexit(void (*func)(void));
int system(const char *command);

double strtod(const char *nptr, char **endptr);
float strtof(const char *nptr, char **endptr);
long double strtold(const char *nptr, char **endptr);

int abs(int j);
long int labs(long int j);
long long int llabs(long long int j);


#endif  /* __STDLIB_H__ */
