/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * (c) Copyright 2006,2007,2008,2009 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2009 Dmitry Gromada <gromada@jarios.org>
 *
 * Block device control decalrations
 */

#ifndef __SYS_BLKDEV_H__
#define __SYS_BLKDEV_H__

#include <stdbool.h>
#include <sys/types.h>
#include <sys/ioctl.h>

#define GENSTOR_CTLS 1

#define TRACE_ID_ANY ((uint64_t)-1)

/* block device ioctls */
#define BLKIOC_RESCAN_PARTITIONS  _IO(GENSTOR_CTLS, CTL_RESCAN_PARTITIONS)

/* trace block layer ioctls */
#define BLKIOC_CLEAR_TRACE        _IO(GENSTOR_CTLS, CTL_CLEAR_TRACE)

/* commands */
#define BLKCMD_RESCAN_PARTITIONS  _IOCMD(GENSTOR_CTLS, CTL_RESCAN_PARTITIONS)
#define BLKCMD_CLEAR_TRACE        _IOCMD(GENSTOR_CTLS, CTL_CLEAR_TRACE)
#define BLKCMD_GET_TRACE_EVENTS   _IOCMD(GENSTOR_CTLS, CTL_GET_TRACE_EVENTS)

/* block device ioctl numbers */
enum {
  CTL_RESCAN_PARTITIONS,
  CTL_CLEAR_TRACE,
  CTL_GET_TRACE_EVENTS
};

typedef enum __merge_type {
  BLK_MERGE_NO,
  BLK_MERGE_FRONT,
  BLK_MERGE_TAIL
} merge_type_t;

/* block layer trace related types and functions */

typedef enum __trace_layer {
  TRACE_LAYER_GENERIC,
  TRACE_LAYER_IOSCHED,
  TRACE_LAYER_DRV
} trace_layer_t;

typedef enum __trace_event_type {
  TRACE_EVENT_ADD,
  TRACE_EVENT_MERGE,
  TRACE_EVENT_DISPATCH,
  TRACE_EVENT_STARVE,
  TRACE_EVENT_START_PROC,
  TRACE_EVENT_END_PROC,
  TRACE_EVENT_SCHEDULE
} trace_event_type_t;

/* search key types */
typedef enum __skey_type {
  SKEY_TYPE_SECTOR,
  SKEY_TYPE_ID
} skey_type_t;

typedef enum __merge_obj_type {
  MERGE_OBJ_TYPE_RQ,
  MERGE_OBJ_TYPE_BLKIO
} merge_obj_type_t;

struct trace_key {
  uint64_t value;
  skey_type_t type;
};

typedef struct __trace_entry_data {
  uint64_t orig_start_sect;
  uint64_t curr_start_sect;
  uint32_t orig_nr_sects;
  uint32_t curr_nr_sects;
  uint32_t id;
  uint32_t nr_events;
} trace_entry_data_t;

typedef struct __merge_data {
  bool with_blkio;
  merge_type_t type;
  /*
   * id of request with which the request
   * is merged, or undefined if merged with a
   * blkio
   */
  uint32_t rq_id;
  short m_prio;
  uint64_t old_start_sect;
  uint64_t new_start_sect;
  uint32_t old_nr_sects;
  uint32_t new_nr_sects;
} merge_data_t;

typedef struct __trace_event_data_t {
  trace_event_type_t type;
  trace_layer_t layer;
  ulong_t flags;
  short prio;
  uint16_t ext_data_size;
  uint64_t jiffies;
  uint8_t ext_data[0];
} trace_event_data_t;

int get_blkdev_trace_events(int fd, uint64_t key, skey_type_t key_type,
                            void *buf, size_t buf_size);

#endif
