/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * (c) Copyright 2006,2007,2008,2009 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2009 Paul Graphov <graphov@jarios.org>
 *
 * console_ioctl.h: ioctl constants for vga and vesa console.
 *
 */

#ifndef __CONSOLE_H__
#define __CONSOLE_H__

#include <sys/types.h>

// console colors
typedef enum __term_colors {
  TC_BLACK = 0,
  TC_BLUE,
  TC_GREEN,
  TC_CYAN,
  TC_RED,
  TC_MAGENTA,
  TC_BROWN,
  TC_WHITE,
  TC_LIGHT_BLUE,
  TC_LIGHT_GREEN,
  TC_LIGHT_RED,
  TC_LIGHT_BROWN,
  TC_LIGHT_GREY,
} term_color_t;


/* font attributes */
#define FONT_BOLD        0x1
#define FONT_UNDERSCOPE  0x2
#define FONT_ITALIC      0x4
#define FONT_BLINK       0x8

#define SCROLL_UP    0x1
#define SCROLL_DOWN  0x2

typedef struct __cschar_t {
  wchar_t ch; /* charaster */
  uint8_t cattr; /* color attributes -> 4bit background, 4bit foreground */
  uint8_t xattr; /* extra attributes */
} __attribute__ ((packed)) cschar_t;

typedef enum {
  PHYT_WRITE = 0, /* write */
  PHYT_CLEAR, /* clear physical terminal */
  PHYT_SETCURPOSITION, /* set cursor position */
  PHYT_SHOWCURSOR, /* show/hide cursor */
  PHYT_HIDECURSOR,
  PHYT_WRITEPOS, /* write at given position */
  PHYT_DISABLEAUTOSCROLL, /* disable automatical scroll */
  PHYT_ENABLEAUTOSCROLL, /* enable scroll */
  PHYT_SCROLL, /* scroll */
  PHYT_SETCOLORS, /* set colors */
  PHYT_WRITEP, /* write with parameters if arg0 > 0 do all parameters set */
  PHYT_SETVIEWPORT, /* set viewport area arg0,arg1 -> offset from text term, arg3, arg4 - width, height */
  PHYT_GETSIZE, /* get console size arg0->width, arg1->height */
} phytermio_command_id_t;

typedef struct __rect_t {
  uint16_t x1;
  uint16_t y1;
  uint16_t x2;
  uint16_t y2;
} rect_t;

typedef struct __scroll_desc_t {
  // define scroll region:
  uint16_t startline;
  uint16_t stopline;
  // scroll amount, negative means scroll down
  int16_t lines;
} scroll_desc_t;

typedef struct __coord_t {
  uint16_t x; // maybe width in some cases
  uint16_t y; // maybe height
} coord_t;

typedef struct __clear_desc_t {
  rect_t rect;
  uint8_t color;
} clear_desc_t;


#define CON_IOCTL 0x01

#define CIOC_CLEAR _IOW(CON_IOCTL, PHYT_CLEAR, rect_t)
#define CIOC_SCROLL _IOW(CON_IOCTL, PHYT_SCROLL, scroll_desc_t)
#define CIOC_SETCURPOSITION _IOW(CON_IOCTL, PHYT_SETCURPOSITION, coord_t)
#define CIOC_SHOWCURSOR _IO(CON_IOCTL, PHYT_SHOWCURSOR)
#define CIOC_HIDECURSOR _IO(CON_IOCTL, PHYT_HIDECURSOR)
#define CIOC_ENABLEAUTOSCROLL _IO(CON_IOCTL, PHYT_ENABLEAUTOSCROLL)
#define CIOC_DISABLEAUTOSCROLL _IO(CON_IOCTL, PHYT_DISABLEAUTOSCROLL)
#define CIOC_GETSIZE _IOR(CON_IOCTL, PHYT_GETSIZE, coord_t)

#endif
