/*
 * This library is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU Lesser General Public License as 
 * published by the Free Software Foundation; either version 2.1 of 
 * the License.
 *
 * This library is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public 
 * License along with this library; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 *
 * (c) Copyright 2006,2007,2008,209 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2009 Michael Tsymbalyuk <mtzaurus@gmail.com>
 *
 * LibOS:
 * include/sys/dma.h: Prototypes for DMA-related functions.
 *
 */

#ifndef __SYS_DMA_H__
#define  __SYS_DMA_H__

#include <sys/types.h>

typedef unsigned long dma_addr_t;

void *dma_alloc_pages(int numpages);
void dma_free_pages(void *addr,int num_pages);

/**
 * @def dma_alloc_consistent(size,dma_handle)
 * @brief Allocate physically contiguous DMA-capable memory.
 *
 * This function allocates the requested amount of bytes DMA-capable memory
 * physically contiguous memory (rounded to the closest page-size
 * amount), maps it to the caller's virtual space (thus making it
 * available for accessing directly from userspace), and stores
 * the first address of this physical memory range to target location.
 *
 * @param size   - Amount of bytes requested for allocation.
 * @param dma_handle  - Location where to store the first physical
 *                      address of an allocated DMA region.
 * @return Virtual address where target DMA region was mapped in case of
 *         successfull allocation, or MAP_FAILED otherwise.
 */
void *dma_alloc_consistent(size_t size,dma_addr_t *dma_handle);

#endif
