/*
 * This library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * (c) Copyright 2006-2009 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2009 Stanislav M. Ivankin <grepz@jarios.org>
 *
 * Fslinker userspace library functions.
 *
 */

#ifndef FSLINKER_H
#define FSLINKER_H

#include <stdint.h>

/* RPC header:
 * All arguments are in byte size
 *
 * arg0 - Fstype len
 *
 * arg1 - Link point path len
 * arg2 - Link point path points num
 *
 * arg3 - Link subject path len
 * arg4 - Link subject path points num
 *
 * arg5 - Opts string len
 * arg6 - Opts num
 *
 * Message:
 * <rpc><type><link point path><link point path points>    ...
 *       ... <link subject path><link subject path points> ...
 *       ... <option string><options len array>            .
 */

#define FS_OPTS_MAX     32
#define FSTYPE_LEN_MAX  64
#define PATH_POINTS_MAX 64

typedef enum {
  FSLINK_REQ_MOUNT,
  FSLINK_REQ_UMOUNT,
  FSLINK_REQ_REMOUNT,
  FSLINK_REQ_SYNC,
  FSLINK_REQ_MAX, /* Not really used */
} fs_link_req_t;

typedef struct __fs_link_pp fs_link_pp_t;
struct __fs_link_pp
{
  int plen;
  int p_num;
  int *pp;
  char *path;
};

typedef struct __fs_link_entry fs_link_entry_t;
struct __fs_link_entry {
  int fstype_len;
  char *fstype;
  fs_link_pp_t mountp;
  fs_link_pp_t mounts;
  int optstr_len;
  int opts_n;
  int *opt_len;
  char *options;
};

/* Takes raw (command line like) arguments, allocate and fills
 * fs_link_entry_t structure.
 * Be sure to set not used parameters to NULL. Also don't forget, that
 * strings must be \0 terminated
 */
fs_link_entry_t *fslink_fill_struct(const char *type,
				    const char *lsubj, const char *lpoint,
				    const char *optstr);

/* Assembles and sends request to fslinker service */
int fslink_send_request(const fs_link_entry_t *fs_entry, fs_link_req_t req)
  __attribute__((nonnull (1)));

void fslink_free_entry(fs_link_entry_t *entry);

/* Path points routines */
int fslink_pp_fill(const char *path, fs_link_pp_t *link_pp);
int fslink_pp_alloc(const char *path, fs_link_pp_t *entry_pp);
void fslink_free_pp(fs_link_pp_t *link_pp);

/* Takes raw option string as argument(opt1,opt2,opt3,...).
 * Fills fs_link_entry_t option string(opt1opt2opt3...) and options
 * length array as a result
 */
int fslink_alloc_opts(const char *optstr, fs_link_entry_t *entry);
void fslink_count_opts(const char *optstr, int *optnum, int *optlen);

#endif
