/*
 * This library is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU Lesser General Public License as 
 * published by the Free Software Foundation; either version 2.1 of 
 * the License.
 *
 * This library is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public 
 * License along with this library; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 *
 * (c) Copyright 2006,2007,2008 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2008 Michael Tsymbalyuk <mtzaurus@gmail.com>
 * (c) Copyright 2008 Dmitry Gromada <gromada@jarios.org>
 *
 * LibOS:
 * include/ievent.h: Prototypes for 'interrupt event', a machanism
 *                   for delivering hardware interrupts to userspace.
 *
 */

#ifndef __IEVENT_H__
#define  __IEVENT_H__

#include <stdbool.h>
#include <sys/types.h>
#include <datalib/usrtc.h>

typedef ulong_t irq_event_mask_t;
typedef ulong_t irq_counter_t;
typedef ulong_t irq_t;

typedef struct __irq_array {
  irq_event_mask_t ev_mask;
  irq_counter_t counters[];
} irq_array_t;

typedef bool (*irq_handler_t)(void *priv);

#define IEV_UP  0xCAFEBABE
#define IEV_DOWN  0xDEADBEEF

typedef struct __interrupt_event {
  ulong_t magic;
  irq_array_t *irq_array;
  uint16_t num_irqs;
  irq_t *irqs;
  usrtc_t **handlers;
  ulong_t array_id;
  ulong_t lock;
} interrupt_event_t;

typedef struct __irq_client {
  irq_handler_t handler;
  void *data;
} irq_client_t;

status_t interrupt_event_create(interrupt_event_t **e,irq_t *irqs,
                                uint16_t num_irqs);
status_t interrupt_event_destroy(interrupt_event_t *e);
status_t interrupt_event_wait(interrupt_event_t *e);
status_t interrupt_event_add_handler(interrupt_event_t *e,irq_t irq,
                                     irq_handler_t handler,void *priv);
void *interrupt_event_set_handler(interrupt_event_t *e,irq_t irq,
                                  irq_handler_t old_handler,
                                  irq_handler_t handler,void *priv);

#endif
