/*
 * This library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * (c) Copyright 2006,2007,2008,2009 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2009 Paul Graphov <graphov@jarios.org>
 *
 * LibC:
 * include/sys/ioctl.h: ioctl operations
 */

#ifndef __SYS_IOCTL_H__
#define __SYS_IOCTL_H__

/* linux-like ioctl macros */

/*
 * IOCTL number is represented as follows (Jari specific):
 * <1 bit - direction> <15 bits - size> <8 bits - type> <8 bits >
 */

#define _IOC_NRBITS	8
#define _IOC_TYPEBITS	8

/*
 * Let any architecture override either of the following before
 * including this file.
 */

#ifndef _IOC_SIZEBITS
# define _IOC_SIZEBITS	14
#endif

/*
 * _IOC_SIZEMAX means that size should be specified via third parameter
 * to ioctl library call.
 */

#define _IOC_SIZEMAX ((1U << _IOC_SIZEBITS) - 1)

#ifndef _IOC_DIRBITS
# define _IOC_DIRBITS	2
#endif

#define _IOC_NRMASK	((1 << _IOC_NRBITS)-1)
#define _IOC_TYPEMASK	((1 << _IOC_TYPEBITS)-1)
#define _IOC_SIZEMASK	((1 << _IOC_SIZEBITS)-1)
#define _IOC_DIRMASK	((1 << _IOC_DIRBITS)-1)

#define _IOC_NRSHIFT	0
#define _IOC_TYPESHIFT	(_IOC_NRSHIFT+_IOC_NRBITS)
#define _IOC_SIZESHIFT	(_IOC_TYPESHIFT+_IOC_TYPEBITS)
#define _IOC_DIRSHIFT	(_IOC_SIZESHIFT+_IOC_SIZEBITS)

#define _IOC_READ	0U
#define _IOC_WRITE 1U
#define _IOC_WRFIXED 2U
#define _IOC_WRARBIT 3U

#define _IOC(dir,type,nr,size)  \
	(((dir)  << _IOC_DIRSHIFT) |  \
	 ((type) << _IOC_TYPESHIFT) | \
	 ((nr)   << _IOC_NRSHIFT) |   \
	 ((size) << _IOC_SIZESHIFT))

#define _IOCMD(type,nr) (((type) << _IOC_TYPESHIFT) | \
                         ((nr) << _IOC_NRSHIFT))
#define _IOCMD_TYPE(cmd) (cmd >> _IOC_TYPESHIFT)

#define _IOC_TYPECHECK(t) (sizeof(t))

/* used to create numbers */
#define _IO(type,nr)		_IOC(_IOC_READ,(type),(nr),0)
#define _IOR(type,nr,size)	_IOC(_IOC_READ,(type),(nr),(_IOC_TYPECHECK(size)))
#define _IOW(type,nr,size)	_IOC(_IOC_WRITE,(type),(nr),(_IOC_TYPECHECK(size)))
#define _IOWRFIX(type,nr,size)	_IOC(_IOC_WRFIXED,(type),(nr),(_IOC_TYPECHECK(size)))
#define _IOWRARB(type,nr,size)	_IOC(_IOC_WRARBIT,(type),(nr),(_IOC_TYPECHECK(size)))

/* used to decode ioctl numbers.. */
#define _IOC_DIR(nr)		(((nr) >> _IOC_DIRSHIFT) & _IOC_DIRMASK)
#define _IOC_TYPE(nr)		(((nr) >> _IOC_TYPESHIFT) & _IOC_TYPEMASK)
#define _IOC_NR(nr)		(((nr) >> _IOC_NRSHIFT) & _IOC_NRMASK)
#define _IOC_SIZE(nr)		(((nr) >> _IOC_SIZESHIFT) & _IOC_SIZEMASK)

#define _IOC_TYPENR(nr)  ((nr) & ((1U << _IOC_SIZESHIFT) - 1) )

#define _RPC_IOCDIR(rpc) ((rpc)->arg5)
#define _RPC_IOCTYPE(rpc) ((rpc)->arg3)
#define _RPC_IOCNR(rpc) ((rpc)->arg4)
#define _RPC_IOCMD(rpc) (((rpc)->arg3 << _IOC_TYPESHIFT) | \
                         ((rpc)->arg4 << _IOC_NRSHIFT))


// FIXME: should be changed to "int ioctl(int d, int request, ...);" after
// devfs implementation.
int ioctl(int fildes, int request, ...);

#endif
