/*
 * This library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * (c) Copyright 2006,2007,2008,2009 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2009 Michael Tsymbalyuk <mtzaurus@gmail.com>
 *
 * LibC:
 * include/sys/kcontrol.h: Kernel control interface.
 */

#ifndef __KCONTROL_H__
#define __KCONTROL_H__

#include <sys/types.h>

typedef struct __kcontrol_args {
  int *name;
  unsigned name_len;
  void *old_data;
  unsigned int *old_data_size;
  void *new_data;
  unsigned int new_data_size;
} kcontrol_args_t;

#define KCTRL_MAX_NAME_LEN  8

/* Top level node related to all kernel parameters. */
#define KCTRL_KERNEL             0
   /* Kernel parameters related to boot process. */
   #define KCTRL_BOOT_INFO          0
     /* Start physical page frame of the initial RAM disk loaded by the boot loader. */
     #define KCTRL_INITRD_START_PAGE  0
     /* Size of initial RAM disk in pages. */
     #define KCTRL_INITRD_SIZE        1
  /* Generic system data. */
  #define KCTRL_SYSTEM_INFO      1
    /* Address of the SWKS area. */
    #define KCTRL_SWKS_ADDR           0

/* Top level node related to kernel debug parameters. */
#define KCTRL_DEBUG             100000
  /* Echoeing of target string to debug console. */
  #define KCTRL_DEBUG_ECHO      0

int kcontrol(kcontrol_args_t *ka);

#define KLOG_MAX_SIZE  512

/* This function prints target message to kernel debug console.
 * Returns 0 on success, or -ENOSYS if kernel doesn't allow accessing
 * debug console form userspace.
 */
int klog(const char *fmt, ...);

#endif
