/*
 * This library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License.
   *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * (c) Copyright 2006,2007,2008 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2008 MadTirra <madtirra@jarios.org>
 *
 * LibOS:
 * include/sys/mman.h: mmap and related functions and defines
 *
 */

#ifndef __SYS_MMAN_H__
#define __SYS_MMAN_H__

#ifndef __ASM__
#include <sys/types.h>
#include <ipc/port.h>
#endif /* __ASM__ */

/* task related defines*/
#define USER_START_VIRT  0x1001000
#define USER_STACK_SIZE  4
#define PAGE_WIDTH       12
#define PAGE_SIZE        0x1000
#define PAGE_MASK        (PAGE_SIZE - 1)
#define PAGE_ALIGN(addr) (((addr) + PAGE_MASK) & ~PAGE_MASK)

#ifndef __ASM__

enum mman_prot {
  PROT_NONE    = 0x01,
  PROT_READ    = 0x02,
  PROT_WRITE   = 0x04,
  PROT_EXEC    = 0x08,
  PROT_NOCACHE = 0x10,
};

enum mman_flags {
  MAP_FIXED    = 0x001,
  MAP_ANON     = 0x002,
  MAP_PRIVATE  = 0x004,
  MAP_SHARED   = 0x008,
  MAP_PHYS     = 0x010,
  MAP_STACK    = 0x020,
  MAP_POPULATE = 0x040,
  MAP_CANRECPAGES  = 0x080,
};

enum {
  MEMOBJ_DPC        = 0x01, /* unused */
  MEMOBJ_BACKENDED  = 0x02, /* memory object with backend */
  MEMOBJ_SHARED     = 0x04, /* shared memory */
  MEMOBJ_SLAVE      = 0x08, /* unused */
};

#define MAP_ANONYMOUS MAP_ANON

typedef unsigned long memobj_id_t;

typedef enum __mm_event {
  MMEV_PAGE_FAULT   = 0x01, /* <- this */
  MMEV_ALLOC_PAGE   = 0x02,
  MMEV_FREE_PAGE    = 0x04,
  MMEV_MSYNC        = 0x08, /* <- this */
  MMEV_TRUNCATE     = 0x10,
} mm_event_t;

struct mmev_hdr {
  mm_event_t event;
  memobj_id_t memobj_id;
  long private;
};

typedef long pgoff_t;
struct mmev_fault {
  struct mmev_hdr hdr;
  uint32_t pfmask;
  pgoff_t offset;
};

struct mmev_msync {
  struct mmev_hdr hdr;
  pgoff_t offset;
};

/* predefined system values */
#define NOFD  0
#define MAP_FAILED ((void *)-1)

/* Memory object Commands */
enum {
  MEMOBJ_CTL_PUTPAGE = 1,
  MEMOBJ_CTL_TRUNC,
  MEMOBJ_CTL_GET_PAGE,
  MEMOBJ_CTL_GET_INFO,
  MEMOBJ_CTL_CHANGE_INFO,
  MEMOBJ_CTL_SET_BACKEND,
  MEMOBJ_CTL_GET_BACKEND,
};

/* Arguments for mmap()-related syscall. */
struct mmap_args {
  uintptr_t addr;
  size_t size;
  int prot;
  int flags;
  off_t offset;
};


typedef enum memobj_nature {
  MMO_NTR_GENERIC = 1,
  MMO_NTR_SRV,
  MMO_NTR_PCACHE,
  MMO_NTR_SLAVE,
  MMO_NTR_PROXY,
} memobj_nature_t;

enum memobj_life {
  MMO_LIFE_SPIRIT = 1,
  MMO_LIFE_STICKY,
  MMO_LIFE_LEECH,
  MMO_LIFE_IMMORTAL,
};

struct memobj_info {
  long id;
  size_t size;
  uid_t owner_uid;
  gid_t owner_gid;
  mode_t acc_mode;
  int users;
  enum memobj_nature nature;
  enum memobj_life lifetype;  
  uint32_t flags;
  long backend_port;
  long private; 
};

/* function prototypes */
void *mmap(void *addr, size_t length, int prot, int flags, int fd, off_t offset);
int munmap(void *base,size_t size);

/* Jari-OS specific low-level map-related functions. */
void *mmap_pid(pid_t victim,void *addr,size_t length,int prot,int flags,int fd,off_t offset);
int grant_pages(void *vaddr_from, size_t length, pid_t victim_pid, uintptr_t victim_addr);
/* low level raw syscall wrapper */
void *mk_mmap(void *addr, size_t length, int prot, int flags, int memobj_id, off_t offset);
/* memory object specific funcs */
int memobj_create_wbackend(prtid_t serve,size_t size,int lifetype,int flags,long private);


#endif /* __ASM__ */

#endif /* __SYS_MMAN_H__ */
