/*
 * This library is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU Lesser General Public License as 
 * published by the Free Software Foundation; either version 2.1 of 
 * the License.
 *
 * This library is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public 
 * License along with this library; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 *
 * (c) Copyright 2006,2007,2008 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2008 MadTirra <madtirra@jarios.org>
 *
 * LibOS:
 * include/sys/proc.h: process related low level functions 
 *
 */

#ifndef __SYS_PROC_H__
#define __SYS_PROC_H__

#include <sys/types.h>

status_t create_task(uint32_t flags);

#define SYS_PR_CTL_SET_ENTRYPOINT      0x0
#define SYS_PR_CTL_SET_STACK           0x1
#define SYS_PR_CTL_GET_ENTRYPOINT      0x2
#define SYS_PR_CTL_GET_STACK           0x3
#define SYS_PR_CTL_ADD_EVENT_LISTENER  0x4
#define SYS_PR_CTL_SET_PERTASK_DTAA    0x5
#define SYS_PR_CTL_DISINTEGRATE_TASK   0x6
#define SYS_PR_CTL_REINCARNATE_TASK    0x7
#define SYS_PR_CTL_CANCEL_TASK         0x8
#define SYS_PR_CTL_SET_CANCEL_STATE    0x9
#define SYS_PR_CTL_SET_CANCEL_TYPE     0xA
#define SYS_PR_CTL_DEL_EVENT_LISTENER  0xB
#define SYS_PR_CTL_GETPID              0xC
#define SYS_PR_CTL_GETPPID             0xD
#define SYS_PR_CTL_GETTID              0xE
#define SYS_PR_CTL_SET_SHORTNAME       0xF
#define SYS_PR_CTL_SET_UIDGID          0x10
#define SYS_PR_CTL_GET_UIDGID          0x11

#define TASK_SHORTNAME_LEN  32

/* Data structure sent after performing SYS_PR_CTL_DISINTEGRATE_TASK command */
typedef struct __disintegration_req_packet {
  pid_t pid;
  ulong_t status;
} disintegration_req_packet_t;

typedef struct __uidgid {
  uid_t uid;
  gid_t gid;
} uidgid_t;

typedef struct __task_details {
  pid_t pid,ppid;
  int state;
} task_details_t;

#define __DR_EXITED  1 /* Target task exited before disintegrating itself. */

/* Mask of task events. */
#define TASK_EVENT_TERMINATION  0x1  /* Task terminated */

typedef struct __task_event_ctl_arg {
  ulong_t ev_mask;
  ulong_t port;
} task_event_ctl_arg_t;

typedef struct __task_event_descr {
  pid_t pid;
  tid_t tid;
  ulong_t ev_mask;
} task_event_descr_t;

status_t task_ctl(pid_t pid, tid_t tid, uint64_t cmd,uint64_t args);
long get_task_details(pid_t pid,task_details_t *info);
int get_task_uidgid(pid_t pid,uidgid_t *uidgid);
int set_task_uidgid(pid_t pid,uidgid_t *uidgid);

#endif /* __SYS_PROC_H__ */

