/*
 * This library is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU Lesser General Public License as 
 * published by the Free Software Foundation; either version 2.1 of 
 * the License.
 *
 * This library is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public 
 * License along with this library; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 *
 * (c) Copyright 2008 Jari OS core team <http://jarios.org> 
 *
 */

#ifndef __SCHEDULER_H__
#define __SCHEDULER_H_

#include <sys/types.h>

#define SYS_SCHED_CTL_SET_POLICY         0x0
#define SYS_SCHED_CTL_GET_POLICY         0x1
#define SYS_SCHED_CTL_SET_PRIORITY       0x2
#define SYS_SCHED_CTL_GET_PRIORITY       0x3
#define SYS_SCHED_CTL_MONOPOLIZE_CPU     0x4
#define SYS_SCHED_CTL_DEMONOPOLIZE_CPU   0x5
#define SYS_SCHED_CTL_SET_AFFINITY_MASK  0x6
#define SYS_SCHED_CTL_GET_AFFINITY_MASK  0x7
#define SYS_SCHED_CTL_SET_MAX_TIMESLICE  0x8
#define SYS_SCHED_CTL_GET_MAX_TIMESLICE  0x9
#define SYS_SCHED_CTL_GET_STATE          0xA
#define SYS_SCHED_CTL_SET_STATE          0xB
#define SYS_SCHED_CTL_GET_CPU            0xC
#define SYS_SCHED_CTL_SET_CPU            0xD

/* Main scheduling policies. */
typedef enum __sched_discipline {
  SCHED_RR = 0,  /* Round-robin discipline. */
  SCHED_FIFO = 1, /* FIFO discipline. */
  SCHED_OTHER = 2, /* Default 'O(1)-like' discipline. */
} sched_discipline_t;


typedef enum __task_state {
  TASK_STATE_RUNNING = 0x1,
  TASK_STATE_RUNNABLE = 0x2,
  TASK_STATE_JUST_BORN = 0x4,
  TASK_STATE_SLEEPING = 0x8,
  TASK_STATE_STOPPED = 0x10,
  TASK_STATE_SUSPENDED = 0x20,
  TASK_STATE_ZOMBIE = 0x8000,
} task_state_t;

status_t task_scheduler_control(pid_t pid,tid_t,ulong_t cmd,ulong_t arg);

#endif
