/*
 * This library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * (c) Copyright 2006,2007,2008,2009 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2008 MadTirra <madtirra@jarios.org>
 * (c) Copyright 2009 Dmitry Balakshin <dmitry.balakshin@jarios.org>
 *
 * LibC:
 * include/sys/stat.h: FIle information (stat et al)
 */

#ifndef __SYS_STAT_H__
#define __SYS_STAT_H__

#include <sys/types.h>

/* File types */
#define S_IFMT          0170000        /* These bits determine file type */

#define S_IFBLK         0060000        /* Block device */
#define S_IFCHR         0020000        /* Character device */
#define S_IFIFO         0010000        /* FIFO */
#define S_IFREG         0100000        /* Regular file */
#define S_IFDIR         0040000        /* Directory */
#define S_IFLNK         0120000        /* Symbolic link */
#define S_IFSOCK        0140000        /* Socket */

/* Test file type */
#define S_ISBLK(m)      (((m) & S_IFMT) == S_IFBLK)
#define S_ISCHR(m)      (((m) & S_IFMT) == S_IFCHR)
#define S_ISDIR(m)      (((m) & S_IFMT) == S_IFDIR)
#define S_ISFIFO(m)     (((m) & S_IFMT) == S_IFIFO)
#define S_ISREG(m)      (((m) & S_IFMT) == S_IFREG)
#define S_ISLNK(m)      (((m) & S_IFMT) == S_IFLNK)
#define S_ISSOCK(m)     (((m) & S_IFMT) == S_IFSOCK)

/* File mode bits */
#define S_IRWXU         (S_IRUSR | S_IWUSR | S_IXUSR)
#define S_IRUSR         00400
#define S_IWUSR         00200
#define S_IXUSR         00100

#define S_IRWXG         (S_IRGRP | S_IWGRP | S_IXGRP)
#define S_IRGRP         (S_IRUSR >> 3)
#define S_IWGRP         (S_IWUSR >> 3)
#define S_IXGRP         (S_IXUSR >> 3)

#define S_IRWXO         (S_IROTH | S_IWOTH | S_IXOTH)
#define S_IROTH         (S_IRGRP >> 3)
#define S_IWOTH         (S_IWGRP >> 3)
#define S_IXOTH         (S_IXGRP >> 3)

/* S_IXUGO is a common extension to POSIX */
#define S_IXUGO         (S_IXUSR | S_IXGRP | S_IXOTH)

#define S_IRWXUGO       (S_IRWXU | S_IRWXG | S_IRWXO)

#define S_ISUID         04000
#define S_ISGID         02000
#define S_ISVTX         01000

struct stat {
  dev_t st_dev;                        /* ID of device containing file */
  ino_t st_ino;                        /* inode number */
  mode_t st_mode;                      /* protection */
  nlink_t st_nlink;                    /* number of hard links */
  uid_t st_uid;                        /* user ID of owner */
  gid_t st_gid;                        /* group ID of owner */
  dev_t st_rdev;                       /* device ID (if special file) */
  off_t st_size;                       /* total size, in bytes */
  blksize_t st_blksize;                /* blocksize for file system I/O */
  blkcnt_t st_blocks;                  /* number of blocks allocated */
  time_t st_atime;                     /* time of last access */
  time_t st_mtime;                     /* time of last modification */
  time_t st_ctime;                     /* time of last status change */
};

int chmod(const char *path, mode_t mode);
int lchmod(const char *path, mode_t mode);
int fchmod(int fildes, mode_t mode);

int mkdir(const char *path, mode_t mode);

int stat(const char *path, struct stat *buf);
int lstat(const char *path, struct stat *buf);
int fstat(int fildes, struct stat *buf);

mode_t umask(mode_t cmask);

#endif  /* __SYS_STAT_H__ */
