/*
 * This library is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU Lesser General Public License as 
 * published by the Free Software Foundation; either version 2.1 of 
 * the License.
 *
 * This library is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public 
 * License along with this library; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 *
 * (c) Copyright 2006,2007,2008 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2008 MadTirra <madtirra@jarios.org>
 *
 * LibOS:
 * include/sys/syscall.h: syscall function
 *
 */

#ifndef __SYS_SYSCALL_H__
#define __SYS_SYSCALL_H__

#include <sys/types.h>

/* muistring system calls */
#define SC_CREATE_TASK         0
#define SC_TASK_CONTROL        1
#define SC_MMAP                2
#define SC_CREATE_PORT         3
#define SC_PORT_SEND           4
#define SC_PORT_RECEIVE        5
#define SC_PORT_REPLY          6
#define SC_ALLOCATE_IOPORTS    7
#define SC_FREE_IOPORTS        8
#define SC_CREATE_IRQ_ARRAY    9
#define SC_WAIT_ON_IRQ_ARRAY   10
#define SC_IPC_PORT_POLL       11
#define SC_NANOSLEEP           12
#define SC_SCHED_CONTROL       13
#define SC_EXIT                14
#define SC_OPEN_CHANNEL        15
#define SC_CLOSE_CHANNEL       16
#define SC_CLOSE_PORT          17
#define SC_CONTROL_CHANNEL     18
#define SC_PORT_SEND_IOV       19
#define SC_SYNC_CREATE         20
#define SC_SYNC_CONTROL        21
#define SC_SYNC_DESTROY        22
#define SC_KILL                23
#define SC_SIGNAL              24
#define SC_SIGRETURN           25
#define SC_PORT_SEND_IOV_V     26
#define SC_PORT_REPLY_IOV      27
#define SC_SIGACTION           28
#define SC_THREAD_KILL         29
#define SC_SIGPROCMASK         30
#define SC_THREAD_EXIT         31
#define SC_TIMER_CREATE        32
#define SC_TIMER_CONTROL       33
#define SC_MUNMAP              34
#define SC_THREAD_WAIT         35
#define SC_PORT_MSG_READ       36
#define SC_KERNEL_CONTROL      37
#define SC_TIMER_DELETE        38
#define SC_SIGWAITINFO         39
#define SC_SCHED_YIELD         40
#define SC_MEMOBJ_CREATE       41
#define SC_FORK                42
#define SC_GRANT_PAGES         43
#define SC_WAITPID             44
#define SC_ALLOC_DMA           45
#define SC_FREE_DMA            46

/* general wrapper for syscall */
extern status_t __syscall(syscall_arg_t arg0,syscall_arg_t arg1,
                          syscall_arg_t arg2,syscall_arg_t arg3,
                          syscall_arg_t arg4,syscall_arg_t arg5,
                          syscall_id_t syscall_id);

/* useful trick to bind syscall with different number of arguments */
#define __syscall0(id)                    __syscall(0, 0, 0, 0, 0, 0, id)
#define __syscall1(id,a0)                 __syscall(a0,0, 0, 0, 0, 0, id)
#define __syscall2(id,a0,a1)              __syscall(a0,a1,0, 0, 0, 0, id)
#define __syscall3(id,a0,a1,a2)           __syscall(a0,a1,a2,0, 0, 0, id)
#define __syscall4(id,a0,a1,a2,a3)         __syscall(a0,a1,a2,a3,0, 0, id)
#define __syscall5(id,a0,a1,a2,a3,a4)      __syscall(a0,a1,a2,a3,a4,0, id)
#define __syscall6(id,a0,a1,a2,a3,a4,a5)   __syscall(a0,a1,a2,a3,a4,a5,id)

#define __SYSCALL_RETURN_ERR_PATH(r)  if((r) < 0 ) {errno=-(r);return-1;} else {return (r);}

#endif /* __SYS_SYSCALL_H__ */

