/*
 * This library is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU Lesser General Public License as 
 * published by the Free Software Foundation; either version 2.1 of 
 * the License.
 *
 * This library is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public 
 * License along with this library; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 *
 * (c) Copyright 2006,2007,2008 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2008 Michael Tsymbalyuk <mtzaurus@gmail.com>
 *
 * LibOS:
 * include/thread.h: Low-level functions/prototypes related to threads.
 *
 */

#ifndef __THREAD_H__
#define  __THREAD_H__

#include <sys/types.h>
#include <sys/mman.h>

typedef status_t (*thread_fn_t)(void *data);

#define STACK_PAGES  16

typedef enum __task_creation_flag_t {
  CLONE_MM       = 0x01,
  CLONE_IPC      = 0x02,
  CLONE_SIGINFO  = 0x04,
  CLONE_COW      = 0x08,
  CLONE_POPULATE = 0x10,
  CLONE_SHMEM    = 0x20,
  CLINE_PHYS     = 0x40,
} task_creation_flags_t;

/* Task creation attributes. */
typedef struct __task_attrs {
  uint8_t run_immediately;
} task_attrs_t;

#define __EXEC_ATTRS_COW       0x1
#define __EXEC_ATTRS_COPY_IPC  0x2

typedef struct __exec_attrs {
  uintptr_t stack,entrypoint,destructor,arg1,arg2;
  uintptr_t per_task_data;
  long flags;
} exec_attrs_t;

typedef struct __task_creation_attrs {
  task_attrs_t task_attrs;
  exec_attrs_t exec_attrs;
} task_creation_attrs_t;

#define __ATTR_OFF  0  /**< Attribute is enabled **/
#define __ATTR_ON   1  /**< Attribute is disabled **/

status_t thread_create(thread_fn_t fn,void *data,ulong_t flags);

#endif
