/*
 * This library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * (c) Copyright 2006,2007,2008,2009 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2009 Dmitry Balakshin <dmitry.balakshin@jarios.org>
 *
 * LibC:
 * sys/types.h: Various data types used elsewhere.
 */

#ifndef __SYS_TYPES_H__
#define __SYS_TYPES_H__

#include <stddef.h>

/* NOTE: sys/types.h is not allowed to declare bool type.
 *       Include stdbool.h instead.
 */

/* According to POSIX we can not provide intN_t/uintN_t types here.
 * But we will do it anyway.
 */
#include <stdint.h>
#include <arch/types.h>

#ifndef __GID_T_DEFINED
typedef unsigned short  gid_t;
#define __GID_T_DEFINED
#endif

#ifndef __UID_T_DEFINED
typedef unsigned short  uid_t;
#define __UID_T_DEFINED
#endif

#ifndef __PID_T_DEFINED
typedef int             pid_t;
#define __PID_T_DEFINED 1
#endif

#if BITS_PER_LONG == 64
typedef signed long int ssize_t;
#endif

typedef unsigned short  dev_t;
typedef unsigned int    mode_t;
typedef unsigned int    nlink_t;

#ifndef __OFF_T_DEFINED
#if BITS_PER_LONG == 64
typedef long            off_t;
#define __OFF_T_DEFINED 1
#endif
#endif

#ifndef __CLOCK_T_DEFINED
#if BITS_PER_LONG == 64
typedef long clock_t;
#define __CLOCK_T_DEFINED 1
#endif
#endif

#ifndef __TIME_T_DEFINED
#if BITS_PER_LONG == 64
typedef long time_t;
#define __TIME_T_DEFINED 1
#endif
#endif

#ifndef __CLOCKID_T_DEFINED
typedef unsigned int clockid_t;
#define __CLOCKID_T_DEFINED 1
#endif

#ifndef __TIMER_T_DEFINED
typedef unsigned int timer_t;
#define __TIMER_T_DEFINED 1
#endif

#ifndef __SUSECONDS_T_DEFINED
typedef long suseconds_t;
#define __SUSECONDS_T_DEFINED 1
#endif

typedef unsigned short  blksize_t;

#if BITS_PER_LONG == 64
typedef unsigned long   blkcnt_t;
#endif

#if BITS_PER_LONG == 64
typedef unsigned long   ino_t;
#endif

typedef struct __pthread_mutexattr {
  uint8_t __type;
} pthread_mutexattr_t;

typedef struct {
  __sync_id_t __id;
  pthread_mutexattr_t __attrs;
} pthread_mutex_t;

typedef struct __pthread {
  tid_t __tid;
} pthread_t;

typedef struct __pthread_attr {
  int foo;
} pthread_attr_t;


/* JariOS specific */

typedef size_t          ulong_t;

typedef uint64_t        count_t;
typedef uint64_t        index_t;

typedef uint32_t        usec_t;

typedef uint32_t        syscall_id_t;
typedef unative_t       syscall_arg_t;

typedef uint32_t        shm_id_t;

/* Bit-related types */
typedef uint32_t        bit_idx_t;

/* Generic file system type */
typedef enum {
  FT_REGULAR = 0,
  FT_DIRECTORY,
  FT_CHARDEV,
  FT_BLOCKDEV,
  FT_NETDEV,
  FT_VIDEODEV,
  FT_FIFO,
  FT_SOCK,
  FT_LOCK,
  FT_HARDLINK,
  FT_SYMLINK
} file_type_t;

#endif  /* __SYS_TYPES_H__ */
