/*
 * This library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * (c) Copyright 2006,2007,2008,2009 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2009 MadTirra <madtirra@jarios.org>
 *
 * LibC:
 * include/sys/v2.h: v2 VFS internal system definions
 */

#ifndef __SYSTEM_V2__
#define __SYSTEM_V2__

#include <sys/types.h>

/**
 * defines for file system attach and mounts
 */
#define V2FS_SYS  (1 << 0)
#define V2FS_RO   (1 << 1)
#define V2FS_RW   (1 << 2)

/**
 * @desc structure used for sync with 
 *       VFS v2 service
 */
typedef struct __l_v2IO_t {
  uint16_t fd;    /* file descriptor id */
  uint16_t lid;   /* link connection id */
  uint16_t pid;   /* pid of end point */
  uint16_t prtid; /* port id of end point */
} __attribute__ ((packed)) l_v2IO_t;


/* dirent-related data */

#define DD_NOTSTART  0
#define DD_ENDODIR  -1

#define DIRENT_RR  0x1 /* indicate that directory should be reridden */
#define DIRENT_RN  0x0 /* indicate that all operations are ok */
#define DIRENT_EL  0x2 /* indicate that there are no entries */

/* 
 * **NOTE:**
 * the following data flow <tr_dirent><...name[tr_dirent->d_namlen]...>....
 * size will indicate amount of flow chunk 
 */
struct tr_dirent { /* this structure used for fs <-> client transmittions */
  ino_t d_ino;
  off_t d_off;
  unsigned short d_namlen;
  unsigned short d_rec_len;
};



#endif /* __SYSTEM_V2__ */
