/*
 * This library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * (c) Copyright 2006,2007,2008,2009 Jari OS Core Team <http://jarios.org>
 *
 * LibC:
 * include/sys/wait.h: Status of terminated child process
 */

#ifndef __SYS_WAIT_H__
#define __SYS_WAIT_H__

#include <sys/types.h>

#define WNOHANG    0x01  /* Don't sleep if there are no available zombies. */

#define WCONTINUED 0x00	/* TODO: implement it */
#define WUNTRACED  0x00 /* TODO: implement it */

/* TODO: implement following macros */

/* Evaluates to a non-zero value if status was returned for a child process
 * that terminated normally.
 */
#define WIFEXITED(status) (status)

/* Evaluates to the low-order 8 bits of the status argument that the child
 * process passed to _exit() or exit(), or the value the child process returned
 * from main().
 */
#define WEXITSTATUS(status) (status)

/* Evaluates to a non-zero value if status was returned for a child process
 * that terminated due to the receipt of a signal that was not caught.
 */
#define WIFSIGNALED(status) (status)

/* Evaluates to the number of the signal that caused the termination of the
 * child process.
 */
#define WTERMSIG(status) (status)

/* Evaluates to a non-zero value if status was returned for a child process
 * that is currently stopped.
 */
#define WIFSTOPPED(status) (status)

/* Evaluates to the number of the signal that caused the child  process to
 * stop.
 */
#define WSTOPSIG(status) (status)

/* Evaluates to a non-zero value if status was returned for a child process
 * that has continued from a job control stop.
 */
#define WIFCONTINUED(status) (status)



pid_t waitpid(pid_t pid, int *stat_loc, int options);

#endif  /* __SYS_WAIT_H__ */
