/*
 * This library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * (c) Copyright 2006,2007,2008,2009 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2008 Michael Tsymbalyuk <mtzaurus@gmail.com>
 *
 * LibC:
 * include/time.h  contains types and prototypes for dealing with time.
 */

#ifndef __TIME_H__
#define __TIME_H__

#include <stddef.h>
#include <signal.h>

#ifndef __CLOCK_T_DEFINED
#if BITS_PER_LONG == 64
typedef long clock_t;
#define __CLOCK_T_DEFINED 1
#endif
#endif

#ifndef __TIME_T_DEFINED
#if BITS_PER_LONG == 64
typedef long time_t;
#define __TIME_T_DEFINED 1
#endif
#endif

#ifndef __CLOCKID_T_DEFINED
typedef unsigned int clockid_t;
#define __CLOCKID_T_DEFINED 1
#endif

#ifndef __TIMER_T_DEFINED
typedef unsigned int timer_t;
#define __TIMER_T_DEFINED 1
#endif

struct tm
{
  int tm_sec;           /* Seconds.     [0-60] (1 leap second) */
  int tm_min;           /* Minutes.     [0-59] */
  int tm_hour;          /* Hours.       [0-23] */
  int tm_mday;          /* Day.         [1-31] */
  int tm_mon;           /* Month.       [0-11] */
  int tm_year;          /* Year - 1900.  */
  int tm_wday;          /* Day of week. [0-6] */
  int tm_yday;          /* Day of year.[0-365] */
  int tm_isdst;         /* DST.         [-1/0/1]*/
};

struct timespec {
  time_t tv_sec;        /* seconds */
  long   tv_nsec;       /* nanoseconds */
};

struct itimerspec {
  struct timespec it_interval;
  struct timespec it_value;
};

#define CLOCK_REALTIME  0
#define TIMER_ABSTIME   1  /* Absolute time value. */

typedef struct timespec timeval_t;

time_t time(time_t *tloc);

int nanosleep(const struct timespec *rqtp, struct timespec *rmtp);

char *asctime(const struct tm *timeptr);
char *ctime(const time_t *clock);
struct tm *localtime(const time_t *timer);
time_t mktime(struct tm *timeptr);

double difftime(time_t time1, time_t time0);

int timer_create(clockid_t clockid, struct sigevent *evp, timer_t *timerid);
int timer_settime(timer_t timerid, int flags, const struct itimerspec *value,
                  struct itimerspec *ovalue);
int timer_gettime(timer_t timerid, struct itimerspec *value);
int timer_getoverrun(timer_t timerid);

/* JariOS extension */
int settime(time_t tloc);

#endif  /* __TIME_H__ */
