/* (c) Copyright 2008 MadTirra <madtirra@jarios.org>
 * (c) Copyright 2008 Dmitry Gromada <gromada@jarios.org>
 *
 * LibC:
 * include/unistd.h: Various essential POSIX functions and constants
 */

#ifndef __UNISTD_H__
#define __UNISTD_H__

#include <stddef.h>

pid_t getpid(void);
tid_t gettid(void);
pid_t getppid(void);

unsigned int sleep(unsigned int secs);
int usleep(ulong_t usecs);

char* getcwd(char *buf, size_t size);
int chdir(const char *path);
int fchdir(int fildes);

int chown(const char *path, uid_t owner, gid_t group);
int lchown(const char *path, uid_t owner, gid_t group);
int fchown(int fildes, uid_t owner, gid_t group);

#define STDIN_FILENO 0
#define STDOUT_FILENO 1
#define STDERR_FILENO 2

int dup(int fildes);
int dup2(int fildes, int fildes2);

int ftruncate(int fildes, off_t length);
int truncate(const char *path, off_t length);

int isatty(int fd);
int close(int fildes);

ssize_t pread(int fildes, void *buf, size_t nbyte, off_t offset);
ssize_t read(int fildes, void *buf, size_t nbyte);

ssize_t pwrite(int fildes, const void *buf, size_t nbyte, off_t offset);
ssize_t write(int fildes, const void *buf, size_t nbyte);

/* Values for the WHENCE argument to lseek */
#ifndef SEEK_SET
#define SEEK_SET 0
#endif

#ifndef SEEK_CUR
#define SEEK_CUR 1
#endif

#ifndef SEEK_END
#define SEEK_END 2
#endif

off_t lseek(int fildes, off_t offset, int whence);

int rmdir(const char *path);

void sync(void);

int unlink(const char *path);

/* Values for the second argument to access() */
#define F_OK 0                  /* Test for existance */
#define R_OK (1 << 2)           /* Test for read permission */
#define W_OK (1 << 1)           /* Test for write permission */
#define X_OK (1 << 0)           /* Test for execute permission */

int access(const char *path, int amode);

pid_t fork(void);

/* Process environment variables */
extern char *environ[];

int execv(const char *path, char *const argv[]);
int execve(const char *filename,char *const argv[], char *const envp[]);

int execl(const char *path, const char *arg0, ... /*, (char *)0 */);
int execle(const char *path, const char *arg0, ... /*, (char *)0, char *const envp[]*/);

int execlp(const char *file, const char *arg0, ... /*, (char *)0 */);
int execvp(const char *file, char *const argv[]);

void _exit(int status);

ssize_t readlink(const char *path, char *buf, size_t bufsize);

uid_t getuid(void);
uid_t geteuid(void);

gid_t getgid(void);
gid_t getegid(void);

int setuid(uid_t uid);
int setgid(gid_t gid);

/* This is GNU extension */
int euidaccess(const char *pathname, int mode);

unsigned int alarm(unsigned int seconds);

void start_service(void);

#endif  /* __UNISTD_H__ */

