/*
 * This library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * (c) Copyright 2006,2007,2008,2009 Jari OS Core Team <http://jarios.org>
 *
 * LibC:
 * include/wchar.h: wide character related types and functions
 */

#ifndef __WCHAR_H__
#define __WCHAR_H__

#include <stddef.h>
#include <stdarg.h>
#include <wctype.h>

typedef struct __mbstate_t {
  int count;            /* Number of bytes needed for the current character */
  wint_t value;         /* Value so far */
} mbstate_t;

int swprintf(wchar_t *str, size_t size, const wchar_t *format, ...);
int vswprintf(wchar_t *str, size_t size, const wchar_t *format, va_list ap);

size_t wcslen(const wchar_t *s);

wchar_t *wcscpy(wchar_t *s,const wchar_t *d);

int wcscmp(const wchar_t *s1,const wchar_t *s2);

int wcsncmp(const wchar_t *s1,const wchar_t *s2,int n);

wchar_t *wcschr(const wchar_t *wcs,wchar_t ch);

wchar_t *wcsrchr(const wchar_t *wcs,wchar_t ch);

wchar_t *wcsdup(const wchar_t *s);

wchar_t *wcpcpy(wchar_t *dst,const wchar_t *src);

wchar_t *wcpncpy(wchar_t *dst,const wchar_t *src,size_t n);

wchar_t *wcscat(wchar_t *dst,const wchar_t *src);

wchar_t *wcsncat(wchar_t *dst,const wchar_t *src,size_t n);

wchar_t *wcsncpy(wchar_t *s,const wchar_t *d,size_t n);

wchar_t *wcsstr(const wchar_t *haystack, const wchar_t *needle);

int wcscasecmp(const wchar_t *s1, const wchar_t *s2);

int wcsncasecmp(const wchar_t *s1, const wchar_t *s2, int n);

int mbsinit(mbstate_t *ps);
size_t mbrtowc(wchar_t *pwc, const char *s, size_t n, mbstate_t *ps);
size_t mbsrtowcs(wchar_t *dest, const char **src, size_t len, mbstate_t *ps);
size_t mbstowcs(wchar_t *dest, const char *src, size_t n);
size_t mbrlen(const char *s, size_t n, mbstate_t *ps);
size_t mbsnrtowcs(wchar_t *dest, const char **src, size_t nms, size_t len, mbstate_t *ps);

int wctomb(char *s, wchar_t wc);
size_t wcrtomb(char *s, wchar_t wc, mbstate_t *ps);
size_t wcstombs(char *dest, const wchar_t *src, size_t n);

wchar_t* wmemcpy(wchar_t *ws1, const wchar_t *ws2, size_t n);
wchar_t* wmemset(wchar_t *ws, wchar_t wc, size_t n);

#endif  /* __WCHAR_H__ */
