/*
 * This library is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU Lesser General Public License as 
 * published by the Free Software Foundation; either version 2.1 of 
 * the License.
 *
 * This library is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public 
 * License along with this library; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 *
 * (c) Copyright 2006,2007,2008 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2008 MadTirra <madtirra@jarios.org>
 *
 * LibC:
 * __lowlevel.h: low level port and channel operations
 *
 */

#ifndef __INT___LOWLEVEL_H__
#define __INT___LOWLEVEL_H__

#include <sys/types.h>
#include <ipc/port.h>
#include <ipc/channel.h>
#include <ipc/iov.h>

/*
 * Internal channels and their ids:
 */
extern int __ntochannels;
extern ipc_channel_t **__ntochan;

/*
 * Internal ports and their ids:
 */
extern ipc_port_t **__toports;
extern int __ntoports;

extern int __channels_init(void);

/* port operations */
/*
 * __create_ipc_port:
 * create ipc port with given flags
 */
status_t __create_ipc_port(ulong_t flags,ulong_t queue_size);
/*
 * __close_ipc_port:
 * close ipc port
 */
status_t __close_ipc_port(ulong_t port);
/*
 * __rcvmsg_ipc_port:
 * receive message from port sleep while message 
 * is not sent
 */
status_t __rcvmsg_ipc_port(ulong_t port,void *recv_buf,size_t recv_size,
			   ipc_port_msg_info_t *msg_info,int flags);
/*
 * __reply_ipc_port:
 * reply to a given message 
 */
status_t __reply_ipc_port(ulong_t port,ulong_t msg_id,
			  void *reply_buf,size_t reply_len);

/*
 * __read_msg_ipc_port:
 * read message from a port with given offset and len
 * returns 0 if ok, and error on error
 */
status_t __read_msg_ipc_port(ulong_t port,ulong_t msgid,
			     void *recv_buf,size_t recv_len,size_t msg_off);

/* channel operations */
/* 
 * __open_ipc_channel:
 * opens a channel to pid,port
 */
status_t __open_ipc_channel(pid_t pid,ulong_t port,ulong_t flags);
/*
 * __close_ipc_channel:
 * close ipc channel connected to pid and port
 */
status_t __close_ipc_channel(ulong_t channel);
/*
 * __msgsnd_ipc_channel:
 * send a message via channel and sleep while reply
 * was sent
 */
status_t __msgsnd_ipc_channel(ulong_t channel,void *send_buf,
			      size_t send_size,void *rcv_buf,
			      size_t rcv_size);
/*
 * __evtsnd_ipc_channel:
 * send non-blocking event via channel
 */
status_t __evtsnd_ipc_channel(ulong_t channel,void *send_buf,
			      size_t send_size);

/*
 * __ipc_channel_send_iov:
 * send iovector via channel and receive reply in single buffer
 */
unsigned int __ipc_channel_send_iov(chid_t channel,iovec_t *snd_vecs,int snd_nvec,
                                    void *rcv_vec, size_t rcv_nvec);

/*
 * __ipc_channel_send_iov:
 * send and receive iovector via channel
 */
unsigned int __ipc_channel_send_iov_iov(chid_t channel,iovec_t *snd_vecs, int snd_nvec,
                                        iovec_t *rcv_vecs, int rcv_nvec);

/*
 * __reply_ipc_port_iov:
 * reply with iovevtor
 */
unsigned int __reply_ipc_port_iov(ulong_t port, ulong_t msg_id, iovec_t *repl_iov,
                                  ulong_t repl_nvec);

#endif /* __INT___LOWLEVEL_H__ */
