/*
 * This library is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU Lesser General Public License as 
 * published by the Free Software Foundation; either version 2.1 of 
 * the License.
 *
 * This library is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public 
 * License along with this library; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 *
 * (c) Copyright 2006,2007,2008 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2008 MadTirra <madtirra@jarios.org>
 *
 * LibC:
 * __port.c: low level port operations
 *
 */

#include <sys/types.h>
#include <sys/syscall.h>
#include <errno.h>
#include <time.h>

#include <ipc/port.h>

/* creates IPC port, returns port number, or negative value on fail */
status_t __create_ipc_port(ulong_t flags,ulong_t queue_size)
{
  if(queue_size > (1 << 21))
    return -EINVAL;
  else
    return __syscall2(SC_CREATE_PORT,flags,queue_size);
}

status_t __close_ipc_port(ulong_t port)
{
  return __syscall1(SC_CLOSE_PORT,port);
}

status_t __rcvmsg_ipc_port(ulong_t port,void *recv_buf,size_t recv_size,
			   ipc_port_msg_info_t *msg_info,int flags)
{
  if(!recv_buf && recv_size>0)
      return -EINVAL;
  
  return __syscall5(SC_PORT_RECEIVE,port,flags,(syscall_arg_t)recv_buf,
                    recv_size,(syscall_arg_t)msg_info);
}

status_t __reply_ipc_port(ulong_t port,ulong_t msg_id,
			  void *reply_buf,size_t reply_len)
{
  if(!reply_buf)
    return -EINVAL;
  else
    return __syscall4(SC_PORT_REPLY,port,msg_id,
		      (syscall_arg_t)reply_buf,reply_len);
}

status_t __read_msg_ipc_port(ulong_t port,ulong_t msgid,
			     void *recv_buf,size_t recv_len,size_t msg_off)
{
  if(!recv_buf)
    return -EINVAL;
  else
    return __syscall5(SC_PORT_MSG_READ,port,msgid,(syscall_arg_t)recv_buf,
		      recv_len,msg_off);
}
