/*
 * This library is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU Lesser General Public License as 
 * published by the Free Software Foundation; either version 2.1 of 
 * the License.
 *
 * This library is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public 
 * License along with this library; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 *
 * (c) Copyright 2006,2007,2008 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2008 MadTirra <madtirra@jarios.org>
 *
 * LibC:
 * iov.c: iov channel operations
 *
 */

#include <sys/types.h>
#include <sys/syscall.h>
#include <errno.h>
#include <time.h>

#include <ipc/port.h>
#include <ipc/channel.h>
#include <ipc/iov.h>

#include "__lowlevel.h"

int ipc_iov_send(chid_t ch,iovec_t *snd_iov, int snd_iovn, void *rcv_msg, size_t size)
{
  if(!snd_iov || snd_iovn<=0)
    return -EINVAL;
  if(!rcv_msg || size<=0)
    return -EINVAL;

  return __ipc_channel_send_iov(ch, snd_iov, snd_iovn, rcv_msg, size);
}

int ipc_iov_iov_send(chid_t ch,iovec_t *snd_iov, int snd_iovn, iovec_t *rcv_iov, int rcv_iovn)
{
  if(!snd_iov || snd_iovn<=0)
    return -EINVAL;
  if(!rcv_iov || rcv_iovn<=0)
    return -EINVAL;

  return __ipc_channel_send_iov_iov(ch,snd_iov, snd_iovn, rcv_iov, rcv_iovn);
}

int ipc_port_reply_nmsg_iov(prtid_t id, iovec_t *repl_iov, int repl_iovn, int msg_id)
{  
  if(repl_iovn<=0 || !repl_iov)
    return -EINVAL;

  return __reply_ipc_port_iov(id, msg_id, repl_iov, repl_iovn);
}

int ipc_port_reply_iov(prtid_t id, iovec_t *repl_iov, int repl_iovn)
{  
  if(repl_iovn<=0 || !repl_iov)
    return -EINVAL;

  return __reply_ipc_port_iov(id, __toports[id]->msginfo.msg_id, repl_iov, repl_iovn);
}
