/*
 * This library is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU Lesser General Public License as 
 * published by the Free Software Foundation; either version 2.1 of 
 * the License.
 *
 * This library is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public 
 * License along with this library; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 *
 * (c) Copyright 2006,2007,2008 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2008 MadTirra <madtirra@jarios.org>
 *
 * LibC:
 * port.c: higher level port operations
 *
 */

#include <sys/types.h>
#include <sys/syscall.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <time.h>

#include <ipc/port.h>
#include <datalib/usrtc.h>

#include "__lowlevel.h"

ipc_port_t **__toports=NULL;
int __ntoports=0;
/* local functions prototypes */
static int __init_global_toport(void);
static int __incr_global_toport(void);

prtid_t ipc_port_open(ulong_t flags,size_t kqueue_size)
{
  prtid_t id;

  if(!__ntoports)
    if(__init_global_toport()!=0)
      return -ENOMEM;

  id=__create_ipc_port(flags,kqueue_size);
  if(id<0)
    return id;
  else {
    if(id>=__ntoports) {
      if(__incr_global_toport()!=0)
        goto __erro_goo;
    }
    __toports[id]=malloc(sizeof(ipc_port_t));
    if(!__toports[id]) {
    __erro_goo:
      __close_ipc_port(id);
      return -ENOMEM;
    }
    __toports[id]->id=id;
    __toports[id]->flags=flags;
  }

  return id;
}

int ipc_port_close(prtid_t id)
{
  if(__toports[id]==NULL) 
    return -EINVAL;
  else {
    free(__toports[id]);
    __toports[id]=NULL;
    return __close_ipc_port(id);
  }

  return 0;
}

int ipc_port_reply(prtid_t id,size_t msg_size,void *msg)
{
  if(__toports[id]==NULL || msg_size==0)
    return -EINVAL;

  return __reply_ipc_port(id,__toports[id]->msginfo.msg_id,msg,msg_size);
}

int ipc_port_reply_nmsg(prtid_t id,size_t msg_size,void *msg,int msgid)
{
  if(__toports[id]==NULL || msg_size==0)
    return -EINVAL;

  if(msgid<0)
    return -EINVAL;

  return __reply_ipc_port(id,msgid,msg,msg_size);
}

int ipc_port_rcvmsg(prtid_t id,size_t msg_size,void *msg)
{
  status_t r;

  if((id < 0 || id >= __ntoports) || __toports[id]==NULL || msg_size==0) {
    *(int*)22 = 33;
    return -EINVAL;
  }

  r=__rcvmsg_ipc_port(id,msg,msg_size,
		      &__toports[id]->msginfo,__toports[id]->flags);

  if(r<0)
    return r;

  return __toports[id]->msginfo.msg_len;
}

int ipc_port_rcvmsg_mi(prtid_t id, size_t msg_size, void * msg, ipc_port_msg_info_t *minfo)
{
  int r;

  r=__rcvmsg_ipc_port(id,msg,msg_size, minfo,__toports[id]->flags);
  if (r < 0)
    return r;

  return minfo->msg_len;
}

int ipc_port_rcvmsg_mi_cs(prtid_t id, size_t msg_size, void * msg,
			  ipc_port_msg_info_t *minfo,int flags)
{
  int r;

  r=__rcvmsg_ipc_port(id,msg,msg_size, minfo,flags);
  if (r < 0)
    return r;

  return minfo->msg_len;
}

int ipc_port_read_msg(prtid_t id,int msg_id,size_t offset,void *msg,size_t recv_len)
{
  return __read_msg_ipc_port(id,msg_id,msg,recv_len,offset);
}

ipc_port_msg_info_t ipc_port_getinfo(prtid_t id)
{
  return __toports[id]->msginfo;
}

/* [ak]TODO: fix up and redesign */
status_t ipc_port_poll(pollfd_t *fds,ulong_t nfds,timeval_t *timeout)
{
  if( !fds || !nfds ) {
    return -EINVAL;
  }

  return __syscall3(SC_IPC_PORT_POLL,(ulong_t)fds,nfds,(ulong_t)timeout);
}

/* local functions */
static int __init_global_toport(void)
{
  __toports=malloc(sizeof(unative_t)*DTOPORTS);
  if(!__toports)
    return -1;
  else
    memset(__toports,0,sizeof(unative_t)*DTOPORTS);

  __ntoports=DTOPORTS;

  return 0;
}

static int __incr_global_toport(void)
{
  __toports=realloc(__toports,sizeof(unative_t)*(__ntoports+DTOPORTS));
  if (!__toports)
    return -ENOMEM;

  __ntoports+=DTOPORTS;

  return 0;
}
