/*
 * This library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * (c) Copyright 2006-2009 Jari OS Core Team <http://jarios.org>
 * Copyright (C) 2009 Dmitry Balakshin <balakshin@jarios.org>
 *
 */

#include <limits.h>
#include <sys/types.h>
#include <bitwise.h>
#include <stdlib.h>

#include "__fildesc.h"

static uint8_t bitmap[OPEN_MAX / 8] = {};
fds_t tfds[OPEN_MAX] = {};

int take_fd(void)
{
  int i, j;

  if ((bitmap[OPEN_MAX / 8 - 1] & 0x80) == 0) {
    /* find last non-zero byte */
    for (i = OPEN_MAX / 8 - 1; i >= 0; i--)
      if (bitmap[i] != 0)
        break;
    if ((i == -1) || (bitmap[i] == 0xff))
      i++;
  } else {
    /* find first non-0xff byte */
    for (i = 0; i < OPEN_MAX / 8; i++)
      if (bitmap[i] != 0xff)
        break;
    if (i == OPEN_MAX)
      return -1;
  }

  for (j = 0; j < 7; j++)
    if (!bit_test(bitmap + i, j))
      break;

  return take_exact_fd(i * 8 + j);
}

int take_exact_fd(int fd)
{
  release_fd(fd);

  bit_set(bitmap + fd / 8, fd % 8);

  tfds[fd].offset = 0;
  tfds[fd].pid = 0;
  tfds[fd].port_id = 0;
  tfds[fd].ufsid = 0;
  tfds[fd].chid = -1;
  tfds[fd].dir = NULL;

  return fd;
}

/* take 'above-or-equal' fd */
int take_ae_fd(int fd)
{
  for (; fd < OPEN_MAX; fd++)
    if (!bit_test(bitmap + fd / 8, fd % 8))
      return take_exact_fd(fd);

  return -1;
}

void release_fd(int fd)
{
  tfds[fd].offset = 0;
  tfds[fd].pid = 0;
  tfds[fd].port_id = 0;
  tfds[fd].ufsid = 0;
  tfds[fd].chid = -1;
  if (tfds[fd].dir != NULL) {
    free(tfds[fd].dir);
    tfds[fd].dir = NULL;
  }

  bit_clear(bitmap + fd / 8, fd % 8);
}

int valid_fd(int fd)
{
  if ((fd < 0) || (fd >= OPEN_MAX))
    return 0;

  return bit_test(bitmap + fd / 8, fd % 8);
}
