/*
 * This library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * (c) Copyright 2006,2007,2008 Jari OS Core Team <http://jarios.org>
 * Copyright (C) 2009 Dmitry Balakshin <balakshin@jarios.org>
 *
 */

#include <limits.h>
#include <sys/types.h>
#include <errno.h>
#include <unistd.h>
#include <string.h>

#include <ns/ns.h>
#include <ipc/port.h>
#include <ipc/channel.h>
#include <sys/v2.h>

#include "__fildesc.h"
#include "__pidfd.h"

#define VFS_PORT 0
chid_t vfs_channel;

static int fetch_fds(void);

int __connect_to_vfs(void)
{
  pid_t vfs;

  /* name server doesn't need vfs */
  if (getpid() == 1)
    return 0;

  vfs = ns_getvfspid();
  if (vfs == 0)
    return -ESRCH;	/* No such process */

  vfs_channel = ipc_channel_open(vfs, VFS_PORT, IPC_BLOCKED_ACCESS);
  if (vfs_channel < 0)
    return vfs_channel;

  fetch_fds();

  return 0;
}

static int fetch_fds(void)
{
  l_v2IO_t rc[OPEN_MAX];
  int r, i, fd;

  memset(rc, 0, sizeof(rc));
  r = __pidfd_get(rc, OPEN_MAX);
  if (r == -1)
    return -1;

  for (i = 0; i < r; i++) {
    fd = take_exact_fd(rc[i].fd);

    tfds[fd].pid = rc[i].pid;
    tfds[fd].port_id = rc[i].prtid;
    tfds[fd].ufsid = rc[i].lid;

    tfds[fd].chid = ipc_channel_open(tfds[fd].pid, tfds[fd].port_id,
                  IPC_BLOCKED_ACCESS);
    if (tfds[fd].chid < 0) {
      release_fd(fd);
      /* TODO: report such situation */
    }

  }

  return 0;
}
