/*
 * This library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * (c) Copyright 2006-2009 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2005,2008,2009 MadTirra <madtirra@jarios.org>
 * Copyright (C) 2009 Dmitry Balakshin <balakshin@jarios.org>
 *
 */

#include <sys/types.h>
#include <errno.h>

#include <sys/v2.h>
#include <ipc/rpc.h>
#include <ipc/iov.h>
#include <ipc/channel.h>

#include "__init.h"
#include "__pidfd.h"

int __pidfd_get(l_v2IO_t *pidlist, size_t nr_pids)
{
  uni_rpc_t rpc;
  iovec_t send, recv[2];
  int r;

  rpc.rpc_call = RPC_PIDFDGET;
  rpc.size = nr_pids;

  send.iov_base = &rpc;
  send.iov_len  = sizeof(uni_rpc_t);
  recv[0].iov_base = &rpc;
  recv[0].iov_len  = sizeof(uni_rpc_t);
  recv[1].iov_base = pidlist;
  recv[1].iov_len = sizeof(l_v2IO_t) * nr_pids;

  r = ipc_iov_iov_send(vfs_channel, &send, 1, recv, 2);
  if (r < 0) {
    errno = -r;
    return -1;
  }
  if (rpc.rpc_call != 0) {
    errno = rpc.rpc_call;
    return -1;
  }

  return (int)rpc.size;
}

int __pidfd_clone(pid_t pid, pid_t newpid)
{
  uni_rpc_t rpc;
  int r;

  rpc.rpc_call = RPC_PIDFDCLONE;
  rpc.arg0 = pid;
  rpc.arg1 = newpid;

  r = ipc_channel_msgsnd(vfs_channel, sizeof(uni_rpc_t), &rpc,
          sizeof(uni_rpc_t), &rpc);
  if (r < 0) {
    errno = -r;
    return -1;
  }
  if (rpc.rpc_call != 0) {
    errno = rpc.rpc_call;
    return -1;
  }

  return 0;
}

int __pidfd_reset(pid_t pid)
{
  uni_rpc_t rpc;
  int r;

  rpc.rpc_call = RPC_PIDFDRESET;
  rpc.arg0 = pid;

  r = ipc_channel_msgsnd(vfs_channel, sizeof(uni_rpc_t), &rpc,
          sizeof(uni_rpc_t), &rpc);
  if (r < 0) {
    errno = -r;
    return -1;
  }
  if (rpc.rpc_call != 0) {
    errno = rpc.rpc_call;
    return -1;
  }

  return 0;
}

int __pidfd_kill(pid_t pid)
{
  uni_rpc_t rpc;
  int r;

  rpc.rpc_call = RPC_PIDFDKILL;
  rpc.arg0 = pid;

  r = ipc_channel_msgsnd(vfs_channel, sizeof(uni_rpc_t), &rpc,
          sizeof(uni_rpc_t), &rpc);
  if (r < 0) {
    errno = -r;
    return -1;
  }
  if (rpc.rpc_call != 0) {
    errno = rpc.rpc_call;
    return -1;
  }

  return 0;
}
