/* This library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * (c) Copyright 2006,2007,2008,2009 Jari OS Core Team <http://jarios.org>
 * Copyright (C) 2009 Dmitry Balakshin <balakshin@jarios.org>
 *
 * libc:
 *   general/libc/jarifs/access.c
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <errno.h>
#include <unistd.h>

int access(const char *path, int amode)
{
  struct stat st;
  int rc = 0;

  if ((amode != F_OK) && ((amode & ~(R_OK | W_OK | X_OK)) != 0)) {
    errno = EINVAL;
    return -1;
  }

  if (stat(path, &st) == -1)
    return -1;

  if (amode == F_OK)
    return 0;

  /* TODO use getuid() & getgid() */
  if (amode & R_OK) {
    if ((st.st_mode & S_IRUSR) || (st.st_mode & S_IRGRP) ||
                    (st.st_mode & S_IROTH))
      rc |= R_OK;
  }
  if (amode & W_OK) {
    if ((st.st_mode & S_IWUSR) || (st.st_mode & S_IWGRP) ||
                    (st.st_mode & S_IWOTH))
      rc |= W_OK;
  }
  if (amode & X_OK) {
    if ((st.st_mode & S_IXUSR) || (st.st_mode & S_IXGRP) ||
                    (st.st_mode & S_IXOTH))
      rc |= X_OK;
  }

  if (rc == amode)
    return 0;

  errno = EACCES;
  return -1;
}

/* This is GNU extension */
int euidaccess(const char *pathname, int mode)
{
  /* TODO: implement function properly */

  return access(pathname, mode);
}
