/*
 * This library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * (c) Copyright 2006,2007,2008,2009 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2009 Dmitry Balakshin <balakshin@jarios.org>
 *
 */

#include <limits.h>
#include <sys/types.h>
#include <errno.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>

#include <ipc/rpc.h>
#include <ipc/iov.h>

#include "__fildesc.h"
#include "__init.h"
#include "__iovpath.h"

#include <stdio.h>

extern char __cwd[PATH_MAX];

int chdir(const char *path)
{
  int err;
  uni_rpc_t rpc;
  iovec_t send[3], recv;

  if ((path == NULL) || (*path == '\0')) {
    errno = EINVAL;
    return -1;
  }
  if (strlen(path) >= PATH_MAX) {
    errno = EINVAL;
    return -1;
  }

  rpc.rpc_call = RPC_SETPWD;
  send[0].iov_base = &rpc;
  send[0].iov_len = sizeof(uni_rpc_t);

  if (make_iovpath(path, &rpc, send + 1, send + 2) == -1) {
    err = errno;
    goto done;
  }

  recv.iov_base = &rpc;
  recv.iov_len = sizeof(rpc);

  err = ipc_iov_iov_send(vfs_channel, send, 3, &recv, 1);
  if (err < 0) {
    err = -err;
    goto done;
  }
  if (rpc.rpc_call != 0) {
    err = rpc.rpc_call;
    goto done;
  }

  memcpy(__cwd, send[2].iov_base, send[2].iov_len);
  __cwd[send[2].iov_len] = '\0';
  err = 0;

done:
  free_iovpath(send + 1, send + 2);
  errno = err;
  return (err == 0) ? 0 : -1;
}

int fchdir(int fildes)
{
  if (!valid_fd(fildes)) {
    errno = EBADF;
    return -1;
  }

  if (tfds[fildes].dir == NULL) {
    errno = ENOTDIR;
    return -1;
  }

  return chdir(tfds[fildes].dir);
}
