/*
 * This library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * (c) Copyright 2006-2009 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2009 Graphov Paul <graphov@jarios.org>
 *
 * deviofs ioctls, used by drivers when registering devices.
 */

#include <limits.h>
#include <sys/types.h>
#include <errno.h>
#include <unistd.h>

#include <ipc/rpc.h>
#include <ipc/iov.h>

#include <devio/devio.h>

#include "__fildesc.h"

int deviofs_assign(int fildes, uint16_t cmd_port, uint16_t io_port, uint16_t dev_type, uint16_t dev_id)
{
  uni_rpc_t rpc;
  int rc;
  
  if (!valid_fd(fildes)) {
    errno = EBADF;
    return -1;
  }

  rpc.rpc_call = RPC_IOCTL;
  rpc.arg0 = DEV_ASSIGN;
  rpc.arg1 = tfds[fildes].ufsid;
  rpc.arg2 = getpid();
  rpc.arg3 = io_port;
  rpc.arg4 = dev_id;
  rpc.arg5 = dev_type;
  rpc.arg6 = cmd_port;

  rc = ipc_channel_msgsnd(tfds[fildes].chid, sizeof(rpc), &rpc, sizeof(rpc), &rpc);
  
  if (rc < 0) {
    errno = -rc;
    return -1;
  }
  
  if (rpc.rpc_call != 0) {
    errno = rpc.rpc_call;
    return -1;
  }

  return 0;
}
