/*
 * This library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * (c) Copyright 2006,2007,2008,2009 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2009 Dmitry Balakshin <balakshin@jarios.org>
 *
 */

#include <limits.h>
#include <sys/types.h>
#include <errno.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdarg.h>

#include <ipc/rpc.h>
#include <ipc/channel.h>

#include "__init.h"
#include "__fildesc.h"

int fcntl(int fildes, int cmd, ...)
{
  va_list ap;
  uni_rpc_t rpc;
  int err, arg, fd;

  if (!valid_fd(fildes)) {
    errno = EBADF;
    return -1;
  }
  switch (cmd) {
  case F_DUPFD:
  case F_SETFD:
  case F_SETFL:
  case F_SETOWN:
    va_start(ap, cmd);
    arg = va_arg(ap, int);
    va_end(ap);
    if ((arg < 0) || (arg >= OPEN_MAX)) {
      errno = EBADF;
      return -1;
    }
    break;
  }

  rpc.rpc_call = RPC_FCNTL;
  rpc.arg0 = cmd;
  rpc.arg1 = fildes;

  switch (cmd) {
  case F_DUPFD:
      fd = take_ae_fd(arg);
      if (fd == -1) {
	errno = EMFILE;
	return -1;
      }
    rpc.arg2 = fd;
    break;
  }

  err = ipc_channel_msgsnd(vfs_channel, sizeof(rpc), &rpc,
          sizeof(rpc), &rpc);
  if (err < 0) {
    errno = -err;
    goto failed;
  }
  if (rpc.rpc_call != 0) {
    errno = rpc.rpc_call;
    goto failed;
  }

  switch (cmd) {
  case F_DUPFD:
    tfds[fd] = tfds[fildes];
    return fd;
  case F_GETFD:
    return rpc.arg0;
  default:
    return 0;
  }


failed:
  switch (cmd) {
  case F_DUPFD:
    release_fd(fd);
    break;
  }

  return -1;
}
