/*
 * This library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * (c) Copyright 2006-2009 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2009 Dmitry Balakshin <balakshin@jarios.org>
 *
 */

#include <limits.h>
#include <sys/types.h>
#include <errno.h>
#include <unistd.h>

#include <ipc/rpc.h>
#include <ipc/channel.h>

#include "__fildesc.h"
#include "__init.h"

int ftruncate(int fildes, off_t length)
{
  int err;
  uni_rpc_t rpc;
  iovec_t send, recv;

  if (!valid_fd(fildes)) {
    errno = EBADF;
    return -1;
  }
  if (length < 0) {
    errno = EINVAL;
    return -1;
  }

  rpc.rpc_call = RPC_FTRUNCATE;
  rpc.arg1 = tfds[fildes].ufsid;
  rpc.offset = length;

  send.iov_base = &rpc;
  send.iov_len = sizeof(uni_rpc_t);

  recv.iov_base = &rpc;
  recv.iov_len = sizeof(uni_rpc_t);

  err = ipc_iov_iov_send(tfds[fildes].chid, &send, 1, &recv, 1);
  if (err < 0) {
    errno = -err;
    return -1;
  }
  if (rpc.rpc_call != 0) {
    errno = rpc.rpc_call;
    return -1;
  }

  return 0;
}
