/*
 * This library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * (c) Copyright 2006,2007,2008,2009 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2009 Dmitry Balakshin <balakshin@jarios.org>
 * (c) Copyright 2009 Alfeiks Kaanoken (MadTirra) <madtirra@jarios.org>
 *
 */

#include <limits.h>
#include <sys/types.h>
#include <errno.h>
#include <string.h>
#include <unistd.h>

#include <ipc/rpc.h>
#include <ipc/iov.h>

#include "__init.h"

char __cwd[PATH_MAX] = {};

int __getcwd(char *buf, size_t size)
{
  uni_rpc_t rpc;
  iovec_t send, recv[2];
  int r;

  rpc.rpc_call = RPC_GETCWD;
  rpc.size = size;

  send.iov_base = &rpc;
  send.iov_len = sizeof(rpc);

  recv[0].iov_base = &rpc;
  recv[0].iov_len = sizeof(rpc);
  recv[1].iov_base = buf;
  recv[1].iov_len = size;

  r = ipc_iov_iov_send(vfs_channel, &send, 1, recv, 2);
  if (r < 0) {
    errno = -r;
    return -1;
  }
  if (rpc.rpc_call != 0) {
    errno = rpc.rpc_call;
    return -1;
  }

  return 0;
}

char *getcwd(char *buf, size_t size)
{
  if ((buf == NULL) || (size == 0)) {
    errno = EINVAL;
    return NULL;
  }

  if (*__cwd == '\0') {
    if (__getcwd(__cwd, PATH_MAX) != 0)
      return NULL;
  }

  strncpy(buf, __cwd, size);
  return buf;
}
