/*
 * This library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * (c) Copyright 2006-2009 Jari OS Core Team <http://jarios.org>
 * Copyright (C) 2009 Dmitry Balakshin <balakshin@jarios.org>
 */

#include <limits.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <errno.h>
#include <unistd.h>

#include <ipc/rpc.h>
#include <ipc/iov.h>

#include "__fildesc.h"

off_t lseek(int fildes, off_t offset, int whence)
{
  struct stat st;

  if (!valid_fd(fildes)) {
    errno = EBADF;
    return (off_t)-1;
  }

  switch (whence) {
  case SEEK_SET:
    if (offset < 0) {
      errno = EINVAL;
      return (off_t)-1;
    }
    tfds[fildes].offset = offset;
    break;
  case SEEK_CUR:
    if (tfds[fildes].offset + offset < 0) {
      errno = EINVAL;
      return (off_t)-1;
    }
    tfds[fildes].offset += offset;
    break;
  case SEEK_END:
    if (fstat(fildes, &st) == -1)
      return (off_t)-1;
    if (st.st_size + offset < 0) {
      errno = EINVAL;
      return (off_t)-1;
    }
    tfds[fildes].offset = st.st_size + offset;
    break;
  default:
    errno = EINVAL;
    return (off_t)-1;
  }

  return tfds[fildes].offset;
}
