/*
 * This library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * (c) Copyright 2006-2009 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2009 Dmitry Balakshin <balakshin@jarios.org>
 *
 */

#include <limits.h>
#include <sys/types.h>
#include <errno.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <ipc/rpc.h>
#include <ipc/iov.h>

#include "__iovpath.h"
#include "__init.h"

/* see general/libc/jarifs/umask.c */
extern mode_t __umask;

int mkdir(const char *path, mode_t mode)
{
  int err;
  uni_rpc_t rpc;
  iovec_t send[3];

  if ((path == NULL) || (*path == '\0')) {
    errno = ENOENT;
    return -1;
  }

  mode &= ~__umask;

  rpc.rpc_call = RPC_CREATEDIR;
  rpc.arg0 = mode;
  send[0].iov_base = &rpc;
  send[0].iov_len = sizeof(rpc);

  if (make_iovpath(path, &rpc, send + 1, send + 2) == -1) {
    err = errno;
    goto done;
  }

  err = ipc_iov_send(vfs_channel, send, 3, &rpc, sizeof(rpc));
  if (err < 0) {
    err = -err;
    goto done;
  }
  if (rpc.rpc_call != 0) {
    err = rpc.rpc_call;
    goto done;
  }

  err = 0;

done:
  free_iovpath(send + 1, send + 2);
  errno = err;
  return (err == 0) ? 0 : -1;
}
