/*
 * This library is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU Lesser General Public License as 
 * published by the Free Software Foundation; either version 2.1 of 
 * the License.
 *
 * This library is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public 
 * License along with this library; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 *
 * (c) Copyright 2006,2007,2008 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2005,2008 MadTirra <madtirra@jarios.org> 
 *
 * mount functions for vfs (user level)
 * 
 */

#include <sys/types.h>
#include <stdlib.h>
#include <stdio.h>
#include <errno.h>
#include <wchar.h>
#include <sys/mount.h>
#include <string.h>
#include <unistd.h>
#include <time.h>

#include <datalib/usrtc.h>
#include <ipc/port.h>
#include <ipc/channel.h>
#include <ipc/rpc.h>
#include <ns/ns.h>
#include <devio/devio.h>

#include "__glob_fs.h"

int mount_s(const char *dev,const char *point,char *fstype,int flags)
{
  char *devno,*_devno;
  char *mpoint,*_mpoint;
  wchar_t *wdevno,*wpoint;
  int ld,lp,e,i;

  if(!dev || !point)
    return -1;

  /* operate with incoming parameters */
  if(*dev==FNSD_S) {
    devno=strdup(dev);
    if(!devno)      return -1;
    _devno=_normalize_path_s(devno);
    if(!_devno){
      free(devno);
      return -1;
    }
    devno=_devno;
  } else 
    devno=(char *)dev;

  mpoint=strdup(point);
  if(!mpoint) {
    free(devno);
    return -1;
  }
  _mpoint=_normalize_path_s(mpoint);
  if(!_mpoint) {
    free(devno);
    free(mpoint);
    return -1;
  }
  mpoint=_mpoint;

  /* vfs didn't should make a converting stuff, task should */
  ld=strlen(devno);
  lp=strlen(mpoint);
  wdevno=malloc(sizeof(wchar_t)*(ld+1));
  wpoint=malloc(sizeof(wchar_t)*(lp+1));
  if(!wdevno || !wpoint) {
    free(devno);
    free(mpoint);
    return -1;
  } else {
    memset(wdevno,0,sizeof(wchar_t)*(ld+1));
    memset(wpoint,0,sizeof(wchar_t)*(lp+1));
    for(i=0;i<ld;i++)
      wdevno[i]=(wchar_t)devno[i];
    for(i=0;i<lp;i++)
      wpoint[i]=(wchar_t)mpoint[i];
    free(devno);
    free(mpoint);
  }

  e=_rmount_w(wdevno,wpoint,fstype,flags);

  return e;
}

int mount_w(const wchar_t *dev,const wchar_t *point,char *fstype,int flags)
{
  /* TODO: implement */
  return -1;
}

