/*
 * This library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * (c) Copyright 2006-2009 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2005,2008,2009 MadTirra <madtirra@jarios.org>
 * Copyright (C) 2009 Dmitry Balakshin <balakshin@jarios.org>
 *
 */

#include <limits.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <errno.h>
#include <fcntl.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>

#include <ipc/rpc.h>
#include <ipc/iov.h>

#include "__fildesc.h"
#include "__init.h"
#include "__iovpath.h"

/* see general/libc/jarifs/umask.c */
extern mode_t __umask;

int open(const char *path, int oflags, ...)
{
  va_list ap;
  mode_t omode;
  int err, fd = -1;
  uni_rpc_t rpc;
  iovec_t send[3];
  struct stat st;

  omode = 0;
  if (oflags & O_CREAT) {
    va_start(ap, oflags);
    omode = va_arg(ap, mode_t);
    va_end(ap);
    omode &= ~__umask;
  }

  fd = take_fd();
  if (fd == -1) {
    err = EMFILE;
    goto error;
  }

  rpc.rpc_call = RPC_OPEN;
  rpc.arg0 = oflags;
  rpc.arg1 = omode;
  rpc.size = fd;
  send[0].iov_base = &rpc;
  send[0].iov_len = sizeof(rpc);

  if (make_iovpath(path, &rpc, send + 1, send + 2) == -1) {
    err = errno;
    goto error;
  }

  err = ipc_iov_send(vfs_channel, send, 3, &rpc, sizeof(rpc));
  if (err < 0) {
    err = -err;
    goto error;
  }
  if (rpc.rpc_call != 0) {
    err = rpc.rpc_call;
    goto error;
  }

  tfds[fd].pid = rpc.arg2;
  tfds[fd].port_id = rpc.arg3;
  tfds[fd].ufsid = rpc.arg4;

  tfds[fd].chid = ipc_channel_open(tfds[fd].pid, tfds[fd].port_id,
                  IPC_BLOCKED_ACCESS);
  if (tfds[fd].chid < 0) {
    err = -tfds[fd].chid;
    goto error;
  }

  if (fstat(fd, &st) == -1) {
    err = errno;
    goto error;
  }

  tfds[fd].offset = 0;
  if ((oflags & O_APPEND) != 0) {
    tfds[fd].offset = st.st_size;
  }

  if (S_ISDIR(st.st_mode)) {
    tfds[fd].dir = calloc(1, send[2].iov_len + 1);
    if (tfds[fd].dir != NULL)
      memcpy(tfds[fd].dir, send[2].iov_base, send[2].iov_len);
  }

  err = 0;
  goto done;

error:
  if (fd != -1) {
    release_fd(fd);
    fd = -1;
  }

done:
  free_iovpath(send + 1, send + 2);
  errno = err;
  return fd;
}
