/*
 * This library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * (c) Copyright 2006-2009 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2009 Dmitry Balakshin <balakshin@jarios.org>
 * A.K. 2009-04-03: fixed bug with incorrect parameters sending <madtirra@jarios.org>
 *
 */

#include <limits.h>
#include <sys/types.h>
#include <errno.h>
#include <unistd.h>

#include <ipc/rpc.h>
#include <ipc/iov.h>

#include "__fildesc.h"

ssize_t pwrite(int fildes, const void *buf, size_t nbyte, off_t offset)
{
  uni_rpc_t rpc;
  iovec_t send[2], recv;
  int rc;

  if (!valid_fd(fildes)) {
    errno = EBADF;
    return -1;
  }

  rpc.rpc_call = RPC_WRITE;
  rpc.arg1 = tfds[fildes].ufsid;
  rpc.size = nbyte;
  rpc.offset = offset;

  send[0].iov_base = &rpc;
  send[0].iov_len = sizeof(uni_rpc_t);
  send[1].iov_base = (void *)buf;
  send[1].iov_len = nbyte;

  recv.iov_base = &rpc;
  recv.iov_len = sizeof(uni_rpc_t);

  rc = ipc_iov_iov_send(tfds[fildes].chid, send, 2, &recv, 1);
  if (rc < 0) {
    errno = -rc;
    return -1;
  }
  if (rpc.rpc_call != 0) {
    errno = rpc.rpc_call;
    return -1;
  }

  return rpc.size;
}
