/*
 * This library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * (c) Copyright 2006-2009 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2009 Dmitry Balakshin <balakshin@jarios.org>
 *
 */

#include <limits.h>
#include <sys/types.h>
#include <errno.h>
#include <string.h>
#include <dirent.h>

#include <ipc/rpc.h>
#include <ipc/iov.h>
#include <sys/v2.h>

#include "__fildesc.h"
#include "__dirstream.h"

static int __readdir(struct __dirstream *ds);

struct dirent *readdir(DIR *dirp)
{
  struct __dirstream *ds = dirp;
  int r;
  struct tr_dirent *p;

  if (ds == NULL) {
    errno = EBADF;
    return NULL;
  }
  if (!valid_fd(ds->fd)) {
    errno = EBADF;
    return NULL;
  }

  if (ds->buf_n == ds->buf_limit) {
    if (!ds->more)
      return NULL;
    r = __readdir(ds);
    if (r <= 0)
      return NULL;
    ds->buf_n = 0;
    ds->buf_limit = r;
    ds->buf_ptr = ds->buf;
  }

  p = (struct tr_dirent*)(ds->buf_ptr);
  ds->buf_ptr += sizeof(struct tr_dirent);

  ds->dd_dir.d_ino = p->d_ino;
  ds->dd_dir.d_off = p->d_off;
  ds->dd_dir.d_namlen = p->d_namlen;
  ds->dd_dir.d_rec_len = p->d_rec_len;

  memset(ds->dd_dir.d_name, 0, sizeof(ds->dd_dir.d_name));
  if (p->d_namlen > sizeof(ds->dd_dir.d_name))
    ds->dd_dir.d_namlen = sizeof(ds->dd_dir.d_name);
  strncpy(ds->dd_dir.d_name, ds->buf_ptr, ds->dd_dir.d_namlen);

  ds->buf_ptr += p->d_namlen;
  ds->buf_n++;

  return &(ds->dd_dir);
}

static int __readdir(struct __dirstream *ds)
{
  uni_rpc_t rpc;
  iovec_t send, recv[2];
  int rc;

begin:

  memset(ds->buf, 0, sizeof(ds->buf));

  rpc.rpc_call = RPC_READDIR;
  rpc.arg0 = tfds[ds->fd].ufsid;
  rpc.size = NR_ENTRIES_IN_STREAM;
  rpc.offset = tfds[ds->fd].offset;

  send.iov_base = &rpc;
  send.iov_len  = sizeof(uni_rpc_t);

  recv[0].iov_base = &rpc;
  recv[0].iov_len  = sizeof(uni_rpc_t);
  recv[1].iov_base = ds->buf;
  recv[1].iov_len  = sizeof(ds->buf);

  rc = ipc_iov_iov_send(tfds[ds->fd].chid, &send, 1, recv, 2);
  if (rc < 0) {
    errno = -rc;
    return -1;
  }
  if (rpc.rpc_call != 0) {
    errno = rpc.rpc_call;
    return -1;
  }

  ds->more = 1;

  if (rpc.size == 0) {
    /* end of directory */
    ds->more = 0;
    return 0;
  }

  if (rpc.arg1 == DIRENT_RR) {
    tfds[ds->fd].offset = 0;
    goto begin;
  }
  if (rpc.arg1 == DIRENT_EL)
    ds->more = 0;

  tfds[ds->fd].offset += rpc.size;
  return rpc.size;
}
