/* This library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * (c) Copyright 2006,2007,2008,2009 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2009 Dmitry Balakshin <balakshin@jarios.org>
 *
 */

#include <limits.h>
#include <sys/types.h>
#include <errno.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>

#include <ipc/rpc.h>
#include <ipc/iov.h>

#include "__init.h"
#include "__iovpath.h"

ssize_t readlink(const char *path, char *buf, size_t bufsize)
{
  uni_rpc_t rpc;
  iovec_t send[3], recv[2];
  int err;
  char *tmpbuf = NULL;

  if ((path == NULL) || (*path == '\0')) {
    errno = ENOENT;
    return -1;
  }
  if (strlen(path) > PATH_MAX) {
    errno = ENAMETOOLONG;
    return -1;
  }
  if ((buf == NULL) && (bufsize != 0)) {
    errno = EINVAL;
    return -1;
  }

  if (buf != NULL)
    memset(buf, 0, bufsize);
  else {
    tmpbuf = calloc(1, PATH_MAX + 1);
    if (tmpbuf == NULL)
      return -1;
  }

  rpc.rpc_call = RPC_READLINK;
  send[0].iov_base = &rpc;
  send[0].iov_len = sizeof(uni_rpc_t);

  if (make_iovpath(path, &rpc, send + 1, send + 2) == -1) {
    err = errno;
    goto failed;
  }

  recv[0].iov_base = &rpc;
  recv[0].iov_len = sizeof(uni_rpc_t);
  if (buf != NULL) {
    recv[1].iov_base = buf;
    recv[1].iov_len = bufsize;
  } else {
    recv[1].iov_base = tmpbuf;
    recv[1].iov_len = PATH_MAX;
  }

  err = ipc_iov_iov_send(vfs_channel, send, 3, recv, 2);
  if (err < 0) {
    err = -err;
    goto failed;
  }
  if (rpc.rpc_call != 0) {
    err = rpc.rpc_call;
    goto failed;
  }

  free_iovpath(send + 1, send + 2);
  if (tmpbuf != NULL)
    free(tmpbuf);

  return recv[1].iov_len;

failed:
  free_iovpath(send + 1, send + 2);
  if (tmpbuf != NULL)
    free(tmpbuf);
  errno = err;
  return -1;
}
