/*
 * This library is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU Lesser General Public License as 
 * published by the Free Software Foundation; either version 2.1 of 
 * the License.
 *
 * This library is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public 
 * License along with this library; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 *
 * Based on newlib implementation of memcpy.
 *                                (c) 2007 Hans Kester  <hans.kester@ellips.nl>
 *                                (c) 2007 Copyright by Ellips BV.
 * (c) Copyright 2006,2007,2008 Jari Core Team <http://jarios.org>
 * (c) Copyright 2005,2008 Tirra <madtirra@jarios.org>
 * (c
 *
 *
 * LibC:
 * platform/amd64/memcpy.S: optimized memcpy for amd64
 *
 */
	
.global memmove
.type memmove STT_FUNC

memmove:

		cmp     %rdi, %rsi
		jb      go_memmove

		call    *memcpy@GOTPCREL(%rip)
		ret

go_memmove:			
				
		movq    %rdi, %rax                /* Store destination in return value */

		leaq		-1(%rsi, %rdx, 1), %rsi
		leaq		-1(%rdi, %rdx, 1), %rdi

    std
	  cmpq    $16, %rdx
	  jb      byte_copy

	  movq    %rdi, %rcx                 /* Align destination on quad word boundary */
	  andq    $7, %rcx
		cmp     $7, %rcx
		je      quadword_aligned
		inc     %rcx
	  subq    %rcx, %rdx
	  rep     movsb

quadword_aligned:
	  cmpq    $256, %rdx
	  jb      quadword_copy

	  movq    %rax, -8  (%rsp)
	  movq    %r12, -16 (%rsp)
	  movq    %r13, -24 (%rsp)
	  movq    %r14, -32 (%rsp)

	  movq    %rdx, %rcx                /* Copy 128 bytes at a time with minimum cache polution */
	  shrq    $7, %rcx

    subq    $127, %rdi
    subq    $127, %rsi
				
	  .p2align 4
loop:
	  prefetchnta   -640 (%rsi)
	  prefetchnta   -704 (%rsi)

	  movq     64 (%rsi), %rax
	  movq     72 (%rsi), %r8
	  movq     80 (%rsi), %r9
	  movq     88 (%rsi), %r10
	  movq     96 (%rsi), %r11
	  movq    104 (%rsi), %r12
	  movq    112 (%rsi), %r13
	  movq    120 (%rsi), %r14

	  movntiq %rax,  64 (%rdi)
	  movntiq %r8 ,  72 (%rdi)
	  movntiq %r9 ,  80 (%rdi)
	  movntiq %r10,  88 (%rdi)
	  movntiq %r11,  96 (%rdi)
	  movntiq %r12, 104 (%rdi)
	  movntiq %r13, 112 (%rdi)
	  movntiq %r14, 120 (%rdi)
				
	  movq       (%rsi), %rax
	  movq     8 (%rsi), %r8
	  movq    16 (%rsi), %r9
	  movq    24 (%rsi), %r10
	  movq    32 (%rsi), %r11
	  movq    40 (%rsi), %r12
	  movq    48 (%rsi), %r13
	  movq    56 (%rsi), %r14

	  movntiq %rax,    (%rdi)
	  movntiq %r8 ,  8 (%rdi)
	  movntiq %r9 , 16 (%rdi)
	  movntiq %r10, 24 (%rdi)
	  movntiq %r11, 32 (%rdi)
	  movntiq %r12, 40 (%rdi)
	  movntiq %r13, 48 (%rdi)
	  movntiq %r14, 56 (%rdi)

	  leaq    -128 (%rsi), %rsi
	  leaq    -128 (%rdi), %rdi

	  dec     %rcx
	  jnz     loop

	  sfence
	  movq    %rdx, %rcx
		addq    $127, %rdi
    addq    $127, %rsi

	  andq    $127, %rcx
	  rep     movsb
	  movq    -8  (%rsp), %rax
	  movq    -16 (%rsp), %r12
	  movq    -24 (%rsp), %r13
	  movq    -32 (%rsp), %r14
		cld
	  ret


byte_copy:
	  movq    %rdx, %rcx
	  rep     movsb
		cld
	  ret

quadword_copy:
		subq		$7, %rdi
		subq    $7, %rsi
	  movq    %rdx, %rcx
	  shrq    $3, %rcx
	  .p2align 4
	  rep     movsq
	  movq    %rdx, %rcx
	  andq    $7, %rcx
		addq    $7, %rdi
		addq    $7, %rsi				
	  rep     movsb                   /* Copy the remaining bytes */
		cld
	  ret

