/*
 * This library is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU Lesser General Public License as 
 * published by the Free Software Foundation; either version 2.1 of 
 * the License.
 *
 * This library is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public 
 * License along with this library; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 *
 * (c) Copyright 2006,2007,2008 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2005,2008 MadTirra <madtirra@jarios.org> 
 *
 * name server related functions
 * 
 */

#include <sys/types.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <time.h>
#include <errno.h>
#include <ipc/port.h>
#include <ipc/channel.h>
#include <ipc/rpc.h>
#include <ns/ns.h>

chid_t nx_ch;
uint8_t nx_init=0;

static void __init_nx_connection(void)
{
  if(nx_init) /* already initied */
    return;

  nx_ch=ipc_channel_open(NX_PID,0,IPC_BLOCKED_ACCESS);
  if(nx_ch<0)
    return;

  nx_init+=1;

  return;
}

int ns_register_port(prtid_t port,char *name)
{
  uni_rpc_t *rpc;
  char *sndbuf;
  int len,r;

  if(!nx_init)
    __init_nx_connection();
  if(!name) {
    errno=EINVAL;
    return -1;
  }
  len=strlen(name)+sizeof(*rpc);
  if((sndbuf=malloc(strlen(name)+sizeof(*rpc)))==NULL) {
    errno=ENOMEM;
    return -1;
  } else memset(sndbuf,0,strlen(name)+sizeof(*rpc));

  rpc=(uni_rpc_t*)sndbuf;
  /* complete rpc call */
  rpc->rpc_call=RPC_IOCTL;
  rpc->arg0=NX_REGPORT;
  rpc->arg1=strlen(name);
  rpc->arg2=port;
  memcpy(sndbuf+sizeof(*rpc),name,strlen(name));

  r=ipc_channel_msgsnd(nx_ch,len,sndbuf,sizeof(*rpc),rpc);
  if(r<0) {
    errno=-r;
    free(sndbuf);
    return -1;
  }
  if(rpc->rpc_call!=0) {
    errno=rpc->rpc_call;
    r=-1;
  } else r=0;

  free(sndbuf);

  return r;
}

int ns_unregister_port(char *name)
{
  uni_rpc_t *rpc;
  char *sndbuf;
  int len,r;

  if(!nx_init)
    __init_nx_connection();
  if(!name) {
    errno=EINVAL;
    return -1;
  }
  len=strlen(name)+sizeof(*rpc);
  if((sndbuf=malloc(strlen(name)+sizeof(*rpc)))==NULL) {
    errno=ENOMEM;
    return -1;
  } else memset(sndbuf,0,strlen(name)+sizeof(*rpc));

  rpc=(uni_rpc_t*)sndbuf;
  /* complete rpc call */
  rpc->rpc_call=RPC_IOCTL;
  rpc->arg0=NX_RMPORT;
  rpc->arg1=strlen(name);
  memcpy(sndbuf+sizeof(*rpc),name,strlen(name));

  r=ipc_channel_msgsnd(nx_ch,len,sndbuf,sizeof(*rpc),rpc);
  if(r<0) {
    errno=-r;
    free(sndbuf);
    return -1;
  }
  if(rpc->rpc_call!=0) {
    errno=rpc->rpc_call;
    r=-1;
  } else r=0;

  free(sndbuf);

  return r;
}

int ns_lookup_portid(pid_t pid,char *name)
{
  uni_rpc_t *rpc;
  char *sndbuf;
  int len,r;

  if(!nx_init)
    __init_nx_connection();
  if(!name) {
    errno=EINVAL;
    return -1;
  }
  len=strlen(name)+sizeof(*rpc);
  if((sndbuf=malloc(strlen(name)+sizeof(*rpc)))==NULL) {
    errno=ENOMEM;
    return -1;
  } else memset(sndbuf,0,strlen(name)+sizeof(*rpc));

  rpc=(uni_rpc_t*)sndbuf;
  /* complete rpc call */
  rpc->rpc_call=RPC_IOCTL;
  rpc->arg0=NX_SRCHPORT;
  rpc->arg1=strlen(name);
  rpc->arg2=pid;
  memcpy(sndbuf+sizeof(*rpc),name,strlen(name));

  r=ipc_channel_msgsnd(nx_ch,len,sndbuf,sizeof(*rpc),rpc);
  if(r<0) {
    errno=-r;
    free(sndbuf);
    return -1;
  }
  if(rpc->rpc_call!=0) {
    errno=rpc->rpc_call;
    r=-1;
  } else r=rpc->arg0;

  free(sndbuf);

  return r;
}

int ns_setattrs(int type,uint8_t subsys)
{
  uni_rpc_t rpc;
  int r;

  if(!nx_init)
    __init_nx_connection();

  rpc.rpc_call=RPC_IOCTL;
  rpc.arg0=NX_SETATTR;
  rpc.arg1=type;
  rpc.arg2=subsys;

  r=ipc_channel_msgsnd(nx_ch,sizeof(rpc),&rpc,sizeof(rpc),&rpc);
  if(r<0) {
    errno=-r;
    return -1;
  }
  if(rpc.rpc_call!=0) {
    errno=rpc.rpc_call;
    return -1;
  }

  return 0;
}

static inline pid_t __nsgetsubsys(int type)
{
  uni_rpc_t rpc;
  int r;

  if(!nx_init)
    __init_nx_connection();

  rpc.rpc_call=RPC_INFO;
  rpc.arg0=type;

  r=ipc_channel_msgsnd(nx_ch,sizeof(rpc),&rpc,sizeof(rpc),&rpc);

  errno=rpc.rpc_call;

  return rpc.arg0;
}

pid_t ns_gettermxpid(void)
{
  return __nsgetsubsys(SYS_TERM);
}

pid_t ns_gethwxpid(void)
{
  return __nsgetsubsys(SYS_DEVIO);
}

pid_t ns_getvfspid(void)
{
  return __nsgetsubsys(SYS_VFS);
}

pid_t ns_getprocpid(void)
{
  return __nsgetsubsys(SYS_PROC);
}

pid_t ns_getnetpid(void)
{
  return __nsgetsubsys(SYS_NET);
}

pid_t ns_getgixpid(void)
{
  return __nsgetsubsys(SYS_GIX);
}

pid_t ns_getfslpid(void)
{
  return __nsgetsubsys(SYS_FSL);
}

int ns_register(pid_t pid,uid_t uid,gid_t gid,int type,int subsystem,
		char *pathname,char **environ,int env_amount)
{
  uni_rpc_t *rpc;
  int r,i,len=0;
  char *hmsg,*epack;

  if(!nx_init)
    __init_nx_connection();

  if((hmsg=malloc(NX_BUFSIZE))==NULL) {
    errno=ENOMEM;
    return -1;
  } else memset(hmsg,0,NX_BUFSIZE);
  /* complete rpc message */
  rpc=(uni_rpc_t*)hmsg;
  rpc->rpc_call=RPC_REGISTER;
  rpc->arg0=pid;
  rpc->arg1=uid;
  rpc->arg2=gid;
  rpc->arg3=type;
  rpc->arg4=subsystem;
  /* lengths */
  if(pathname==NULL) {
    strcat(hmsg+sizeof(uni_rpc_t),"<nil>");
    rpc->arg5=strlen(hmsg);
  } else {
    strcat(hmsg+sizeof(uni_rpc_t),pathname);
    rpc->arg5=strlen(pathname);
  }
  len=sizeof(*rpc)+rpc->arg5;
  /* let's deal with environ */
  if(!env_amount) rpc->arg6=0;
  else {
    epack=hmsg+sizeof(uni_rpc_t)+rpc->arg5;
    for(i=0;i<env_amount;i++) {
      strcpy(epack,environ[i]);
      epack+=strlen(environ[i])+sizeof(char);
      len+=strlen(environ[i])+sizeof(char);
    }
    rpc->arg6=env_amount;
  }

  r=ipc_channel_msgsnd(nx_ch,len,hmsg,sizeof(*rpc),rpc);
  if(r<0) {
    errno=-r;
    free(hmsg);
    return -1;
  }
  if(rpc->rpc_call!=0) {
    errno=rpc->rpc_call;
    r=-1;
  } else r=0;

  free(hmsg);

  return r;
}

int uname(struct utsname *buf)
{
  char *str, *p;
  uni_rpc_t rpc;
  iovec_t send, recv[2];
  int err;

  if (!nx_init)
    __init_nx_connection();

  str = malloc(NX_BUFSIZE);
  if (str == NULL)
    return -1;
  memset(str, 0, NX_BUFSIZE);

  rpc.rpc_call = RPC_UNAME;
  rpc.size = NX_BUFSIZE;

  send.iov_base = &rpc;
  send.iov_len = sizeof(uni_rpc_t);

  recv[0].iov_base = &rpc;
  recv[0].iov_len = sizeof(uni_rpc_t);
  recv[1].iov_base = str;
  recv[1].iov_len = NX_BUFSIZE;

  err = ipc_iov_iov_send(nx_ch, &send, 1, recv, 2);
  if (err < 0) {
    err = -err;
    goto done;
  }
  if (rpc.rpc_call != 0) {
    err = rpc.rpc_call;
    goto done;
  }

  err = 0;

  p = str;
  strncpy(buf->sysname, p, sizeof(buf->sysname));
  p += strlen(p) + 1;
  strncpy(buf->nodename, p, sizeof(buf->nodename));
  p += strlen(p) + 1;
  strncpy(buf->release, p, sizeof(buf->release));
  p += strlen(p) + 1;
  strncpy(buf->version, p, sizeof(buf->version));
  p += strlen(p) + 1;
  strncpy(buf->machine, p, sizeof(buf->machine));
  p += strlen(p) + 1;

done:
  free(str);
  errno = err;
  return (err == 0) ? 0 : -1;
}
