/* This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * (c) Copyright 2006,2007,2008,2009 Jari OS Core Team <http://jarios.org>
 * Copyright (C)2009 Dmitry Balakshin <balakshin@jarios.org>
 *
 * LibC:
 * general/libc/passwd/__passwd.c
 */

#include <sys/types.h>
#include <errno.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <pwd.h>

#include "__passwd.h"

FILE* __passwd_file = NULL;


struct passwd* __alloc_passwd(void)
{
  struct passwd *p;

  p = malloc(sizeof(struct passwd));
  if (p == NULL)
    return NULL;

  p->pw_name = NULL;
  p->pw_uid = (uid_t)-1;
  p->pw_gid = (gid_t)-1;
  p->pw_dir = NULL;
  p->pw_shell = NULL;

  return p;
}

void __free_passwd(struct passwd *p)
{
  if (p != NULL) {
    if (p->pw_name != NULL)
      free(p->pw_name);
    if (p->pw_dir != NULL)
      free(p->pw_dir);
    if (p->pw_shell != NULL)
      free(p->pw_shell);

    free(p);
  }
}

char* __strdup(const char *str, struct buffer *membuf)
{
  char *rc;

  if (membuf == NULL)
    return strdup(str);

  if ((str == NULL) || (membuf->buf == NULL) || (membuf->ofs == membuf->size))
    return NULL;
  if (membuf->ofs + strlen(str) + 1 >= membuf->size)
    return NULL;

  rc = membuf->buf + membuf->ofs;
  memcpy(rc, str, strlen(str));
  membuf->ofs += strlen(str);
  *(membuf->buf + membuf->ofs) = '\0';
  membuf->ofs++;

  return rc;
}

/* Returns:
 *   1 - ok,
 *   0 - invalid string,
 *  -1 - memory allocation error
 */
int __parse_pwent(char *str, struct passwd *pwd, struct buffer *membuf)
{
  char *o, *p;
  int err;

  o = str;

  /* account name */
  p = strchr(o, ':');
  if (p == NULL)
    return 0;
  *p = '\0';
  if (strlen(o) == 0)
    return 0;
  pwd->pw_name = __strdup(o, membuf);
  if (pwd->pw_name == NULL)
    return -1;
  o = p + 1;

  /* password, must be '*' */
  if ((o[0] != '*') || (o[1] != ':'))
    return 0;
  o += 2;

  /* uid */
  p = strchr(o, ':');
  if (p == NULL)
    return 0;
  *p = '\0';
  err = errno;
  errno = 0;
  pwd->pw_uid = (uid_t)strtoul(o, NULL, 10);
  if (errno != 0) {
    errno = err;
    return 0;
  }
  errno = err;
  o = p + 1;

  /* gid */
  p = strchr(o, ':');
  if (p == NULL)
    return 0;
  *p = '\0';
  err = errno;
  errno = 0;
  pwd->pw_gid = (gid_t)strtoul(o, NULL, 10);
  if (errno != 0) {
    errno = err;
    return 0;
  }
  errno = err;
  o = p + 1;

  /* gecos, skip it */
  p = strchr(o, ':');
  if (p == NULL)
    return 0;
  o = p + 1;

  /* home directory */
  p = strchr(o, ':');
  if (p == NULL)
    return 0;
  *p = '\0';
  if (strlen(o) != 0)
    pwd->pw_dir = __strdup(o, membuf);
  else
    pwd->pw_dir = __strdup("", membuf);
  if (pwd->pw_dir == NULL)
    return -1;
  o = p + 1;

  /* shell */
  p = strchr(o, ':');
  if (p != NULL)
    return 0;
  if (strlen(o) != 0)
    pwd->pw_shell = __strdup(o, membuf);
  else
    pwd->pw_shell = __strdup("", membuf);
  if (pwd->pw_shell == NULL)
    return -1;

  return 1;
}
