/* This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * (c) Copyright 2006,2007,2008,2009 Jari OS Core Team <http://jarios.org>
 * Copyright (C)2009 Dmitry Balakshin <balakshin@jarios.org>
 *
 * LibC:
 * general/libc/passwd/pwent.c
 */

#include <limits.h>
#include <sys/types.h>
#include <errno.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <pwd.h>

#include "__passwd.h"

static struct passwd *entry = NULL;

/* Expected /etc/passwd format string is
 *     account:password:UID:GID:GECOS:directory:shell
 * where password MUST be an asterisk (*).
 */

void setpwent(void)
{
  if (__passwd_file == NULL) {
    __passwd_file = fopen(PASSWD_FILE, "r");
    if (__passwd_file == NULL)
      return;
  } else {
    if (fseek(__passwd_file, 0, SEEK_SET) == -1) {
      fclose(__passwd_file);
      __passwd_file = NULL;
      return;
    }
  }
}

void endpwent(void)
{
  if (entry != NULL) {
    __free_passwd(entry);
    entry = NULL;
  }

  if (__passwd_file != NULL) {
    fclose(__passwd_file);
    __passwd_file = NULL;
  }
}

struct passwd* getpwent(void)
{
  char buf[LINE_MAX];
  struct passwd *pwd;
  int i;

  if (__passwd_file == NULL) {
    setpwent();
    if (__passwd_file == NULL)
      return NULL;
  }

  pwd = __alloc_passwd();
  if (pwd == NULL)
    return NULL;

  for (;;) {
    if (pwd->pw_name != NULL) {
      free(pwd->pw_name);
      pwd->pw_name = NULL;
    }
    if (pwd->pw_dir != NULL) {
      free(pwd->pw_dir);
      pwd->pw_dir = NULL;
    }
    if (pwd->pw_shell != NULL) {
      free(pwd->pw_shell);
      pwd->pw_shell = NULL;
    }

    memset(buf, 0, LINE_MAX);
    if (fgets(buf, LINE_MAX, __passwd_file) == NULL) {
      __free_passwd(pwd);
      return NULL;
    }
    if (buf[strlen(buf) - 1] == '\n')
      buf[strlen(buf) - 1] = '\0';

    i = __parse_pwent(buf, pwd, NULL);
    if (i == -1) {
      __free_passwd(pwd);
      return NULL;
    }
    if (i == 1)
      break;
  }

  if (entry != NULL)
    __free_passwd(entry);
  entry = pwd;

  return entry;
}
