/* This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * (c) Copyright 2006,2007,2008,2009 Jari OS Core Team <http://jarios.org>
 * Copyright (C)2009 Dmitry Balakshin <balakshin@jarios.org>
 *
 * LibC:
 * general/libc/passwd/pwnam.c
 */

#include <limits.h>
#include <sys/types.h>
#include <errno.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <pwd.h>

#include "__passwd.h"

static char buf[LINE_MAX];
static struct passwd entry;

struct passwd* getpwnam(const char *name)
{
  int err;
  struct passwd *p = NULL;

  err = getpwnam_r(name, &entry, buf, LINE_MAX, &p);
  if (err != 0)
    errno = err;

  return p;
}

int getpwnam_r(const char *name, struct passwd *pwd, char *buffer,
                size_t bufsize, struct passwd **result)
{
  struct buffer membuf;
  int i;

  setpwent();
  if (__passwd_file == NULL)
    return errno;

  membuf.buf = buffer;
  membuf.size = bufsize;
  membuf.ofs = 0;

  *result = NULL;

  for (;;) {
    memset(buf, 0, LINE_MAX);
    membuf.ofs = 0;

    if (fgets(buf, LINE_MAX, __passwd_file) == NULL) {
      if (feof(__passwd_file))
        break;
      return errno;
    }
    if (buf[strlen(buf) - 1] == '\n')
      buf[strlen(buf) - 1] = '\0';

    i = __parse_pwent(buf, pwd, &membuf);
    if ((i == 1) && (strcmp(pwd->pw_name, name) == 0)) {
      *result = pwd;
      return 0;
    }
    if (i == -1)
      return ERANGE;
  }

  return ENOENT;
}
