/* This library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * (c) Copyright 2006,2007,2008,2009 Jari OS Core Team <http://jarios.org>
 * Copyright (C) 2009 Michael Tsymbalyuk <mtzaurus@gmail.com>
 * Copyright (C) 2009 Dmitry Balakshin <balakshin@jarios.org>
 *
 * libc:
 *   general/libc/proc/fork.c
 *
 * fork() function
 */

#include <sys/types.h>
#include <errno.h>
#include <unistd.h>

#include <ns/ns.h>
#include <ipc/channel.h>
#include <ipc/iov.h>
#include <sys/syscall.h>
#include <sys/procmgr.h>

#include "__procmgr.h"

pid_t fork(void)
{
  pid_t pid;
  pm_cmd_header_t rqh;
  pm_fork_request_t rq;
  iovec_t iov[2];
  int err = 0;

  if (__proc_mgr_chid == (chid_t)-1) {
    open_pm_connection();
    if (__proc_mgr_chid == (chid_t)-1) {
      err = EPIPE;
      goto done;
    }
  }

  err = __syscall0(SC_FORK);
  if (err < 0) {
    err = -err;
    goto done;
  }
  pid = err;

  if (pid == 0)
    /* Child will begin execution from here. */
    return 0;

  rqh.cmd = __PROCMGR_CMD_FORK;
  rq.child_pid = pid;

  iov[0].iov_base = &rqh;
  iov[0].iov_len = sizeof(rqh);
  iov[1].iov_base = &rq;
  iov[1].iov_len = sizeof(rq);

  err = ipc_iov_send(__proc_mgr_chid, iov, 2, &pid, sizeof(pid));
  if (err < 0) {
    err = -err;
    goto done;
  }
  if (pid < 0) {
    err = -pid;
    goto done;
  }

  err = 0;

done:
  if (err) {
    errno = err;
    return -1;
  }

  return pid;
}
