/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * (c) Copyright 2006,2007,2008,2009 MString Core Team <http://mstring.berlios.de>
 * (c) Copyright 2008 Michael Tsymbalyuk <mtzaurus@gmail.com>
 * Copyright (C) 2009 Dmitry Balakshin <balakshin@jarios.org>
 *
 * LibC:
 * signal/signal.c: functions for dealing with POSIX signals.
 */

#include <sys/types.h>
#include <sys/syscall.h>
#include <signal.h>
#include <errno.h>
#include <pthread.h>

sighandler_t signal(int signum,sighandler_t handler)
{
  return (sighandler_t)__syscall2(SC_SIGNAL,
              (syscall_arg_t)signum,(syscall_arg_t)handler);
}

int kill(pid_t pid,int sig)
{
  status_t r=__syscall3(SC_KILL,(syscall_arg_t)pid,
              (syscall_arg_t)sig,(syscall_arg_t)NULL);
  POSIX_ERRNO_RETURN_PATH(r);
}

int sigprocmask(int how,const sigset_t *set,sigset_t *oldset)
{
  status_t r=__syscall3(SC_SIGPROCMASK,(syscall_arg_t)how,
               (syscall_arg_t)set, (syscall_arg_t)oldset);
  POSIX_ERRNO_RETURN_PATH(r);
}

status_t kill_siginfo(pid_t pid,int sig,siginfo_t *siginfo)
{
  return __syscall3(SC_KILL,(syscall_arg_t)pid,(syscall_arg_t)sig,
               (syscall_arg_t)siginfo);
}

status_t thread_kill(pid_t pid,tid_t tid,int sig)
{
  return __syscall3(SC_THREAD_KILL,(syscall_arg_t)pid,
               (syscall_arg_t)tid, (syscall_arg_t)sig);
}

int sigemptyset(sigset_t *set)
{
  *set=0;
  return 0;
}

int sigfillset(sigset_t *set)
{
  *set=(sigset_t)~0;
  return 0;
}

int sigaddset(sigset_t *set,int signum)
{
  if( !valid_signal(signum) ) {
    errno=EINVAL;
    return -1;
  }
  *set |= (1<<signum);
  return 0;
}

int sigdelset(sigset_t *set,int signum)
{
  if( !valid_signal(signum) ) {
    errno=EINVAL;
    return -1;
  }
  *set &= ~(1<<signum);
  return 0;
}

int sigwait(const sigset_t *set,int *sig)
{
  status_t r=__syscall4(SC_SIGWAITINFO,(syscall_arg_t)set,
               (syscall_arg_t)sig,(syscall_arg_t)NULL,
	       (syscall_arg_t)NULL);
  POSIX_ERRNO_RETURN_PATH(r);
}

int sigwaitinfo(const sigset_t *set,siginfo_t *info)
{
  status_t r=__syscall4(SC_SIGWAITINFO,(syscall_arg_t)set,
               (syscall_arg_t)NULL,(syscall_arg_t)info,
	       (syscall_arg_t)NULL);
  POSIX_ERRNO_RETURN_PATH(r);
}

int sigaction(int signum,const struct sigaction *act,struct sigaction *oldact)
{
  int r=__syscall3(SC_SIGACTION,signum,(syscall_arg_t)act,(syscall_arg_t)oldact);
  POSIX_ERRNO_RETURN_PATH(r);
}

int raise(int sig)
{
  // or kill(getpid(), sig)

  pthread_t pth;

  pth.__tid = pthread_self();
  return pthread_kill(pth, sig);
}

int sigtimedwait(const sigset_t *set,siginfo_t *info,const struct timespec *timeout)
{
  int r=__syscall4(SC_SIGWAITINFO,(syscall_arg_t)set,
                   (syscall_arg_t)NULL,(syscall_arg_t)info,
	           (syscall_arg_t)timeout);
  POSIX_ERRNO_RETURN_PATH(r);
}
