/* This library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * (c) Copyright 2006,2007,2008,2009 Jari OS Core Team <http://jarios.org>
 * Copyright (C) 2009 Dmitry Balakshin <balakshin@jarios.org>
 *
 * LibC:
 * general/libc/stdio/vprintf.c
 */

#include <limits.h>
#include <sys/types.h>
#include <errno.h>
#include <stdarg.h>
#include <stdint.h>
#include <stdlib.h>
#include <stdio.h>

int vprintf(const char *format, va_list ap)
{
  return vfprintf(stdout, format, ap);
}

int vfprintf(FILE *stream, const char *format, va_list ap)
{
  int s, rc;
  char *buf;
  va_list ap_copy;

  va_copy(ap_copy, ap);

  s = vsnprintf(NULL, 0, format, ap);
  if (s == -1)
    return -1;

  buf = malloc(s + 1);
  if (buf == NULL)
    return -1;

  rc = vsnprintf(buf, s + 1, format, ap_copy);
  va_end(ap_copy);

  if (rc != s) {
    free(buf);
    errno = ENOMEM;
    return -1;
  }

  rc = fputs(buf, stream);
  if (rc != EOF)
    free(buf);
  else {
    s = errno;
    free(buf);
    errno = s;
  }

  return rc;
}

int vsprintf(char *str, const char *format, va_list ap)
{
  return vsnprintf(str, SIZE_MAX, format, ap);
}
