/*
 * This library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * (c) Copyright 2006,2007,2008,2009 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2008 Michael Tsymbalyuk <sentinel_san@jarios.org>
 * (c) Copyright 2008 MadTirra <madtirra@jarios.org>
 * Copyright (C) 2009 Dmitry Balakshin <balakshin@jarios.org>
 *
 * LibC:
 * lib/exit.c: exit call, atexit functionality
 *
 */

#include <limits.h>
#include <sys/types.h>
#include <errno.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>

#include <sys/syscall.h>

typedef void (*atexit_fn)(void);
static atexit_fn list[ATEXIT_MAX] = {};

void abort(void)
{
  (*(int *)0x666 = errno);

  /* TODO: POSIX'ly correct implementation */
}

void _exit(int status)
{
  fflush(NULL);
  __syscall1(SC_EXIT, status);
}

void exit(int status)
{
  size_t i;

  fflush(NULL);

  for (i = ATEXIT_MAX; i != 0; i--)
    if (list[i - 1] != NULL)
      break;
  if (i != 0) {
    for (; i != 0; i--)
      list[i - 1]();
  }

  __syscall1(SC_EXIT, status);
}

int atexit(void (*func)(void))
{
  size_t i;

  if (func == NULL) {
    errno = EINVAL;
    return -1;
  }

  for (i = 0; i < ATEXIT_MAX; i++)
    if (list[i] == NULL)
      break;
  if (i == ATEXIT_MAX) {
    errno = ENOMEM;
    return -1;
  }

  list[i] = func;

  return 0;
}
