/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * (c) Copyright 2006,2007,2008 Jari OS Core Team <http://jarios.org>
 * Copyright (C) 2009 Dmitry Balakshin <balakshin@jarios.org>
 *
 */

#include <sys/types.h>
#include <errno.h>
#include <wchar.h>
#include <inttypes.h>

intmax_t strtoimax(const char *str, char **endptr, int base)
{
  intmax_t rc = 0, v;
  intmax_t nrc;
  const char *start;
  int neg = 0;
  int consumed = 0;

  if ((str == NULL) || (*str == '\0')) {
    errno = EINVAL;
    return 0;
  }

  start = str;

  if((*str == '+')) {
    str++;
  } else if((*str == '-')) {
    neg = 1;
    str++;
  }

  if (base == 0) {
    if ((*str >= '1') && (*str <= '9'))
      base = 10;
    else if (*str == '0') {
      base = 8;
      str++;
      if ((*str == 'x') || (*str == 'X')) {
        base = 16;
        str++;
      }
    }
    else {
      errno = EINVAL;
      str = start;
      goto error;
    }
  }

  if ((base < 2) || (base > 36)) {
    errno = EINVAL;
    str = start;
    rc = 0;
    goto error;
  }

  do {
    if((*str >= '0') && (*str <= '9')) {
      v = *str - '0';
    } else if((*str >= 'A') && (*str <= 'Z')) {
      v = *str - 'A' + 10;
    } else if((*str >= 'a') && (*str <= 'z')) {
      v = *str - 'a' + 10;
    } else {
      goto out;
    }

    if(v >= base) {
      goto out;
    }

    consumed++;

    nrc = rc * base + v;

    if(nrc < 0) {
      errno = ERANGE;
      rc = INTMAX_MAX;
      goto error;
    } else {
      rc = nrc;
    }

    str++;
  } while (*str != 0);

  out:
  if(!consumed) {
    errno = EINVAL;
    str = start;
    goto error;
  }

  if(neg) {
    rc = -rc;
    if (rc > 0) {
      errno = ERANGE;
      rc = INTMAX_MIN;
    }
  }

  error:
  if (endptr != NULL) {
    *endptr = (char*)str;
  }
  return rc;


}

uintmax_t strtoumax(const char *str, char **endptr, int base)
{
  uintmax_t rc = 0, v;
  uintmax_t nrc;
  const char *start;
  int neg = 0;
  int consumed = 0;

  if ((str == NULL) || (*str == '\0')) {
    errno = EINVAL;
    return 0;
  }

  start = str;

  if((*str == '+')) {
    str++;
  } else if((*str == '-')) {
    neg = 1;
    str++;
  }

  if (base == 0) {
    if ((*str >= '1') && (*str <= '9'))
      base = 10;
    else if (*str == '0') {
      base = 8;
      str++;
      if ((*str == 'x') || (*str == 'X')) {
        base = 16;
        str++;
      }
    }
    else {
      errno = EINVAL;
      str = start;
      goto error;
    }
  }

  if ((base < 2) || (base > 36)) {
    errno = EINVAL;
    str = start;
    rc = 0;
    goto error;
  }

  do {
    if((*str >= '0') && (*str <= '9')) {
      v = *str - '0';
    } else if((*str >= 'A') && (*str <= 'Z')) {
      v = *str - 'A' + 10;
    } else if((*str >= 'a') && (*str <= 'z')) {
      v = *str - 'a' + 10;
    } else {
      goto out;
    }

    if(v >= base) {
      goto out;
    }

    consumed++;

    nrc = rc * base + v;

    if(nrc < rc) {
      errno = ERANGE;
      rc = UINTMAX_MAX;
      goto error;
    } else {
      rc = nrc;
    }

    str++;
  } while (*str != 0);

  out:
  if(!consumed) {
    errno = EINVAL;
    str = start;
    goto error;
  }

  if(neg) {
    rc = ~rc;
  }

  error:
  if (endptr != NULL) {
    *endptr = (char*)str;
  }
  return rc;

}
