/*
 * This library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * (c) Copyright 2006,2007,2008,2009 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2008 Stepan Zastupov <redchrom@jarios.org>
 *
 */
#include <sys/wait.h>
#include <sys/types.h>
#include <signal.h>
#include <unistd.h>
#include <stdlib.h>

int system(const char *command)
{
	pid_t pid;
	int status;
	struct sigaction intr, quit, sa;
	sigset_t omask;

	sa.sa_handler = SIG_IGN;
	sa.sa_flags = 0;
	sigemptyset(&sa.sa_mask);

	if (sigaction(SIGINT, &sa, &intr) < 0)
		return -1;
	if (sigaction(SIGQUIT, &sa, &quit) < 0) {
		status = -1;
		goto restore_sigint;
	}
	sigaddset(&sa.sa_mask, SIGCHLD);
	if (sigprocmask(SIG_BLOCK, &sa.sa_mask, &omask) < 0) {
		status = -1;
		goto restore_sigquit;
	}

	switch ((pid = fork())) {
	case 0:						/* child */
		execlp("/bin/sh", "/bin/sh", "-c", command, (char*)NULL);
		/* Riached on error, i.e. shell not found */
		exit(127);
	case -1:					/* fork fail */
		status = -1;			/* errno should be set by fork */
		goto restore;
	default:
		waitpid(pid, &status, 0);
	}

restore:
	sigprocmask(SIG_SETMASK, &omask, NULL);
restore_sigquit:
	sigaction(SIGQUIT, &quit, NULL);
restore_sigint:
	sigaction(SIGINT, &intr, NULL);

	return status;
}
