/*
 * This library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * (c) Copyright 2006,2007,2008,2009 Jari OS Core Team <http://jarios.org>
 * Copyright (C) 2009 Dmitry Balakshin <balakshin@jarios.org>
 *
 */

#include <sys/types.h>
#include <errno.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>

#include "__stream.h"

int fflush(FILE *stream)
{
  ssize_t wr;

  if (stream == NULL)
    return flush_all_streams();

  if (stream->error)
    return EOF;

  if (stream->out.buf == NULL) {
    stream->ofs += stream->in.pos;
    return 0;
  }
  if (!stream->out.dirty) {
    stream->ofs += stream->out.pos;
    return 0;
  }

  wr = pwrite(stream->fd, stream->out.buf, stream->out.pos,
          stream->ofs);
  if (wr != stream->out.pos) {
    stream->error = 1;
    return EOF;
  }

  stream->out.dirty = 0;
  stream->ofs += stream->out.pos;

  if (stream->in.buf != NULL) {
    memmove(stream->in.buf, stream->in.buf + stream->in.pos,
      stream->in.limit - stream->in.pos);
    stream->in.limit -= stream->in.pos;
    stream->in.pos = 0;
  }
  memmove(stream->out.buf, stream->out.buf + stream->out.pos,
    stream->out.limit - stream->out.pos);
  stream->out.limit -= stream->out.pos;
  stream->out.pos = 0;

  return 0;
}
