/*
 * This library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * (c) Copyright 2006,2007,2008,2009 Jari OS Core Team <http://jarios.org>
 * Copyright (C) 2009 Dmitry Balakshin <balakshin@jarios.org>
 *
 */

#include <limits.h>
#include <sys/types.h>
#include <errno.h>
#include <unistd.h>
#include <stdio.h>

#include "__stream.h"

int fgetc(FILE *stream)
{
  int c;

  if (stream == NULL) {
    errno = EBADF;
    return EOF;
  }
  if (stream->eof || stream->error)
    return EOF;
  if (stream->in.buf == NULL) {
    errno = EBADF;
    return EOF;
  }

  if (stream->in.pos == stream->in.limit) {
    if (fflush(stream) == EOF)
      return EOF;
    if (fill_stream(stream) == -1)
      return EOF;
    if (stream->eof)
      return EOF;
  }

  c = stream->in.buf[stream->in.pos++];
  if (stream->out.buf != NULL)
    stream->out.pos++;

  return c;
}

int getc(FILE *stream)
{
  return fgetc(stream);
}

int getchar(void)
{
  return fgetc(stdin);
}
