/*
 * This library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * (c) Copyright 2006,2007,2008,2009 Jari OS Core Team <http://jarios.org>
 * Copyright (C) 2009 Dmitry Balakshin <balakshin@jarios.org>
 *
 */

#include <limits.h>
#include <sys/types.h>
#include <errno.h>
#include <unistd.h>
#include <stdio.h>

#include "__stream.h"

char *fgets(char *s, int n, FILE *stream)
{
  char *ptr = s;

  if (stream == NULL) {
    errno = EBADF;
    return NULL;
  }
  if ((stream->fd < 0) || (stream->fd >= OPEN_MAX)) {
    errno = EBADF;
    return NULL;
  }
  if ((s == NULL) || (n <= 0)){
    errno = EINVAL;
    return NULL;
  }
  if (feof(stream) || ferror(stream))
    return NULL;

  for ( ; n != 0; n--) {
    *ptr = fgetc(stream);
    if (ferror(stream)) {
      *ptr = '\0';
      return NULL;
    }
    if (feof(stream)) {
      *ptr = '\0';
      return NULL;
    }
    if (*ptr == '\n')
      break;

    ptr++;
  }

  ptr++;
  *ptr = '\0';
  return s;
}

char *gets(char *s)
{
  char *ptr = s;

  if (s == NULL) {
    errno = EINVAL;
    return NULL;
  }
  if (feof(stdin) || ferror(stdin))
    return NULL;

  for (;;) {
    *ptr = fgetc(stdin);
    if (ferror(stdin)) {
      *ptr = '\0';
      return NULL;
    }
    if (feof(stdin)) {
      *ptr = '\0';
      return NULL;
    }
    if (*ptr == '\n')
      break;

    ptr++;
  }

  if (*ptr != '\n')
    ptr++;
  *ptr = '\0';
  return s;
}
