/*
 * This library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * (c) Copyright 2006,2007,2008,2009 Jari OS Core Team <http://jarios.org>
 * Copyright (C) 2009 Dmitry Balakshin <balakshin@jarios.org>
 *
 */

#include <sys/types.h>
#include <errno.h>
#include <string.h>

char *strerror(int errnum)
{
  switch (errnum) {
  case 0:               return "Success";
  case EPERM:           return "Operation not permitted";
  case ENOENT:          return "No such file or directory";
  case ESRCH:           return "No such process";
  case EINTR:           return "Interrupted system call";
  case EIO:             return "I/O error";
  case ENXIO:           return "No such device or address";
  case E2BIG:           return "Argument list too long";
  case ENOEXEC:         return "Exec format error";
  case EBADF:           return "Bad file number";
  case ECHILD:          return "No child processes";
  case EAGAIN:          return "Try again";
  case ENOMEM:          return "Out of memory";
  case EACCES:          return "Permission denied";
  case EFAULT:          return "Bad address";
  case ENOTBLK:         return "Block device required";
  case EBUSY:           return "Device or resource busy";
  case EEXIST:          return "File exists";
  case EXDEV:           return "Cross-device link";
  case ENODEV:          return "No such device";
  case ENOTDIR:         return "Not a directory";
  case EISDIR:          return "Is a directory";
  case EINVAL:          return "Invalid argument";
  case ENFILE:          return "File table overflow";
  case EMFILE:          return "Too many open files";
  case ENOTTY:          return "Not a typewriter";
  case ETXTBSY:         return "Text file busy";
  case EFBIG:           return "File too large";
  case ENOSPC:          return "No space left on device";
  case ESPIPE:          return "Illegal seek";
  case EROFS:           return "Read-only file system";
  case EMLINK:          return "Too many links";
  case EPIPE:           return "Broken pipe";
  case EDOM:            return "Math argument out of domain of func";
  case ERANGE:          return "Math result not representable";

  case ENOSYS:          return "Not implemented system functionality";
  case EWOULDBLOCK:     return "Call would block";
  case EDEADLK:         return "Resource deadlock would occur";

  case ENOSLOT:         return "No free slot found to mount";
  case EPOSTPONED:      return "Call was postponed";

  case ENAMETOOLONG:    return "File or device name is too long";
  case ENOLCK:          return "No locks available";

  case ENOTEMPTY:       return "Directory not empty";
  case ELOOP:           return "Too many levels of symbolic links";
  case ENOMSG:          return "No message of the desired type";
  case EIDRM:           return "Identifier removed";

  case ENOSTR:          return "Not a STREAM";
  case ENODATA:         return "No message is available on the STREAM head read queue";
  case ETIME:           return "Stream ioctl() timeout";
  case ENOSR:           return "No STREAM resources";
  case EPROTO:          return "Protocol error";
  case EBADMSG:         return "Bad message";
  case EOVERFLOW:       return "Value too large to be stored in data type";
  case EILSEQ:          return "Bad character sequence";
  case ENOTSOCK:        return "Not a socket";
  case EDESTADDRREQ:    return "Destination address required";
  case EMSGSIZE:        return "Message too large";
  case EPROTOTYPE:      return "Protocol wrong type for socket";
  case ENOPROTOOPT:     return "Protocol not available";
  case EPROTONOSUPPORT: return "Protocol not supported";
  case EOPNOTSUPP:      return "Operation not supported on socket";
  case ENOTSUP:         return "Not supported";
  case EAFNOSUPPORT:    return "Address family not supported";
  case EADDRINUSE:      return "Address in use";
  case EADDRNOTAVAIL:   return "Address not available";
  case ENETDOWN:        return "Network is down";
  case ENETUNREACH:     return "Network unreachable";
  case ENETRESET:       return "Connection aborted by network";
  case ECONNABORTED:    return "Connection aborted";
  case ECONNRESET:      return "Connection reset";
  case ENOBUFS:         return "No buffer space available";
  case EISCONN:         return "Socket is connected";
  case ENOTCONN:        return "The socket is not connected";
  case ESHUTDOWN:       return "Cannot send after endpoint shutdown";
  case ETIMEDOUT:       return "Connection timed out";
  case ECONNREFUSED:    return "Connection refused";
  case EHOSTUNREACH:    return "Host is unreachable";
  case EALREADY:        return "Connection already in progress";
  case EINPROGRESS:     return "Operation in progress";
  case ECANCELED:       return "Operation canceled";
  case ENOINFO:         return "No info found";

  case ENOLINK:
  case EMULTIHOP:
  case ESTALE:
  case EDQUOT:          return "<reserved>";

  default:
    errno = EINVAL;
    return "";
  }
}

int strerror_r(int errnum, char *buf, size_t buflen)
{
  char *s;

  if ((buf == NULL) || (buflen == 0))
    return ERANGE;

  s = strerror(errnum);
  if (*s == '\0')
    return EINVAL;

  if (strlen(s) + 1 > buflen)
    return ERANGE;

  strcpy(buf, s);
  return 0;
}
