/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * (c) Copyright 2006,2007,2008 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2008 Michael Tsymbaluck <sentinel_san@jarios.org>
 * (c) Copyright 2009 Dmitry Gromada <gromada@jarios.org>
 *
 * LibOS:
 * syslib/misc.c: Miscellaneous system functions.
 */

#include <sys/types.h>
#include <sys/swks.h>
#include <sys/kcontrol.h>
#include <string.h>
#include <stdarg.h>
#include <stdio.h>

static swks_t *__swks=NULL;

#if BITS_PER_LONG == 64

static inline uint64_t __get_sys_ticks(swks_t *swks)
{
  return swks->system_clock_ticks;
}

#elif BITS_PER_LONG == 32

static inline uint64_t __get_sys_ticks(void)
{
  /* yet dummy */
  return 0;
}

#else
#error "The current_sys_ticks function is not implemented " \
       "on the current architecture!"
#endif

swks_t *get_swks(void)
{
  if( !__swks ) {
    kcontrol_args_t ka;
    int n[]={KCTRL_KERNEL,KCTRL_SYSTEM_INFO,KCTRL_SWKS_ADDR};

    memset(&ka,0,sizeof(ka));
    ka.name=n;
    ka.name_len=3;
    ka.old_data=&__swks;

    kcontrol(&ka);
  }

  return __swks;
}

uint64_t current_sys_ticks(swks_t *swks)
{
  return __get_sys_ticks(swks);
}

int klog(const char *fmt, ...)
{
  kcontrol_args_t ka;
  char buf[KLOG_MAX_SIZE];
  int n[]={KCTRL_DEBUG,KCTRL_DEBUG_ECHO};
  va_list ap;

  va_start(ap,fmt);
  vsprintf(buf,fmt,ap);

  memset(&ka,0,sizeof(ka));
  ka.name=n;
  ka.name_len=2;
  ka.new_data=(void *)buf;
  ka.new_data_size=strlen(buf)+1;
  va_end(ap);

  return kcontrol(&ka);
}
