/*
 * This library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * (c) Copyright 2006,2007,2008 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2008 MadTirra <madtirra@jarios.org>
 *
 * LibOS:
 * lib/proc.c: process related system functions
 *
 */

#include <sys/types.h>
#include <sys/syscall.h>
#include <sys/scheduler.h>
#include <unistd.h>
#include <sys/proc.h>
#include <errno.h>

/**
 * @func getpid()
 * 
 * return the caller's pid
 */
pid_t getpid(void)
{
  return task_ctl(0, 0, SYS_PR_CTL_GETPID, 0);
}

pid_t getppid(void)
{
  return task_ctl(0, 0, SYS_PR_CTL_GETPPID, 0);
}

tid_t gettid(void)
{
  return task_ctl(0, 0, SYS_PR_CTL_GETTID, 0);
}

status_t create_task(uint32_t flags)
{
  return __syscall1(SC_CREATE_TASK,flags);
}

status_t task_ctl(pid_t pid, tid_t tid, uint64_t cmd,uint64_t args)
{
  long r=__syscall4(SC_TASK_CONTROL,pid,tid,cmd,args);

  if( r < 0 ) {
    errno=-r;
    return -1;
  }
  return r;
}

long get_task_details(pid_t pid,task_details_t *info)
{
  info->pid=pid;
  info->ppid=10;
  info->state=TASK_STATE_JUST_BORN;
  return 0;
}

int get_task_uidgid(pid_t pid,uidgid_t *uidgid)
{
  long r=__syscall4(SC_TASK_CONTROL,pid,0,SYS_PR_CTL_GET_UIDGID,
                    (syscall_arg_t)uidgid);

  if( r < 0 ) {
    errno=-r;
    return -1;
  }
  return r;
}

int set_task_uidgid(pid_t pid,uidgid_t *uidgid)
{
  long r=__syscall4(SC_TASK_CONTROL,pid,0,SYS_PR_CTL_SET_UIDGID,
                    (syscall_arg_t)uidgid);

  if( r < 0 ) {
    errno=-r;
    return -1;
  }
  return r;
}
