/*
 * This library is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU Lesser General Public License as 
 * published by the Free Software Foundation; either version 2.1 of 
 * the License.
 *
 * This library is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public 
 * License along with this library; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 *
 * (c) Copyright 2006,2007,2008 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2008 Michael Tsymbalyuk <mtzaurus@gmail.com>
 *
 * libsys/uevent.c: functions related to raw kernel events.
 *
 */

#include <sys/types.h>
#include <sys/uevent.h>
#include <errno.h>
#include <sys/syscall.h>
#include <pthread.h>

status_t event_init(event_t *e)
{
  if( !e ) {
    return -EINVAL;
  }
  return __syscall4(SC_SYNC_CREATE,__SO_RAWEVENT,
                    (syscall_arg_t)e,0,0);
}

status_t event_destroy(event_t *e)
{
  status_t r;

  if( !e ) {
    return -EINVAL;
  }

  r=SYNC_DESTROY(e);
  if( !r ) {
    e->__id=__SO_UNINITIALIZED_ID;
  }
  return r;
}

status_t event_wait(event_t *e)
{
  if( !e || e->__id == __SO_UNINITIALIZED_ID ) {
    return -EINVAL;
  }
  return SYNC_CTRL(e->__id,__SYNC_CMD_EVENT_WAIT,0);
}

status_t event_signal(event_t *e)
{
  if( !e || e->__id == __SO_UNINITIALIZED_ID ) {
    return -EINVAL;
  }
  return SYNC_CTRL(e->__id,__SYNC_CMD_EVENT_SIGNAL,0);
}
