/*
 * This library is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU Lesser General Public License as 
 * published by the Free Software Foundation; either version 2.1 of 
 * the License.
 *
 * This library is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public 
 * License along with this library; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 *
 * (c) Copyright 2006,2007,2008 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2008 Michael Tsymbalyuk <mtzaurus@gmail.com>
 *
 * LibOS:
 * lib/time.c: Sleep functions.
 *
 */

#include <sys/types.h>
#include <sys/syscall.h>
#include <errno.h>
#include <time.h>
#include <unistd.h>
#include <string.h>
#include <mach/amd64/atomic.h>

static timer_t __alarm_timer = -1;

int nanosleep(const struct timespec *rqtp, struct timespec *rmtp)
{
  return __syscall2(SC_NANOSLEEP,(ulong_t)rqtp,(ulong_t)rmtp);
}

unsigned int sleep(unsigned int secs)
{
  timeval_t tv;
  int r;
  
  tv.tv_sec = secs;
  tv.tv_nsec = 0;
  if((r=nanosleep(&tv,&tv))==0) 
    return 0;
  if(r==EINTR)
    return tv.tv_sec;
    
  return -1;
}

int usleep(ulong_t usecs)
{
  timeval_t tv;
  int r;
  
  tv.tv_sec=usecs/1000000;
  tv.tv_nsec=(usecs%1000000)*1000;
  if((r=nanosleep(&tv,&tv))==0) 
    return 0;
  if(r==EINTR)
    return tv.tv_sec;


  return -1;
}

unsigned int alarm(unsigned int seconds)
{
  if( __alarm_timer == -1 ) {
    struct sigevent ev;

    memset(&ev,0,sizeof(ev));
    ev.sigev_notify = SIGEV_SIGNAL;
    ev.sigev_signo = SIGALRM;

    /* Currently alarm() implemented via POSIX timers but it will be later
     * move in the kernel to its own interval timers API.
     */
    timer_create(CLOCK_REALTIME,&ev,&__alarm_timer);
  }
  if( __alarm_timer != -1 ) {
    struct itimerspec v1,v2;

    v1.it_value.tv_sec = seconds;
    v1.it_value.tv_nsec = 0;
    v1.it_interval.tv_sec = 0;
    v1.it_interval.tv_nsec = 0;
    if( !timer_settime(__alarm_timer,0,&v1,&v2) ) {
      return v2.it_value.tv_sec;
    }
  }
  return 0;
}
