/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * (c) Copyright 2006,2007,2008 MString Core Team <http://mstring.berlios.de>
 * (c) Copyright 2008 Michael Tsymbalyuk <mtzaurus@gmail.com>
 *
 * LibC:
 * time/timer.c: implementation of POSIX timers-related function.
 */

#include <sys/types.h>
#include <sys/syscall.h>
#include <signal.h>
#include <time.h>
#include <errno.h>
#include <pthread.h>

#define __POSIX_TIMER_SETTIME 0
#define __POSIX_TIMER_GETTIME 1
#define __POSIX_TIMER_GETOVERRUN 2

int timer_create(clockid_t clockid,struct sigevent *evp,timer_t *timerid)
{
  long res=__syscall3(SC_TIMER_CREATE,clockid,(syscall_arg_t)evp,
                      (syscall_arg_t)timerid);
  __SYSCALL_RETURN_ERR_PATH(res);
}

int timer_settime(timer_t timerid,int flags,const struct itimerspec *value,
                  struct itimerspec *ovalue)
{
  long res=__syscall5(SC_TIMER_CONTROL,timerid,__POSIX_TIMER_SETTIME,flags,
                      (syscall_arg_t)value,(syscall_arg_t)ovalue);
  __SYSCALL_RETURN_ERR_PATH(res);
}

int timer_gettime(timer_t timerid,struct itimerspec *value)
{
  long res=__syscall5(SC_TIMER_CONTROL,timerid,__POSIX_TIMER_GETTIME,
                      (syscall_arg_t)value,0,0);
  __SYSCALL_RETURN_ERR_PATH(res);
}

int timer_getoverrun(timer_t timerid)
{
  long res=__syscall5(SC_TIMER_CONTROL,timerid,__POSIX_TIMER_GETOVERRUN,
                      0,0,0);
  __SYSCALL_RETURN_ERR_PATH(res);
}
