/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * (c) Copyright 2006,2007,2008 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2008 Paul Graphov <graphov@jarios.org>
 *:
 * circular.c: circullar buffer implementation, one element is always
 * unused to differentiate between full and empty buffer
 *
 */

#include <sys/types.h>
#include <datalib/circular.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>

status_t circular_init(circular_t *circ, size_t size)
{
  circ->data = (char *)malloc(size + 1);
  if(!circ->data) {
    return -ENOMEM;
  }
  
  circ->size = size+1;
  circ->write_pos = 0;
  circ->read_pos = 0;

  return 0;
}

void circular_destroy(circular_t *circ)
{
  free(circ->data);
}

size_t circular_read(circular_t *circ, char *buf, size_t size)
{
  size_t start = circ->read_pos;
  size_t end = circ->write_pos;
  size_t len;

  len = (circ->size + end - start) % circ->size;
  if(len != 0) {
    if(len > size) {
      end = (start + size) % circ->size;
      len = size;
    }
    if(start>end) {
      memcpy(buf, &circ->data[start],(circ->size - start));
      memcpy(&buf[circ->size - start], circ->data, end);
    } else {
      memcpy(buf, &circ->data[start], (end - start));
    }
    circ->read_pos = end;
  } else {

    len = 0;
  }
  return len;
}

size_t circular_write(circular_t *circ, char *buf, size_t size)
{
  size_t start = circ->write_pos;
  size_t end = (circ->size + circ->read_pos-1) % circ->size;
  size_t len;
  
  len = (circ->size + end - start) % circ->size;
  if(len != 0) {
    if(len > size) {
      end = (start + size) % circ->size;
      len = size;
    }
    if(start>end) {
      memcpy(&circ->data[start], buf, circ->size - start);
      memcpy(circ->data, &buf[circ->size - start], end);
    } else {
      memcpy(&circ->data[start], buf, end - start);
    }
    circ->write_pos = end;
  } else {
    len = 0;
  }
  return len;
}

void circular_flush(circular_t *circ)
{
  circ->read_pos = circ->write_pos;
}



