/*
 * This library is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU Lesser General Public License as 
 * published by the Free Software Foundation; either version 2.1 of 
 * the License.
 *
 * This library is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public 
 * License along with this library; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 *
 * (c) Copyright 2008 Jari OS core team <http://jarios.org>
 * (c) Copyright Paul Graphov <graphov@jarios.org>
 *
 */

#include <sys/types.h>
#include <wchar.h>
#include <string.h>
#include <errno.h>

static const unsigned char table[] =
{
  0,1,2,2,3,3,3,3,4,4,4,4,4,4,4,4,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
  6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,
  7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
  7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
  8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,
  8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,
  8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,
  8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8
};

static int fls(int i) {
  int n = 4;
  while(n-- > 0) {
    int m = table[(i>>(n*8)) & 0xFF];
    if(m) return m+n*8;
  }
  return 0;
}

int wcntomb(char *s, wchar_t wc, int n)
{
  int fbit, len, cnt;

  if(!s) return 0;

  if(!n) return -2;

  fbit = fls(wc);

  if(fbit <= 7) {
    *s = (char)wc;
    return 1;
  } else if(fbit <= 31) {
    len = (fbit-2)/5;
  } else {
    return -1;
  }

  if(n < len+1) return -2;
  
  for(cnt = 0; cnt < len; cnt++) {
    s[len-cnt] = (wc & 0x3F) | 0x80;
    wc >>= 6;
  }
  
  *s = (char)wc | (0xFF << (7 - len));
  return len+1;
}

size_t wcrtomb(char *s, wchar_t wc, mbstate_t *ps)
{
  int r = wcntomb(s, wc, 6);
  if(r == -1) {
    errno = EILSEQ;
    return (size_t)-1;
  }
  return (size_t)r;
}

size_t wcsrtombs(char *dest, const wchar_t **src,  size_t n, mbstate_t *ps)
{
  int pos = 0;
  int ret;
  char fdest[6];
  char *destp;

  if(!dest) {
    destp = &fdest[0];
    n = 6;
  } else destp = dest;
  
  do {
    ret = wcntomb(&destp[pos], **src, n);
    if(ret == -2) { // not enough room
      return pos;
    }
    if(ret == -1) {
      errno = EILSEQ;
      return -1;
    }
    if(destp[pos] == 0) {
      *src = NULL;
      return pos;
    }
    (*src)++;
    pos += ret;
    if(dest) n -= ret;
    else destp -= ret;
      
  } while(1);
}

size_t wcstombs(char *dest, const wchar_t *src, size_t n)
{
  int save_errno = errno;
  int ret;
  ret = wcsrtombs(dest, &src, n, NULL);
  if(src == NULL) return ret;
  if(ret == -1) {
    errno = save_errno;
    return -1;
  }
  return ret;
}
