/*
 * This library is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU Lesser General Public License as 
 * published by the Free Software Foundation; either version 2.1 of 
 * the License.
 *
 * This library is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public 
 * License along with this library; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 *
 * (c) Copyright 2006,2007,2008 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2008 MadTirra <madtirra@jarios.org> 
 *
 * LibC:
 * lib/wchar.c: wide char related functions
 *
 */

#include <sys/types.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <wchar.h>
#include <wctype.h>

size_t wcslen(const wchar_t *s)
{
  register const wchar_t *t;

  for(t=s;*t;t++);

  return t-s;
}

wchar_t *wcscpy(wchar_t *s,const wchar_t *d)
{
  register wchar_t *src=s;
  register wchar_t *dest=(wchar_t*)d;

  do {
    *src=*dest++;
  } while(*src++!=0);

  return s;
}

int wcscmp(const wchar_t *s1,const wchar_t *s2)
{
  register wchar_t *s=(wchar_t*)s1;
  register wchar_t *d=(wchar_t*)s2;

  while (*((wchar_t *)s) == *((wchar_t *)d)) {
    if (!*s++) 
      return 0;
    ++d;
  }
  
  return (*((wchar_t *)s) < *((wchar_t *)d)) ? -1 : 1;
}

int wcsncmp(const wchar_t *s1,const wchar_t *s2,int n)
{
  register wchar_t *s=(wchar_t*)s1;
  register wchar_t *d=(wchar_t*)s2;

  while (n && (*((wchar_t *)s) == *((wchar_t *)d))) {
    if (!*s++) 
      return 0;
    ++d;
    --n;
  }
  
  return (n == 0) ? 0 : ((*((wchar_t *)s) < *((wchar_t *)d)) ? -1 : 1);
}

wchar_t *wcschr(const wchar_t *wcs,wchar_t ch)
{
  register wchar_t *t=(wchar_t*)wcs;

  do {
    if(*t==ch)
      return (wchar_t*)t;
  } while(*t++);

  return NULL;
}

wchar_t *wcsrchr(const wchar_t *wcs,wchar_t ch)
{
  register wchar_t *r=NULL;
  register wchar_t *t=(wchar_t*)wcs;

  do {
    if(*t==ch)
      r=t;
  } while(*t++);

  return (wchar_t*) r;
}

wchar_t *wcsdup(const wchar_t *s)
{
  register wchar_t *r=NULL;
  register size_t len=(wcslen((const wchar_t*)s)+1)*sizeof(wchar_t);

  if((r=malloc(len))!=NULL) {
    memcpy(r,s,len);
  }

  return r;
}

wchar_t *wcpcpy(wchar_t *dst,const wchar_t *src)
{
  do {
    *dst=*src++;
  } while(*dst++!=0);

  return dst-1;
}

wchar_t *wcpncpy(wchar_t *dst,const wchar_t *src,size_t n)
{
  wchar_t *d=dst;
  wchar_t *s=(wchar_t *)src;

  while(n--) {
    if ((*d=*dst)!=0) src++;
    ++d;
  }

  return dst+(src-s);
}

wchar_t *wcscat(wchar_t *dst,const wchar_t *src)
{
  register wchar_t *d=dst;
  register int i=0;

  while(*d++);
  --d;
  while((*d=src[i++])!=0) d++;

  return dst;
}

wchar_t *wcsncat(wchar_t *dst,const wchar_t *src,size_t n)
{
  register wchar_t *d=dst;
  register int i=0;

  while(*d++);
  --d;
  while(n-- && (*d=src[i++])!=0) d++;

  *d=0;

  return dst;
}

wchar_t *wcsncpy(wchar_t *s,const wchar_t *d,size_t n)
{
  register wchar_t *src=s;
  register wchar_t *dest=(wchar_t*)d;

  do {
    *src=*dest++;
  } while(n--);

  return s;
}

wchar_t *wcsstr(const wchar_t *haystack, const wchar_t *needle)
{
  register const wchar_t *s = haystack;
  register const wchar_t *p = needle;

  for (;;) {
    if (!*p)
      return (wchar_t*)haystack;
    if (*p == *s) {
      p++;
      s++;
    } else {
      p = needle;
      if (!*s)
        return NULL;
      s = ++haystack;
    }
  }
}

int wcscasecmp(const wchar_t *s1, const wchar_t *s2)
{
  while (towupper(*s1) == towupper(*s2)) {
    if (!*s1++)
      return 0;
    ++s2;
  }

  return (*s1 < *s2) ? -1 : 1;
}

int wcsncasecmp(const wchar_t *s1, const wchar_t *s2, int n)
{
  while (n && (towupper(*s1) == towupper(*s2))) {
    if (!*s1++)
      return 0;
    ++s2;
    --n;
  }

  if (n == 0)
    return 0;

  return (*s1 < *s2) ? -1 : 1;
}
