/*
 * This library is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU Lesser General Public License as 
 * published by the Free Software Foundation; either version 2.1 of 
 * the License.
 *
 * This library is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public 
 * License along with this library; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 *
 * (c) Copyright 2006,2007,2008 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2008 MadTirra <madtirra@jarios.org>
 * (c) Copyright 2008 Patrick Madden <patrick_madden@jarios.org>
 *
 * LibC:
 * lib/ctype.c: simple C types funcs
 *
 */

#include <sys/types.h>
#include <ctype.h>

int isdigit(int c)
{
  return ((c <= '9') && (c >= '0'));
}

int isxdigit(int c)
{
  if(isdigit(c))    return 1;
  if(((c >= 'a') && (c <= 'f')) || 
     ((c >= 'A') && (c <= 'F')))    return 1;

  return 0;
}

int isspace(int c)
{
  switch(c) {
  case ' ':
  case '\f':
  case '\n':
  case '\r':
  case '\t':
  case '\v':
    return 1;
  }

  return 0;
}

int isupper(int c)
{
  return((c >= 'A') && (c <= 'Z'));
}

int islower(int c)
{
  return((c >= 'a') && (c <= 'z'));
}

int isalpha(int c)
{
  return (isupper(c) || islower(c));
}

int isalnum(int c)
{
  return (isalpha(c) || isdigit(c));
}

int isascii(int c)
{
  return ((c >= 0) && (c <= 127));
}

int isblank(int c)
{
  return ((c == ' ') || (c == '\t'));
}

int iscntrl(int c)
{
  return ((c <= 31) || (c == 127));
}

int isprint(int c)
{
  return ((c >= 32) && (c <= 126));
}

int isgraph(int c)
{
  return ((c >= 33) && (c <= 126));
}

int ispunct(int c)
{
  return (isgraph(c) && !isalnum(c));
}

int toupper(int c)
{
  if (islower(c))
    c &= 0xdf;
  return c;
}

int tolower(int c)
{
  if (isupper(c))
    c |= 0x20;
  return c;
}
