/*
 * This library is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU Lesser General Public License as 
 * published by the Free Software Foundation; either version 2.1 of 
 * the License.
 *
 * This library is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public 
 * License along with this library; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 *
 * (c) Copyright 2002 Red Hat Incorporated.
 * (c) Copyright 2006,2007,2008 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2008 MadTirra <madtirra@jarios.org> 
 *
 * LibC:
 * lib/wctype.c: wide char related functions
 *
 */

#include <sys/types.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <wchar.h>
#include <ctype.h>
#include <wctype.h>

#include "utf8alpha.h"
#include "utf8print.h"
#include "utf8punct.h"

int iswalpha(wint_t c)
{
  unsigned const char *table;
  unsigned char *ptr;
  unsigned char ctmp;
  int size;
  wint_t x = (c >> 8);
  
  /* for some large sections, all characters are alphabetic so handle them here */
  if ((x >= 0x34 && x <= 0x4c) ||
      (x >= 0x4e && x <= 0x9e) ||
      (x >= 0xac && x <= 0xd6) ||
      (x >= 0x200 && x <= 0x2a5))
    return 1;
  
  switch (x)
    {
    case 0x00:
      table = a_u0;
      size = sizeof(u0);
      break;
    case 0x01:
    case 0x15:
    case 0xa0:
    case 0xa1:
    case 0xa2:
    case 0xa3:
    case 0xf9:
    case 0xfc:
    case 0x2f8:
    case 0x2f9:
      return 1;
    case 0x02:
      table = a_u2;
      size = sizeof(a_u2);
      break;
    case 0x03:
      table = a_u3;
      size = sizeof(a_u3);
      break;
    case 0x04:
      table = a_u4;
      size = sizeof(a_u4);
      break;
    case 0x05:
      table = a_u5;
      size = sizeof(a_u5);
      break;
    case 0x06:
      table = a_u6;
      size = sizeof(a_u6);
      break;
    case 0x07:
      table = a_u7;
      size = sizeof(a_u7);
      break;
    case 0x09:
      table = a_u9;
      size = sizeof(a_u9);
      break;
    case 0x0a:
      table = a_ua;
      size = sizeof(a_ua);
      break;
    case 0x0b:
      table = a_ub;
      size = sizeof(a_ub);
      break;
    case 0x0c:
      table = a_uc;
      size = sizeof(a_uc);
      break;
    case 0x0d:
      table = a_ud;
      size = sizeof(a_ud);
      break;
    case 0x0e:
      table = a_ue;
      size = sizeof(a_ue);
      break;
    case 0x0f:
      table = a_uf;
      size = sizeof(a_uf);
      break;
    case 0x10:
      table = a_u10;
      size = sizeof(a_u10);
      break;
    case 0x11:
      table = a_u11;
      size = sizeof(a_u11);
      break;
    case 0x12:
      table = a_u12;
      size = sizeof(a_u12);
      break;
    case 0x13:
      table = a_u13;
      size = sizeof(a_u13);
      break;
    case 0x14:
      table = a_u14;
      size = sizeof(a_u14);
      break;
    case 0x16:
      table = a_u16;
      size = sizeof(a_u16);
      break;
    case 0x17:
      table = a_u17;
      size = sizeof(a_u17);
      break;
    case 0x18:
      table = a_u18;
      size = sizeof(a_u18);
      break;
    case 0x1e:
      table = a_u1e;
      size = sizeof(a_u1e);
      break;
    case 0x1f:
      table = a_u1f;
      size = sizeof(a_u1f);
      break;
    case 0x20:
      table = a_u20;
      size = sizeof(a_u20);
      break;
    case 0x21:
      table = a_u21;
      size = sizeof(a_u21);
      break;
    case 0x24:
      table = a_u24;
      size = sizeof(a_u24);
      break;
    case 0x30:
      table = a_u30;
      size = sizeof(a_u30);
      break;
    case 0x31:
      table = a_u31;
      size = sizeof(a_u31);
      break;
    case 0x4d:
      table = a_u4d;
      size = sizeof(a_u4d);
      break;
    case 0x9f:
      table = a_u9f;
      size = sizeof(a_u9f);
      break;
    case 0xa4:
      table = a_ua4;
      size = sizeof(a_ua4);
      break;
    case 0xd7:
      table = a_ud7;
      size = sizeof(a_ud7);
      break;
    case 0xfa:
      table = a_ufa;
      size = sizeof(a_ufa);
      break;
    case 0xfb:
      table = a_ufb;
      size = sizeof(a_ufb);
      break;
    case 0xfd:
      table = a_ufd;
      size = sizeof(a_ufd);
      break;
    case 0xfe:
      table = a_ufe;
      size = sizeof(a_ufe);
      break;
    case 0xff:
      table = a_uff;
      size = sizeof(a_uff);
      break;
    case 0x103:
      table = a_u103;
      size = sizeof(a_u103);
      break;
    case 0x104:
      table = a_u104;
      size = sizeof(a_u104);
      break;
    case 0x1d4:
      table = a_u1d4;
      size = sizeof(a_u1d4);
      break;
    case 0x1d5:
      table = a_u1d5;
      size = sizeof(a_u1d5);
      break;
    case 0x1d6:
      table = a_u1d6;
      size = sizeof(a_u1d6);
      break;
    case 0x1d7:
      table = a_u1d7;
      size = sizeof(a_u1d7);
      break;
    case 0x2a6:
      table = a_u2a6;
      size = sizeof(a_u2a6);
      break;
    case 0x2fa:
      table = a_u2fa;
      size = sizeof(a_u2fa);
      break;
    default:
      return 0;
    }
  /* we have narrowed down to a section of 256 characters to check */
  /* now check if c matches the alphabetic wide-chars within that section */
  ptr = (unsigned char *)table;
  ctmp = (unsigned char)c;
  while (ptr < table + size)    {
    if (ctmp == *ptr)
      return 1;
    if (ctmp < *ptr)
      return 0;
    /* otherwise c > *ptr */
    /* look for 0x0 as next element which indicates a range */
    ++ptr;
    if (*ptr == 0x0)	{
      /* we have a range..see if c falls within range */
      ++ptr;
      if (ctmp <= *ptr)
	return 1;
      ++ptr;
    }
  }
  /* not in table */
  return 0;
}

int iswdigit(wint_t c)
{
  return (c >= (wint_t)'0' && c <= (wint_t)'9');
}

int iswalphanum(wint_t c)
{
  return (iswalpha (c) || iswdigit (c));
}

int iswblank(wint_t c)
{
  return (c == 0x0009 || c == 0x0020 || c == 0x1680 ||
	  (c >= 0x2000 && c <= 0x2006) ||
	  (c >= 0x2008 && c <= 0x200b) ||
	  c == 0x205f || c == 0x3000);
}

int iswcntrl(wint_t c)
{
  return ((c >= 0x0000 && c <= 0x001f) || 
	  (c >= 0x007f && c <= 0x009f) ||
	  c == 0x2028 || c == 0x2029);
}

int iswprint(wint_t c)
{
  unsigned const char *table;
  unsigned char *ptr;
  unsigned char ctmp;
  int size;
  wint_t x = (c >> 8);
  
  /* for some large sections, all characters are printuation so handle them here */
  if ((x >= 0x34 && x <= 0x4c) ||
      (x >= 0x4e && x <= 0x9e) ||
      (x >= 0xac && x <= 0xd6) ||
      (x >= 0xe0 && x <= 0xf9) ||
      (x >= 0x200 && x <= 0x2a5) ||
      (x >= 0xf00 && x <= 0xffe) ||
      (x >= 0x1000 && x <= 0x10fe))
    return 1;
  
  switch (x)
    {
    case 0x01:
    case 0x15:
    case 0x22:
    case 0x25:
    case 0x28:
    case 0x29:
    case 0x2a:
    case 0xa0:
    case 0xa1:
    case 0xa2:
    case 0xa3:
    case 0xfc:
    case 0x2f8:
    case 0x2f9:
      return 1;
    case 0x00:
      table = p_u0;
      size = sizeof(p_u0);
      break;
    case 0x02:
      table = p_u2;
      size = sizeof(p_u2);
      break;
    case 0x03:
      table = p_u3;
      size = sizeof(p_u3);
      break;
    case 0x04:
      table = p_u4;
      size = sizeof(p_u4);
      break;
    case 0x05:
      table = p_u5;
      size = sizeof(p_u5);
      break;
    case 0x06:
      table = p_u6;
      size = sizeof(p_u6);
      break;
    case 0x07:
      table = p_u7;
      size = sizeof(p_u7);
      break;
    case 0x09:
      table = p_u9;
      size = sizeof(p_u9);
      break;
    case 0x0a:
      table = p_ua;
      size = sizeof(p_ua);
      break;
    case 0x0b:
      table = p_ub;
      size = sizeof(p_ub);
      break;
    case 0x0c:
      table = p_uc;
      size = sizeof(p_uc);
      break;
    case 0x0d:
      table = p_ud;
      size = sizeof(p_ud);
      break;
    case 0x0e:
      table = p_ue;
      size = sizeof(p_ue);
      break;
    case 0x0f:
      table = p_uf;
      size = sizeof(p_uf);
      break;
    case 0x10:
      table = p_u10;
      size = sizeof(p_u10);
      break;
    case 0x11:
      table = p_u11;
      size = sizeof(p_u11);
      break;
    case 0x12:
      table = p_u12;
      size = sizeof(p_u12);
      break;
    case 0x13:
      table = p_u13;
      size = sizeof(p_u13);
      break;
    case 0x14:
      table = p_u14;
      size = sizeof(p_u14);
      break;
    case 0x16:
      table = p_u16;
      size = sizeof(p_u16);
      break;
    case 0x17:
      table = p_u17;
      size = sizeof(p_u17);
      break;
    case 0x18:
      table = p_u18;
      size = sizeof(p_u18);
      break;
    case 0x1e:
      table = p_u1e;
      size = sizeof(p_u1e);
      break;
    case 0x1f:
      table = p_u1f;
      size = sizeof(p_u1f);
      break;
    case 0x20:
      table = p_u20;
      size = sizeof(p_u20);
      break;
    case 0x21:
      table = p_u21;
      size = sizeof(p_u21);
      break;
    case 0x23:
      table = p_u23;
      size = sizeof(p_u23);
      break;
    case 0x24:
      table = p_u24;
      size = sizeof(p_u24);
      break;
    case 0x26:
      table = p_u26;
      size = sizeof(p_u26);
      break;
    case 0x27:
      table = p_u27;
      size = sizeof(p_u27);
      break;
    case 0x2e:
      table = p_u2e;
      size = sizeof(p_u2e);
      break;
    case 0x2f:
      table = p_u2f;
      size = sizeof(p_u2f);
      break;
    case 0x30:
      table = p_u30;
      size = sizeof(p_u30);
      break;
    case 0x31:
      table = p_u31;
      size = sizeof(p_u31);
      break;
    case 0x32:
      table = p_u32;
      size = sizeof(p_u32);
      break;
    case 0x33:
      table = p_u33;
      size = sizeof(p_u33);
      break;
    case 0x4d:
      table = p_u4d;
      size = sizeof(p_u4d);
      break;
    case 0x9f:
      table = p_u9f;
      size = sizeof(p_u9f);
      break;
    case 0xa4:
      table = p_ua4;
      size = sizeof(p_ua4);
      break;
    case 0xd7:
      table = p_ud7;
      size = sizeof(p_ud7);
      break;
    case 0xfa:
      table = p_ufa;
      size = sizeof(p_ufa);
      break;
    case 0xfb:
      table = p_ufb;
      size = sizeof(p_ufb);
      break;
    case 0xfd:
      table = p_ufd;
      size = sizeof(p_ufd);
      break;
    case 0xfe:
      table = p_ufe;
      size = sizeof(p_ufe);
      break;
    case 0xff:
      table = p_uff;
      size = sizeof(p_uff);
      break;
    case 0x103:
      table = p_u103;
      size = sizeof(p_u103);
      break;
    case 0x104:
      table = p_u104;
      size = sizeof(p_u104);
      break;
    case 0x1d0:
      table = p_u1d0;
      size = sizeof(p_u1d0);
      break;
    case 0x1d1:
      table = p_u1d1;
      size = sizeof(p_u1d1);
      break;
    case 0x1d4:
      table = p_u1d4;
      size = sizeof(p_u1d4);
      break;
    case 0x1d5:
      table = p_u1d5;
      size = sizeof(p_u1d5);
      break;
    case 0x1d6:
      table = p_u1d6;
      size = sizeof(p_u1d6);
      break;
    case 0x1d7:
      table = p_u1d7;
      size = sizeof(p_u1d7);
      break;
    case 0x2a6:
      table = p_u2a6;
      size = sizeof(p_u2a6);
      break;
    case 0x2fa:
      table = p_u2fa;
      size = sizeof(p_u2fa);
      break;
    case 0xe00:
      table = p_ue00;
      size = sizeof(p_ue00);
      break;
    case 0xfff:
      table = p_ufff;
      size = sizeof(p_ufff);
      break;
    case 0x10ff:
      table = p_u10ff;
      size = sizeof(p_u10ff);
      break;
    default:
      return 0;
    }
  /* we have narrowed down to a section of 256 characters to check */
  /* now check if c matches the printuation wide-chars within that section */
  ptr = (unsigned char *)table;
  ctmp = (unsigned char)c;
  while (ptr < table + size)    {
    if (ctmp == *ptr)
      return 1;
    if (ctmp < *ptr)
      return 0;
    /* otherwise c > *ptr */
    /* look for 0x0 as next element which indicates a range */
    ++ptr;
    if (*ptr == 0x0)	{
      /* we have a range..see if c falls within range */
      ++ptr;
      if (ctmp <= *ptr)
	return 1;
      ++ptr;
    }
  }
  /* not in table */
  return 0;
}

int iswpunct(wint_t c)
{
  unsigned const char *table;
  unsigned char *ptr;
  unsigned char ctmp;
  int size;
  wint_t x = (c >> 8);
  
  /* for some large sections, all characters are punctuation so handle them here */
  if ((x >= 0xe0 && x <= 0xf8) ||
      (x >= 0xf00 && x <= 0xffe) ||
      (x >= 0x1000 && x <= 0x10fe))
    return 1;
  
  switch (x) {
  case 0x22:
  case 0x25:
  case 0x28:
  case 0x29:
  case 0x2a:
    return 1;
  case 0x00:
    table = u0;
    size = sizeof(u0);
    break;
  case 0x02:
    table = u2;
    size = sizeof(u2);
    break;
  case 0x03:
    table = u3;
    size = sizeof(u3);
    break;
  case 0x04:
    table = u4;
    size = sizeof(u4);
    break;
  case 0x05:
    table = u5;
    size = sizeof(u5);
    break;
  case 0x06:
    table = u6;
    size = sizeof(u6);
    break;
  case 0x07:
    table = u7;
    size = sizeof(u7);
    break;
  case 0x09:
    table = u9;
    size = sizeof(u9);
    break;
  case 0x0a:
    table = ua;
    size = sizeof(ua);
    break;
  case 0x0b:
    table = ub;
    size = sizeof(ub);
    break;
  case 0x0c:
    table = uc;
    size = sizeof(uc);
    break;
  case 0x0d:
    table = ud;
    size = sizeof(ud);
    break;
  case 0x0e:
    table = ue;
    size = sizeof(ue);
    break;
  case 0x0f:
    table = uf;
    size = sizeof(uf);
    break;
  case 0x10:
    table = u10;
    size = sizeof(u10);
    break;
  case 0x13:
    table = u13;
    size = sizeof(u13);
    break;
  case 0x16:
    table = u16;
    size = sizeof(u16);
    break;
  case 0x17:
    table = u17;
    size = sizeof(u17);
    break;
  case 0x18:
    table = u18;
    size = sizeof(u18);
    break;
  case 0x1f:
    table = u1f;
    size = sizeof(u1f);
    break;
  case 0x20:
    table = u20;
    size = sizeof(u20);
    break;
  case 0x21:
    table = u21;
    size = sizeof(u21);
    break;
  case 0x23:
    table = u23;
    size = sizeof(u23);
    break;
  case 0x24:
    table = u24;
    size = sizeof(u24);
    break;
  case 0x26:
    table = u26;
    size = sizeof(u26);
    break;
  case 0x27:
    table = u27;
    size = sizeof(u27);
    break;
  case 0x2e:
    table = u2e;
    size = sizeof(u2e);
    break;
  case 0x2f:
    table = u2f;
    size = sizeof(u2f);
    break;
  case 0x30:
    table = u30;
    size = sizeof(u30);
    break;
  case 0x31:
    table = u31;
    size = sizeof(u31);
    break;
  case 0x32:
    table = u32;
    size = sizeof(u32);
    break;
  case 0x33:
    table = u33;
    size = sizeof(u33);
    break;
  case 0xa4:
    table = ua4;
    size = sizeof(ua4);
    break;
  case 0xfb:
    table = ufb;
    size = sizeof(ufb);
    break;
  case 0xfd:
    table = ufd;
    size = sizeof(ufd);
    break;
  case 0xfe:
    table = ufe;
    size = sizeof(ufe);
    break;
  case 0xff:
    table = uff;
    size = sizeof(uff);
    break;
  case 0x103:
    table = u103;
    size = sizeof(u103);
    break;
  case 0x1d0:
    table = u1d0;
    size = sizeof(u1d0);
    break;
  case 0x1d1:
    table = u1d1;
    size = sizeof(u1d1);
    break;
  case 0x1d6:
    table = u1d6;
    size = sizeof(u1d6);
    break;
  case 0x1d7:
    table = u1d7;
    size = sizeof(u1d7);
    break;
  case 0xe00:
    table = ue00;
    size = sizeof(ue00);
    break;
  case 0xfff:
    table = ufff;
    size = sizeof(ufff);
    break;
  case 0x10ff:
    table = u10ff;
    size = sizeof(u10ff);
    break;
  default:
    return 0;
  }
  /* we have narrowed down to a section of 256 characters to check */
  /* now check if c matches the punctuation wide-chars within that section */
  ptr = (unsigned char *)table;
  ctmp = (unsigned char)c;
  while (ptr < table + size)	{
    if (ctmp == *ptr)
      return 1;
    if (ctmp < *ptr)
      return 0;
    /* otherwise c > *ptr */
    /* look for 0x0 as next element which indicates a range */
    ++ptr;
    if (*ptr == 0x0)	    {
      /* we have a range..see if c falls within range */
      ++ptr;
      if (ctmp <= *ptr)
	return 1;
      ++ptr;
    }
  }
  /* not in table */
  return 0;
}

int iswspace(wint_t c)
{
  return ((c >= 0x0009 && c <= 0x000d) || c == 0x0020 || c == 0x1680 ||
	  (c >= 0x2000 && c <= 0x2006) ||
	  (c >= 0x2008 && c <= 0x200b) ||
	  c == 0x2028 || c == 0x2029 ||
	  c == 0x205f || c == 0x3000);
}

int iswgraph(wint_t c)
{
  return (iswprint (c) && !iswspace (c));
}

int iswxdigit(wint_t c)
{
  return ((c >= (wint_t)'0' && c <= (wint_t)'9') ||
	  (c >= (wint_t)'a' && c <= (wint_t)'f') ||
	  (c >= (wint_t)'A' && c <= (wint_t)'F'));
}

int iswlower(wint_t c)
{
  return (towupper (c) != c);
}

int iswupper(wint_t c)
{
  return (towlower (c) != c);
}


