/*
 * This library is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU Lesser General Public License as 
 * published by the Free Software Foundation; either version 2.1 of 
 * the License.
 *
 * This library is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public 
 * License along with this library; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 *
 * (c) Copyright 2006,2007,2008,2009 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2008 Michael Tsymbalyuk <mtzaurus@gmail.com>
 *
 * libpthread/lifecycle.c: functions related to POSIX threads
 *                         (the ones related to thread lifecycle/state management).
 *
 */

#include <sys/types.h>
#include <sys/syscall.h>
#include <pthread.h>
#include <errno.h>
#include <sys/proc.h>

int pthread_join(pthread_t thread,void **value_ptr)
{
  int r=__syscall2(SC_THREAD_WAIT,thread.__tid,(syscall_arg_t)value_ptr);
  return POSIX_ERR(r);
}

void pthread_exit(void *value_ptr)
{
  __syscall1(SC_THREAD_EXIT,(syscall_arg_t)value_ptr);
}

int pthread_cancel(pthread_t thread)
{
  int r;

  r=__syscall3(SC_TASK_CONTROL,thread.__tid,SYS_PR_CTL_CANCEL_TASK,0);
  return POSIX_ERR(r);
}

int pthread_setcanceltype(int type,int *oldtype)
{
  int r;

  if( !oldtype ) {
    return EINVAL;
  }

  if( type == PTHREAD_CANCEL_DEFERRED || type == PTHREAD_CANCEL_ASYNCHRONOUS ) {
    r=__syscall3(SC_TASK_CONTROL,pthread_self(),SYS_PR_CTL_SET_CANCEL_TYPE,type);
    if( r >= 0 ) {
      *oldtype=r;
      r=0;
    } else {
      r=POSIX_ERR(r);
    }
  } else {
    r=EINVAL;
  }
  return r;
}

int pthread_setcancelstate(int state,int *oldstate)
{
  int r;

  if( !oldstate ) {
    return EINVAL;
  }

  if( state == PTHREAD_CANCEL_ENABLE || state == PTHREAD_CANCEL_DISABLE ) {
    r=__syscall3(SC_TASK_CONTROL,pthread_self(),SYS_PR_CTL_SET_CANCEL_STATE,state);
    if( r >= 0 ) {
      *oldstate=r;
      r=0;
    } else {
      r=POSIX_ERR(r);
    }
  } else {
    r=EINVAL;
  }
  return r;
}
