/*
 * This library is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU Lesser General Public License as 
 * published by the Free Software Foundation; either version 2.1 of 
 * the License.
 *
 * This library is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public 
 * License along with this library; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 *
 * (c) Copyright 2006,2007,2008 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2008 Michael Tsymbalyuk <mtzaurus@gmail.com>
 *
 * libpthread/pthreads.c: functions related to POSIX threads.
 *
 */

#include <sys/types.h>
#include <sys/syscall.h>
#include <pthread.h>
#include <errno.h>
#include <bitwise.h>
#include <alloc/imcache.h>
#include <unistd.h>
#include <sys/scheduler.h>
#include <sys/proc.h>
#include <sys/thread.h>
#include <sys/ptd.h>

static void __pthread_gen_entrypoint(void *(*start_routine)(void *),void *arg)
{
  pthread_exit(start_routine(arg));
}

static void __pthread_gen_destructor(void)
{
}

static void __pthread_attrs_to_system_attrs(const pthread_attr_t *attr,
                                            task_creation_attrs_t *attrs)
{
}

int pthread_create(pthread_t *thread,const pthread_attr_t *attr,
                   void *(*start_routine)(void *),void *arg)
{
  status_t r,pid;
  void *stack_addr;
  task_creation_attrs_t attrs;
  per_task_data_t *ptd;
  thread_local_data_t *tld;

  if( !thread || !start_routine ) {
    return EINVAL;
  }

  stack_addr = mmap(NULL, STACK_PAGES * PAGE_SIZE, PROT_READ | PROT_WRITE,
                    MAP_ANON | MAP_STACK | MAP_PRIVATE, NOFD, 0);
  if( stack_addr == MAP_FAILED ) {
    return EAGAIN;
  }

  __pthread_attrs_to_system_attrs(attr,&attrs);

  /* Allocate and initialize new thread private data. */
  stack_addr-=sizeof(thread_local_data_t);
  tld=(thread_local_data_t*)stack_addr;
  /* TODO: [mt] initialize new TLD properly. */

  /* Allocate new per-task data. */
  stack_addr-=sizeof(per_task_data_t);
  ptd=(per_task_data_t *)stack_addr;

  ptd->process_global_data=PTD()->process_global_data;
  /* Initialize new per-task data. */
  ptd->ptd_base=(uintptr_t)ptd;
  ptd->process_global_data=PTD()->process_global_data;
  ptd->task_private_data=tld;
  ptd->_errno=0;  /* TODO: [mt] should we initialize errno explicitely ? */

  /* Insufficient return address on top of task's stack. */
  stack_addr-=8;

  /* Setup common thread attributes. */
  attrs.task_attrs.run_immediately=__ATTR_ON;
  attrs.exec_attrs.entrypoint=(uintptr_t)__pthread_gen_entrypoint;
  attrs.exec_attrs.destructor=(uintptr_t)__pthread_gen_destructor;
  attrs.exec_attrs.stack=(uintptr_t)stack_addr;
  attrs.exec_attrs.arg1=(uintptr_t)start_routine;
  attrs.exec_attrs.arg2=(uintptr_t)arg;
  attrs.exec_attrs.per_task_data=(uintptr_t)ptd;

  pid=__syscall2(SC_CREATE_TASK,CLONE_MM|CLONE_IPC|CLONE_SIGINFO,
                 (syscall_arg_t)&attrs);
  if( pid > 0 ) {
    thread->__tid=pid;
    return 0;
  }

  return POSIX_ERR(r);
}
