/*
 * This library is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU Lesser General Public License as 
 * published by the Free Software Foundation; either version 2.1 of 
 * the License.
 *
 * This library is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public 
 * License along with this library; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 * 
 * (c) Copyright 2006,2007,2008 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2008 Dmitry Gromada <gromada@jarios.org>
 *
 * event locked queue implementation
 */

#include <queue.h>
#include <errno.h>

void evl_queue_add(evl_queue_t *q, list_node_t *l)
{
  bool empty;

  pthread_mutex_lock(&q->mutex);
  empty = list_is_empty(&q->head);
  list_add2tail(&q->head, l);
  pthread_mutex_unlock(&q->mutex);
  if (empty)
    event_signal(&q->event);
}

list_node_t *evl_queue_next(evl_queue_t *q)
{
  list_node_t *l = NULL;

  while (1) {
    pthread_mutex_lock(&q->mutex);
    if (list_is_empty(&q->head)) {
      pthread_mutex_unlock(&q->mutex);
      if (event_wait(&q->event) == -EINTR)
        break;
    } else {
      l = list_node_first(&q->head);
      list_delfromhead(&q->head);
      pthread_mutex_unlock(&q->mutex);
      break;
    }
  }

  return l;
}

int init_evl_queue(evl_queue_t *q)
{
  pthread_mutexattr_t attr = { PTHREAD_PROCESS_PRIVATE };

  list_init_head(&q->head);
  event_init(&q->event);

  return -pthread_mutex_init(&q->mutex, &attr);
}
