/*
 * This library is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU Lesser General Public License as 
 * published by the Free Software Foundation; either version 2.1 of 
 * the License.
 *
 * This library is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public 
 * License along with this library; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 *
 * (c) Copyright 2006,2007,2008,209 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2009 Michael Tsymbalyuk <mtzaurus@gmail.com>
 *
 * LibOS:
 * general/libsys/dma.c: DMA-related functions.
 *
 */

#include <sys/types.h>
#include <sys/dma.h>
#include <sys/syscall.h>
#include <sys/mman.h>
#include <errno.h>

void *dma_alloc_pages(int numpages)
{
  uintptr_t phys_addr=__syscall1(SC_ALLOC_DMA,numpages);

  if( PAGE_ALIGN(phys_addr) ) {
    return (void *)phys_addr;
  }
  errno=-((long)phys_addr);
  return MAP_FAILED;
}

void dma_free_pages(void *addr,int numpages)
{
  __syscall2(SC_FREE_DMA,(syscall_arg_t)addr,numpages);
}

void *dma_alloc_consistent(size_t size,dma_addr_t *dma_handle)
{
  int pages=PAGE_ALIGN(size)>>PAGE_WIDTH;
  void *pa=dma_alloc_pages(pages);
  void *va;

  if( pa != MAP_FAILED ) {
    va=mmap(NULL,PAGE_ALIGN(size),PROT_READ | PROT_WRITE,MAP_PHYS|MAP_PRIVATE,
            NOFD,(long)pa);
    if( va != MAP_FAILED ) {
      *dma_handle=(dma_addr_t)pa;
      return va;
    }
  }

  return NULL;
}

