/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * (c) Copyright 2006,2007,2008 Jari OS Core Team <http://jarios.org>
 * (c) Copyright 2008 Dmitry Gromada <gromada82@gmail.com>
 *
 * LibOS:
 * lib.irq.c: irq control implementation
 */

/* TO DO: implement all API */

#include <sys/types.h>
#include <errno.h>
#include <string.h>
#include <unistd.h>
#include <time.h>

#include <devio/devio.h>
#include <devio/irq.h>
#include <devio/irqioctl.h>
#include <ipc/rpc.h>
#include <sys/ioctl.h>

static int __send_irqcon_request(int request, irq_ctl_t *ctl)
{  
  int r = 0;
  int fd;
  
  fd = arch_open_irqcon();
  if (fd < 0)
    return fd;
  
  r = ioctl(fd, request, ctl);
  
  /* fixme: RPC_CLOSE will be later pent to the target device, uncomment these lines */
  /*rpc->rpc_call = RPC_CLOSE;
    ipc_port_send(&cmdp, sizeof(rpc));*/
  
  return r;
}

int hw_set_irq_prio(uint16_t irq, uint16_t prio)
{
  irq_ctl_t ctl;

  if (prio >= NR_IRQ_VECTORS)
    return -EINVAL;
  
  ctl.irq = irq;
  ctl.vec = prio;
  
  return __send_irqcon_request(IRQIOC_SET_PRIO, &ctl);
}


int hw_mask_irq(uint16_t irq, int mask)
{
  irq_ctl_t ctl;
  
  ctl.irq = irq;
  ctl.mask = (mask != 0);
  
  return __send_irqcon_request(IRQIOC_MASK, &ctl);
}
