/* This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * Copyright (C) 2006,2007,2008,2009 Jari OS Core Team <http://jarios.org>
 * Copyright (C) 2009 Dmitry Balakshin <balakshin@jarios.org>
 *
 * libsys:
 *     service.c - auxiliary functions to use in system
 *     services/devices/filesystems etc.
 */

#include <sys/types.h>
#include <unistd.h>
#include <stdlib.h>
#include <signal.h>
#include <service.h>

int service_start_init(service_start_t *sstart, int signal)
{
  sstart->btsd_pid = getppid();
  sstart->pid      = getpid();
  sstart->chld_pid = -1;
  sstart->signal   = signal;

  if (sigemptyset(&sstart->set) || sigaddset(&sstart->set, signal)) {
    klog("Can't set service wokeup signal for pid (%d).\n",
         sstart->pid);
    return 1;
  }

  if (sigprocmask(SIG_BLOCK, &sstart->set, NULL)) {
    klog("Can't block wokeup signal for pud (%d).\n",
         sstart->pid);
    return 1;
  }

  return 0;
}

int service_start_finished(service_start_t *sstart)
{
  if (kill(sstart->pid, sstart->signal) == -1) {
    klog("Failed to send signal to parent pid (%d).\n",
         sstart->pid);
    return 1;
  }

  return 0;
}

void launch_service(service_start_t *sstart)
{
  pid_t pid;
  siginfo_t sinfo;
  int ret, sig = -1;

  sstart->chld_pid = fork();
  if (sstart->chld_pid < 0)
    abort();
  if (sstart->chld_pid != 0) {
    ret = sigwaitinfo(&sstart->set, &sinfo);
    if (ret == -1)
      exit(EXIT_FAILURE);
    if (sinfo.si_pid != sstart->chld_pid) {
      klog("Signal sender pid is not what we waitning for.\n"
           "(child %d = recv %d).\n",
           sstart->chld_pid, sinfo.si_pid);
      exit(EXIT_FAILURE);
    }

    exit(EXIT_SUCCESS);
  }
}

/* Temprorary stub */

void start_service(void)
{
  pid_t f;

  f = fork();
  if (f < 0) abort();
  if (f != 0)
    exit(EXIT_SUCCESS);
}
